
/**
 * Starts examples of the application of the <a href="http://www.ai-center.com/projects/dragonbreath/"><font face="Times">DragonBreath</font> engine</a>.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.6 $
 */


import java.io.*;

import com.ai_center.dragonbreath.Registry;


public class StartExamples {
    public static void main(String[] args) {

        System.out.println("\n***** DragonBreath Engine - Version "
                           + Registry.VERSION + "\n");

        int selection;

        if (args.length == 1) {

            System.out.println("Using command-line parameter for example choice...");

            try {
                selection = (new Integer(args[0])).intValue();
            } catch (NumberFormatException e) {
                selection = -1;
            }

        } else {

            // Printing choice options

            System.out.println("Please choose one of the following examples:\n");
            System.out.println("  1 Interactive use of the engine");
            System.out.println("  2 Problem specification basics");
            System.out.println("  3 Scheduling");
            System.out.println("  4 Demo of the Task Constraint");
            System.out.println("  5 Structural Constraints");
            System.out.print("\nYour choice: ");

            // User input

            final int maxIndex = 100;
            byte[] byteInput = new byte[maxIndex];
            int length = 0;
            try {
                length = System.in.read(byteInput);
            } catch (IOException e) {
                e.printStackTrace();
            }

            // Eat leading spaces

            int offset = 0;
            while (byteInput[offset] == 32) {
                offset++;
            }

            // Check length

            int count = offset;
            while ((count < length)
                   && (byteInput[count] >= '0') && (byteInput[count] <= '9')) {
                count++;
            }
            count -= offset;

            // Retrieving user selection

            if (count > 0) {
                try {
                    selection = (new Integer(new String(byteInput, offset, count))).intValue();
                } catch (NumberFormatException e) {
                    selection = -1;
                }
            } else {
                selection = -1;
            }

        }

        if (selection < 1) {
            System.out.println("\nERROR: Invalid input (the example's number is required)!");
        } else {

            // Executing user selection

            System.out.println();
            System.out.println("Trying to start example " + selection + "...\n");

            switch (selection) {
            case 1: {
                com.ai_center.dragonbreath.examples.interaction.Guide guide = new com.ai_center.dragonbreath.examples.interaction.Guide();
                break;
            }
            case 2: {
                com.ai_center.dragonbreath.examples.specificationbasics.Guide guide = new com.ai_center.dragonbreath.examples.specificationbasics.Guide();
                break;
            }
            case 3: {
                com.ai_center.dragonbreath.examples.scheduling.Guide guide = new com.ai_center.dragonbreath.examples.scheduling.Guide();
                break;
            }
            case 4: {
                com.ai_center.dragonbreath.examples.taskconstraintdemo.Guide guide = new com.ai_center.dragonbreath.examples.taskconstraintdemo.Guide();
                break;
            }
            case 5: {
                com.ai_center.dragonbreath.examples.structuralConstraintExample.Guide guide = new com.ai_center.dragonbreath.examples.structuralConstraintExample.Guide();
                break;
            }
            default:
                System.out.println("ERROR: Invalid input (the example's number is required)!");
            }
        }
        System.out.println();

    }
}
