package com.ai_center.dragonbreath;


import javax.swing.event.ChangeListener;




/** The RealTimeControl controls the planning speed.<p> 
   
  The RealTimeControl interface manages the communication
  from the GlobalSearchControl to the RealTimeControl and from
  the user interface to the RealTimeControl. <p>
  
    @author Daniel Weiss
  @version $$
 */
public interface RealTimeControl
{

    /** Single step mode. */
    public final int STEPPING = 1;

    /** Pause, wait for user interaction. */
    public final int PAUSE = STEPPING;

    /** Play mode with some milliseconds pause between two breakpoints. */
    public final int PLAY = 2;

    /** Fastforward mode. */
    public final int FASTFORWARD = 3;


    
    /** Adds a ChangeListener to the RealTimeControl.<p>
     
        All ChangeListeners get informed about mode changes.
       @param l
     */
    public void addChangeListener(ChangeListener l);

    /** This function sets a breakpoint and informs all RealTimeControl
       listeners.<p>
       @param breakpointID the id of the breakpoint, specified in the Registry.
       @param changedObject The last changed object to highlight on screen.
     */
    public void beakpoint(int breakpointNumber, Object changedObject);

    /** Returns a list of all breakpoint names.<p>
     
       @return A list of all breapint names.
     */
    public String[] getBreakpoints();

    /** Gets the state of the give breakpoint.<p>
     
       @param breakpointNumber The number of the breakpoint to query.
       @return True if this breakpoint is active else false.
     */
    public boolean getBreakpointState(int breakpointNumber);

    public int getMode();

    public int getPlayPauseTime();

    /**
     * Continiues the execution of the dragonbreath engine to the next breakpoint.
     *
     */
    public void next();

    public void removeChangeListener(ChangeListener l);

    public void setBeakpointActive(int breakpointNumber, boolean _active);

    /**
     * Sets the break-mode. 
     * @param mode i.e. RealTimeControl.STEPPING, .PLAY or .FASTFORWARD
     */
    public void setMode(int mode);

    /**
     * Sets the time to wait between two ATOMIC_CHANGE events.
     * @param pause pause time in milliseconds.
     */
    public void setPlayPauseTime(int pause);
}