
package com.ai_center.dragonbreath.constraintgraph;

/** 
 * Represents a connected node (via an edge; type or instance).
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.2 $
 * 
 * @see dragonbreath
 */


public class ConnectedNode {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /** 
     * Type of the connected node.
     * @see Registry
     */
    public int nodeTypeID;

    /** 
     * The connected node. This is a variable or a
     * constraint.
     */
    public Node node = null;

    /** 
     * Type of the edge.
     * @see Registry
     */
    public int edgeID;

    /** 
     * Direction of the edge WRT to the other node which is not represented 
     * in this structure.
     * @see Registry
     */
    public boolean direction;

    /**
     * Every edge between two nodes is represented by two ConnectedNodes;
     * for every direction one.
     * This is the link to the other one.
     */
    public ConnectedNode other = null;
    

    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /** 
     * Creates a connection instance.
     * @param node  The connected node.
     * @param nodeTypeID  Type of the connected node.
     * @param edgeID  Type of the edge.
     * @param direction  Direction of the edge.
     * @see Registry
     */
    public ConnectedNode(Node node, int nodeTypeID, int edgeID, 
			 boolean direction) {
	this.nodeTypeID = nodeTypeID;
	this.node = node;
	this.edgeID = edgeID;
	this.direction = direction;
    }

    /** 
     * Creates a connection type.
     * @param nodeTypeID  Type of the connected node.
     * @param edgeID  Type of the edge.
     * @param direction  Direction of the edge.
     * @see Registry
     */
    public ConnectedNode(int nodeTypeID, int edgeID, boolean direction) {
	this.nodeTypeID = nodeTypeID;
	this.edgeID = edgeID;
	this.direction = direction;
    }
    
    /** 
     * Creates a connection after the given example.
     * @param example  The example this ConnectedNode should be a copy of.
     * @see Registry
     */
    public ConnectedNode(ConnectedNode example) {
	this.nodeTypeID = example.nodeTypeID;
	this.edgeID = example.edgeID;
	this.direction = example.direction;
        this.node = example.node;
    }

    /**
     * Tests wheter the types of the two ConnectedNodes are the same,
     * i.e. nodeTypeID, edgeID and direction are compared.
     * @param ther
     */
    public boolean isSameType(ConnectedNode other) {
        return     (this.nodeTypeID == other.nodeTypeID)
                && (this.edgeID == other.edgeID)
                && (this.direction == other.direction);
    }
    
}
