
package com.ai_center.dragonbreath.constraintgraph;

/** 
 * Graphical interface for a conventional constraint. 
 * 
 * @author Alexander Nareyek
 * @version $Revision: 2.0 $ 
 */


import java.awt.*;
import java.awt.event.*;
import java.math.BigInteger;
import javax.swing.*;

import com.ai_center.dragonbreath.constraints.IntegerConstraint;
import com.ai_center.dragonbreath.constraints.RangeConstraint;
import com.ai_center.dragonbreath.costs.CostFunctionReference;
import com.ai_center.dragonbreath.GlobalSearchControl;


public class ConventionalConstraintGraphics extends NodeGraphics {
    
    // **********************
    // ***** ATTRIBUTES *****
    // **********************
    
    /** 
     * The Constraint reference.
     */
    private final ConventionalConstraint constraint;
    /** 
     * Displays the constraint's costs.
     */
    public JLabel valueLabel;
    /** 
     * The main panel window.
     */
    public JPanel mainPanel;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /** 
     * Creates basic graphical objects.
     * @param constraint_ref  The related (specialized) 
     *                        ConventionalConstraint object.
     * @param gsc_ref The related GlobalSearchControl object.
     */
    public ConventionalConstraintGraphics(ConventionalConstraint 
					  constraint_ref, 
					  GlobalSearchControl gsc_ref) {

	constraint = constraint_ref;
	gsc = gsc_ref;

	setBackground(Color.cyan);
	setBorder(BorderFactory.createCompoundBorder(BorderFactory.createRaisedBevelBorder(), BorderFactory.createEmptyBorder(5, 5, 5, 5)));
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	JPanel p;
	p = new JPanel();
	p.setLayout(new BorderLayout());
	activator = new JButton(constraint.getTypeName());
	activator.setMinimumSize(new Dimension(100, 20));
        activator.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    gsc.graphObject.constraintButtonPressed(constraint);
		}});
	activator.setEnabled(false);
	p.add(activator, BorderLayout.CENTER);
	add(p);
	add(Box.createRigidArea(new Dimension(0, 2)));

	mainPanel = new JPanel();
	mainPanel.setLayout(new BorderLayout());
	mainPanel.setBackground(Color.cyan);
	valueLabel = new JLabel();
	valueLabel.setHorizontalAlignment(SwingConstants.CENTER);
	if ((constraint instanceof IntegerConstraint) || 
	    (constraint instanceof RangeConstraint)) {
	    valueLabel.setText("(hard)");
	    valueLabel.setForeground(Color.lightGray);
	} else {
	    updateCosts();
	}
	mainPanel.add(valueLabel, BorderLayout.CENTER);
	add(mainPanel);

    }


    // *******************
    // ***** METHODS *****
    // *******************

    public void updateCosts() {

	// Retrieving costs to be displayed

	BigInteger costs = new BigInteger("0");
	if (gsc.lastUnsatisfiedCostFunction != null) {

	    int level = gsc.lastUnsatisfiedCostFunction.level;
	    int maxID = constraint.getNumberOfCosts();
	    CostFunctionReference cfr;

	    for (int i = 0; i < maxID; i++) {
		cfr = constraint.getCostFunctionReference(i);
		if (cfr != null) {
		    if (cfr.costFunction.level == level) {
			costs = costs.add(cfr.objectiveCosts);
		    }
		}
	    }
	}

	// Displaying costs

	if (!(constraint instanceof IntegerConstraint) &&
	    !(constraint instanceof RangeConstraint)) {
	    if (!costs.equals(BigInteger.ZERO)) {
		valueLabel.setForeground(Color.red);
	    } else {
		valueLabel.setForeground(Color.lightGray);
	    }
	    valueLabel.setText("Costs = " + costs);
	}
    }

}


/* 
 * DO _NOT_ EDIT ANYTHING BELOW THIS POINT! 
 * 
 * $Log: ConventionalConstraintGraphics.java,v $
 * Revision 2.0  2002/09/17 00:33:06  nareyek
 * work on beta version began: resource type, state tasks and precondition tasks introduced
 *
 * Revision 1.3  2002/05/22 21:59:12  nareyek
 * constraints display now the costs of the current improvement level
 *
 * Revision 1.2  2002/04/09 01:19:37  nareyek
 * Every cost item can now have a mapping; constraints can have multiple costs
 *
 * Revision 1.1  2002/04/04 09:25:36  nareyek
 * general constraint interface included; cost workflow improved
 * 
 * 
 */ 
