
package com.ai_center.dragonbreath.constraintgraph;

/** 
 * Represents a node (variable or constraint; type or instance) of the CSP. 
 * 
 * @author Alexander Nareyek
 * @version $Revision: 2.4 $
 */

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

import org.jdom.Element;
import com.ai_center.dragonbreath.GlobalSearchControl;


public class Node extends NodeType {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

  
    /** 
     * Indicates whether this is a node type or node instance.
     */
    public boolean isType = true;

    /** 
     * The Global Search Control reference.
     */
    public GlobalSearchControl gsc = null;

    /** 
     * The node's connections.
     */
    public Connections connections = null;

    /** 
     * Panel for visual tracking. A value of "null" means that the graphics 
     * are not turned on and no graphical information has been provided for
     * the node. An object type of NodeGraphicsCache means that the graphics 
     * are not turned on but graphical information has been provided and is
     * stored in this object. An object type of NodeGraphics means that the
     * the graphics are turned on and the object represents the graphical
     * representation of the node.
     */
    public Object graphObject = null;

    /** A list of EventListeners, which get informed about changes. */
    private EventListenerList listenerList = null;

    
    
    // *******************
    // ***** METHODS *****
    // *******************

    /**
     * When a Constraint is loaded or created via structuralConstraint-repair
     * this function is called with the stored data for the node.
     * If a Constraint wants to store special data, as the ranges for RangeConstraints
     * or the Variable value for Variables, it has to do it in the getAuxiliaryNodeData() function, and restore
     * it in the setAuxiliaryNodeData() function.
     * The parameter is a jdom xml-Element.
     * To get the integer parameter "value" you would call nodeData.getAttribute("value").getIntValue();
     * See in the jdom documentation for more information. www.jdom.org/docs/apidocs/
     * @param nodeData  The nodeData as stored in file. 
     */
    public void setAuxiliaryNodeData(Element nodeData) {
    }
    

    /**
     * Before storing a node this function is called, and the node can add its own data to the XML-data.
     * To set the parameter "value" to 2 you would call nodeData.setAttribute("value", "2");
     * See in the jdom documentation for more information. www.jdom.org/docs/apidocs/
     * @param nodeData  Where the data should be stored.
     */
    public void getAuxiliaryNodeData(Element nodeData) {
    }
    
    /**
     * Adds a mark to the node's graphical representation.
     */
    public void mark() {
        if (graphObject instanceof NodeGraphics) {
            ((NodeGraphics) graphObject).mark();
        }
    }
    
    /**
     * Removes a mark from the node's graphical representation.
     */
    public void unmark() {
        if (graphObject instanceof NodeGraphics) {
            ((NodeGraphics) graphObject).unmark();
        }
    }
    /** Adds a ChangeListener to observe this object.<p>
    
    @param l The ChangeListener to add.
    */
   public void addChangeListener(ChangeListener l)
   {
       if (listenerList == null)
       {
           listenerList = new EventListenerList();
       }
       listenerList.add(ChangeListener.class, l);
   }

   /** Removes a ChangeListener.<p>
    
    @param l The ChangeListener to be removed.
    */
   public void removeChangeListener(ChangeListener l)
   {
       if (listenerList != null)
       {
           listenerList.remove(ChangeListener.class, l);
       }
   }

   /** Informs all ChangeListeners about a change.
     @param changedObject The changed object.
    */
   protected void fireChange(Object changedObject)
   {
       if (listenerList != null)
       {

           ChangeEvent simpleChangeEvent = new ChangeEvent(changedObject);


           Object[] listeners = listenerList.getListenerList();
           ChangeListener listener = null;

           for (int i = listeners.length - 2; i >= 0; i -= 2)
           {
               if (listeners[i] == ChangeListener.class)
               {

                   ((ChangeListener) listeners[i + 1])
                           .stateChanged(simpleChangeEvent);

               }

           }
       }
   }

}


/* 
 * DO _NOT_ EDIT ANYTHING BELOW THIS POINT! 
 * 
 * $Log: Node.java,v $
 * Revision 2.4  2004/05/06 22:49:37  weissd
 * Documentation updated
 *
 * Revision 2.3  2004/04/07 12:53:37  weissd
 * Added capability to observe the datastructures
 *
 * Revision 2.2  2003/02/21 16:55:23  teutates
 * improved auxiliary data load for nodes
 *
 * Revision 2.1  2003/02/21 01:32:59  teutates
 * structural repair initial release
 *
 * Revision 2.0  2002/09/17 00:33:06  nareyek
 * work on beta version began: resource type, state tasks and precondition tasks introduced
 *
 * Revision 1.6  2002/08/28 07:52:53  nareyek
 * engine can now also be run without a graphical display
 *
 * Revision 1.5  2002/08/07 16:19:28  teutates
 * added structural constraint access from GSCGraphics
 *
 * Revision 1.4  2002/07/25 01:23:09  teutates
 * at last..
 * a new version of the dockingGraph matching algorithm.
 *
 * Revision 1.3  2002/06/01 01:12:28  nareyek
 * initial cost type selection for graphics added
 *
 * Revision 1.2  2002/04/09 23:17:56  teutates
 * NodeType as parentclass for Node added
 *
 * Revision 1.1  2002/04/04 04:55:58  nareyek
 * directories costs and constraintgraph introduced
 *
 * Revision 1.7  2002/04/04 03:04:58  nareyek
 * new constraint directory; VariableComparator removed and integrated into Variable
 *
 * Revision 1.6  2002/02/04 18:31:28  teutates
 * connection between two representations of same edge added
 *
 * Revision 1.5  2001/12/18 16:04:09  nareyek
 * Minor improvements of the previous change
 *
 * Revision 1.4  2001/12/18 15:02:54  teutates
 * anyNode replaced by non abstract Node class
 *
 * Revision 1.3  2001/12/12 14:40:12  teutates
 * functions mark()
 * and unmark() for tracking added.
 *
 * Revision 1.2  2001/11/22 18:55:02  nareyek
 * Major Registry revision: Dynamic constraint class addition; Registry is now a static object only
 * 
 * 
 */ 
