
package com.ai_center.dragonbreath.constraintgraph;

/** 
 * Graphical interface for a variable. 
 * 
 * @author Alexander Nareyek
 * @version 0.1
 */


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import com.ai_center.dragonbreath.GlobalSearchControl;


public class VariableGraphics extends NodeGraphics {
    
    // **********************
    // ***** ATTRIBUTES *****
    // **********************
    
    /** 
     * The Variable reference.
     */
    private final Variable v;

    /** 
     * Displays the variable's value.
     */
    public JLabel valueLabel;

    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /** 
     * Creates basic graphical objects.
     * @param v_ref  The related Variable object.
     * @param gsc_ref The related GlobalSearchControl object.
     */
    public VariableGraphics(Variable v_ref, GlobalSearchControl gsc_ref) {

	v = v_ref;
	gsc = gsc_ref;

	setBackground(Color.yellow);
	setBorder(BorderFactory.createCompoundBorder(BorderFactory.createRaisedBevelBorder(), BorderFactory.createEmptyBorder(5, 5, 5, 5)));
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	JPanel p;
	p = new JPanel();
	p.setLayout(new BorderLayout());
	activator = new JButton(v.getName());
	activator.setMinimumSize(new Dimension(100, 20));
        activator.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    gsc.graphObject.variableButtonPressed(v);
		}});
	activator.setEnabled(false);
	p.add(activator, BorderLayout.CENTER);
	add(p);
	add(Box.createRigidArea(new Dimension(0,2)));

	p = new JPanel();
	p.setLayout(new BorderLayout());
	p.setBackground(Color.yellow);
	if (v.integerConstraints != 0) {
	    valueLabel = new JLabel("Int = " + v.integerValue);
	} else {
	    valueLabel = new JLabel("Real = " + v.realValue);
	}
	valueLabel.setHorizontalAlignment(SwingConstants.CENTER);
	p.add(valueLabel, BorderLayout.CENTER);
	add(p);

    }
    
    
    
    // *******************
    // ***** METHODS *****
    // *******************

    /**
     * Sets the variables name.
     */
    public void updateVariableName(String newName) {
        activator.setText(newName);
    }
    
    /** 
     * Updates the display of the variable's value.
     */
    public void updateValue() {
	if (v.integerConstraints != 0) {
	    valueLabel.setText("Int = " + v.integerValue);
	} else {
	    valueLabel.setText("Real = " + v.realValue);
	}
    }

}
