
package com.ai_center.dragonbreath.constraints;

/** 
 * An object constraint representing an action, involving precondition tasks,
 * action tasks, state tasks, and indicating the action's type by an action 
 * type variable.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.2 $
 * 
 * @see Variable
 * @see PreconditionTaskObjectConstraint
 * @see ActionTaskObjectConstraint
 * @see StateTaskObjectConstraint
 */


import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.Registry;


public class ActionObjectConstraint extends ObjectConstraint {


    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /** 
     * The constraint type's name.
     */
    private static final String TYPE_NAME = new String("Action");

    /** 
     * The constraint's base graph.
     */
    private static final Embedding BASE_GRAPH = new Embedding(1);

    /** 
     * The constraint's number of extension graphs.
     */
    private static final int EXTENSION_GRAPH_NUMBER = 3;
    /** 
     * The constraint's extension graphs.
     */
    private static final Embedding[] EXTENSION_GRAPH = 
	new Embedding[EXTENSION_GRAPH_NUMBER];


    // *******************
    // ***** METHODS *****
    // *******************


    public void create_graphs() {

	// Creating the base graph

	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.ACTIONTYPE_EDGE, 
				    Registry.INCOMING);

	// Creating the extension graphs
        
	EXTENSION_GRAPH[0] = new Embedding(1);
	EXTENSION_GRAPH[0].addConnectedNode(Registry.PRECONDITIONTASK_OBJECTCONSTRAINT, Registry.PLAIN_EDGE, Registry.INCOMING);

	EXTENSION_GRAPH[1] = new Embedding(1);
	EXTENSION_GRAPH[1].addConnectedNode(Registry.ACTIONTASK_OBJECTCONSTRAINT, Registry.PLAIN_EDGE, Registry.INCOMING);

	EXTENSION_GRAPH[2] = new Embedding(1);
	EXTENSION_GRAPH[2].addConnectedNode(Registry.STATETASK_OBJECTCONSTRAINT, Registry.PLAIN_EDGE, Registry.INCOMING);

    }


    public String getTypeName() {
	return TYPE_NAME;
    }


    public Embedding getBaseGraph() {
	return BASE_GRAPH;
    }


    public int getExtensionGraphNumber() {
	return EXTENSION_GRAPH_NUMBER;
    }


    public Embedding getExtensionGraph(int graphID) {
	return EXTENSION_GRAPH[graphID];
    }


    public int getMaximalConnectionNumber() {
	/* There is no maximum for extensions */
	return Integer.MAX_VALUE;
    }


}
