
package com.ai_center.dragonbreath.constraints;

/** 
 * An object constraint representing an action task with a temporal begin
 * and duration variable, an operation variable, a consumption variable, 
 * a resource type variable, and an execution state variable.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.3 $
 * 
 * @see Variable
 */


import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.Registry;


public class ActionTaskObjectConstraint extends ObjectConstraint {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /** 
     * The constraint type's name.
     */
    private static final String TYPE_NAME = new String("Action Task");

    /** 
     * The constraint's base graph.
     */
    private static final Embedding BASE_GRAPH = new Embedding(6);


    // *******************
    // ***** METHODS *****
    // *******************

    public void create_graphs() {

	// Creating the base graph

	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.RESOURCETYPE_EDGE, 
				    Registry.INCOMING);
	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.OPERATION_EDGE, 
				    Registry.INCOMING);
	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.BEGIN_EDGE, Registry.INCOMING);
	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.DURATION_EDGE, Registry.INCOMING);
	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.CONSUMPTION_EDGE, 
				    Registry.INCOMING);
	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.EXECUTIONSTATE_EDGE, 
				    Registry.INCOMING);
    }

    public String getTypeName() {
	return TYPE_NAME;
    }

    public Embedding getBaseGraph() {
	return BASE_GRAPH;
    }

    public int getExtensionGraphNumber() {
	return 0;
    }

    public Embedding getExtensionGraph(int graphID) {
	return null;
    }

    public int getMaximalConnectionNumber() {
	/* There is no maximum for extensions */
	return Integer.MAX_VALUE;
    }

}
