
package com.ai_center.dragonbreath.constraints;


import java.lang.Math;

import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.Registry;


/**
 * This unary constraint forces the affected variable to
 * take an odd value.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.2 $
 * @see Variable
 */
public class OddConstraint extends ConventionalConstraint
{


    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /**
     * The constraint type's name.
     */
    private static final String TYPE_NAME = new String( "Odd" );

    /**
     * The identifier for the constraint's satisfaction-related cost
     * component.
     */
    public static final int ODD_INCONSISTENCY = 0;
    /**
     * Descriptions of the cost components.
     */
    public static String[] costDescription = new String[ 1 ];
    static
    {
        costDescription[ ODD_INCONSISTENCY ] = new String( "inconsistency" );
    }

    /**
     * The constraint's base graph.
     */
    private static final Embedding BASE_GRAPH = new Embedding( 1 );

    /**
     * The affected variable.
     */
    private Variable variable = null;


    // *******************
    // ***** METHODS *****
    // *******************


    public void create_graphs()
    {

        // Creating the base graph

        BASE_GRAPH.addConnectedNode( Registry.VARIABLE ,
            Registry.PLAIN_EDGE , Registry.INCOMING );

    }


    public int getNumberOfCosts()
    {
        return 1;
    }


    public String getCostName( int costID )
    {
        if ( costID < 1 )
        {
            return costDescription[ costID ];
        }
        else
        {
            throw new Error( "Invalid cost ID" );
        }
    }


    public String getTypeName()
    {
        return TYPE_NAME;
    }


    public Embedding getBaseGraph()
    {
        return BASE_GRAPH;
    }


    public int getExtensionGraphNumber()
    {
        return 0;
    }


    public Embedding getExtensionGraph( int graphID )
    {
        return null;
    }


    public int getMaximalConnectionNumber()
    {
        // Making the constraint non-extensible
        return BASE_GRAPH.connectedNodeNumber;
    }


    public void initializeCostType( int costID )
    {
        // Nothing to do
    }


    /**
     * Calculates the constraint's costs.
     */
    private void updateCosts()
    {
        long intValue = Math.round( variable.value() );
        if ( ( intValue == variable.value() ) && ( ( intValue % 2 ) != 0 ) )
        {
            costs[ ODD_INCONSISTENCY ] = 0;
        }
        else
        {
            costs[ ODD_INCONSISTENCY ] = 1;
        }
        costFunctionReference[ ODD_INCONSISTENCY ].updateGlobalCosts();
    }


    public void update( double oldValue , ConnectionReference first ,
        ConnectionReference last )
    {
        updateCosts();
    }


    public void connectNode( ConnectionReference first ,
        ConnectionReference last )
    {
        this.variable = (Variable)first.node;
        updateCosts();
    }


    public void disconnectNode( ConnectionReference first ,
        ConnectionReference last )
    {
        this.variable = null;
        costs[ ODD_INCONSISTENCY ] = 0;
        costFunctionReference[ ODD_INCONSISTENCY ].updateGlobalCosts();
    }


    public void improveCosts( int costID )
    {

        // Checking for valid cost ID

        if ( costID != ODD_INCONSISTENCY )
        {
            throw new Error( "The cost ID  \"" + costID
                + "\" is not supported by this constraint" );
        }

        if ( !variable.isConstant() )
        {
            long intValue = Math.round( variable.value() );
            if ( ( intValue != variable.value() )
                && ( ( intValue % 2 ) != 0 ) )
            {
                gsc.setVariable_internal( variable , intValue );
            }
            else
            {
                int shift;
                if ( gsc.random.nextInt( 2 ) == 0 )
                {
                    shift = -1;
                }
                else
                {
                    shift = 1;
                }
                gsc.setVariable_internal( variable , intValue + shift );
            }
        }

    }


}
