
package com.ai_center.dragonbreath.constraints;

/** 
 * An object constraint representing a precondition task with a temporal 
 * reference variable, a state variable and a resource type variable.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.4 $
 * 
 * @see Variable
 */


import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.Registry;


public class PreconditionTaskObjectConstraint extends ObjectConstraint {


    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /** 
     * The constraint type's name.
     */
    private static final String TYPE_NAME = new String("Precondition Task");

    /** 
     * The constraint's base graph.
     */
    private static final Embedding BASE_GRAPH = new Embedding(3);


    // *******************
    // ***** METHODS *****
    // *******************


    public void create_graphs() {

	// Creating the base graph

	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.RESOURCETYPE_EDGE, 
				    Registry.INCOMING);
	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.TEMPORALREFERENCE_EDGE, 
				    Registry.INCOMING);
	BASE_GRAPH.addConnectedNode(Registry.VARIABLE, 
				    Registry.STATE_EDGE, 
				    Registry.INCOMING);
    }


    public String getTypeName() {
	return TYPE_NAME;
    }


    public Embedding getBaseGraph() {
	return BASE_GRAPH;
    }


    public int getExtensionGraphNumber() {
	return 0;
    }


    public Embedding getExtensionGraph(int graphID) {
	return null;
    }


    public int getMaximalConnectionNumber() {
	/* There is no maximum for extensions */
	return Integer.MAX_VALUE;
    }


}
