
package com.ai_center.dragonbreath.constraints;

/** 
 * Graphical interface for a range constraint. 
 * 
 * @author Alexander Nareyek
 * @version $Revision: 2.0 $
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.GlobalSearchControl;


public class RangeConstraintGraphics extends ConventionalConstraintGraphics {
    
    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /** 
     * The related RangeConstraint object.
     */
    public RangeConstraint rangeConstraint;

    /** 
     * Displays the constraint's range.
     */
    public JLabel rangeLabel;
    /** 
     * The panel for the range display.
     */
    public JPanel rangePanel;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /** 
     * Creates basic graphical objects.
     * @param constraint  The related RangeConstraint object.
     * @param gsc The related GlobalSearchControl object.
     */
    public RangeConstraintGraphics(RangeConstraint constraint, 
				   GlobalSearchControl gsc) {
	super(constraint, gsc);
	rangeConstraint = constraint;

	add(Box.createRigidArea(new Dimension(0,2)));
	rangePanel = new JPanel();
	rangePanel.setLayout(new BorderLayout());
	rangePanel.setBackground(Color.cyan);
	rangeLabel = new JLabel();
	updateRange();
	rangeLabel.setHorizontalAlignment(SwingConstants.CENTER);
	rangePanel.add(rangeLabel, BorderLayout.CENTER);
	add(rangePanel);

    }


    // *******************
    // ***** METHODS *****
    // *******************

    /** 
     * Updates the display of the constraint's range.
     */
    public void updateRange() {
	if (rangeConstraint.lowerBound != rangeConstraint.upperBound) {
	    String lowerRange;
	    if (rangeConstraint.lowerBound != Double.NEGATIVE_INFINITY) {
		lowerRange = 
		    new String("[" + rangeConstraint.lowerBound + ";");
	    } else {
		lowerRange = new String("]-inf" + ";");
	    }
	    String range;
	    if (rangeConstraint.upperBound != Double.POSITIVE_INFINITY) {
		range = 
		    new String(lowerRange + rangeConstraint.upperBound + "]");
	    } else {
		range = new String(lowerRange + "inf[");
	    }
	    rangeLabel.setText(range);
	} else {
	    rangeLabel.setText("Constant: " + rangeConstraint.lowerBound);
	}
    }

}


/* 
 * DO _NOT_ EDIT ANYTHING BELOW THIS POINT! 
 * 
 * $Log: RangeConstraintGraphics.java,v $
 * Revision 2.0  2002/09/17 00:33:06  nareyek
 * work on beta version began: resource type, state tasks and precondition tasks introduced
 *
 * Revision 1.3  2002/04/04 09:25:36  nareyek
 * general constraint interface included; cost workflow improved
 * 
 * 
 */ 
