
package com.ai_center.dragonbreath.constraints.discreteresourceconstraint;

/** 
 * An interval representation for a DiscreteResourceConstraint's projection
 * of its assignments.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.1 $
 */


import java.util.*;

import com.ai_center.dragonbreath.utilities.OrderedShiftDLLElement;


public class DRCInterval implements Cloneable {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /** 
     * The DRCTasks that are part of the interval.
     */
    public HashSet tasks = new HashSet();

    /** 
     * The number of tasks that are part of the interval.
     */
    public long claims = 0;

    /** 
     * The subjective capacity costs involved in this interval.
     */
    public long capacityCosts = 0;

    /**
     * The back link to the DiscreteResourceConstraint's 
     * capacityCostsIntervalList element.
     */
    public OrderedShiftDLLElement capacityCostsIntervalLE = null;

    /**
     * The back link to the DiscreteResourceConstraint's 
     * claimsIntervalList element.
     */
    public OrderedShiftDLLElement claimsIntervalLE = null;

    /** 
     * Variable that can be used by heuristics as short-term memory.
     * Should not be used after a heuristic is left because other 
     * heuristics may override this value.
     */
    public long tempValue = 0;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************
    
    /** 
     * Constructor that builds an empty task set.
     */
    public DRCInterval() {
	tasks = new HashSet();
    }
    
    /** 
     * Constructor that copies a task set.
     * @param tasks  The task set to be copied.
     */
    public DRCInterval(HashSet tasks) {
	this.tasks = (HashSet) tasks.clone();
    }

    
    // *******************
    // ***** METHODS *****
    // *******************
    
    /** 
     * Prints claims, the task list and the interval's costs.
     */
    public String toString() {
	return capacityCosts + "/" + claims + ":" + tasks.toString();
    }
    
    /** 
     * Returns a copy of the object.
     */
    public Object clone() {
	DRCInterval interval = new DRCInterval(tasks);
	interval.claims = claims;
	return interval;
    }

    /** 
     * Calculates the capacity costs.
     * @param duration  The interval's duration.
     * @param capacity  The available capacity.
     */
    public void calculateCapacityCosts(long duration, long capacity) {
	if (claims > capacity) {
	    capacityCosts = (claims - capacity) * duration;
	} else {
	    capacityCosts = 0;
	}
    }

}
