
package com.ai_center.dragonbreath.constraints.discreteresourceconstraint;

/** 
 * A DiscreteResourceConstraint's internal representation of a task that has
 * been assigned to it.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.3 $
 */


import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.ActionTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.DiscreteResourceConstraint;
import com.ai_center.dragonbreath.utilities.IntervalListElement;
import com.ai_center.dragonbreath.utilities.OrderedShiftDLLElement;


public class DRCTask {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************
    
    /** 
     * The related task constraint with the connection to the variables.
     */
    public ActionTaskObjectConstraint taskConstraint;

    /** 
     * The related variable for the begin of the task.
     */
    public Variable beginVar = null;
    /** 
     * The internally stored value of the task's begin. Is updated when
     * beginVar changes and can be used for faster value access.
     */
    public long begin = 0;

    /** 
     * The related variable for the duration of the task.
     */
    public Variable durationVar = null;
    /** 
     * The internally stored value of the task's duration. Is updated when
     * durationVar changes and can be used for faster value access.
     */
    public long duration = 0;

    /** 
     * The end of the task. Is updated when begin or duration changes.
     */
    public long end = 0;

    /** 
     * The related variable for the consumption of the task.
     */
    public Variable consumptionVar = null;
    /** 
     * The internally stored value of the task's consumption.
     * Necessary for update situations.
     */
    public long consumption = 0;

    /** 
     * The related variable for the execution state of the task.
     */
    public Variable executionStateVar = null;
    /** 
     * The internally stored value of the task's execution state.
     */
    public int executionState = DiscreteResourceConstraint.PENDING;

    /** 
     * The related variable for the resource type of the task.
     */
    public Variable resourceTypeVar = null;

    /** 
     * The related variable for the operationID of the task.
     */
    public Variable operationVar = null;

    /** 
     * The first list interval element which contains this task.
     */
    public IntervalListElement firstInterval;
    /** 
     * The last list interval element which contains this task.
     */
    public IntervalListElement lastInterval;

    /** 
     * All capacity costs of the task's intervals.
     */
    public long capacityCosts = 0;

    /**
     * The back link to the DiscreteResourceConstraint's taskList element.
     */
    public OrderedShiftDLLElement taskLE = null;

    /** 
     * Variable that can be used by heuristics as short-term memory.
     * Should not be used after a heuristic is left because other 
     * heuristics may override this value.
     */
    public long tempValue = 0;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************
    
    /** 
     * Initializes the new task.
     * @param tc  The related task constraint.
     */
    public DRCTask(ActionTaskObjectConstraint tc) {
	taskConstraint = tc;
    }

    
    // *******************
    // ***** METHODS *****
    // *******************
    
    /** 
     * Prints the begin and end variable.
     */
    public String toString() {

	String desc = null;
	switch (executionState) {
	case DiscreteResourceConstraint.PENDING: desc = new String("Pending");
	    break;
	case DiscreteResourceConstraint.IN_EXECUTION: desc = new String("InExecution");
	    break;
	case DiscreteResourceConstraint.DONE: desc = new String("Done");
	}

	return desc + ":" + begin + ".." + end + ":(" + capacityCosts +")";
    }

}
