
package com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc;

/** 
 * A SymbolicSRC's internal representation of a state transition.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.3 $
 */


import java.util.*;


public class SymSRCStateTransition {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /**
     * The predecessor state.
     */
    public SymSRCState predecessorState;

    /**
     * The event that has to happen for this transition.
     */
    public SymSRCEvent event;

    /**
     * The successor state.
     */
    public SymSRCState successorState;

    /**
     * The ID of the action that causes this transition.
     */
    public int actionTypeID;

    /**
     * The action's total duration.
     */
    public long duration;

    /**
     * The heuristic difficulty value.
     */
    public int difficulty;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /**
     * Initializes the transition's variables.
     * @param predecessorState  The predecessor state.
     * @param event  The event that has to happen for this transition.
     * @param successorState  The successor state.
     * @param actionTypeID  The ID of the action that causes this transition.
     * @param difficulty  The heuristic difficulty value.
     */
    public SymSRCStateTransition(SymSRCState predecessorState, 
				 SymSRCEvent event, 
				 SymSRCState successorState, 
				 int actionTypeID, 
				 long duration, int difficulty) {
	this.predecessorState = predecessorState;
        this.event = event;
        this.successorState = successorState;
        this.actionTypeID = actionTypeID;
        this.duration = duration;
        this.difficulty = difficulty;
    }


    // *******************
    // ***** METHODS *****
    // *******************

    /**
     * Returns the transitions states.
     */
    public String toString() {
        return new String(predecessorState + " -> " + successorState);
    }

}
