
package com.ai_center.dragonbreath.constraints.taskconstraint;


import java.util.*;

import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.ActionTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.DiscreteResourceConstraint;


/**
 * A TaskConstraint's internal representation of an action task that
 * has been connected to it.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.7 $
 */
public class TCActionTaskInstance
{


    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /**
     * The corresponding ActionTaskObjectConstraint.
     */
    public ActionTaskObjectConstraint constraint;

    /**
     * An array of the task's begin and duration variables.
     */
    public Variable[] tempVariable = new Variable[ 2 ];
    /**
     * An array of the internally stored values of the task's begin,
     * duration and end.
     */
    public long[] tempValue = new long[ 3 ];

    /**
     * The task's resource type variable.
     */
    public Variable resourceTypeVariable;
    /**
     * The internally stored value of the task's resource type.
     */
    public long resourceType;

    /**
     * The task's operation ID variable.
     */
    public Variable operationVariable;
    /**
     * The internally stored value of the task's operation ID.
     */
    public long operation;

    /**
     * The task's consumption variable.
     */
    public Variable consumptionVariable;
    /**
     * The internally stored value of the task's consumption.
     */
    public double consumption;

    /**
     * The task's execution state variable.
     */
    public Variable executionStateVariable = null;
    /**
     * The internally stored value of the task's execution state.
     */
    public int executionState = DiscreteResourceConstraint.PENDING;

    /**
     * The costs related to the task's execution state.
     */
    public long executionStateCosts = 0;

    /**
     * The list of temporal relations which are related to this
     * task.
     */
    public HashSet temporalRelations = new HashSet();

    /**
     * Indicates whether the task has been connected by external
     * sources or by the constraint's heuristics.
     */
    public boolean externallyConnected;

    /**
     * Marking for internal search purposes.
     */
    public TCActionTask found;

    /**
     * Indicating the time at which the task's temporal variable was
     * externally changed last time.
     */
    public int externalTaskMoveCount = 0;

    /**
     * Indicating if the task has already been repaired during the
     * current improvement iteration.
     */
    public boolean repaired;


    // *******************
    // ***** METHODS *****
    // *******************


    /**
     * Prints the task's resource type, operation ID, begin, duration and end.
     */
    public String toString()
    {
        return new String( "[RT: " + resourceType + " OP: " + operation + " @ " + tempValue[ 0 ] + "..(" + tempValue[ 1 ] + ").." + tempValue[ 2 ] + "]" );
    }


}
