
package com.ai_center.dragonbreath.constraints.taskconstraint;


import java.util.*;


/**
 * A TaskConstraint's internal representation of a user-registered
 * potential action type.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.9 $
 */
public class TCActionType
{


    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /**
     * The ID of the action type.
     */
    public int ID;

    /**
     * The name of the action.
     */
    public String actionName;

    /**
     * The transition ID if this is a world transition, and -1 if this is
     * an action actively executed by the agent.
     */
    public int transitionID = -1;

    /**
     * The set of precondition tasks.
     */
    public HashSet preconditionTasks = new HashSet();
    /**
     * The number of precondition tasks.
     */
    public long preconditionTaskNumber = 0;

    /**
     * The set of action tasks.
     */
    public HashSet actionTasks = new HashSet();
    /**
     * The number of action tasks.
     */
    public long actionTaskNumber = 0;

    /**
     * The set of state tasks.
     */
    public HashSet stateTasks = new HashSet();
    /**
     * The number of state tasks.
     */
    public long stateTaskNumber = 0;

    /**
     * The set of temporal relations.
     */
    public HashSet temporalRelations = new HashSet();

    /**
     * The time point at which the orientation task should be placed.
     * This variable is only used while selecting a desired action type
     * and should not be expected to have a valid entry elsewhere.
     */
    public long temporalAnchor = 0;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************


    /**
     * Initializes a new action type.
     * @param ID  The ID for the new action type.
     */
    public TCActionType( int ID )
    {
        this.ID = ID;
    }


    // *******************
    // ***** METHODS *****
    // *******************


    /**
     * Prints the ID, task sets, and temporal relations.
     */
    public String toString()
    {
        return new String( "ID: " + ID + " PTs: " + preconditionTasks
            + " ATs: " + actionTasks + " STs: " + stateTasks
            + " TRs: " + temporalRelations );
    }


}
