
package com.ai_center.dragonbreath.constraints.taskconstraint;

/** 
 * A TaskConstraint's internal representation of a state task that 
 * has been connected to it.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.4 $
 */


import java.util.*;

import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.StateTaskObjectConstraint;


public class TCStateTaskInstance {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************
    
    /** 
     * The corresponding StateTaskObjectConstraint.
     */
    public StateTaskObjectConstraint constraint;

    /** 
     * The temporal reference variable.
     */
    public Variable tempVariable;
    /**
     * The temporal reference variable's current value.
     */
    public long tempValue;

    /** 
     * The task's resource type variable.
     */
    public Variable resourceTypeVariable;
    /**
     * The resource type variable's current value.
     */
    public long resourceType;

    /** 
     * The task's contribution variable.
     */
    public Variable contributionVariable;
    /**
     * The contribution variable's current value.
     */
    public long contribution;

    /** 
     * The list of temporal relations which are related to this
     * task.
     */
    public HashSet temporalRelations = new HashSet();

    /**
     * Indicates whether the task has been connected by external
     * sources or by the constraint's heuristics.
     */
    public boolean externallyConnected;

    /**
     * Marking for internal search purposes.
     */
    public TCStateTask found;

    /** 
     * Indicating the time at which the task's temporal variable was
     * externally changed last time. 
     */
    public int externalTaskMoveCount = 0;

    /**
     * Indicating if the task has already been repaired during the
     * current improvement iteration.
     */
    public boolean repaired;


    // *******************
    // ***** METHODS *****
    // *******************
    
    /** 
     * Prints the task's resource type, contribution, and temporal reference.
     */
    public String toString() {
        return new String("[RT: " + resourceType + " CO: " + contribution + " @ " + tempValue + "]");
    }

}
