
package com.ai_center.dragonbreath.constraints.taskconstraint;


/**
 * A TaskConstraint's internal representation of an action type's
 * tasks. CLass is inherited by TCPreconditionTask, TCActionTask and
 * TCStateTask.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.3 $
 */
public abstract class TCTask
{


    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /**
     * The required resource type for the task.
     */
    public long resourceType = 0;

    /**
     * The relative temporal shift with respect to the action's
     * orientation task.
     */
    public long relativeShift = Long.MIN_VALUE;

    /**
     * The earliest possible relative time with respect to the action's
     * orientation task. Only used during action specification.
     */
    public long shiftMin = Long.MIN_VALUE;

    /**
     * The latest possible relative time with respect to the action's
     * orientation task. Only used during action specification.
     */
    public long shiftMax = Long.MAX_VALUE;


    // *******************
    // ***** METHODS *****
    // *******************


    /**
     * Prints current shift information.
     */
    public String shiftInfo()
    {
        return new String( "[" +  shiftMin + " <" + relativeShift + "> "
            + shiftMax + "]" );
    }


}
