
package com.ai_center.dragonbreath.constraints.taskconstraint;


import com.ai_center.dragonbreath.constraints.TaskConstraint;


/**
 * A TaskConstraint's internal representation of an action type's
 * temporal relation.
 * The represented relation is "var1 comparator var2 + const". All
 * comparators are required to be transformed beforehand such that
 * only EQUAL and LESS are used.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.2 $
 */
public class TCTemporalRelation
{

    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /**
     * The task of the first variable.
     */
    public TCTask task1;
    /**
     * The task of the second variable.
     */
    public TCTask task2;

    /**
     * Indicates the first task's variable that is involved.
     */
    public int variableID1;
    /**
     * Indicates the second task's variable that is involved.
     */
    public int variableID2;

    /**
     * The constant to be added to the second variable.
     */
    public long constant;

    /**
     * The relation's comparator.
     */
    public int comparator;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************


    /**
     * Initializes the new relation.
     * @param task1  The task of the first variable.
     * @param variableID1  Indicates the first task's variable to be involved.
     * @param comparator  The relation's comparator. Only EQUAL and LESS
     *     should be used.
     * @param task2  The task of the second variable.
     * @param variableID2  Indicates the second task's variable to be involved.
     * @param constant The constant to be added to the second variable.
     */
    public TCTemporalRelation( TCTask task1 , int variableID1 ,
        int comparator , TCTask task2 , int variableID2 , long constant )
    {
        this.task1 = task1;
        this.variableID1 = variableID1;
        this.comparator = comparator;
        this.task2 = task2;
        this.variableID2 = variableID2;
        this.constant = constant;
   }


    // *******************
    // ***** METHODS *****
    // *******************


    /**
     * Prints a description of the temporal relation.
     */
    public String toString()
    {

        String desc1;
        if ( variableID1 == TaskConstraint.END )
        {
            desc1 = new String( "end-of-(" + task1 + ")" );
        }
        else
        {
            if ( task1 instanceof TCActionTask )
            {
                desc1 = new String( "begin-of-(" + task1 + ")" );
            }
            else
            {
                desc1 = new String( "(" + task1 + ")" );
            }
        }
        String desc2;
        if ( variableID2 == TaskConstraint.END )
        {
            desc2 = new String( "end-of-(" + task2 + ")" );
        }
        else
        {
            if ( task2 instanceof TCActionTask )
            {
                desc2 = new String( "begin-of-(" + task2 + ")" );
            }
            else
            {
                desc2 = new String( "(" + task2 + ")" );
            }
        }

        return new String( desc1 + ( ( comparator == TaskConstraint.LESS_THAN )?" < ":" = " ) + desc2 + ( ( constant < 0 )?( " - " + -constant ):( " + " + constant ) ) );

    }


}
