
package com.ai_center.dragonbreath.constraints.transitionconstraint;


import java.util.HashSet;
import java.util.Iterator;


/**
 * An interval representation for a TransitionConstraint's timeline of
 * all watched states.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.7 $
 */
public class TCInterval implements Cloneable
{


    // **********************
    // ***** ATTRIBUTES *****
    // **********************


    /**
     * The states that are valid during the interval, including
     * those of the newStates set (object types: TCState).
     */
    public HashSet states;

    /**
     * The states that start to be valid with the interval (object
     * types: TCState).
     */
    public HashSet newStates;

    /**
     * The transitions that occur at the beginning of the interval.
     */
    public HashSet transitions;


    // *******************
    // ***** METHODS *****
    // *******************


    /**
     * Returns a string including a description of all states that are
     * valid during the interval, and the transitions that occur at the
     * beginning of the interval. A star after a state description
     * indicates states that begin to be valid with the interval.
     */
    public String toString()
    {

        StringBuffer s = new StringBuffer( "ST: [" );

        if ( states != null )
        {
            TCState st;
            Iterator i = states.iterator();
            while ( i.hasNext() )
            {
                st = (TCState)i.next();
                if ( ( newStates != null ) && ( newStates.contains( st ) ) )
                {
                    s.append( "NEW:" );
                }
                s.append( "<" + st.toString() + ">" );
                if ( i.hasNext() )
                {
                    s.append( " " );
                }
            }
        }

        if ( transitions != null )
        {
            s.append( "] TR: [" );
            Iterator i = transitions.iterator();
            while ( i.hasNext() )
            {
                s.append( "T" + ( (TCTransition)i.next() ).transitionType.ID );
                if ( i.hasNext() )
                {
                    s.append( " " );
                }
            }
        }
        s.append( "]" );

        return s.toString();

    }


    /**
     * Returns a copy of the object. Note that the newStates set and the
     * transitions set are not copied.
     */
    public Object clone()
    {
        TCInterval interval = new TCInterval();
        if ( states != null )
        {
            interval.states = (HashSet)states.clone();
        }
        else
        {
            interval.states = null;
        }
        return interval;
    }

}
