
package com.ai_center.dragonbreath.constraints.transitionconstraint;


import java.util.*;


/**
 * A TransitionConstraint's internal representation of a user-registered
 * potential transition type.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.6 $
 */
public class TCTransitionType
{

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /**
     * The ID of the transition type.
     */
    public int ID;

    /**
     * The ID of the related TaskConstraint's action type.
     */
    public int actionTypeID;

    /**
     * The set of precondition tasks (object types: TCStateType).
     */
    public HashSet preconditionTasks = new HashSet();
    /**
     * The number of precondition tasks.
     */
    public long preconditionTaskNumber = 0;

    /**
     * The set of state tasks (object types: TCEventType).
     */
    public HashSet stateTasks = new HashSet();
    /**
     * The number of state tasks.
     */
    public long stateTaskNumber = 0;


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************


    /**
     * Initializes a new transition type.
     * @param ID  The ID for the new transition type.
     * @param actionTypeID  The ID of the related TaskConstraint's action type.
     */
    public TCTransitionType( int ID , int actionTypeID )
    {
        this.ID = ID;
        this.actionTypeID = actionTypeID;
    }


    // *******************
    // ***** METHODS *****
    // *******************


    /**
     * Prints the ID, the related action type ID, the precondition task
     * set and the state task set.
     */
    public String toString()
    {
        return new String( "ID: " + ID + " AT-ID: " + actionTypeID
            + " PTs: " + preconditionTasks + " STs: " + stateTasks );
    }

}
