
package com.ai_center.dragonbreath.examples.specificationbasics;


import java.math.BigDecimal;

import com.ai_center.dragonbreath.*;
import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.constraints.DiscreteResourceConstraint;
import com.ai_center.dragonbreath.costs.CostFunction;
import com.ai_center.dragonbreath.costs.CostMapping;
import com.ai_center.dragonbreath.costs.FactorCostMapping;


/**
 * Sample application of the <a href="http://www.ai-center.com/projects/dragonbreath/"><font face="Times">DragonBreath</font> engine</a>.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.3 $
 */
public class SpecificationBasics
{


    // *******************
    // ***** METHODS *****
    // *******************


    public static void main( String[] args )
    {

        // Initialization

        GlobalSearchControl gsc = new GlobalSearchControl( 0 );

        // Variable creation

        Variable vTest = gsc.createVariable( "Test" );
        Variable vBegin1 = gsc.createVariable( "a1" , 60 , 350 );
        Variable vDuration1 = gsc.createVariable( "a2" , 360 , 350 );
        Variable vEnd1 = gsc.createVariable( "a3" , 520 , 350 );
        Variable vOperation1 = gsc.createVariable( "a4" , 60 , 250 );
        Variable vConsumption1 = gsc.createVariable( "a5" , 60 , 150 );
        Variable vExecutionState1 = gsc.createVariable( "a6" , 370 , 450 );
        Variable vBegin2 = gsc.createVariable( "b1" , 350 , 150 );
        Variable vDuration2 = gsc.createVariable(" b2" , 510 , 150 );
        Variable vEnd2 = gsc.createVariable( "b3" , 810 , 150 );
        Variable vOperation2 = gsc.createVariable( "b4" , 810 , 250 );
        Variable vConsumption2 = gsc.createVariable( "b5" , 810 , 330 );
        Variable vExecutionState2 = gsc.createVariable( "b6" , 500 , 50 );
        Variable vCapacity = gsc.createVariable( "c" , 665 , 330 );
        Variable vResourceType = gsc.createVariable( "r" , 435 , 220 );
        Variable vCurrentTime = gsc.createVariable( "time" , 210 , 50 );

        // Setting variables

        gsc.setVariable( vTest , 5 );
        gsc.setVariable( vBegin1 , 14.6 );
        gsc.setVariable( vDuration1 , -7.0 );
        gsc.setVariable( vEnd1 , 3.0 );
        gsc.setVariable( vOperation1 , 1 );
        gsc.setVariable( vExecutionState1 ,
            DiscreteResourceConstraint.PENDING );
        gsc.setVariable( vBegin2 , 280 );
        gsc.setVariable( vDuration2 , 1 );
        gsc.setVariable( vEnd2 , 5 );
        gsc.setVariable( vOperation2 , 2 );
        gsc.setVariable( vExecutionState2
            , DiscreteResourceConstraint.PENDING );
        gsc.setVariable( vCapacity , 2.3 );
        gsc.setVariable( vResourceType , 1 );
        gsc.setVariable( vCurrentTime , 0 );

        System.out.println( gsc.getVariableDescription( vTest ) );

        // Variable deletion

        gsc.deleteVariable( vTest );

        System.out.println( gsc.getVariableDescriptions() );

        // Cost-mapping-class addition

        int SQR_COSTMAPPING = Registry.registerCostMapping( "com.ai_center.dragonbreath.costs.SquareFactorCostMapping" , "Squared Factorized Cost Mapping" );

        // Creating new cost mapping to exchange the default cost mapping
        // for makespan costs

        CostMapping newSCM = gsc.createCostMapping( SQR_COSTMAPPING , "My New Squared Factorized Cost Mapping" );

        // Changing default factors for a factor cost mapping

        FactorCostMapping cm
            = (FactorCostMapping)gsc.getSelectedCostMapping();
        cm.setDefaultFactor( Registry.SUM_CONSTRAINT , new BigDecimal( "2" ) );

        // Creating cost functions

        gsc.selectCostMapping( newSCM );
        CostFunction mcf = gsc.createCostFunction( "Makespan Cost Function" );

        gsc.selectDefaultCostMapping();
        CostFunction tcf
            = gsc.createCostFunction( "Action Task Cost Function" );
        tcf.setConstraintSelectionMethod( CostFunction.SELECTION_PROBABILITY_DISTRIBUTION );

        System.out.println( gsc.getCostFunctionInfo() );

        // Constraint creation

        gsc.selectCostFunction (mcf );
        gsc.selectDefaultCostMapping();

        Embedding eTest = new Embedding( 1 );
        gsc.extendEmbedding( eTest , vBegin1 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cTest
            = gsc.createConstraint( Registry.INTEGER_CONSTRAINT , eTest );

        // Creating interactive graphics display

        gsc.createGraphics();

        // Constraint deletion

        gsc.deleteConstraint( cTest );

        // (Adding "action task" structure 1)

        gsc.selectCostFunction( tcf );
        gsc.selectCostMappingOfCostFunction();

        Embedding eRangeBegin1 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeBegin1 , vBegin1 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cRangeBegin1
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT , eRangeBegin1 ,
            60 , 450 );

        // Playing with a range's bounds

        gsc.setLowerBound( cRangeBegin1 , 10 );
        gsc.setUpperBound( cRangeBegin1 , 20 );
        gsc.deactivateUpperBound( cRangeBegin1 );
        gsc.setBounds( cRangeBegin1 , 0 , 100 );

        // (Going on with adding "action task" structure 1)

        Embedding eRangeDuration1 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeDuration1 , vDuration1 ,
            Registry.PLAIN_EDGE , Registry.INCOMING );
        ConventionalConstraint cRangeDuration1
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT ,
            eRangeDuration1 , 220 , 350 );
        gsc.setBounds( cRangeDuration1 , 40 , 40 );

        Embedding eRangeEnd1 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeEnd1 , vEnd1 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cRangeEnd1
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT , eRangeEnd1 ,
            520 , 450 );
        gsc.setBounds( cRangeEnd1 , 0 , 100 );

        Embedding eSum1 = new Embedding( 2 );
        gsc.extendEmbedding( eSum1 , vEnd1 , Registry.SUM_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eSum1 , vBegin1 , Registry.ADD_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cSum1
            = gsc.createConstraint( Registry.SUM_CONSTRAINT , eSum1 ,
            220 , 450 );

        // Constraint extensions

        Embedding eSum1_1 = new Embedding( 1 );
        gsc.extendEmbedding( eSum1_1 , vDuration1 , Registry.ADD_EDGE ,
            Registry.INCOMING );
        gsc.extendConstraint( cSum1 , eSum1_1 );

        Embedding eSum1_Test1 = new Embedding( 1 );
        gsc.extendEmbedding( eSum1_Test1 , vBegin2 , Registry.ADD_EDGE ,
            Registry.INCOMING );
        gsc.extendConstraint( cSum1 , eSum1_Test1 );

        // Constraint reductions

        Embedding eSum1_Test2 = new Embedding( 1 );
        gsc.extendEmbedding( eSum1_Test2 , vBegin2 , Registry.ADD_EDGE ,
            Registry.INCOMING );
        gsc.reduceConstraint( cSum1 , eSum1_Test2 );

        // Switching graphics off

        gsc.deleteGraphics();

        // (Going on with adding "action task" structure 1)

        Embedding eRangeConsumption1 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeConsumption1 , vConsumption1 ,
            Registry.PLAIN_EDGE , Registry.INCOMING );
        ConventionalConstraint cRangeConsumption1
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT ,
            eRangeConsumption1 , 60 , 50 );
        gsc.setBounds( cRangeConsumption1 , 2 , 2 );

        Embedding eTask1 = new Embedding( 6 );
        gsc.extendEmbedding( eTask1 , vBegin1 , Registry.BEGIN_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eTask1 , vDuration1 , Registry.DURATION_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eTask1 , vOperation1 , Registry.OPERATION_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eTask1 , vConsumption1 ,
            Registry.CONSUMPTION_EDGE , Registry.INCOMING );
        gsc.extendEmbedding( eTask1 , vResourceType ,
            Registry.RESOURCETYPE_EDGE , Registry.INCOMING );
        gsc.extendEmbedding( eTask1 , vExecutionState1 ,
            Registry.EXECUTIONSTATE_EDGE , Registry.INCOMING );
        ConventionalConstraint cTask1
            = gsc.createConstraint( Registry.ACTIONTASK_OBJECTCONSTRAINT ,
            eTask1 , 220 , 250 );

        // (Adding "action task" structure 2)

        Embedding eRangeBegin2 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeBegin2 , vBegin2 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cRangeBegin2
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT , eRangeBegin2 ,
            350 , 50 );
        gsc.setBounds( cRangeBegin2 , 0 , 100 );

        Embedding eRangeDuration2 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeDuration2 , vDuration2 ,
            Registry.PLAIN_EDGE , Registry.INCOMING );
        ConventionalConstraint cRangeDuration2
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT ,
            eRangeDuration2 , 650 , 150 );
        gsc.setBounds( cRangeDuration2 , 40 , 40 );

        Embedding eRangeEnd2 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeEnd2 , vEnd2 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cRangeEnd2
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT , eRangeEnd2 ,
            810 , 50 );
        gsc.setBounds( cRangeEnd2 , 0 , 100 );

        Embedding eSum2 = new Embedding( 2 );
        gsc.extendEmbedding( eSum2 , vEnd2 , Registry.SUM_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eSum2 , vBegin2 , Registry.ADD_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cSum2
            = gsc.createConstraint( Registry.SUM_CONSTRAINT , eSum2
            , 650 , 50 );
        Embedding eSum2_1 = new Embedding( 1 );
        gsc.extendEmbedding( eSum2_1 , vDuration2 , Registry.ADD_EDGE ,
            Registry.INCOMING );
        gsc.extendConstraint( cSum2 , eSum2_1 );

        Embedding eRangeConsumption2 = new Embedding( 1 );
        gsc.extendEmbedding( eRangeConsumption2 , vConsumption2 ,
            Registry.PLAIN_EDGE , Registry.INCOMING );
        ConventionalConstraint cRangeConsumption2
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT ,
            eRangeConsumption2 , 810 , 430 );
        gsc.setBounds( cRangeConsumption2 , 1 , 1 );

        Embedding eTask2 = new Embedding( 6 );
        gsc.extendEmbedding( eTask2 , vBegin2 , Registry.BEGIN_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eTask2 , vDuration2 , Registry.DURATION_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eTask2 , vOperation2 , Registry.OPERATION_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eTask2 , vConsumption2 ,
            Registry.CONSUMPTION_EDGE , Registry.INCOMING );
        gsc.extendEmbedding( eTask2 , vResourceType ,
            Registry.RESOURCETYPE_EDGE , Registry.INCOMING );
        gsc.extendEmbedding( eTask2 , vExecutionState2 ,
            Registry.EXECUTIONSTATE_EDGE , Registry.INCOMING );
        ConventionalConstraint cTask2
            = gsc.createConstraint( Registry.ACTIONTASK_OBJECTCONSTRAINT ,
            eTask2 , 650 , 250 );

        Embedding eRangeCapacity = new Embedding( 1 );
        gsc.extendEmbedding( eRangeCapacity , vCapacity , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cRangeCapacity
            = gsc.createConstraint( Registry.RANGE_CONSTRAINT ,
            eRangeCapacity , 665 , 430 );
        gsc.setBounds( cRangeCapacity , 2 , 2 );

        // Adding the Current Time object

        Embedding eTime = new Embedding( 1 );
        gsc.extendEmbedding( eTime , vCurrentTime , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cTime
            = gsc.createConstraint( Registry.CURRENTTIME_OBJECTCONSTRAINT ,
            eTime , 210 , 150 );

        // Note that by the DiscreteResourceConstraint, additional Integer
        // constraints are automatically added to the variables.

        Embedding eDRC = new Embedding( 3 );
        gsc.extendEmbedding( eDRC , vResourceType ,
            Registry.RESOURCETYPE_EDGE , Registry.INCOMING );
        gsc.extendEmbedding( eDRC , vCapacity , Registry.CAPACITY_EDGE ,
            Registry.INCOMING );
        gsc.extendEmbedding( eDRC , cTime , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cDRC
            = gsc.createConstraint( Registry.DISCRETERESOURCE_CONSTRAINT ,
            eDRC , DiscreteResourceConstraint.CAPACITY_OVERLOAD ,
            new BigDecimal( "1.5" ) , 435 , 280 );
        Embedding eDRC_1 = new Embedding( 1 );
        gsc.extendEmbedding( eDRC_1 , cTask1 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        gsc.extendConstraint( cDRC , eDRC_1 );
        Embedding eDRC_2 = new Embedding( 1 );
        gsc.extendEmbedding( eDRC_2 , cTask2 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        gsc.extendConstraint( cDRC , eDRC_2 );

        // Switching graphics on again

        gsc.createGraphics();

        // Using additional costs

        gsc.selectCostFunction( mcf );

        gsc.addCosts( cDRC , DiscreteResourceConstraint.MAKESPAN );

        // Customizing DiscreteResourceConstraint

        DiscreteResourceConstraint.setRepairTimeBuffer( 10 );

        // Saving the (structural) constraint problem

        gsc.saveSCP( "sample.scp" );

        // Searching for a solution

        System.out.println( "Starting optimization!\n" );
        System.out.println( gsc.getCostFunctionInfo() );

        int iteration = 0;
        while ( ( iteration < 1000 ) && gsc.executeImprovementIteration() )
        {
            System.out.println( "After iteration " + ++iteration + ":\n" );
            System.out.println( gsc.getCostFunctionInfo() );
        }

        System.out.println();
        System.out.println( gsc.getVariableDescriptions() );

        // Constraint-class addition

        int ODD_CONSTRAINT = Registry.produceNodeTypeID();
        // ATTENTION! For the following line to work, you have to
        // disable the corresponding line in "Registry.java" because
        // otherwise, the constraint will already be loaded and cause
        // an error.
        Registry.registerNodeType( ODD_CONSTRAINT , "com.ai_center.dragonbreath.constraints.OddConstraint" );

        Embedding eNew = new Embedding( 1 );
        gsc.extendEmbedding( eNew , vBegin1 , Registry.PLAIN_EDGE ,
            Registry.INCOMING );
        ConventionalConstraint cNew
            = gsc.createConstraint( ODD_CONSTRAINT , eNew , 60 , 250 );

        // Deleting the complete CSP

        gsc.clearCSP();

        // Loading the old (structural) constraint problem

        gsc.loadSCP( "sample.scp" );

        // Disabling graph changes by improvement heuristics

        gsc.disableStructuralChanges();

    }


}
