
package com.ai_center.dragonbreath.examples.taskconstraintdemo;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;

import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.examples.utilities.ExampleDisplay;


/**
 * Guides through the task constraint demo of the <a href="http://www.ai-center.com/projects/dragonbreath/"><font face="Times">DragonBreath</font> engine</a>.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.2 $
 */
public class Guide extends ExampleDisplay
{


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************


    public Guide()
    {

        // Initializing text display

        super( "Task Constraint Demo" , "taskconstraintdemo" , "TaskConstraintDemo.java" );
        appendIntroduction( "This program demonstrates some functionality of the \"TaskConstraint\".  The constraint's purpose is to establish one of a set of given configurations of appended \"PreconditionTask\"s, \"ActionTask\"s and \"StateTask\"s (connected via an \"Action\" object constraint), to satisfy respective temporal relations between these tasks, and to maintain a correct execution state of the \"ActionTask\"s." );
        appendIntroduction( "\n\nThis example has only two possible action type configurations for the \"TaskConstraint\": an action to unlock a door and one to force the door open. The initial example graph currently displayed already contains a \"TaskConstraint\" with two \"PreconditionTask\"s (testing if the agent has the key and if the door is locked) and one \"StateTask\" (modifying the door to be unlocked/passable). The \"Action Type\" variable is set to -1, indicating that the \"TaskConstraint\" is free to select the concrete action configuration, and implying costs for the \"TaskConstraint\" thereby." );
        appendIntroduction( "\n\nWhen an improvement step is performed, the \"TaskConstraint\" selects a possible configuration to be established, which is the action to unlock the door in this case. This is expressed by setting the \"Action Type\" variable to 0. Please perform this improvement step now." );
        appendIntroduction( "\n\nThe action to unlock the door, however, also requires an \"ActionTask\" in its configuration. The current costs of the \"TaskConstraint\" are representing this. In the next improvement step, the \"TaskConstraint\" will add this \"ActionTask\" to the graph. The temporal relations between the tasks will also be repaired according to the action specification because the action's configuration is completed after the task's addition. Please perform this improvement step now and move the produced graph elements to some free space of the display." );
        appendIntroduction( "\n\nAs a side note - if the \"DiscreteResource\" constraint with a value for the \"Resource Type\" variable that equals the required one of the \"ActionTask\" would not have existed, such a \"DiscreteResource\" constraint is created in addition." );
        appendIntroduction( "\n\nThe costs of the \"TaskConstraint\" now indicate that the \"Execution State\" of the \"ActionTask\" is not set correctly. It should equal 1, representing that the task is under execution, because the time point overlaps with the \"Current Time\". Please perform this improvement step in which this variable is repaired by the \"TaskConstraint\"." );
        appendIntroduction( "\n\nThe \"StateResource\" constraint's costs currently cannot be repaired (you can try it) because the \"StateResource\" constraint blocks the deletion of an action under execution. We thus modify one of the tasks to start at a later point in time. For example, please set the \"begin\" of the \"ActionTask\" to 10." );
        appendIntroduction( "\n\nThe temporal relations between the tasks are now violated, implying costs for the \"TaskConstraint\". The relations get repaired in the next improvement iteration, after which the execution state will be repaired again. Please perform these two improvement steps now." );
        appendIntroduction( "\n\nThe \"StateResource\" constraint can now perform its improvement - deleting the connected \"PreconditionTask\" because the tested state (0 means having the key) is not compatible with the real state projected by the \"StateResource\" (1 means that the agent does not have the key). Please perform this improvement step now." );
        appendIntroduction( "\n\nAfter the deletion of the task, the action's task configuration is of course no more valid, which leads to costs for the \"TaskConstraint\". In the next improvement iteration, the \"TaskConstraint\" could theoretically change its configuration to the other possible configuration of forcing the door open. However, because the deleted precondition task was initially connected to the action, the task constraint assumes that this task was an important reason for the existence of whole action, and thus, deletes the action instead." );
        appendIntroduction( "\n\n" );

        // Run demo

        TaskConstraintDemo demo = new TaskConstraintDemo();
        demo.main( null );

    }


}
