
package com.ai_center.dragonbreath.examples.utilities;

/** 
 * A blueprint class for displaying application examples.
 *
 * @author Alexander Nareyek
 * @version $Revision: 1.2 $
 */


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;

import com.ai_center.dragonbreath.Registry;


public class ExampleDisplay extends JFrame {

    public JPanel panel;
    public JTextArea introduction;
    public JScrollPane introductionScrollPane;
    public JTextArea source;
    public JScrollPane sourceScrollPane;
    public JPanel buttonPanel;
    public JButton exitButton;

    /**
     * Displays the example description windows.
     * @param exampleName  A short description of the example (3-4 words).
     * @param exampleDir  The example's directory name in the example
     *                    directory.
     * @param sourceFile  The name of the source file to be displayed.
     */
    public ExampleDisplay(String exampleName, String exampleDir, 
			  String sourceFile) {

	super("Example Info");

	panel = new JPanel();
	panel.setLayout(new BorderLayout());
	panel.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));

	// Introduction text

	introduction = new JTextArea();
	introduction.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));
	introduction.setColumns(80);
	introduction.setEditable(false);
	introduction.setLineWrap(true);
	introduction.setWrapStyleWord(true);
	introduction.append("***** DragonBreath Engine - Version " 
			   + Registry.VERSION + "\n");
	introduction.append("***** " + exampleName + "\n\n");

	int rows = introduction.getLineCount();
	if (rows > 10) {
	    rows = 10;
	}
	introduction.setRows(rows);

	introductionScrollPane = new JScrollPane(introduction);
	introductionScrollPane.setViewportBorder(BorderFactory.createLoweredBevelBorder());

	panel.add(introductionScrollPane, BorderLayout.NORTH);

	// Source file display

	source = new JTextArea();
	source.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));
	source.setColumns(80);
	source.setEditable(false);
	source.setLineWrap(true);
	source.setWrapStyleWord(false);

	File inputFile = new File(System.getProperty("user.dir")
				  + System.getProperty("file.separator")
				  + "com"
				  + System.getProperty("file.separator")
				  + "ai_center"
				  + System.getProperty("file.separator")
				  + "dragonbreath"
				  + System.getProperty("file.separator")
				  + "examples"
				  + System.getProperty("file.separator")
				  + exampleDir
				  + System.getProperty("file.separator")
				  + sourceFile);
	BufferedReader in = null;
	try {
	    in = new BufferedReader(new FileReader(inputFile));
	    String line = null;
	    while ((line = in.readLine()) != null) {
		source.append(line + "\n");
	    }
	    in.close();
	} catch (IOException e) {
	    source.append("ERROR: Source file could not be retrieved!");
	}

	rows = source.getLineCount();
	if (rows > 20) {
	    rows = 20;
	}
	source.setRows(rows);

	sourceScrollPane = new JScrollPane(source);
	sourceScrollPane.setViewportBorder(BorderFactory.createLoweredBevelBorder());

	panel.add(sourceScrollPane, BorderLayout.CENTER);

	// Interactive buttons

	buttonPanel = new JPanel();
	buttonPanel.setLayout(new FlowLayout());

	exitButton = new JButton("Exit");
        exitButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    System.exit(0);
		}});
	buttonPanel.add(exitButton, BorderLayout.CENTER);

	panel.add(buttonPanel, BorderLayout.SOUTH);

	// Display graphics and run example

	getContentPane().add(panel, BorderLayout.CENTER);
        pack();
        setVisible(true);

    }

    /**
     * Adds additional content to the introduction.
     * @param s  The string to be appended to the introduction.
     */
    public void appendIntroduction(String s) {
	introduction.append(s);
	int rows = introduction.getLineCount();
	if (rows > 10) {
	    rows = 10;
	}
	introduction.setRows(rows);
    }

}
