/*
 * SimpleNodeChooser.java
 *
 * Created on 5. Dezember 2002, 17:17
 */
package com.ai_center.dragonbreath.graphRepair;

import com.ai_center.dragonbreath.*;
import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.*;
import com.ai_center.dragonbreath.utilities.*;

import java.util.*;

import org.jdom.Element;


/**
 * Allways chooses to create a new node. Or the first of the options.
 * 
 * @author tim baermann
 */
public class AllwaysCreateNodeChooser implements NodeChooseInterface {
    // Instance variables *****************************************************

    /** 
     * Link to the GlobalSearchControl of this application. 
     */
    GlobalSearchControl myGSC;

    // Constructors ***********************************************************

    /**
     * Creates a new instance of SimpleNodeChooser
     * 
     * @param gsc The GlobalSearchControl to be used.
     */
    public AllwaysCreateNodeChooser(GlobalSearchControl gsc) {
        myGSC = gsc;
    }

    // Methods ****************************************************************

    /**
     * When a node was created the GraphRepairer asks via getAuxiliaryNodeData
     * what value to set as default for the new Variable, or constraint.  As
     * this nodchooser is only used to created randomnodes there is no default
     * data for any node.
     * 
     * @param n The node that was created and has to be initialized now.
     * 
     * @return null.
     */
    public Element getAuxiliaryNodeData(Node n) {
        return null;
    }


    /**
     * Returns either an allready existing Node, or null, if the GraphRepairer
     * should create its own new one.  Returns allways null, as this is the
     * AllwaysCreateNodeChooser.
     * 
     * @param nodeTypeID The id of the node, as in Registry.
     * 
     * @return null
     */
    public Node getFittingNode(int nodeTypeID) {
        return null;
    }


    /**
     * Returns either an allready existing Node, that can be used to connect n
     * to, or null, if the GraphRepairer should create its own new one.
     * Returns allways null, as this is the AllwaysCreateNodeChooser.
     * 
     * @param n  The node the edge comes from. May be null, if Node not yet
     *        created.
     * @param cn  The edge.
     * 
     * @return null.
     */
    public Node getFittingNodeForEdgeCreation(Node n, ConnectedNode cn) {
        return null;
    }


    /**
     * Before a StructuralConstraintInstance is repairing its TestGraph it
     * registered at the nodeChooser, so that when a new node is to be added
     * it is the first to be asked for a candidate.  Does nothing, as this is
     * the AllwaysCreateNodeChooser.
     * 
     * @param instance Not used
     */
    public void setMainNodeSource(NodeChooseInterface instance) {
    }


    /**
     * When a node is chosen to be used it can be tested for compliance with
     * the nodeChooser. Allways called by a nodechooser from its
     * mainNodeSource. Returns allways true, as this is the
     * AllwaysCreateNodeChooser.
     * 
     * @param n The tested node.
     * 
     * @return true
     */
    public boolean isNodeOk(Node n) {
        return true;
    }


    /**
     * Inherited, does nothing.
     * 
     * @param n The node that was replaced.
     */
    public void setNodeReplaced(Node n) {
    }


    /**
     * Each time a NodeChooseInterface was called with
     * getFittingNodeForEdgeCreation() or getFittingNode() and returned null
     * it has to be called again as soon as the requested node was produced
     * with the new node. Inherited, does nothing.
     * 
     * @param node The new node.
     */
    public void registerNewNode(Node node) {
    }


    /**
     * Returns whether the node should be removed, or the edge added. Called if
     * an edge should be removed, but can not due to the fact that it is part
     * of the basegraph of the node. In such a case the addition of a new edge
     * of the same type could help, as it could then be used for the
     * basegraph.  Returns allways true, as this is the
     * AllwaysCreateNodeChooser.
     * 
     * @param node  The node.
     * @param edge  The edge that should be removed, but can not.
     * 
     * @return true
     */
    public boolean removeNodeOrElseAddEdge(Node node, Object edge) {
        return true;
    }
}