/*
 * CusotmDockinggraphRepairer.java
 *
 * Created on 23. April 2003, 17:47
 */
package com.ai_center.dragonbreath.graphRepair;

import com.ai_center.dragonbreath.*;
import com.ai_center.dragonbreath.Constraint;
import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.costs.CostFunctionReference;
import com.ai_center.dragonbreath.structuralConstraintHandling.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.SCIDockingGraph;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.testGraph.*;
import com.ai_center.dragonbreath.utilities.*;

import java.util.*;


/**
 * Example for a customized TestGraphRepairer. Does the same as the standard
 * TestGraphRepairer.
 * 
 * @version $Revision: 1.1 $
 * @author tim baermann
 */
public class CustomTestGraphRepairer extends TestGraphRepairer {
    // Methods ****************************************************************

    /**
     * Creates a new instance of TestGraphRepairer
     * 
     * @param scm  The StructuralConstraintManagement to be used.
     * @param gsc  The GlobalSearchControl to be used.
     * @param scGraphRepairer The GraphRepairer that is used to change the
     *        CSP-Graph.
     */
    public void setConstructionParameters(StructuralConstraintManagement scm, 
                                          GlobalSearchControl gsc, 
                                          SCGraphRepairer scGraphRepairer) {
        super.setConstructionParameters(scm, gsc, scGraphRepairer);
    }


    /**
     * Adds missing Nodes/Edges, or removes NACs from the given testGraph. If
     * after a certain period of time no solution is found it returns. The
     * structural error remains.
     * 
     * @param repairedTestGraph  The testgraph that should be repaired.
     * @param instance  The StructuralConstraintInstance the testGraph belongs
     *        to.
     */
    public void repair(TestGraph repairedTestGraph, 
                       StructuralConstraintInstance instance) {
        java.lang.System.out.println("CUSTOM TESTGRAPHREPAIRER");
        repairedInstance = instance;
        startTimer();

        if (repairedTestGraph.getNACError() == 0) {
            mySCGraphRepairer.completeSubgraph(repairedTestGraph.getMainParts(), 
                                               instance.millisForRepair);
            repairedTestGraph.resetInterpretations();

            if (repairedTestGraph.testTestGraph()) {
                instance.createNewTestGraphInstance(repairedTestGraph);
            } // else a NAC might have been created, repair in next iteration
        } else {
            destroyNACs(repairedTestGraph);

            if (repairedTestGraph.findOneEmbedding()) {
                instance.createNewTestGraphInstance(repairedTestGraph);
            }
        }
    }
}