/*
 * DockingGraphRepairer.java
 *
 * Created on 12. Februar 2003, 14:55
 */
package com.ai_center.dragonbreath.graphRepair;

import com.ai_center.dragonbreath.*;
import com.ai_center.dragonbreath.Constraint;
import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.costs.CostFunctionReference;
import com.ai_center.dragonbreath.structuralConstraintHandling.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.SCIDockingGraph;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.testGraph.*;
import com.ai_center.dragonbreath.utilities.*;

import java.util.*;


/**
 * Used to destroy a dockinggraph of a StructuralConstraintInstance.
 * 
 * @version $Revision: 1.3 $
 * @author tim baermann
 */
public class DockingGraphRepairer {
    // Instance variables *****************************************************

    /** 
     * Used for debug output. 
     */
    DebugLogger dl = new DebugLogger(true);

    /** 
     * The GlobalSearchControl. 
     */
    GlobalSearchControl myGSC;

    /** 
     * The Part of the repairedTestGraph that is actually beeing repaired. 
     */
    MovableSubgraphPart repairedPart;

    /**
     * Where the GraphRepairer should ask for nodes that were not called to be
     * changed, but have to be for the demanded change to work.
     */
    NodeChooseInterface myNodeChooser;

    /** 
     * Used to change the graphs 
     */
    SCGraphRepairer mySCGraphRepairer;

    /** 
     * The instance the repairedTestGraph is part of. 
     */
    StructuralConstraintInstance repairedInstance;

    /** 
     * The StructuralConstraintManagement 
     */
    StructuralConstraintManagement mySCM;

    /** 
     * The testGraph currently beeing repaired. 
     */
    TestGraph repairedTestGraph;

    /** 
     * When did the repair start? 
     */
    long startRepairTime;

    // Methods ****************************************************************

    /**
     * Creates a new instance of DockingGraphRepairer
     * 
     * @param scm  The StructuralConstraintManagement to be used.
     * @param gsc The GlobalSearchControl to be used.
     * @param scGraphRepairer Repair is realized via the scGraphRepairer.
     */
    public void setConstructionParameters(StructuralConstraintManagement scm, 
                                          GlobalSearchControl gsc, 
                                          SCGraphRepairer scGraphRepairer) {
        mySCM             = scm;
        myGSC             = gsc;
        mySCGraphRepairer = scGraphRepairer;

        //dl.switchOn(1); // uncomment for debug output
    }


    /**
     * Adds missing Nodes/Edges, or removes NACs from the given testGraph. If
     * after a certain period of time no solution is found it returns. The
     * structural error remains.
     * 
     * @param dockingGraph  The testgraph that should be repaired.
     * @param instance the StructuralConstraintInstance the DockingGraph is
     *        part of.
     */
    public void destroy(SCIDockingGraph dockingGraph, 
                        StructuralConstraintInstance instance) {
        repairedInstance = instance;
        dl.print(1, "\n destroy docking");

        int numNACs = dockingGraph.NACs.size();

        if ((numNACs > 0) && 
                (myGSC.random.nextDouble() < instance.createDockingNACProbability)) {
            MovableSubgraph nac = (MovableSubgraph) dockingGraph.NACs.elementAt(
                                          myGSC.random.nextInt(numNACs));
            mySCGraphRepairer.completeSubgraph(nac, instance.millisForRepair);

            if (nac.findOneEmbedding()) {
                instance.dockingGraphInvalid();
            } else {
                dl.print(1, "\nGraph could not be destroyed");
            }
        } else {
            if (destroyMainPart(dockingGraph)) {
                instance.dockingGraphInvalid();
            } else {
                dl.print(1, "\nGraph could not be destroyed");
            }
        }
    }


    /**
     * Destroys one of the mainParts of the given DockingGraph.
     * 
     * @param dockingGraph The dockingGraph the mainParts are part of.
     * 
     * @return true if a mainPart could be destroyed, else false.
     */
    protected boolean destroyMainPart(SCIDockingGraph dockingGraph) {
        int                           numParts = dockingGraph.mainParts.size();
        int                           partNr = myGSC.random.nextInt(numParts);
        StructuralConstraintGraphPart part   = (StructuralConstraintGraphPart) dockingGraph.mainParts.elementAt(
                                                       partNr);

        if (part instanceof NullMainPart) {
            return false;
        }

        mySCGraphRepairer.destroyGraphPart(part, 
                                           repairedInstance.removeNodeProbability);

        return true;
    }
}