/*
 * NodeChooseInterface.java
 *
 * Created on 5. Dezember 2002, 15:35
 */
package com.ai_center.dragonbreath.graphRepair;

import com.ai_center.dragonbreath.*;
import com.ai_center.dragonbreath.constraintgraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.*;
import com.ai_center.dragonbreath.utilities.*;

import java.util.*;

import org.jdom.Element;


/**
 * When the GraphRepairer creates a plan to add or delete nodes or edges, an
 * Object of this interface is asked, whether to use second hand nodes, that
 * allready exist in the GlobalSearchControl and which ones, or to rather
 * create new ones.
 * 
 * @version $Revision: 1.3 $
 * @author teutates
 */
public interface NodeChooseInterface {
    // Methods ****************************************************************

    /**
     * When a Node was created the GraphRepairer asks via
     * getAuxiliaryNodeData() what values to set as default.
     * 
     * @param node The new node.
     * 
     * @return A jdom.Element with the relevant data.
     */
    public Element getAuxiliaryNodeData(Node node);


    /**
     * Returns either an allready existing Node, or null, if the GraphRepairer
     * should create its own new one. The node should contain all the edges in
     * edges.
     * 
     * @param nodeTypeID The id of the node, as in Registry.
     * 
     * @return A node of the given type or null.
     */
    public Node getFittingNode(int nodeTypeID);


    /**
     * Returns either an allready existing Node, that can be used to connect n
     * to, or null, if the GraphRepairer should create its own new one.
     * 
     * @param node  The node the edge comes from. May be null, if Node not yet
     *        created.
     * @param cn  The edge. The returned node will later be the node member of
     *        that edge.
     * 
     * @return Either a node from GlobalSearchControl or null.
     */
    public Node getFittingNodeForEdgeCreation(Node node, ConnectedNode cn);


    /**
     * Registers a NodeChooser to be asked first to decide for a node. Normally
     * this is the TestGraphRepairer. It is the first to be asked for a
     * candidate.
     * 
     * @param nodeSource The new mainNodeSource.
     */
    public void setMainNodeSource(NodeChooseInterface nodeSource);


    /**
     * When a node is chosen to be used it can be tested for compliance with
     * the nodeChooser. Especially used to not use nodes for repair of a
     * subgraph, that were allready assigned to an other subgraph. Allways
     * called by a nodechooser from its mainNodeSource.
     * 
     * @param node The checked node.
     * 
     * @return Can the node be used?
     */
    public boolean isNodeOk(Node node);


    /**
     * When a node should get a new edge, but that edge can only be created via
     * the basegraph The node is removed by the graphRepairer, and the
     * nodeChooser informed via the setNodeReplaced() method. it then has to
     * inform the repaired graph that the interpretation of said node has
     * changed.
     * 
     * @param node The replaced node.
     */
    public void setNodeReplaced(Node node);


    /**
     * Each time a NodeChooseInterface was called with
     * getFittingNodeForEdgeCreation() or getFittingNode() and returned null
     * it has to be called again as soon as the requested node was produced
     * with this method.
     * 
     * @param node The new node.
     */
    public void registerNewNode(Node node);


    /**
     * Returns whether the node should be removed, or the edge added. Called if
     * an edge should be removed, but can not due to the fact that it is part
     * of the basegraph of the node. In such a case the addition of a new edge
     * of the same type could help, as it could then be used for the
     * basegraph.
     * 
     * @param node  The node.
     * @param edge  The edge that should be removed, but can not.
     * 
     * @return true if the node should be removed, false if an edge should be
     *         added.
     */
    public boolean removeNodeOrElseAddEdge(Node node, Object edge);
}