/*
 * StructuralConstraintDisplayer.java
 *
 * Created on 7. August 2002, 11:52
 */
package com.ai_center.dragonbreath.structuralConstraintHandling;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;


/**
 * Displays a list of the loaded structural constraints. When a constraint is
 * selected all dockinggraphs of all instances are highlighted.
 * 
 * @version $revision:$
 * @author tim baermann
 */
public class StructuralConstraintDisplayer extends javax.swing.JDialog {
    // Static variables/initializers ******************************************

    /** 
     * The only instance of this class 
     */
    private static StructuralConstraintDisplayer singleton;

    // Instance variables *****************************************************

    /** 
     * The structuralConstraints that should be displayed with this dialog. 
     */
    Object[] structuralConstraints;

    /** 
     * The last marked StructuralConstraint had this index 
     */
    int oldIndex = -1;

    /** 
     * The number of constraints in structuralConstraints 
     */
    int structuralConstraintsSize;

    /** 
     * ok 
     */
    private javax.swing.JButton okButton;

    /** 
     * List with structural constraints 
     */
    private javax.swing.JList scList;

    /** 
     * Holds the ok button 
     */
    private javax.swing.JPanel jPanel1;

    /** 
     * Holds jScrollpane1. 
     */
    private javax.swing.JPanel jPanel2;

    /** 
     * Holds the list of constraints 
     */
    private javax.swing.JScrollPane jScrollpane1;

    // Constructors ***********************************************************

    /**
     * Creates new form StructuralConstraintDisplayer
     * 
     * @param SCs List of structural constraints that are displayed.
     */
    private StructuralConstraintDisplayer(Collection SCs) {
        super(new javax.swing.JFrame(), true);
        this.structuralConstraints = SCs.toArray();
        structuralConstraintsSize  = SCs.size();
        initComponents();

        String[] scNames = new String[structuralConstraintsSize];

        for (int i = 0; i < structuralConstraintsSize; i++) {
            StructuralConstraintType actualSC = (StructuralConstraintType) structuralConstraints[i];
            scNames[i] = "(" + actualSC.numInstances() + ") " + 
                         actualSC.getName();
        }

        scList       = new JList(scNames);
        jScrollpane1 = new javax.swing.JScrollPane(scList);
        jPanel2.add(jScrollpane1, java.awt.BorderLayout.CENTER);

        //Provide minimum sizes for the two components in the split pane
        Dimension minimumSize = new Dimension(100, 50);
        jScrollpane1.setMinimumSize(minimumSize);

        MouseListener mouseListener = new myMouseAdapter(this);
        scList.addMouseListener(mouseListener);
    }

    // Methods ****************************************************************

    /**
     * Displays the dialog, if called more then once the dialog is only
     * displayed the first time.
     * 
     * @param SCs  A Set of structural constraints.
     */
    public static void displayDialog(Collection SCs) {
        if (singleton == null) {
            singleton = new StructuralConstraintDisplayer(SCs);
            singleton.show();
        }
    }


    /**
     * Closes the dialog, unmarks all constraint instances.
     * 
     * @param evt inherited, not used.
     */
    private void closeDialog(java.awt.event.WindowEvent evt) { //GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
        singleton = null;

        if (oldIndex != -1) {
            ((StructuralConstraintType) this.structuralConstraints[oldIndex]).unmarkInstances();
        }
    } //GEN-LAST:event_closeDialog


    /**
     * Calls closeDialog.
     * 
     * @param evt Not used.
     */
    private void exitDialog(java.awt.event.MouseEvent evt) { //GEN-FIRST:event_exitDialog
        closeDialog(null);
    } //GEN-LAST:event_exitDialog


    /**
     * This method is called from within the constructor to initialize the
     * form. WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() { //GEN-BEGIN:initComponents
        jPanel1  = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        jPanel2  = new javax.swing.JPanel();

        setTitle("Structural Constraints");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        okButton.setText("Ok");
        okButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                exitDialog(evt);
            }
        });

        jPanel1.add(okButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel2.setMinimumSize(new java.awt.Dimension(400, 200));
        jPanel2.setPreferredSize(new java.awt.Dimension(320, 200));
        jPanel2.setAutoscrolls(true);
        getContentPane().add(jPanel2, java.awt.BorderLayout.NORTH);

        pack();
    } //GEN-END:initComponents

    // Classes ****************************************************************

    class myMouseAdapter extends MouseAdapter {
        StructuralConstraintDisplayer myDisplayer;

        public myMouseAdapter(StructuralConstraintDisplayer d) {
            myDisplayer = d;
        }

        public void mouseClicked(MouseEvent e) {
            int index = scList.locationToIndex(e.getPoint());

            if (oldIndex != -1) {
                ((StructuralConstraintType) myDisplayer.structuralConstraints[oldIndex]).unmarkInstances();
            }

            ((StructuralConstraintType) myDisplayer.structuralConstraints[index]).markInstances();

            myDisplayer.oldIndex = index;
        }
    }

    // End of variables declaration
}