/*
 * connectingPartInterface.java
 *
 * Created on 24. Juni 2002, 18:33
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph;

import java.util.*;


/**
 * Interface for StructuralConstraintGraphParts. Is implemented by mainParts
 * and connectingPACParts. Is used during search for an embedding for the
 * whole graph, in order to find the parts that fit together.
 * 
 * @version $Revision: 1.3 $
 * @author tim baermann
 */
public interface ConnectingPartInterface {
    // Methods ****************************************************************

    /**
     * Returns all the frames of the part that fit to the actual interpretation
     * of  the linked nodes of this part i.e. have the same interpretaiton.
     * 
     * @return All the found frames.
     */
    public Stack getFittingFrames();


    /**
     * Returns all Parts that are linked to or are linked from by this part.
     * 
     * @return All Parts that are linked to or are linked from by this part.
     */
    public Set getInnerGraphLinkParts();


    /**
     * If there exists a frame that fits to the actual interpretation of  the
     * linked nodes of part i.e. have the same interpretaiton, this function
     * returns true, else false.
     * 
     * @param part  The other part connected to this one.
     * 
     * @return There is at least one frame fitting to part.
     */
    public boolean existFittingFrames(StructuralConstraintGraphPart part);


    /**
     * Is there at least one frame for the part?
     * 
     * @return Is there a frame stored?
     */
    public boolean existsFrame();
}