/*
 * SCGPComparator.java
 *
 * Created on 23. April 2002, 22:50
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph;

import java.util.Comparator;


/**
 * Sorts StructuralConstraintGraphParts by the number of other connected parts
 * 
 * @version $Revision: 1.2 $
 * @author tim baermann
 */
public class SCGPComparator implements Comparator {
    // Instance variables *****************************************************

    /** 
     * Is the Comperator sorting ascending? 
     */
    private boolean ascending;

    // Constructors ***********************************************************

    /**
     * Creates a new instance of SCGPComparator that sorts the parts  by the
     * number of other connected parts.
     * 
     * @param isAscending  Should the comparator order ascending or descending?
     */
    public SCGPComparator(boolean isAscending) {
        ascending = isAscending;
    }

    // Methods ****************************************************************

    /**
     * Brings two StructuralConstraintGraphParts in order, either ascending or
     * descending, depending on the comparator.
     * 
     * @param a One element.
     * @param b The other element.
     * 
     * @return a > b or b > a
     */
    public int compare(Object a, Object b) {
        StructuralConstraintGraphPart ga = (StructuralConstraintGraphPart) a;
        StructuralConstraintGraphPart gb = (StructuralConstraintGraphPart) b;

        if (ascending) {
            return ga.numberOfConnectedParts() - gb.numberOfConnectedParts();
        } else {
            return gb.numberOfConnectedParts() - ga.numberOfConnectedParts();
        }
    }
} //class
