/*
 * UnconnectedGraphPart.java
 *
 * Created on 4. Oktober 2002, 17:59
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph;

import com.ai_center.dragonbreath.constraintgraph.Node;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.SCNode;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.StructuralConstraintGraphPart;

import java.util.*;


/**
 * MainParts and ConnectingPACParts in DockingGraphs. These are parts of the
 * graph that are used for the puzzling in SCTDockingGraph.
 * 
 * @author tim baermann
 */
public interface UnconnectedGraphPartInterface {
    // Methods ****************************************************************

    /**
     * Is the given interpretation allready an seedInterpretations?
     * 
     * @param n The checked node.
     * 
     * @return true if n is in seedInterpretations.
     */
    public boolean isSeedInterpretation(Node n);


    /**
     * Returns the seedInterpretations of the part.
     * 
     * @return The seedInterpretations.
     */
    public HashSet getSeedInterpretations();


    /**
     * Called before findNextEmbedding(). Causes that the search process does
     * not proceed if no new seed interpretation could be found. Speeds up a
     * little bit the search.
     * 
     * @param on  Switch on or off.
     */
    public void findNewSeedsOn(boolean on);


    /**
     * Is the current search searching for new seeds?
     * 
     * @return findNewSeeds
     */
    public boolean findNewSeedsOn();


    /**
     * Finds next valid embedding for the AC part using next seedInterpretation
     * if with current seed no more embeddings can be found. Uses also
     * interpretations allready used by other parts.
     * 
     * @return true if an embedding was found.
     */
    public boolean findNextEmbeddingWithSeedInterpretations();


    /**
     * The number of interpretations found for the seed.
     * 
     * @return A positive number.
     */
    public int nrSeedInterpretations();


    /**
     * Unconnected AC parts store all the possible interpretations for their
     * seed, that resulted in a complete match. So when searching for an
     * embedding for all AC parts the part can easily find a valid embedding.
     */
    public void registerSeedInterpretation();


    /**
     * Remove an interpretation from the list of the possible interpretations.
     * 
     * @param n A node that should no longer be an seed interpretation.
     */
    public void removeSeedInterpretation(Node n);


    /**
     * Removes all the seedInterpretations that were deleted.
     * 
     * @param deletedNodes Nodes that were deleted.
     */
    public void testForDeletedSeeds(Collection deletedNodes);


    /**
     * Sets the seedInterpretation to be the first one available.
     */
    void setFirstSeedInterpretation();


    /**
     * If the AC is not connected to a mainPart there might be several possible
     * seed interpretation. This function steps through them. If the AC is
     * connected to a mainPart the seed interpretation is copied from the
     * seeds linked node.
     * 
     * @return true if there are more seedInterpretations left.
     */
    boolean setNextSeedInterpretation();


    /**
     * When with the actual seedInterpretaion no embedding could be found the
     * interpretation is invalid, so delete it with this function. Deletes the
     * actual element of lastUsedSeedInterpretationIt.
     */
    void deleteActualSeedInterpretation();


    /**
     * Calls first setNextSeedInterpretation() and then findOneEmbedding(). If
     * a seed interpretation didnt result in a match and that was not due to
     * an intersection it is deleted.
     * 
     * @return true if an embedding was found, else false.
     */
    boolean findEmbeddingWithNextSeedInterpretation();


    /**
     * finds one embedding trying all the seed interpretations.
     * 
     * @return found.
     */
    boolean findOneEmbeddingWithSeedInterpretations();


    /**
     * Registeres the given interpretation as seedInterpretation. Does not
     * check wheter seedInterpretation is null
     * 
     * @param seedInterpretation  The new seedInterpretation.
     */
    void registerSeedInterpretation(Node seedInterpretation);
}