/*
 * ApplicationCondition.java
 *
 * Created on 1. Maerz 2002, 15:24
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition;

import com.ai_center.dragonbreath.constraintgraph.Node;
import com.ai_center.dragonbreath.constraintgraph.NodeType;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * This interface is implemented by all a structural graph parts that determine
 * whether  a found match of the rest of the graph is applicable, or not.
 * 
 * @version $Revision: 1.2 $
 * @author tim baermann
 */
public interface ApplicationConditionPart {
    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return An ApplicationCondition.
     */
    public ApplicationCondition getApplicationCondition();


    /**
     * Called from the ApplicationCondition if the Part was found.
     * 
     * @return true if further search is necessary, eg. if no more mainParts are
     *         missing, else false.
     */
    public boolean handleMatch();


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame);


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame);
} //class
