/*
 * UnconnectedACPart.java
 *
 * Created on 13. Oktober 2002, 17:40
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition;

import com.ai_center.dragonbreath.constraintgraph.Node;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * A part of an ApplicationCondition that
 * 
 * @version $Revision: 1.2 $
 * @author tim baermann
 */
public abstract class UnconnectedACPart
    extends UnconnectedMovableSubgraphPart implements ApplicationConditionPart {
    // Instance variables *****************************************************

    /**
     * All the mainParts that were not created due to this
     * ApplicationConditionPart.
     */
    protected PartFrameContainer mainPartsMissingThisACPart = 
            new PartFrameContainer();

    // Constructors ***********************************************************

    /**
     * Creates a new instance of UnconnectedACPart
     * 
     * @param ac The ApplicationCondition this Part belongs to.
     */
    public UnconnectedACPart(ApplicationCondition ac) {
        super((MovableSubgraph) ac);
    }

    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return The ApplicationCondition this part belongs to.
     */
    public ApplicationCondition getApplicationCondition() {
        return (ApplicationCondition) mySubgraph;
    }


    /**
     * When no match was found for the part this funciton is called to prepare
     * the search of the part, and the recheck of the mainPart, once the part
     * is found. Called by ApplicationCondition.testPartAC()
     * 
     * @param mainPartFrame  A frame of the mainPart that is in need of this
     *        part.
     */
    public void registerMainPartFrame(GraphPartFrame mainPartFrame) {
        if (mainPartFrame != null) {
            mainPartsMissingThisACPart.add(mainPartFrame);
        }
    }


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        return findOneEmbeddingAllParts();
    }


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        return testAC(mainPartFrame);
    }
}