/*
 * SCINAC.java
 *
 * Created on 22. Juli 2002, 18:11
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.instance;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type.TypeDockingNAC;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * A NAC of the instance of a DockingGraph.
 * 
 * @version $Revision: 1.2 $
 * @author tim baermann
 */
public class InstanceDockingNAC extends ApplicationCondition {
    // Instance variables *****************************************************

    /** 
     * A mainPartFrame of the match that lead to this instance. 
     */
    GraphPartFrame typeMainPartFrame;

    /** 
     * The NAC of the type that correspondens to this NAC. 
     */
    TypeDockingNAC typeNAC;

    // Constructors ***********************************************************

    /**
     * Creates a new StructuralConstraintInstance Negative Application
     * Condition.
     * 
     * @param scg The StructuralConstraintGraph, a DockingGraph, this part
     *        belongs to.
     */
    public InstanceDockingNAC(DockingGraph scg) {
        super(scg);
        scg.addNAC(this);
    }

    // Methods ****************************************************************

    /**
     * Copies the interpretations of the other graph to this one. The two
     * graphs must be exactly identical in structure. Enters the source as
     * typePart.
     * 
     * @param source the graph with the interpretations
     * 
     * @throws Error Thrown if source has diffent structure then this.
     */
    public void copyInterpretations(StructuralConstraintGraphPart source) {
        typeNAC           = (TypeDockingNAC) source;
        typeMainPartFrame = typeNAC.getAnyConnectedMainPartFrame();

        Iterator destIt   = allParts.iterator();
        Iterator srcIt = ((ApplicationCondition) source).allParts.iterator();

        while (destIt.hasNext()) {
            if (! srcIt.hasNext()) {
                throw new Error(
                        "ApplicationConditions should have the same stucture.");
            }

            try {
                InstanceDockingUnconnectedNACPart part = 
                        (InstanceDockingUnconnectedNACPart) destIt.next();
                part.copyInterpretations(
                        (StructuralConstraintGraphPart) srcIt.next());
            } catch (ClassCastException e) {
                srcIt.next();
            } //do nothing
        }
    }


    /**
     * Checks whether the NAC can be found now. If nac got invalid
     * 
     * @return true if the NAC is still valid, else false.
     */
    public boolean testAC() {
        boolean  found = true;
        Iterator it = allParts.iterator();

        while (it.hasNext() && found) { // while all parts are found

            MovableSubgraphPart acp = (MovableSubgraphPart) it.next();


            // no check necessary if newNodes or expanded nodes are part of the part, 
            // as every single part might have existed before.
            found = acp.findOneEmbeddingAllParts();
        }

        if (found) {
            Set nodesInMatch = new HashSet();
            it = allParts.iterator();

            while (it.hasNext()) {
                MovableSubgraphPart g = (MovableSubgraphPart) it.next();
                nodesInMatch.addAll(g.getInterpretedNodes().keySet());
            } //while

            typeNAC.registerMatch(typeMainPartFrame, nodesInMatch);
        }

        return ! found;
    }
} //class
