/*
 * SCIPAC.java
 *
 * Created on 21. Juli 2002, 21:03
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.instance;

import com.ai_center.dragonbreath.constraintgraph.Node;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.DockingGraph;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * A PAC of an instance of a DockingGraph.
 * 
 * @version $Revision: 1.3 $
 * @author tim baermann
 */
public class InstanceDockingPAC extends ApplicationCondition {
    // Constructors ***********************************************************

    /**
     * Creates a new StructuralConstraintInstance Positive Application
     * Condition.
     * 
     * @param scg The StructuralConstraintGraph, a DockingGraph, this part
     *        belongs to.
     */
    public InstanceDockingPAC(DockingGraph scg) {
        super((StructuralConstraintGraph) scg);
        scg.addPAC(this);
    }

    // Methods ****************************************************************

    /**
     * Checks whether one of the parts is deleted due to a deleted node. If pac
     * got invalid
     * 
     * @param deletedNodes  Nodes that are no longer in the problemgraph
     * @param reducedNodes  Nodes that lost a connection.
     * 
     * @return true if the PAC is still valid, else false.
     */
    public boolean checkAC(Set deletedNodes, Set reducedNodes) {
        boolean  valid   = true;
        Iterator partsIt = allParts.iterator();

        while (partsIt.hasNext() && valid) {
            MovableSubgraphPart pacPart = (MovableSubgraphPart) partsIt.next();
            valid = checkPart(pacPart, deletedNodes) && 
                    checkPart(pacPart, reducedNodes);
        }

        return valid;
    }


    /**
     * Checks the given part. Called from checkAC().
     * 
     * @param pacPart  The part.
     * @param changedNodes The nodes that could be part of the part.
     * 
     * @return true if the part can be found, false else.
     */
    private boolean checkPart(MovableSubgraphPart pacPart, Set changedNodes) {
        boolean  valid = true;
        Iterator it = changedNodes.iterator();

        while (it.hasNext() && valid) {
            Node n = (Node) it.next();

            if (pacPart.partContainsInterpretation(n)) { // the pac of the instance uses the given interpretation.
                pacPart.resetInterpretations(); // try to find it again

                if (! pacPart.findOneEmbeddingAllParts()) { // if no substitude can be found
                    ((InstanceDockingPACPart) pacPart).registerMissingMatch();
                    valid = false;
                } // if findOneEmbedding
            } //if pac.contains
        } //while it

        return valid;
    }
} //class
