/*
 * SCIPACPart.java
 *
 * Created on 20. Juli 2002, 18:56
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.instance;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type.TypeDockingACPart;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * A part of a PAC of a DockingGraphInstance.
 * 
 * @version $Revision: 1.2 $
 * @author tim baermann
 */
public class InstanceDockingPACPart extends MovableSubgraphPart
    implements ApplicationConditionPart {
    // Instance variables *****************************************************

    /**
     * The ApplicationConditionPart in the Type DockingGraph that
     * correspondends to this one.
     */
    ApplicationConditionPart typePart;

    /**
     * The MainPartFrame of one mainPart connected to the typePart, at the
     * moment of the match.
     */
    GraphPartFrame typeMainPartFrame;

    // Constructors ***********************************************************

    /**
     * Creates a new instance of SCIPACPart.
     * 
     * @param ac The ApplicationCondition this part belongs to.
     */
    public InstanceDockingPACPart(ApplicationCondition ac) {
        super(ac);
    }

    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return The mySubgraph member.
     */
    public ApplicationCondition getApplicationCondition() {
        return (ApplicationCondition) mySubgraph;
    }


    /**
     * Copies the interpretations of the other graph to this one. The two
     * graphs must be exactly identical in structure. Enters the source as
     * typePart.
     * 
     * @param source the graph with the interpretations
     */
    public void copyInterpretations(StructuralConstraintGraphPart source) {
        super.copyInterpretations(source);
        typePart          = (ApplicationConditionPart) source;
        typeMainPartFrame = typePart.getApplicationCondition()
                                    .getAnyConnectedMainPartFrame();
    }


    /**
     * If the ApplicationConditionPart is not valid any more this function
     * registeres it at the Type for search, so that the Instance can be
     * recreated when the pac is valid again.
     */
    public void registerMissingMatch() {
        ((TypeDockingACPart) typePart).registerMainPartFrame(typeMainPartFrame);
    }


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        return true;
    }


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        return true;
    }
} //class
