/*
 * SCTNACPart.java
 *
 * Created on 18. Juli 2002, 13:11
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.ApplicationCondition;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.ApplicationConditionPart;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * A part of a NAC of a dockingGraphType.
 * 
 * @version $Revision: 1.2 $
 * @author tim baermann
 */
public class TypeDockingNACPart extends MovableSubgraphPart
    implements TypeDockingACPart {
    // Instance variables *****************************************************

    /**
     * All the mainParts that were not created due to this
     * ApplicationConditionPart.
     */
    protected PartFrameContainer mainPartsMissingThisACPart = 
            new PartFrameContainer();

    // Constructors ***********************************************************

    /**
     * Creates a new instance of SCTNACPart
     * 
     * @param ac The NAC this part belongs to.
     */
    public TypeDockingNACPart(ApplicationCondition ac) {
        super(ac);
    }

    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return The mySubgraph member.
     */
    public ApplicationCondition getApplicationCondition() {
        return (ApplicationCondition) mySubgraph;
    }


    /**
     * When no match was found for the part this funciton is called to prepare
     * the search of the part, and the recheck of the mainPart, once the part
     * is found. Called by ApplicationCondition.testPartAC()
     * 
     * @param mainPartFrame  A frame of the mainPart that is in need of this
     *        part.
     */
    public void registerMainPartFrame(GraphPartFrame mainPartFrame) {
        if (mainPartFrame != null) {
            mainPartsMissingThisACPart.add(mainPartFrame);
        }
    }


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        return findOneEmbeddingAllParts();
    }


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        return testAC(mainPartFrame);
    }
} //class
