/*
 * SCTPAC.java
 *
 * Created on 23. Juni 2002, 23:49
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.Iterator;


/**
 * A PAC of a dockingGraph type.
 * 
 * @version $Revision: 1.3 $
 * @author tim baermann
 */
public class TypeDockingPAC extends ApplicationCondition {
    // Constructors ***********************************************************

    /**
     * Creates a new instance of SCTPAC registeres this at the DockingGraph.
     * 
     * @param scg The dockingGraph this PAC belongs to.
     */
    public TypeDockingPAC(SCTDockingGraph scg) {
        super(scg);
        scg.addPAC(this);

        //dl.switchOn(3);// uncomment for debug output
    }

    // Methods ****************************************************************

    /**
     * Same as testAC() but with every missing ACPart the given mainPartFrame
     * is stored.
     * 
     * @param mainPartFrame  the frame of the mainPart that should be tested.
     * 
     * @return true if all parts could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        boolean  found = true;
        Iterator it = allParts.iterator();

        while (it.hasNext() && found) { // while all parts are found

            ApplicationConditionPart acp = (ApplicationConditionPart) it.next();
            found            = acp.testAC(mainPartFrame);
            firstMissingPart = (MovableSubgraphPart) acp;
        }

        if (! found) {
            it = allParts.iterator();

            while (it.hasNext()) { // reset all the parts that have got an interpretation until now.

                MovableSubgraphPart g = (MovableSubgraphPart) it.next();

                if (g == firstMissingPart) { // ok this was the first part missing, so this and the following have no interpretations.

                    break;
                } else {
                    g.resetInterpretations();
                }
            }
        }

        return found;
    } //testAC()


    /**
     * When one MainPart has its nodes assigned, this method tests wheter the
     * whole AC can be embedded in the graph. Is called when the MainPart was
     * found before, but not stored, due to a missing AC, and now the part
     * should be tested again, as the missing AC was found. If false the
     * missing parts are registered for search, the mainPart is registered as
     * partlyFound. If true all the mainParts that might have been in need of
     * this PAC get restored too.
     * 
     * @param mainPartFrame  the frame of the mainPart that should be tested.
     * 
     * @return true if all PACParts were found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        dl.print(3, "\ntestPartAC ");

        boolean                  found            = true;
        ApplicationConditionPart firstMissingPart = null;
        Iterator                 it               = allParts.iterator();

        while (it.hasNext() && found) { // while all parts are found

            ApplicationConditionPart acp = (ApplicationConditionPart) it.next();
            found = acp.testPartAC(mainPartFrame);

            if (! found) {
                firstMissingPart = acp;
            }
        }


        //then the interpretation has to be unassigned. and after this call the interpretation of the PAC doesnt
        // matter anyway.
        it = allParts.iterator();

        while (it.hasNext()) { // reset all the parts that have got an interpretation until now.

            MovableSubgraphPart g = (MovableSubgraphPart) it.next();

            if (g == firstMissingPart) { // ok this was the first part missing, 

                //so this and the following parts have no interpretations.
                break;
            } else {
                g.resetInterpretations();
            }
        }

        return found;
    } // testPartAC()
}