/*
 * PartPACPart.java
 *
 * Created on 26. Juni 2002, 18:03
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type;

import com.ai_center.dragonbreath.constraintgraph.NodeType;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * A part of a PAC of a DockingGraphType. The part is connected to exactly one
 * MainPart.
 * 
 * @version $Revision: 1.3 $
 * @author tim baermann
 */
public class TypeDockingPartPACPart extends MovableSubgraphPart
    implements TypeDockingACPart {
    // Instance variables *****************************************************

    /**
     * All the mainParts that were not created due to this
     * ApplicationConditionPart.
     */
    protected PartFrameContainer mainPartsMissingThisACPart = 
            new PartFrameContainer();

    // Constructors ***********************************************************

    /**
     * Creates a new instance of PartPACPart
     * 
     * @param ac The PAC the part belongs to.
     */
    public TypeDockingPartPACPart(ApplicationCondition ac) {
        super(ac);
    }

    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return The mySubgraph member.
     */
    public ApplicationCondition getApplicationCondition() {
        return (ApplicationCondition) mySubgraph;
    }


    /**
     * Returns all the nodes of this ACPart that have the same type as
     * NodeType.
     * 
     * @param n The nodeType.
     * 
     * @return All nodes of this part of the given type n.
     */
    public Set getSearchedNodes(NodeType n) {
        Set result;

        if (searchedNodes.size() > 0) {
            if (mainPartsMissingThisACPart.size() > 0) {
                // Are there still mainParts in mainPartsMissingThisACPart?
                // they might have been deleted by PartFrameContainer.allContainerCheckDeletedNodes()
                result = searchedNodes.getNodes(n);
            } else {
                searchedNodes.clear(); // there are no mainParts missing this part, so stop searching
                result = new HashSet(0);
            }
        } else {
            result = new HashSet(0);
        }

        return result;
    }


    /**
     * Used to restore all the mainParts in mainPartsMissingThisACPart that
     * have now the missing part assigned. They are registered with the
     * DockingGraph, and there tested after all new nodes were tested.
     */
    public void checkWrongMainParts() {
        Collection fittingMainParts = mainPartsMissingThisACPart.extractFittingParts(
                                              this.getInnerGraphLinkNodes());

        if (mySCGraph instanceof SCTDockingGraph) {
            ((SCTDockingGraph) mySCGraph).registerImprovedMainPartMatches(
                    fittingMainParts);
        }
    } // checkWrongMainParts()


    /**
     * When a new match for the AC is found during
     * ApplicationCondition.checkNodes() if part was missing for mainPart:
     * check main part again.
     * 
     * @return true if further search is necessary, eg. if no more mainParts are
     *         missing.
     */
    public boolean handleMatch() {
        checkWrongMainParts();

        return mainPartsMissingThisACPart.size() > 0; // only if there are still mainParts missing this part 

        // the search must go on
    }


    /**
     * When no match was found for the part this funciton is called to prepare
     * the search of the part, and the recheck of the mainPart, once the part
     * is found. Called by ApplicationCondition.testPartAC()
     * 
     * @param mainPartFrame  A frame of the mainPart that is in need of this
     *        part.
     */
    public void registerMainPartFrame(GraphPartFrame mainPartFrame) {
        mainPartsMissingThisACPart.add(mainPartFrame);

        if (searchedNodes.size() == 0) { // if the part was not searched before start search now
            searchedNodes.addNodes(nodes);
        }
    }


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        boolean found = findOneEmbeddingAllParts();

        if (! found) {
            registerMainPartFrame(mainPartFrame);

            Iterator intersectingPartsIt = getIntersectingPartsIterator();

            while (intersectingPartsIt.hasNext()) {
                TypeDockingACPart missingPart = (TypeDockingACPart) intersectingPartsIt.next();
                missingPart.registerMainPartFrame(mainPartFrame);
            }
        }

        return found;
    }


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        if (! innerGraphLinks.containsKey(mainPartFrame.myGraphPart)) {
            return true; // if the PAC of this Part is tested for a mainPart not connected to this return true.
        } else {
            return testAC(mainPartFrame);
        }
    }
}