/*
 * UnconnectedSCTNACPart.java
 *
 * Created on 17. Juli 2002, 18:03
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;

import java.util.*;


/**
 * A part of a NAC of a dockinggraph type that is not connected to any
 * mainPart.
 * 
 * @version $Revision: 1.2 $
 * @author tim baermann
 */
public class TypeDockingUnconnectedNACPart extends UnconnectedACPart
    implements TypeDockingACPart {
    // Constructors ***********************************************************

    /**
     * Creates a new instance of UnconnectedSCTNACPart
     * 
     * @param ac The ApplicationCondition this part belongs to.
     */
    public TypeDockingUnconnectedNACPart(ApplicationCondition ac) {
        super(ac);
    }

    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return The mySubgraph member.
     */
    public ApplicationCondition getApplicationCondition() {
        return (ApplicationCondition) mySubgraph;
    }


    /**
     * When a new match for the AC is found during
     * ApplicationCondition.checkNodes() if part was missing for mainPart:
     * check main part again.
     * 
     * @return true as unconnected parts are allways searched.
     */
    public boolean handleMatch() {
        registerSeedInterpretation();

        return true; // unconnected parts are allways searched.

        // the search must go on
    }


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        boolean found = findOneEmbeddingAllParts();

        if (! found) {
            registerMainPartFrame(mainPartFrame);

            Iterator intersectingPartsIt = getIntersectingPartsIterator();

            while (intersectingPartsIt.hasNext()) {
                TypeDockingACPart missingPart = (TypeDockingACPart) intersectingPartsIt.next();
                missingPart.registerMainPartFrame(mainPartFrame);
            }
        }

        return found;
    }


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        return testAC(mainPartFrame);
    }
}