/*
 * UnconnectedSCTPACPart.java
 *
 * Created on 26. Juni 2002, 19:04
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;

import java.util.*;


/**
 * A part of a PAC of the type of a dockingGraph that is not connected to a
 * mainPart.
 * 
 * @version $Revision: 1.3 $
 * @author tim baermann
 */
public class TypeDockingUnconnectedPACPart extends UnconnectedACPart
    implements TypeDockingACPart {
    // Instance variables *****************************************************

    /**
     * All the mainParts that were not created due to this
     * ApplicationConditionPart.
     */
    protected PartFrameContainer mainPartsMissingThisACPart = 
            new PartFrameContainer();

    // Constructors ***********************************************************

    /**
     * Creates a new instance of UnconnectedSCTPACPart
     * 
     * @param ac The applicationCondition this part belongs to.
     */
    public TypeDockingUnconnectedPACPart(ApplicationCondition ac) {
        super(ac);
    }

    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return mySubgraph.
     */
    public ApplicationCondition getApplicationCondition() {
        return (ApplicationCondition) mySubgraph;
    }


    /**
     * Used to restore all the mainParts in mainPartsMissingThisACPart that
     * have now the missing part assigned. They are registered with the
     * DockingGraph, and there tested after all new nodes were tested.
     */
    public void checkWrongMainParts() {
        Collection fittingMainParts = mainPartsMissingThisACPart.extractFittingParts(
                                              this.getInnerGraphLinkNodes());

        if (mySCGraph instanceof SCTDockingGraph) {
            ((SCTDockingGraph) mySCGraph).registerImprovedMainPartMatches(
                    fittingMainParts);
        }
    } // checkWrongMainParts()


    /**
     * When a new match for the AC is found during
     * ApplicationCondition.checkNodes() if part was missing for mainPart:
     * check main part again.
     * 
     * @return Is further search necessary? eg. if no more mainParts are
     *         missing.
     */
    public boolean handleMatch() {
        checkWrongMainParts();
        registerSeedInterpretation();

        return true; // unconnected parts are allways searched.

        // the search must go on
    }


    /**
     * When no match was found for the part this funciton is called to prepare
     * the search of the part, and the recheck of the mainPart, once the part
     * is found. Called by ApplicationCondition.testPartAC()
     * 
     * @param mainPartFrame  A frame of the mainPart that is in need of this
     *        part.
     */
    public void registerMainPartFrame(GraphPartFrame mainPartFrame) {
        mainPartsMissingThisACPart.add(mainPartFrame);

        if (searchedNodes.size() == 0) { // if the part was not searched before start search now
            searchedNodes.addNodes(nodes);
        }
    }


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        boolean found = findOneEmbeddingAllParts();

        if (! found) {
            registerMainPartFrame(mainPartFrame);

            Iterator intersectingPartsIt = getIntersectingPartsIterator();

            while (intersectingPartsIt.hasNext()) {
                TypeDockingACPart missingPart = (TypeDockingACPart) intersectingPartsIt.next();
                missingPart.registerMainPartFrame(mainPartFrame);
            }
        }

        return found;
    }


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return true if the ACPart could be found, else false.
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        if (! getInnerGraphLinkParts().contains(mainPartFrame.myGraphPart)) {
            return true; // if the PAC of this Part is tested for a mainPart not connected to this return true.
        } else {
            return testAC(mainPartFrame);
        }
    }
}