/*
 * InstanceTestNACPart.java
 *
 * Created on 6. Oktober 2002, 20:05
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.testGraph.applicationCondition.instance;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.dockingGraph.applicationCondition.type.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.movableSubgraph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.testGraph.*;

import java.util.*;


/**
 * A part of a NAC of an InstanceTestGraph.
 * 
 * @version $Revision: 1.3 $
 * @author tim baermann
 */
public class InstanceTestNACPart extends MovableSubgraphPart
    implements ApplicationConditionPart {
    // Instance variables *****************************************************

    /**
     * All the mainParts that were not created due to this
     * ApplicationConditionPart.
     */
    protected PartFrameContainer mainPartsMissingThisACPart = 
            new PartFrameContainer();

    // Constructors ***********************************************************

    /**
     * Creates a new instance of InstanceTestNACPart
     * 
     * @param ac The ApplicationCondition this belongs to.
     */
    public InstanceTestNACPart(ApplicationCondition ac) {
        super(ac);
    }

    // Methods ****************************************************************

    /**
     * Returns the ApplicationCondition this part belongs to.
     * 
     * @return The mySubgraph member.
     */
    public ApplicationCondition getApplicationCondition() {
        return (ApplicationCondition) mySubgraph;
    }


    /**
     * Produces a new node that can be connected via connectNodes to others in
     * this graph Also adds the node to the searched ones.
     * 
     * @param id the nodeID as defined in Registry
     * @param isLeafNode Is the created node a leaf node? If false it is added
     *        to the searchedNodes.
     * 
     * @return the newly created SCNode
     */
    public SCNode addNode(int id, boolean isLeafNode) {
        SCNode result = super.addNode(id, isLeafNode);

        if (! isLeafNode) {
            searchedNodes.addNode(result);
        }

        return result;
    }


    /**
     * Inherited, does nothing.
     * 
     * @param source the graph with the interpretations
     */
    public void copyInterpretations(StructuralConstraintGraphPart source) {
    }


    /**
     * Called whenever a match was found for the part. Inititiates the check of
     * the whole NAC, so the testGraph can be deleted, if NAC is found.
     * 
     * @return false. All possible interpretations are tried in the checkNAC,
     *         so no findNextPartEmbeddingWithoutCheckedNodes() in
     *         MovableSubgraph is necesary
     */
    public boolean handleMatch() {
        ((TestGraph) mySCGraph).checkNAC(getApplicationCondition());
        mySubgraph.resetInterpretations();

        return false;
    }


    /**
     * When no match was found for the part this funciton is called to prepare
     * the search of the part, and the recheck of the mainPart, once the part
     * is found. Called by ApplicationCondition.testPartAC()
     * 
     * @param mainPartFrame  A frame of the mainPart that is in need of this
     *        part.
     */
    public void registerMainPartFrame(GraphPartFrame mainPartFrame) {
        if (mainPartFrame != null) {
            mainPartsMissingThisACPart.add(mainPartFrame);
        }
    }


    /**
     * Called for every part of AC in ApplicationCondition.testAC(). Searches
     * for an embedding, and if none found initiates search for the ACPart to
     * create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return could the ACPart be found?
     */
    public boolean testAC(GraphPartFrame mainPartFrame) {
        boolean found = findOneEmbeddingAllParts();

        if (! found) {
            registerMainPartFrame(mainPartFrame);

            Iterator intersectingPartsIt = getIntersectingPartsIterator();

            while (intersectingPartsIt.hasNext()) {
                TypeDockingACPart missingPart = (TypeDockingACPart) intersectingPartsIt.next();
                missingPart.registerMainPartFrame(mainPartFrame);
            }
        }

        return found;
    }


    /**
     * Called for every part of AC in ApplicationCondition.testPartAC().
     * Searches for an embedding, and if none found initiates search for the
     * ACPart to create the mainPart when the ACPart is created.
     * 
     * @param mainPartFrame  A frame of the mainPart
     * 
     * @return could the ACPart be found?
     */
    public boolean testPartAC(GraphPartFrame mainPartFrame) {
        return testAC(mainPartFrame);
    }
}