/*
 * InstanceTestUnconnectedPart.java
 *
 * Created on 6. Oktober 2002, 19:34
 */
package com.ai_center.dragonbreath.structuralConstraintHandling.graph.testGraph.applicationCondition.instance;

import com.ai_center.dragonbreath.structuralConstraintHandling.graph.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.applicationCondition.*;
import com.ai_center.dragonbreath.structuralConstraintHandling.graph.testGraph.*;


/**
 * A part of a NAC that is not connected to any mainPart.
 * 
 * @version $Revision: 1.4 $
 * @author tim baermann
 */
public class InstanceTestUnconnectedNACPart extends UnconnectedACPart {
    // Instance variables *****************************************************

    /** 
     * The testGraph the AC of this Part belongs to. 
     */
    StructuralConstraintGraph mySCGraph;

    // Constructors ***********************************************************

    /**
     * Creates a new instance of InstanceTestUnconnectedPart
     * 
     * @param ac The ApplicationCondition this part belongs to.
     */
    public InstanceTestUnconnectedNACPart(ApplicationCondition ac) {
        super(ac);
        mySCGraph = ac.mySCGraph;
    }

    // Methods ****************************************************************

    /**
     * Produces a new node that can be connected via connectNodes to others in
     * this graph Also adds the node to the searched ones.
     * 
     * @param id the nodeID as defined in Registry
     * @param isLeafNode Is the created node a leaf node? If false it is added
     *        to the searchedNodes.
     * 
     * @return the newly created SCNode
     */
    public SCNode addNode(int id, boolean isLeafNode) {
        SCNode result = super.addNode(id, isLeafNode);

        if (! isLeafNode) {
            searchedNodes.addNode(result);
        }

        return result;
    }


    /**
     * Sets the given source as the myPartType member of this, so that its
     * seedInterpretations can be used.
     * 
     * @param source the graph with the interpretations
     */
    public void copyInterpretations(StructuralConstraintGraphPart source) {
        myPartType = (UnconnectedACPart) source;
    }


    /**
     * Called whenever a match was found for the part. Inititiates the check of
     * the whole NAC, so the testGraph can be deleted, if NAC is found.
     * 
     * @return false.
     */
    public boolean handleMatch() {
        ((TestGraph) mySCGraph).checkNAC(getApplicationCondition());

        return false;
    }
}