
package com.ai_center.dragonbreath.utilities;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

/** 
 * The list element represents a temporal interval and stores a link to 
 * a content element.
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.2 $
 *
 * @see IntervalList
 */

public class IntervalListElement {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /** 
     * The beginning of the interval.
     */
    private long beginning;
    /** 
     * The duration of the interval.
     */
    private long duration;
    /** 
     * The end of the interval.
     */
    private long end;

    /** 
     * The previous list element.
     */
    private IntervalListElement previous = null;

    /** 
     * The following list element.
     */
    private IntervalListElement next = null;

    /** 
     * The list element's content element.
     */
    private Object element;

	private EventListenerList listenerList = new EventListenerList();
    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /** 
     * Initializes the IntervalListElement with a given beginning, end
     * and content element.
     * @param beginning  The beginning of the interval.
     * @param end  The end of the interval.
     * @param element  The element for the interval.
     */
    public IntervalListElement(long beginning, long end, Object element) {
	this.beginning = beginning;
	duration = end - beginning;
	this.end = end;
	this.element = element;
    }

    /** 
     * Initializes the IntervalListElement with a given beginning and end
     * but without a content element.
     * @param beginning  The beginning of the interval.
     * @param end  The end of the interval.
     */
    public IntervalListElement(long beginning, long end) {
	this.beginning = beginning;
	duration = end - beginning;
	this.end = end;
	this.element = null;
    }

    /** 
     * Initializes the IntervalListElement without a specific beginning,
     * end, or element.
     */
    public IntervalListElement() {
	beginning = 0;
	duration = 0;
	end = 0;
	element = null;
    }


    // *******************
    // ***** METHODS *****
    // *******************

    /** 
     * Prints the interval information (beginning, duration, end and element).
     */
    public String toString() {
	return new String("[" + beginning + "..(" + duration + ").." + end + "]<" + element + ">");
    }

    /**
     * @return
     */
    public synchronized long getBeginning() {
        return beginning;
    }

    /**
     * @return
     */
	public synchronized long getDuration() {
        return duration;
    }

    /**
     * @return
     */
	public synchronized Object getElement() {
        return element;
    }

    /**
     * @return
     */
	public synchronized long getEnd() {
        return end;
    }

    /**
     * @return
     */
	public synchronized IntervalListElement getNext() {
        return next;
    }

    /**
     * @return
     */
	public synchronized IntervalListElement getPrevious() {
        return previous;
    }

    /**
     * @param l
     */
	public synchronized void setBeginning(long l) {
        beginning = l;
		fireChange(this);
    }

    /**
     * @param l
     */
	public synchronized void setDuration(long l) {
        duration = l;
		fireChange(this);
    }

    /**
     * @param object
     */
	public synchronized void setElement(Object object) {
        element = object;
		fireChange(this);
    }

    /**
     * @param l
     */
	public synchronized void setEnd(long l) {
        end = l;
		fireChange(this);
    }

    /**
     * @param element
     */
	public synchronized void setNext(IntervalListElement element) {
        next = element;
        fireChange(this);
    }

    /**
     * @param element
     */
	public synchronized void setPrevious(IntervalListElement element) {
        previous = element;
		fireChange(this);
    }
    
	public void addChangeListener(ChangeListener l) {
		 listenerList.add(ChangeListener.class, l);
	 }

	 public void removeChangeListener(ChangeListener l) {
		 listenerList.remove(ChangeListener.class,  l);
	 }

	 /**
	  * This method supports normal ChangeListeners
	  */
	 protected void fireChange(Object changedObject) {

		 
		 ChangeEvent simpleChangeEvent = new ChangeEvent(changedObject);

		 
		 Object[] listeners = listenerList.getListenerList();
		 ChangeListener listener = null;

		 for (int i = listeners.length - 2; i >= 0; i -= 2) {
			 if (listeners[i] == ChangeListener.class) {

				 ((ChangeListener) listeners[i + 1]).stateChanged( simpleChangeEvent );

			 }

		 }
	 }


}
