
package com.ai_center.dragonbreath.utilities;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** 
 * A pop-up window to let the user select among a list of alternatives. 
 *
 * @author Alexander Nareyek
 * @version 0.1
 */

public class ListDialog {

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    private String[] options;
    private String title;
    private JPanel mainPanel;
    private JList nameList;
    private JScrollPane scrollPane;

    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /** 
     * Creates the structures without displaying the window.
     * @param options  An array of list elements that can be selected
     * @param title  The window title string
     * @param message  The message to be displayed
     */
    public ListDialog(String[] options, String title, String message)  {

	this.options = options;
	this.title = title;

	mainPanel = new JPanel();
	mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

	JLabel m = new JLabel(message);
	m.setAlignmentX(Component.LEFT_ALIGNMENT);
	m.setForeground(Color.black);
	mainPanel.add(m);
	mainPanel.add(Box.createRigidArea(new Dimension(0,8)));

	nameList = new JList(options);
	nameList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
	nameList.setSelectedIndex(0);
	JScrollPane scrollPane = new JScrollPane(nameList);
	scrollPane.setPreferredSize(new Dimension(250, 80));
	scrollPane.setMinimumSize(new Dimension(250, 80));
	scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
	mainPanel.add(scrollPane);

    }

    // *******************
    // ***** METHODS *****
    // *******************

    /** 
     * Displays the window and lets the user select among the alternatives.
     * Returns -1 if the user has canceled the choice.
     */
    public int select() {
	if (JOptionPane.showConfirmDialog(null, mainPanel, title, JOptionPane.OK_CANCEL_OPTION) != JOptionPane.OK_OPTION) {
	    return -1;
	} else {
	    return nameList.getSelectedIndex();
	}
    }

}
