/*
 * MappaM.java
 *
 * Created on 21. Mai 2002, 19:55
 */

package com.ai_center.dragonbreath.utilities;

import java.util.*;
   /*
    * A map that maps values to keys and vice versa. Ergo a Map and a paM
    * @author  tim baermann
    * @version $Revision: 2.0 $
    */
public class MappaM {
    
    SetHashtable map = new SetHashtable();
    SetHashtable pam = new SetHashtable();
    
    /**
     * Adds the value, mapped to the key.
     *@param key  The key.
     */
    public void add( Object key,  Object value ){
        boolean result;
        map.add( key, value );
        pam.add( value, key );
    }
    
    /**
     * Adds all values to the key.
     * @param key  The key.
     * @param values  The values.
     */
    public void addAll( Object key, Collection values ){
        map.addAll( key, values );
        Iterator it = values.iterator();
        while (it.hasNext() ){
            pam.add( it.next(), key );
        }
    }
    
    /**
     * Adds all the keys to the value.
     * @param keys  The keys that should get the value added.
     * @param value  The value.
     */
    public void addAllKeys( Collection keys, Object value ){
        pam.addAll( value, keys );
        Iterator it = keys.iterator();
        while (it.hasNext() ){
            map.add( it.next(), value );
        }
    }// addAllKeys()
    
    /**
     * The same as calling first get( key ) and then remove ( key )
     *@param key  The key.
     */
    public Set extract( Object key ){
        Set s;
        Object o= map.get( key );
        if ( o instanceof Set ){
            s = (Set) o;
            map.removeKey( key );
            Iterator it = s.iterator();
            while ( it.hasNext() ){
                pam.removeKey( it.next() );
            }
        } else {
            s = new HashSet();
            if ( o != null ){
                s.add( o );
                map.remove( key, o );
                pam.removeKey( o );
            }
        }
        return s;
    }//extract()
    
     /**
     * Returns all the values assoziated with key.
     *@param key  a key
     * @return  All the objects associated with element in a set.
     */
    public Set get( Object key ){
        Set result = null ;
        Object o= map.get( key );
        if ( o instanceof Set ){
            result = (Set) o;
        } else {
            result = new HashSet();
            if ( o != null ){
                result.add(o);
            }
        }
        return result;
    }
    
    /**
     * Removes the given Value and all the assoziated keys, and all the other values assoziated with the keys.
     * @param value  The value that should be removed.
     */
    public void removeValue( Object value ){
        Set s;
        Object o = pam.get( value );
        if ( o instanceof Set ){
            s = (Set) o;
            pam.removeKey( value );
            Iterator it = s.iterator();
            while ( it.hasNext() ){
                map.removeKey( it.next() );
            }
        } else {
            if ( o != null ){
                pam.remove( value, o );
                map.removeKey( o );
            }
        }
    }
    
    /**
     * Removes the given key and all the assoziated values, and all the other keys assoziated with the values.
     * @param key  The key.
     */
    public void remove( Object key ){
        Set s;
        Object o= map.get( key );
        if ( o instanceof Set ){
            s = (Set) o;
            map.removeKey( key );
            Iterator it = s.iterator();
            while ( it.hasNext() ){
                pam.removeKey( it.next() );
            }
        } else {
            if ( o != null ){
                map.remove( key, o );
                pam.removeKey( o );
            }
        }
    }//remove()
    
    
    /**
     * Removes exactly the given pair from map.
     * @param key  The key.
     * @param value  The value.
     */
    public void remove (Object key, Object value ){
        map.remove( key, value );
        pam.remove( value, key );
    }
    
    /**
     * Removes all the entries.
     */
   public void clear(){
       map.clear();
       pam.clear();
   }
    
   /**
    * Returns an Iterator over the keys.
    * @return  The iterator.
    */
   public Iterator keyIterator(){
       return map.iterator();
   }
   
   /**
    * Returns an Iterator over the values ( if a value is stored with multiple keys it is returned by the Iterator only once).
    * @return  The Iterator.
    */
   public Iterator valueIterator(){
       return pam.iterator();
   }
    
}//class

/*
 * DO _NOT_ EDIT ANYTHING BELOW THIS POINT!
 *
 * $Log: MappaM.java,v $
 * Revision 2.0  2002/09/17 00:33:07  nareyek
 * work on beta version began: resource type, state tasks and precondition tasks introduced
 *
 * Revision 1.1  2002/07/24 17:01:05  teutates
 * an m to n map,
 * that works in both ways
 *
 */

