
package com.ai_center.dragonbreath.utilities;

import java.util.*;

/** 
 * Provides a Hashtable with either elements or HastSets of elements.
 * DO NOT insert HastSets as elements!
 *
 * @author Alexander Nareyek
 * @version $Revision: 2.2 $
 * 
 * @see dragonbreath
 */

public class SetHashtable {

    /**
     * Iterator to iterate over all values of SetHashtable.
     */
    private class SetHashtableIterator implements java.util.Iterator {

	// Attributes

        /**
	 * The SetHashtable that is iterated.
	 */
        SetHashtable sht;
        /**
	 * Iterator over the keys.
	 */
        java.util.Iterator keyIt;
        /** 
	 * Iterator over the values.
	 */
        java.util.Iterator valIt;
        
	// Constructors

        /** 
	 * Creates an iterator to iterate over all values of the SetHashtable.
         * @param s  The SetHashtable that should be iterated.
         */
        SetHashtableIterator(SetHashtable s) {
            sht = s;
            keyIt = s.iterator();
        }
        
	// Methods

        /** 
	 * Checks whether further elements are existing.
         * @return Exists next element?
         */
        public boolean hasNext() {
            if (keyIt.hasNext() || ((valIt != null) && valIt.hasNext())) {
                return true;
	    } else {
		return false;
	    }
        }
        
        /** 
	 * Returns the next value.
         * @return The next value object.
         */        
        public Object next() {
            
            if ((valIt != null) && valIt.hasNext()) {
                return valIt.next();
            } else {
                Object o = sht.get(keyIt.next());
                if (o instanceof HashSet) {
                    valIt = ((HashSet) o).iterator();
                    return valIt.next();
                } else {
                    valIt = null;
                    return o;
                }
            }

        }
        
        /** 
	 * Removes the element that was last returned by next().
         */        
        public void remove() {
            if (valIt != null) {
                valIt.remove();
            } else {
                keyIt.remove();
            }
        }
        
    }

    // **********************
    // ***** ATTRIBUTES *****
    // **********************

    /** 
     * Number of elements contained in this structure.
     */
    private int number = 0;

    /** 
     * Hashtable from objects to hashset/elements.
     */
    private Hashtable hashtable = new Hashtable();


    // ************************
    // ***** CONSTRUCTORS *****
    // ************************

    /**
     * Creates a new SetHashtable.
     */
    public SetHashtable() {}

    /**
     * Creates a SetHashtable that has the same content as a given one.
     * @param sh  The original SetHashtable.
     */
    public SetHashtable(SetHashtable sh) {
        this.addAll(sh);
    }
    
   
    // *******************
    // ***** METHODS *****
    // *******************

    /**
     * Prints the Hashtable's content.
     * @return  The textual content.
     */
    public String toString() {
	return hashtable.toString();
    }

    /** 
     * Adds an element.
     * @param key  The key object.
     * @param element  The element to be added for the key object.
     */
    public void add(Object key, Object element) {
	Object retrievedObject;
	synchronized (hashtable) {
	    if ((retrievedObject = hashtable.get(key)) == null) {

		// First element WRT the key object

		hashtable.put(key, element);
                number++;

	    } else {
		if (retrievedObject instanceof HashSet) {

		    // Already a list WRT the key object

                    int oldSize = ((HashSet) retrievedObject).size();
    		    ((HashSet) retrievedObject).add(element);
                    number += ((HashSet) retrievedObject).size() - oldSize;

		} else {

		    // Already one element WRT the key object

                    if (retrievedObject == element) {
			// No addition of an object that is already in set.
			return; 
		    }
		    HashSet retrievedSet = new HashSet();
		    retrievedSet.add(retrievedObject);
		    retrievedSet.add(element); 
                    if (element != retrievedObject) {
                        number++;
                    }
		    hashtable.put(key, retrievedSet);

		}
	    }
	}
	
    }
    /**
     * Adds all the elements in this setHashtable.
     * @param key  The key to be used.
     * @param all  The elements to be added.
     */
    public void addAll(Object key, Collection all) {

        Object retrievedObject;
        HashSet retrievedSet;
        if (all.size() == 0) {
            return;
        }

        synchronized (hashtable) {

             if ((retrievedObject = hashtable.get(key)) == null) {

		 // First element WRT the key object

                 if (all.size() == 1) { 
                     Iterator it = all.iterator();
                     hashtable.put(key, it.next());
                 } else {
                    hashtable.put(key, new HashSet(all));
                 }
		 number += all.size();
                 
            } else {
                if (retrievedObject instanceof HashSet) {

                    // Already a list WRT the key object

                    int sizebefore = ((HashSet) retrievedObject).size();
                    ((HashSet) retrievedObject).addAll(all);
                    number += ((HashSet) retrievedObject).size() - sizebefore;

                } else {

                    // Already one element WRT the key object

                    HashSet value = new HashSet(all);
                    value.add(retrievedObject);
                    if (value.size() > 1) {
                        hashtable.remove(key);
                        hashtable.put(key, value);
                        number += value.size() - 1;    
                    } 
		    // Else do nothing, no new element was added. 
		    // Leave the old one where it is.
                    
                }
            }           
        }
    }
    
    /**
     * Adds all the elements in the given SetHashtable into this.
     * Creates new HashSets, and does not use the ones of the source.
     * @param all  The elements to be added.
     */
    public void addAll(SetHashtable all) {
        Object retrievedObject;
        HashSet retrievedSet;
        synchronized (all.hashtable) { synchronized (hashtable) {
            java.util.Set in = all.hashtable.entrySet();
            java.util.Iterator it = in.iterator();
            while (it.hasNext()) {
                java.util.Map.Entry el = (java.util.Map.Entry) it.next();
                Object value = el.getValue();
                Object key = el.getKey();
                if (value instanceof HashSet) {
                     if ((retrievedObject = hashtable.get(key)) == null) {
                        // First element WRT the key object
                        hashtable.put(key, new HashSet((Collection) value));
                        number += ((HashSet) value).size();
                    } else {
                        if (retrievedObject instanceof HashSet) {
                            // Already a list WRT the key object
                            int sizebefore = 
				((HashSet) retrievedObject).size();
                            ((HashSet) retrievedObject).addAll((HashSet) value);
                            number += ((HashSet) retrievedObject).size() 
				- sizebefore;
                        } else {
                            // Already one element WRT the key object
                            ((HashSet) value).add(retrievedObject);
                            hashtable.put(key, value);
                            number += ((HashSet) value).size() - 1;
                        }
                    }   
                } else {
                    add(key, value);
                }
            }
        }}
    }
   
    /**
     * Adds all the elements in this Map.
     * @param all The elements to be added.
     */
    public void addAll(Map all) {
        Object retrievedObject;
        HashSet retrievedSet;
        synchronized (hashtable) {
            java.util.Set in = all.entrySet();
            java.util.Iterator it = in.iterator();
            while (it.hasNext()) {
                java.util.Map.Entry el = (java.util.Map.Entry) it.next();
                Object value = el.getValue();
                Object key = el.getKey();
		add(key, value);
            }
        }
    }
   
    /**
     * Checks if an object is stored in this SetHashtable.
     * @param  o  The object searched for.
     * @return true if and only if the specified object is in this hashtable, 
     *         as determined by the equals method; false otherwise.
     */
    public boolean contains(Object o) {
        java.util.Iterator it = this.valuesIterator();
        while (it.hasNext()) {
            Object obj = it.next();
            if (o.equals(obj)) {
		return true;
	    }
        }
        return false;
    }

    /**
     *Tests if the specified object is a key in this SetHashtable.
     * @param o  The key element.
     * @return true if and only if the specified object is a key in this 
     *         hashtable, as determined by the equals method; false otherwise.
     */
    public boolean containsKey(Object o) {
        return hashtable.containsKey(o);
    }
    
    /**
     * Is the given pair in this SetHashtable? Faster then contains(value).
     * @param key  The key element.
     * @param value  The value element.
     * @return true if and only if the specified object is a key in this 
     *         hashtable, as determined by the equals method; false otherwise.
     */
    public boolean contains(Object key, Object value) {
        Object o = get(key);
        if (o == null) {
            return false;
        } else {
	    if (o instanceof Set) {
		return ((Set) o).contains(value);
	    } else {
		return o.equals(value);
	    } 
        }
    }
    
    
    /**
     * Returns the object/HashSet for a special key object.
     * @param key  The key object.
     * @return An object, or a HashSet of objects that were stored for key, 
     *         or null if none stored.
     */
    public Object get(Object key) {
	Object retrievedObject;
	synchronized (hashtable) {
	    retrievedObject = hashtable.get(key);
	}
	return retrievedObject;
    } 
    
    /**
     * Returns a Set with the Values of the SetHashtable.
     */
    public Set values() {
        HashSet result = new HashSet();
        Iterator it = hashtable.values().iterator();
        while (it.hasNext()) {
            Object o = it.next();
            if (o instanceof Set) {
                result.addAll((Set) o);
            } else {
                result.add(o);
            }
        }
        return result;
    }
    
        
    /** 
     * Removes an element.
     * @param key  The key object.
     * @param element  The element to be removed from the list retrieved by
     *                 of the key object. 
     * @return Was the Object member before?
     */
    public boolean remove(Object key, Object element) {
	Object retrievedObject;
	synchronized (hashtable) {
	    if ((retrievedObject = hashtable.get(key)) == null) {
		return false;
	    } else {
		if (retrievedObject instanceof HashSet) {
		    // A list of elements WRT the key object
		    HashSet retrievedSet = (HashSet) retrievedObject;
                    
                    // Two elements in the list WRT the key object
                    int sizebefore = retrievedSet.size();
                    boolean result = retrievedSet.remove(element);
                    number -= sizebefore - retrievedSet.size();
                    if (retrievedSet.size() == 0) {
                        // Last element in the list WRT the key object
                        hashtable.remove(key);
                    }
                    return result;
                 } else {
		    // Only one element WRT the key object
                    if (retrievedObject == element) {
                        hashtable.remove(key);
                        number--;
                        return true;
                    } else {
                        return false;
                    }
		}
	    }
	}
    }
    
    /**
     * Removes all elements stored for the given key.
     * @param key The key object.
     */
    public void removeKey(Object key) {
        Object retrievedObject;
        synchronized (hashtable) {
	    if ((retrievedObject = hashtable.get(key)) == null) {
		return;
	    } else {
		if (retrievedObject instanceof HashSet) {
		    HashSet retrievedSet = (HashSet) retrievedObject;
                    number -= retrievedSet.size();
		} else {    
                    number--;
		}                
                hashtable.remove(key);
            }
        }
    }
    
    /**
     * Removes all entries from this SetHashtable
     * @param all the entries from the SethHashtable are removed from this one
     */
    public void removeAll(SetHashtable all){
	Object retrievedObject;
        synchronized(all.hashtable) { 
	    synchronized(hashtable) {
		java.util.Set in = all.hashtable.entrySet();
		java.util.Iterator it = in.iterator();
		while (it.hasNext()) {
		    java.util.Map.Entry el = (java.util.Map.Entry) it.next();
		    Object value = el.getValue();
		    Object key = el.getKey();
		    if (value instanceof HashSet) {
			if ((retrievedObject = hashtable.get(key)) == null) {
			    // No element found WRT the key object
			    throw new Error("Elements have not been stored for key " + key);
			} else {
			    if (retrievedObject instanceof HashSet) {
				// A list of elements WRT the key object
				HashSet retrievedSet = 
				    (HashSet) retrievedObject;
				int sizebefore = 
				    ((HashSet) retrievedObject).size();
				retrievedSet.removeAll((HashSet) value);
				if (retrievedSet.isEmpty()) { 
                                    // last element for key was removed
				    hashtable.remove(key);
				} else {
				    if (retrievedSet.size() == 1) {
				        // Last element in the list WRT the key object
					java.util.Iterator i = retrievedSet.iterator();
					hashtable.remove(key);
					hashtable.put(key,i.next());
				    }
				}
				number -= (sizebefore - retrievedSet.size());
			    } else {
				if (((HashSet)value).contains(retrievedObject)) {
				    // Only one element WRT the key object
				    hashtable.remove(key);
				    number--;
				}
			    }
			}
		    } else {
			remove(key, value);
		    }
		}//while it.hasNext()
           
	    }
	}//synchronized	
   }                               

    /**
     * Clears the whole SetHashtable
     */
    public void clear() {
        hashtable.clear();
        number = 0;
    }
    
    /**
     * Returns the total number of elements.
     * @return Number of elements.
     */
    public int size() {
	return number;
    }

    /**
     * Returns the number of elements for a special key object.
     * @param key The key object.
     * @return The number.
     */
    public int size(Object key) {
	Object retrievedObject;
	synchronized (hashtable) {
	    if ((retrievedObject = hashtable.get(key)) == null) {
		return 0;
	    } else {
		if (retrievedObject instanceof HashSet) {
    		    return ((HashSet) retrievedObject).size();
		} else {
		    return 1;
		}
	    }
	}
    }
   
    /**
     * Returns the number of keys used.
     * @return Number of Keys used.
     */
    public int numberOfKeys(){
        return hashtable.size();
    }
   
    /**
     * Returns a set view on the keys.
     */
    public Set keySet(){
        return hashtable.keySet();
    }
    
    /**
     * Returns an iterator to enumerate all keys of the SetHashtable.
     * @return The iterator over the keys.
     */
    public Iterator iterator() {
	return hashtable.keySet().iterator();
    }

    
    /**
     * Returns an iterator to enumerate all values of the SetHashtable.
     * @return The iterator over the values.
     */
    public Iterator valuesIterator() {
        return new SetHashtableIterator(this);
    }

}
/*
 * DO _NOT_ EDIT ANYTHING BELOW THIS POINT!
 *
 * $Log: SetHashtable.java,v $
 * Revision 2.2  2003/02/21 01:33:05  teutates
 * structural repair initial release
 *
 * Revision 2.1  2002/09/24 05:03:28  nareyek
 * task constraint now fully functional with action types
 *
 * Revision 2.0  2002/09/17 00:33:07  nareyek
 * work on beta version began: resource type, state tasks and precondition tasks introduced
 *
 * Revision 1.11  2002/07/25 01:23:10  teutates
 * at last..
 * a new version of the dockingGraph matching algorithm.
 *
 * Revision 1.10  2002/05/21 01:54:27  nareyek
 * code style corrected
 *
 * Revision 1.9  2002/04/08 12:02:06  teutates
 * member order changed.
 *
 * Revision 1.8  2002/03/30 08:33:09  nareyek
 * basic cost-function functionality added
 *
 * Revision 1.7  2002/03/27 13:09:00  teutates
 * numberOfKeys()
 * valuesIterator
 * and new constructor added
 *
 * Revision 1.6  2002/01/30 00:52:46  teutates
 * add same element twice doesnt create a list any more
 *
 * Revision 1.5  2002/01/29 00:10:09  teutates
 * number is set to 0 in clearAll. @param comments added
 *
 * Revision 1.4  2002/01/25 12:34:29  teutates
 * added addAll, removeAll, clear
 * fixed size bug
 *
 * Revision 1.3  2001/11/19 17:42:24  nareyek
 * Connections class rewritten; added cvs log commands
 *
 *
 */
