package com.ai_center.planpresentation;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

import javax.swing.BorderFactory;
import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.plugins.planplugin.AxisJPanel;

/**
 * The main JFrame of the planpresentation application.
 * <p>
 *
 * This JFrame opens the mainwindow and contains most of the non-plugin gu
 * elements.
 * <p>
 *
 * @author Daniel Weiss
 * @version $Revision: 1.8 $
 */
public class PlanJFrame extends JFrame implements PlanPresentationConstants {

        // non gui attributes

        /**
         * Contains all control elements (the column on the left side) ordered by
         * prority
         */
        private Map controlMap = new TreeMap();

        /** The model which is representet by the progress bars */
        public BoundedRangeModel currentTimeModel = new DefaultBoundedRangeModel();

        /**
         * The main view model represents the scrolling.
         * <p>
         *
         * i.e
         *
         * <pre>
         * value
         * </pre>
         *
         * is the tick on the left side of the scroll area and
         *
         * <pre>
         * value + extent
         * </pre>
         *
         * is the tick on the right side of the scroll area.
         *
         * <pre>
         * minimum
         * </pre>
         *
         * ist always zero, while
         *
         * <pre>
         * maximum
         * </pre>
         *
         * is the highest known ick value. (e.g. if the longest plan was until tick
         * 2000, the maximum is 2000 too.)
         */
        public BoundedRangeModel mainViewModel = new DefaultBoundedRangeModel();

        /** Maps from menu name to JMenu. */
        private HashMap menuItemMap = new HashMap();

        /** Contains all menu names ordered by priority */
        private Map menuMap = new TreeMap();

        /** Maps the menuname to the desired mnemonic (key-shortcut) */
        private Map mnemonicMap = new HashMap();

        /** The link to the PlanPresentation */
        private PlanPresentation planPresentation = null;

        /** Set to true if a full update is requested. */
        public boolean updateAll;

        /** Set to true if the plan paint area should update soon. */
        public boolean updateView;

        /** Contains all views. */
        public List viewList = new LinkedList();

        // functional gui elements
        /** The link to the swimlane control. */
        private SwimlaneControl swimlaneControl;

        /** The progressbar on the top of the plan area. */
        JProgressBar currentTimeJProgressBarNorth = new JProgressBar();

        /** The progressbar on the bottom of the plan area */
        JProgressBar currentTimeJProgressBarSouth = new JProgressBar();

        /** The helpMessage Label on top of the plan area. */
        JLabel helpMessage = new JLabel("", JLabel.CENTER);

        /** The sole Menu Bar. */
        JMenuBar jMainMenuBar = new JMenuBar();

        /** The sole Menu Tool Bar. */
        JToolBar menuJToolBar = new JToolBar();

        /** The link to the GlassPaneArea. */
        public GlassPaneArea planGlassPane = null;

        /** The scrollbar on the top of the plan area. */
        JScrollBar planJScrollBarNorth = new JScrollBar();

        /** The scrollbar on the bottom of the plan area. */
        JScrollBar planJScrollBarSouth = new JScrollBar();

        /** The vertical scrollbar on the right side. */
        JScrollBar planjScrollBarVertical = new JScrollBar();

        /** The plan paint area, where all views are shown. */
        public JPanel planPaintArea = null;

        /** The zoom JSlider on the left side. */
        JSlider zoomJSlider = new JSlider();

        // gui elements without any function (only for layout) and therefore
        // not documented.

        JScrollPane controlJScrollPane = new JScrollPane();

        JPanel controlMainJPanel = new JPanel();

        JPanel controlSubJPanelCenter = new JPanel();

        JPanel controlSubjPanelNorth = new JPanel();

        JPanel controlSubjPanelSouth = new JPanel();

        FlowLayout flowLayout1 = new FlowLayout();

        JPanel planRootJPanel = new JPanel();

        JPanel planScrollRootJPanel = new JPanel();

        JPanel planSubjPanelEast = new JPanel();

        JPanel planSubJPanelNorth = new JPanel();

        JPanel planSubJPanelSouth = new JPanel();

        JPanel planSubJPanelWest = new JPanel();

        JScrollPane planSubJScrollPane = null;

        JPanel spacerjPanelNorth = new JPanel();

        JPanel spacerjPanelSouth = new JPanel();

        private TitledBorder zoomBorder;

        /**
         * The constructor prepares the gui window.
         * <p>
         *
         * But it does not show the gui.
         * <p>
         *
         * @param _planPresentation
         *            The reference to the PlanPresentation.
         */
        public PlanJFrame(PlanPresentation _planPresentation) {
                planPresentation = _planPresentation;
                try {
                        // load all gui elements
                        initGUI();

                        // layout them
                        this.pack();

                        // start with maximizied window
                        this.setExtendedState(getExtendedState() | MAXIMIZED_BOTH);
                } catch (Exception e) {
                        e.printStackTrace();
                }
        }

        /**
         * Builds the whole gui window.
         * <p>
         *
         * If the layout should be changed, the first adress is this function.
         * <p>
         *
         * @throws Exception
         *             Possible execption will be caught in the constructor.
         */
        private void initGUI() throws Exception {

                // initate the swimlanecontrol
                swimlaneControl = new DefaultSwimlaneControl(planPresentation);

                // create the planPaintArea with DoubleBuffering
                planPaintArea = new JPanel(true);

                // Put the planPaintArea inside a ScrollPane
                planSubJScrollPane = new JScrollPane(planPaintArea);

                // initate the GlassPane
                planGlassPane = new GlassPaneArea(swimlaneControl);

                // complete swimlaneControl initiation
                swimlaneControl.setGlassPane(planGlassPane);
                swimlaneControl.setPaintArea(planPaintArea);

                // create a resize listener, which adjust the views, when
                // the window is resized.
                this.addComponentListener(new ComponentAdapter() {

                        public void componentResized(ComponentEvent e) {
                                planPaintArea.setPreferredSize(new Dimension(planSubJScrollPane
                                                .getWidth(), 800));
                                planPaintArea.setMinimumSize(new Dimension(planSubJScrollPane
                                                .getWidth(), 800));
                                updateMainViewModel();

                        }

                });

                // make a link between the maximum of the mainViewModel and the
                // currentTimeModel
                mainViewModel.addChangeListener(new ChangeListener() {

                        /*
                         * (non-Javadoc)
                         *
                         * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
                         */
                        public void stateChanged(ChangeEvent e) {
                                // a full repaint is required. (the axis need a repaint too)
                                updateAll = true;

                                if (mainViewModel.getMaximum() != currentTimeModel.getMaximum()) {
                                        currentTimeModel.setMaximum(mainViewModel.getMaximum());
                                }
                        };
                });

                // init mainViewModel
                mainViewModel.setRangeProperties(0,
                // Time on the left border of the screen
                                0,
                                // (will be overwritten when initalizing updateMainViewModel()
                                // the first time.
                                0, // min-ct to show
                                100, // max-ct to show
                                false // adjusted
                                );

                // link model to the scrollbars. (to make the extent visible)
                planJScrollBarNorth.setModel(mainViewModel);
                planJScrollBarSouth.setModel(mainViewModel);

                // init currentTimeModel
                currentTimeModel.setRangeProperties(0, // current Time
                                1, // extend useless because progressbar
                                0, // min-ct to show
                                1000, // max-ct to show
                                false // adjusted
                                );

                // link model to the progressbars
                currentTimeJProgressBarNorth.setModel(currentTimeModel);
                currentTimeJProgressBarSouth.setModel(currentTimeModel);
                currentTimeModel.setValue(0);

                //close the application, if someone clicks on the close button.
                this.setDefaultCloseOperation(EXIT_ON_CLOSE);

                // some minor adjustments
                this.setLocale(java.util.Locale.getDefault());
                this.setJMenuBar(jMainMenuBar);
                this.setState(Frame.NORMAL);
                this.setTitle("Plan Presentation");
                this.getContentPane().setLayout(new BorderLayout());

                // put the glass pane over the screen
                this.setGlassPane(planGlassPane);
                planGlassPane.setVisible(true);

                // make a zoom slider
                zoomJSlider = new JSlider( 0 , 200 , 100 );
                zoomJSlider.setBounds( new Rectangle( 7 , 17 , 200 , 74 ) );
                zoomBorder = BorderFactory.createTitledBorder( "Zoom" );
                zoomJSlider.setBorder( zoomBorder );
                zoomJSlider.setMajorTickSpacing( 50 );
                zoomJSlider.setMinorTickSpacing( 10 );
                zoomJSlider.setPaintTicks( true );
                zoomJSlider.setInverted( false );

                zoomJSlider.addChangeListener(
                  new ChangeListener()
                  {
                        public void stateChanged( ChangeEvent event )
                        {
                                JSlider source = (JSlider)event.getSource();

                                // don't allow the zoom to be zero.
                                int zoom = source.getValue();
                                if ( zoom < 1 )
                                {
                                    source.setValue( 1 );
                                    zoom = 1;
                                }

                                if ( !source.getValueIsAdjusting() )
                                {

                                    planPresentation.setZoomfactor( new Float( zoom ) );
                                    // repaint after jslider stopps
                                    updateMainViewModel();
                                }
                        }

                  } );

                // layout stuff not documented
                controlMainJPanel.setMinimumSize(new Dimension(200, 200));
                controlMainJPanel.setPreferredSize(new Dimension(200, 200));
                controlMainJPanel.setLayout(new BorderLayout());

                controlSubjPanelSouth.setLayout(null);
                controlSubJPanelCenter.setLayout(new FlowLayout());
                controlSubJPanelCenter.setMinimumSize(new Dimension(200, 100));
                controlSubJPanelCenter.setPreferredSize(new Dimension(200, 100));
                controlSubjPanelSouth.setMaximumSize(new Dimension(32767, 32767));
                controlSubjPanelSouth.setMinimumSize(new Dimension(200, 100));
                controlSubjPanelSouth.setPreferredSize(new Dimension(200, 100));

                menuJToolBar.setToolTipText("Toolbar");
                menuJToolBar.setFloatable(false);
                planSubjPanelEast.setLayout(new BorderLayout());
                spacerjPanelNorth.setMinimumSize(new Dimension(10, 15));
                spacerjPanelNorth.setPreferredSize(new Dimension(10, 15));
                spacerjPanelNorth.setLayout(null);

                planPaintArea.setBackground(Color.white);
                planPaintArea.setMinimumSize(new Dimension(100, 800));
                planPaintArea.setPreferredSize(new Dimension(100, 800));

                planPaintArea.setLayout(flowLayout1);
                planRootJPanel.setLayout(new BorderLayout());

                flowLayout1.setAlignment(FlowLayout.LEFT);
                flowLayout1.setHgap(0);
                flowLayout1.setVgap(0);
                planScrollRootJPanel.setLayout(new BorderLayout());
                planSubJPanelWest.setLayout(null);
                planSubJPanelWest.setMinimumSize(new Dimension(10, 600));
                planSubJPanelWest.setPreferredSize(new Dimension(10, 600));

                spacerjPanelSouth.setMinimumSize(new Dimension(10, 15));
                spacerjPanelSouth.setPreferredSize(new Dimension(10, 15));
                spacerjPanelSouth.setLayout(null);
                planSubJPanelNorth.setLayout(new GridLayout(4, 1, 0, 0));

                planJScrollBarNorth.setOrientation(JScrollBar.HORIZONTAL);
                planJScrollBarSouth.setOrientation(JScrollBar.HORIZONTAL);

                planSubJPanelSouth.setLayout(new GridLayout(3, 1, 0, 0));

                JPanel spacerLeftSouth = new JPanel();
                spacerLeftSouth.setPreferredSize(new Dimension(15, 15));
                JPanel spacerRightSouth = new JPanel();
                spacerRightSouth.setPreferredSize(new Dimension(15, 15));

                this.getContentPane().add(controlJScrollPane, BorderLayout.WEST);
                controlJScrollPane.getViewport().add(controlMainJPanel, null);
                controlMainJPanel.add(controlSubjPanelSouth, BorderLayout.SOUTH);

                controlMainJPanel.add(controlSubJPanelCenter, BorderLayout.CENTER);
                controlSubjPanelSouth.add(zoomJSlider, null);

                this.getContentPane().add(menuJToolBar, BorderLayout.NORTH);
                planRootJPanel.add(planSubjPanelEast, BorderLayout.EAST);
                planRootJPanel.add(planSubJPanelWest, BorderLayout.WEST);
                planSubjPanelEast.add(planjScrollBarVertical, BorderLayout.CENTER);
                planSubjPanelEast.add(spacerjPanelSouth, BorderLayout.SOUTH);
                planSubjPanelEast.add(spacerjPanelNorth, BorderLayout.NORTH);
                planRootJPanel.add(planScrollRootJPanel, BorderLayout.CENTER);

                this.getContentPane().add(planRootJPanel, BorderLayout.CENTER);

                planSubJPanelNorth.add(helpMessage);
                planSubJPanelNorth.add(planJScrollBarNorth, BorderLayout.NORTH);

                planSubJPanelNorth.add(makeProgressPanel(currentTimeJProgressBarNorth),
                                BorderLayout.CENTER);

                planSubJPanelNorth.add(new AxisJPanel(this), BorderLayout.SOUTH);

                planScrollRootJPanel.add(planSubJPanelSouth, BorderLayout.SOUTH);
                planSubJPanelSouth.add(new AxisJPanel(this), BorderLayout.NORTH);
                planSubJPanelSouth.add(makeProgressPanel(currentTimeJProgressBarSouth),
                                BorderLayout.CENTER);
                planSubJPanelSouth.add(planJScrollBarSouth, BorderLayout.SOUTH);

                planScrollRootJPanel.add(planSubJPanelNorth, BorderLayout.NORTH);

                // background color of the plan paint area should be the same as the
                // first view color.

                planPaintArea.setBackground(ALTERNATE_COLOR_1);
                planSubJScrollPane = new JScrollPane(planPaintArea);

                planSubJScrollPane
                                .setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
                planSubJScrollPane
                                .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

                planSubJScrollPane.setMinimumSize(new Dimension(1621, 821));
                planSubJScrollPane.setOpaque(true);
                planSubJScrollPane.setPreferredSize(new Dimension(1621, 821));
                planSubJScrollPane.setToolTipText("");
                planSubJScrollPane.setVerticalScrollBar(planjScrollBarVertical);
                planScrollRootJPanel.add(planSubJScrollPane, BorderLayout.CENTER);

                // create a Repaint Listener
                RepaintActionListener repaintPlanJPanel = new RepaintActionListener(
                                this);

                new Timer(200, repaintPlanJPanel).start();

        }

        /**
         * Put a progress bar in a jpanel with spacer on both sides.
         * <p>
         *
         * @param progressBar
         *            The progressBar which should in the panel.
         * @return The complete panel with the spacers.
         */
        private JPanel makeProgressPanel(JProgressBar progressBar) {
                JPanel progressPanel = new JPanel(new BorderLayout());

                JPanel spacerLeft = new JPanel();
                spacerLeft.setPreferredSize(new Dimension(15, 15));
                JPanel spacerRight = new JPanel();
                spacerRight.setPreferredSize(new Dimension(15, 15));

                progressPanel.setPreferredSize(new Dimension(1600, 15));
                progressPanel.add(spacerLeft, BorderLayout.WEST);
                progressPanel.add(progressBar, BorderLayout.CENTER);
                progressPanel.add(spacerRight, BorderLayout.EAST);

                // Layout ProgressBar
                progressBar.setToolTipText(null);
                progressBar.setOrientation(JProgressBar.HORIZONTAL);
                progressBar.setStringPainted(true);
                progressBar.setString("Current Time: 0");

                return progressPanel;

        }

        /**
         * Adds a control element to the gui.
         * <p>
         *
         * Adds a control element to the left column of the gui. Like zoomslider.
         * <p>
         *
         * @param controlComponent
         *            Can be any JComponent.
         * @param order
         *            the order in which the elements should appear. zero on top.
         */
        public void addControl(JComponent component, int order) {
                controlMap.put(new Integer(order), component);
        }

        /**
         * Adds a menu to the menubar.
         * <p>
         *
         * Creates a new menu if none with the same menu name exists. If you are not
         * sure, if the menu already exist, call addMenu again, this function is
         * idempotent.
         * <p>
         *
         * @param menuname
         *            The name of the menu.
         * @param order
         *            The order, zero on the left side.
         */
        public void addMenu(String menuname, int mnemonic, int order) {
                menuMap.put(new Integer(order), menuname);
                mnemonicMap.put(menuname, new Integer(mnemonic));
        }

        /**
         * Adds a menu item to a give menu.
         * <p>
         *
         * @param jMenuItem
         *            The menu item to be added.
         * @param menuname
         *            The menu name in which this item should be added.
         */
        public void addMenuItem(JMenuItem jMenuItem, String menuname) {
                JMenu subMenu = null;
                // check if this menu already exists
                if (menuItemMap.containsKey(menuname)) {

                        subMenu = (JMenu) menuItemMap.get(menuname);

                } else {
                        // make a new menu
                        subMenu = new JMenu(menuname);
                        menuItemMap.put(menuname, subMenu);

                }

                subMenu.add(jMenuItem);

        }

        /**
         * Adds a button in the toolbar below the menu.
         * <p>
         *
         * The button can also be any other JComponent.
         * <p>
         *
         * @param button
         *            The button to be added.
         * @param order
         *            The order from left to right. Zero is left.
         */
        public void addToolButton(JComponent toolButton, int priority) {

                menuJToolBar.add(toolButton, null);

        } // end addToolButton

        /**
         * Adds a ViewChangeListener.
         * <p>
         *
         * The ViewChangeLister will be informed about all changes corresponding, to
         * the view. Like scrolling or time stepping.
         * <p>
         *
         * @param changeListener
         *            A changeListener which should be informed.
         */
        public void addViewChangeListener(ChangeListener l) {
                mainViewModel.addChangeListener(l); // add view change listener

                // perform a datachange to send initial data to viewchangelistener
                mainViewModel.setValueIsAdjusting(true);
                mainViewModel.setValueIsAdjusting(false);
        }

        /** Generate all control elements from the controlMap. */
        public void generateControls() {
                Iterator it = controlMap.entrySet().iterator();
                while (it.hasNext()) {
                        Map.Entry entry = (Entry) it.next();
                        JComponent component = (JComponent) entry.getValue();
                        controlSubJPanelCenter.add(component);
                }

        }

        /** Generate all menues from the menuMap. */
        public void generateMenus() {
                Iterator it = menuMap.entrySet().iterator();
                while (it.hasNext()) {
                        Map.Entry entry = (Entry) it.next();
                        String menuname = (String) entry.getValue();
                        JMenu menu = (JMenu) menuItemMap.get(menuname);
                        Integer mnemonic = (Integer) mnemonicMap.get(menuname);

                        if (menu != null) {
                                jMainMenuBar.add(menu);
                        }
                        if (mnemonic != null) {
                                menu.setMnemonic(mnemonic.intValue());
                        }
                }

        }

        /**
         * Returns the SwimlaneControl
         *
         * @return The current SwimlaneControl
         */
        public SwimlaneControl getSwimlaneControl() {
                return swimlaneControl;
        }

        /**
         * Removes the ViewChangeListener from the listenerList.
         *
         * @param changeListener
         *            The changeListener to be removed.
         */
        public void removeViewChangeListener(ChangeListener l) {
                mainViewModel.removeChangeListener(l);
        }

        /**
         * Scrolls the view to the right or left in different speeds.
         * <p>
         *
         * Normally scrolling is traced by the RetractableView and the plugin
         * programmer does not need to take care about it.
         * <p>
         *
         * @param scrollstep
         *            A high positive value scrolls fast to the right, a negative
         *            value scrolls to the right.
         */
        public void scroll(int scrollstep, boolean extendViewIfNecessary) {
                // compute the scrolling from pixels to ticks.
                double pixelPerTimeUnit = (double) planPaintArea.getWidth()
                                / (double) mainViewModel.getExtent();
                int offset = (int) (scrollstep / pixelPerTimeUnit);

                // compute new value
                int value = mainViewModel.getValue() + offset;

                // check if maximum needs an adjustment.

                // commented because auto-zooming while mousemovement leads to
                // disorientation of user.
                // disadvante: without auto-zooming and scrolling, the movement of
                // tasks to the right is no longer possible, if the maximum planning
                // time
                // is reached.
                if (extendViewIfNecessary) {
                        if (value + mainViewModel.getExtent() + 1000 >= mainViewModel
                                        .getMaximum()) {
                                mainViewModel.setMaximum(value + mainViewModel.getExtent()
                                                + 1000);
                        }
                }
                mainViewModel.setValue(value);

        }

        /**
         * Performa a view update.
         *
         * First it computes the visible range of the model (extent), then it
         * request a full view update.
         * <p>
         */
        void updateMainViewModel() {
                // computing the extent (the visible range of the scroll area):
                // 1.) if zoomJSlider.getValue() is 200, we want to see all.
                // 2.) if zoomJSlider.getValue() is 1, we want to see a fixed range
                //         e.g. 10 pixel/tick
                //
                // 1.) maximum = m * zoomvalue (with zoomvalue = 200)
                // 2.) fixed_pixel_per_time = m * zoomvalue (with zoomvalue = 1)
                //
                //     extent = maximum * visibilepercent , correcting to start at 1 not
                //     zero (-1) and adding an minimum value (fixed_pixel_per_time)
                //  => extent = maximum * ((zoomvalue - 1) / 200) + fixed_pixel_per_time

                double extent = (mainViewModel.getMaximum() * ((zoomJSlider.getValue() - 1.0) / 200))
                                + PIXEL_PER_TIME_UNIT;

                // avoid errors because of boundary conditions
                if (extent < 10.00) {
                        extent = 10.00;
                }

                // check if maximum is big enough
                if (mainViewModel.getMaximum() < mainViewModel.getValue() + extent) {
                        mainViewModel.setValueIsAdjusting(true);
                        mainViewModel.setValue(mainViewModel.getMaximum()
                                        - new Double(extent).intValue());

                }
                mainViewModel.setExtent(new Double(extent).intValue());

                mainViewModel.setValueIsAdjusting(false);

                // request a full repaint
                updateAll = true;

        }

        public void moveView(RetractableView r, boolean up) {
                RetractableView aboveView = null;
                boolean testViewAlreadyFound = false;
                for (int i = 0; i < viewList.size(); i++) {
                        RetractableView testView = (RetractableView) viewList.get(i);
                        if (testViewAlreadyFound && up == false) {
                                //  move down
                                // the testview was found in the loop before
                                // switch the priority of the view before and the the r-view
                                switchPriorities(r, testView);
                                updateView = true;
                                return;
                        }
                        if (testView == r) {
                                testViewAlreadyFound = true;

                                // ok, we have found the view
                                if (up) {
                                        // move up
                                        if (aboveView == null) {
                                                // Cannot move the view higher, because it is on top
                                                return;
                                        }
                                        switchPriorities(r, aboveView);
                                        updateView = true;
                                        return;
                                }
                        }
                        aboveView = testView; // remind the last view, for the next loop
                }
        }

        private void switchPriorities(RetractableView r1, RetractableView r2) {
                long r1Priority = r1.getPriority().longValue(); // 1000 = 800 + 200 => 1005 => 800 + 205
                long r2Priority = r2.getPriority().longValue(); // 1005 = 900 + 105 => 1000 => 100 + 105
                long diffPriority = r1Priority - r2Priority;

                r1.setPriorityOffset(0);
                r2.setPriorityOffset(0);
                long r1AccessTime = r1.getPriority().longValue();
                long r2AccessTime = r2.getPriority().longValue();
                long r1OldOffset = (r1Priority - r1AccessTime);
                long r2OldOffset = (r2Priority- r2AccessTime);
                long r1Offset =  (r1OldOffset - diffPriority);
                long r2Offset =  (r2OldOffset + diffPriority);
                if (diffPriority >= 0) {
                        r1Offset--;
                } else {
                        r2Offset--;
                }
                r1.setPriorityOffset(r1Offset);
                r2.setPriorityOffset(r2Offset);

        }
}

/**
 * RepaintActionListener permanently updates the views.
 * <p>
 *
 * The run function is called every x microseconds and checks, if a update is
 * necessary. If yes, it puts a repaint request in the AWT-Event-Queue.
 * <p>
 *
 * @author Daniel Weiss
 * @version $Revision: 1.8 $
 */

class RepaintActionListener implements ActionListener,
                PlanPresentationConstants {

        /** The link to the JFrame */
        private PlanJFrame planJFrame = null;

        public RepaintActionListener(PlanJFrame _planJFrame) {
                planJFrame = _planJFrame;

        }

        /*
         * (non-Javadoc)
         *
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent arg0) {
                if (planJFrame != null) {
                        // put this new thread on the AWT-Event-Queue
                        SwingUtilities.invokeLater(new Runnable() {

                                public void run() {
                                        // check if a update is necessary
                                        if (planJFrame.updateView) {
                                                //mark updateView as false
                                                planJFrame.updateView = false;

                                                // sort the viewlist by accesstime
                                                Collections.sort(planJFrame.viewList);

                                                // remove all views from the container
                                                planJFrame.planPaintArea.removeAll();
                                                Iterator it = planJFrame.viewList.iterator();

                                                int panelHeight = 0;
                                                boolean alternate1 = true;
                                                while (it.hasNext()) {

                                                        RetractableView retractableView = (RetractableView) it
                                                                        .next();

                                                        // accumulate all view heights.
                                                        panelHeight += retractableView.getHeight();

                                                        // switch background color
                                                        if (alternate1) {
                                                                retractableView
                                                                                .setBackground(ALTERNATE_COLOR_1);
                                                        } else {
                                                                retractableView
                                                                                .setBackground(ALTERNATE_COLOR_2);
                                                        }
                                                        alternate1 = !alternate1;

                                                        // add the view to the container again
                                                        planJFrame.planPaintArea.add(
                                                                        (JPanel) retractableView, null);
                                                }

                                                // compute new panel height
                                                Dimension planDim = new Dimension(
                                                                planJFrame.planSubJScrollPane.getWidth(),
                                                                panelHeight);
                                                // set plan paint area height new
                                                planJFrame.planPaintArea.setMinimumSize(planDim);
                                                planJFrame.planPaintArea.setMaximumSize(planDim);
                                                planJFrame.planPaintArea.setPreferredSize(planDim);

                                                // call a revalidate to adjust all views
                                                planJFrame.planScrollRootJPanel.revalidate();

                                        }

                                        if (planJFrame.updateAll) {
                                                //check if a full update is necessary
                                                // compute extend new.
                                                planJFrame.updateMainViewModel();
                                                planJFrame.updateAll = false;

                                                // request a repaint for the planning area and the axis
                                                planJFrame.planScrollRootJPanel.repaint();
                                        } else { // only a minor repaint is necessary

                                                // request only a repaint for the planning area
                                                planJFrame.planPaintArea.repaint();
                                        }

                                }
                        });

                }
        }


}
