package com.ai_center.planpresentation;


import java.awt.Color;




/** The constants' file for the planpresentation gui.<p>

 Here are global constants defined and some layout parameters can be changed.
 <p>

 @author Daniel Weiss
 @version $Revision: 1.6 $
 */
public interface PlanPresentationConstants
{


    // values which can be changed without deeper knowledge about the
    // planpresentation application.

    /** The background color of the first and all other odd views */
    public final Color ALTERNATE_COLOR_1 = new Color(230,230,230);//new Color(240,241,215);

    /** The background color of the second and all other even views */
    public final Color ALTERNATE_COLOR_2 = new Color(210,210,210); //new Color(178,219,176);

    /** The background color of the actions */
    public final Color ACTIONSEMANTICGRAPHIC_BACKGROUND = new Color( 200 , 150 , 200 ); //Color(152,222,200);

    public final Color STATETASK_COLOR = new Color( 200 , 150 , 100 );

    public final Color ACTIONTASK_COLOR = new Color( 200 , 190 , 100 );

    public final Color PRECONDITIONTASK_COLOR = new Color( 200 , 230 , 100 );

    public final Color STATE_COLOR = new Color( 250 , 250 , 160 );

    /** the initial pixel/ct value */
    final double PIXEL_PER_TIME_UNIT = 20.0;

    /** the height of a action plus the margin*/
    final int ACTION_LEVEL_HEIGHT = 40;

    /** the height of the view header. i.e. where the name of the view is
     displayed */
    final int HEADER_HEIGHT = 30;

    /** the height of a state in the StateResourceView */
    final int STATE_HEIGHT = 40;

    /** the margin between the header and the states at the
     StateResourceView */
    final int MARGIN_TOP = 5;

    /** the width of the retract button.(To retract a view and expand
     it again). */
    final int RETRACT_BUTTON_WIDTH = 15;

    /** the normal scrolling speed for autoscroll (when moving the cursor to the
     left or right boundary of the planning area) */
    final int SCROLL_NORMAL = 10;

    /** the fast scrolling speed, when moving the cursor really near to the border
      of the planning area */
    final int SCROLL_FAST = 40;

    /** how many pixel from the border the SCROLL_NORMAL mode is active  */
    final int SCROLL_NORMAL_AREA = 40;

    /** how many pixel from the border the SCROLL_FAST mode is active */
    final int SCROLL_FAST_AREA = 10;



    // the following values should only be changed if you know what you
    // are doing.

    /** messageID for sortmode changes */
    final int SORTMODE_CHANGED = 2;

    /** messageID for restart action (currently not supported) */
    final int RESTART_ACTION = 4;

    /** messageID for a exit call */
    final int EXIT_ACTION = 8;

    /** messageID for a current time change */
    final int CURRENT_TIME_CHANGED = 16;

    /** messageID for a zoom change */
    final int ZOOM_CHANGED = 32;


    // GUI commands from PlanPlugin
    /** The messageID when setting sortmode to update */
    final int SORTMODE_UPDATE = 4001;
    /** The messageID when setting sortmode to grouped */
    final int SORTMODE_GROUPED = 4002;



    // the priorityOffsets for different constraints
    /** the accessTime offset for StateResources when grouping */
    final int STATERESOURCE_GROUP = 100000;

    /** the accessTime offset for ActionResources when grouping */
    final int ACTIONRESOURCE_GROUP = 200000;


}
