package com.ai_center.planpresentation;


import java.awt.Graphics;
import java.util.List;

import javax.swing.JComponent;




/** The interface for SwimlaneControls.<p>
 
 This is interface describes the communication from the SwimlaneConnector
 Objects, the GlassPane, the PlanJFrame to the SwimlaneControl object.<p>
 
 @author Daniel Weiss
 @version $Revision: 1.3 $
 */
public interface SwimlaneControl
{


    /** Returns the corresponding gui element to a given datamodel object.<p>
     
     This function maps the datamodel objects to the gui elements, if they
     have already registerd themselves. 
     @param o The object from the datamodel
     @return The SwimlaneConnector which represents the object.
     @see #registerConnector
     */
    SwimlaneConnector getConnector(Object o);

    /** Returns the ctPercent.<p>
     
     The ctPercent multiplicated with the width of the planning area gives the
     x-coordinate, where the dashed line should be painted. 

     @return The ctPercent value between 0..1
     */
    double getCTPercent();

    /** Returns the glassPane */
    JComponent getGlassPane();

    /** Returns the PlanPaintArea (where all views are contained) */
    JComponent getPlanPaintArea();

    /** Returns the highest y value of the planPaintArea, which is visible */
    int maxVisibleY();
    
    /** Returns the highest y value of the planPaintArea, which is visible */
    int minVisibleY();
    
    /** Performs that all swimlanes are painted */
    void paintSwimlanes(Graphics g);

    /** Paints a vertical line at the give x-coordinate. e.g. the current time */
    void paintVerticalLine(int x);

    /** Registers a representative gui object for the given datamodel object.<p>
     
     @param object The object as key, which should be representet.
     @param sc The SwimlaneConnector, which represents the object in the gui.
     */
    void registerConnector(Object object, SwimlaneConnector sc);

    /** Removes a connector for the given object.<p>
     
     If the connector isn't registered, this function does nothing.
     @param object The datamodel object, whos gui representative should be
     removed.
     */
    void removeConnector(Object object);

    /** Removes a swimlane.<p>
     
     The swimlane where connector is the owner will be removed. 	
     @param connector The owner of the swimlane
     @return true if successful.
     */
    boolean removeSwimlane(SwimlaneConnector connector);


    /** Requests a new swimlane to all SwimlaneConnectors given in the
     connectorList.<p>
     
     Every connector gets the same connectionType.<p>
     
     @param connectionType
     @param connectorList
     @return true if the request was successfull 
     */
    boolean requestSwimlane(int connectionType,
            SwimlaneConnector ownerConnector, List connectorList);

    /** Set CTPercent value.<p>
     
     @param ctPercent The new ctPercent Value.
     @see #getCTPercent
     */
    void setCTPercent(double ctPercent);


    /** Sets the GlassPane.<p>
     
     @param planGlassPane The new glassPane
     */
    void setGlassPane(GlassPaneArea planGlassPane);

    /** Sets the PaintArea.<p>
     
     @param planPaintArea The new plnaPaintArea
     */
    void setPaintArea(JComponent planPaintArea);

    /** Sets a new PriorityOffset for the given resourceConstraint. <p>
     
     This function searches the corresponding gui element for the given
     resourceConstraint and sets its priorityOffset new.
     
     @param resourceConstraint The object in the datamodel.
     @param priorityOffset The new priorityOffset.
     */
    void setPriorityOffset(Object resourceConstraint, int priorityOffset);


    /** Sets the sort mode.<p>
     
     @param sortmode The new sort mode.
     @see PlanPresentationConstants#SORTMODE_UPDATE
     @see PlanPresentationConstants#SORTMODE_GROUPED
     */
    void setSortMode(int sortmode);
}