
package com.ai_center.planpresentation.plugins;

import com.ai_center.planpresentation.PlanPresentationAPI;

/** This controller represents one object in the datamodel. <p>
  
 The controller is one of three parts of an bottom-level agent in 
 the presentation- abstraction-control (pac) architecture pattern. 
 It is completed by the view and the model.
 The controller is in the bottom level agent of the architecture and
 is connected to a coordinator as intermediate level agent.
 
  @author Daniel Weiss
  @version $Revision: 1.4 $
 */
public interface Controller {

    /** Gets the last accesstime quoted by the copy of the datamodel<p>
     
       @return The last access time as unix timestamp.(milliseconds since 1970)
     */
    long getAccessTime();

   
    /** Delegate function. Returns the width of the main view panel.
      @return The width of the main view panel.
     */
    int getPanelWidth();


    /** Delegate function. Gets the priority.
     @return The priority is the last access time accumulated with the 
     offset
     */
    Long getPriority();

    /** Delegate function. Scroll to the left or right.
     @param scrollstep Positive values scroll to the right, negative to the 
     left
     @param extendViewIfNecessary If the planing horizont shall be adjusted if necessary.
  
     @see com.ai_center.planpresentation.PlanPresentationAPI#scroll 
     */
    void scroll(int scrollstep, boolean extendViewIfNecessary);


    /** Set the priority offset.<p>
     
     This offset is added to the last access time and gives the priority.<p>
     @param _offset The new offset
     */
    void setPriorityOffset(long _offset);

  
    /** Requests a view update.<p>
     
       Requests a update for an adjustment of a new height. The levelcount
       is used to check, if this update is really necessary.
       @param levelcount The actual number of levels.
     */
    void updateViewSize(int levelcount);


    /** Returns a reference to the PlanPresentationAPI
    
      @return The PlanPresentationAPI
    */
   PlanPresentationAPI getPlanPresentationAPI();

}