package com.ai_center.planpresentation.plugins;


import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.SwimlaneControl;




/** The coordinator connects a controller with the PlanPresentationAPI. <p>
 
 The coordinator is the intermediate level agent of the presentation-
 abstraction-control (pac) architecture pattern.
 A coordinator administrates one or more controllers. Normally coordinator
 doesn't have a corresponding view. It creates new controllers, if a new
 object in the datamodell is added.


 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public interface Coordinator
{


    /** Computes the absolut y coordinate at a give relative y coordinate.<p>
     @param relativeY The y-coordinate IN a view.
     @return absolut y-value in the plan paint area coordinate system.
     */
    public int absoluteY(int relativeY);

    /** Delegate function. Add a view to the plan paint area.<p>
     
     @param view The new view.
     @see PlanPresentationAPI#absoluteY
     */
    public void addView(RetractableView view);

    /** Delegate function. Add a viewChangeListener.<p>
     @param listener The ChangeListener.
     @see PlanPresentationAPI#addViewChangeListener
     */
    public void addViewChangeListener(ChangeListener listener);

    /** Delegate function. Stops the engine.
     @see PlanPresentationAPI#engineStop */
    public void engineStop();

    /** Gets the GlobalSearchControl<p>
     
     @return The GlobalSearchControl
     */
    public GlobalSearchControl getGlobalSearchControl();

    /** Gets the panel width.<p>
     @return The width of the plan panel.
     */
    public int getPanelWidth();

    /** Gets a direct link to the PlanPresentionAPI.<p>
     
     @return The PlanPresentationAPI.
     */
    public PlanPresentationAPI getPlanPresentationAPI();


    /** Gets a direct link to the SwimlaneControl implementation.<p>
     
     @return The SwimlaneControl.
     */
    public SwimlaneControl getSwimlaneControl();

    /** Delegate function. Removes a view
     @param view The View to be removed.
     @see PlanPresentationAPI#removeView
     */
    public void removeView(RetractableView view);

    /** Delegate function. Removes a View Change Listener.
     @param listener The ChangeListener to be removed.
     @see PlanPresentationAPI#removeViewChangeListener
     */
    public void removeViewChangeListener(ChangeListener listener);


    /** Delegate function. Requests a view update
     @see PlanPresentationAPI#updateView  */
    public void updateView();


}