package com.ai_center.planpresentation.plugins;


import com.ai_center.dragonbreath.Constraint;
import com.ai_center.planpresentation.PlanPresentationAPI;




/** Every Plugin has to implement this interface.<p>
 
 This interface specifies the communication from the PlanPresentationAPI 
 to a plugin. There are only three functions to implement:
 initPlugin, addConstraint and removConstraint.
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public interface Plugin
{


    /** The initPlugin function is called, when the PlanPresentation
     finds the plugin at the startup process.<p>
     
     The plugin can here register for constraint creation and
     deletion callbacks. Additional it can register menus, 
     controls and views.   
     
     @param _planPresentationAPI The link to the planpresentationAPI.
     */
    public void initPlugin(PlanPresentationAPI _planPresentationAPI);


    /** This function is called, when the GlobalSearchControl creates a new
     constraint.<p>
     
     But only if this plugin is registered for this constraint type.<p>
     
     @param constraint The link to the new constraint.
     @see PlanPresentationAPI#registerConstraintType
     */
    public void addConstraint(Constraint constraint);

    /** This remove function is called, when the GlobalSearchControl removes
     a constraint from the graph.<p>
     
     But only if this plugin is registered for this constraint type.<p>
     
     @param constraint The link to the removed constraint.
     @see PlanPresentationAPI#registerConstraintType
     */
    public void removeConstraint(Constraint constraint);

}