
package com.ai_center.planpresentation.plugins;

import com.ai_center.planpresentation.PlanPresentationAPI;

/** The view displays the content of one or more datamodel objects.<p>
 
 The View is one of three parts of a bottom-level agent at the
 presentation-abstraction-control (pac) architecture pattern.
 It communicates only with the controller.
 
  @author Daniel Weiss
  @version $Revision: 1.4 $
 */
public interface View {

  
    /** Returns the width of the view.
      @return The width of the view.
     */
    int getWidth();

    /** Returns the height of the view.
      @return The height of the view.
     */
    int getHeight();

    /** Delegate function. Scrolls to the left or right.<p>

        @param scrollstep The direction and speed of the scrolling.
        @param extendViewIfNecessary If the planing horizont shall be adjusted if necessary.
  
        @see com.ai_center.planpresentation.PlanPresentationAPI#scroll
     */
    void scroll(int scrollstep,  boolean extendViewIfNecessary);

    
    /** Delegate function. Stops the engine.<p>
     
       @see com.ai_center.planpresentation.PlanPresentationAPI#engineStop
     */
    void engineStop();

    /** Computes the absolut y coordinate at a give relative y coordinate.<p>
    @param relativeY The y-coordinate IN a view.
    @return absolut y-value in the plan paint area coordinate system.
    */
    int absoluteY(int relativeY);

    /** Gets the extent of the main view model.<p>
     
     	The view is normaly connected as MainViewModel ChangeListener and gets
     	all information about changes in the MainViewModel.
       
       @return The extent of the MainViewModel.
     */
    int getMainViewModelExtent();

    /** Gets the value of the main view model.<p>
     
     	The view is normaly connected as MainViewModel ChangeListener and gets
     	all information about changes in the MainViewModel.<p>
       
       @return The value of the MainViewModel.
     */
    int getMainViewModelValue();

    /** Checks if the view is retracted or expanded 

       @return True if retracted.
     */
    boolean isRetracted();

    

    /** Returns a reference to the PlanPresentationAPI
    
      @return The PlanPresentationAPI
    */
  public PlanPresentationAPI getPlanPresentationAPI();
	
}
