package com.ai_center.planpresentation.plugins.actionconstraint;


import java.awt.Cursor;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Map.Entry;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.Constraint;
import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.ActionObjectConstraint;
import com.ai_center.dragonbreath.constraints.TransitionConstraint;
import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.SwimlaneControl;
import com.ai_center.planpresentation.plugins.Controller;
import com.ai_center.planpresentation.plugins.Coordinator;
import com.ai_center.planpresentation.plugins.Plugin;
import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.utilities.MultiLevelController;
import com.ai_center.planpresentation.utilities.MultiLevelModel;



/** The ActionCoordinatorControlPlugin is a exception, because it merges
  the coordinator and the controller. <p>
  
 This Plugin has only one view, because it shows all actions in one panel.
 
  @author Daniel Weiss
  @version $$
 */
public class ActionCoordinatorControlPlugin
        implements
            Coordinator,
            Controller,
            MultiLevelController,
            Plugin,
            PlanPresentationConstants
{

    /** GUI Action Command to show/hide swimlanes. */
    public static final int CHANGE_SWIMLANES = 985435;

    /** GUI Action Command to start the AddActionDialog. */
    public static final int SHOW_ADDACTIONDIALOG = 985436;

    public static final int REMOVE_ACTION = 985437;
    
    /** A random number generator for miscellaneous purposes. */
    public static Random random = new Random();

    /** The link to the gui model. */
    protected MultiLevelModel multiLevelModel = null;

    /** The link to the view. */
    protected ActionCoordinatorView multiLevelView = null;

    /** The link to the planpresentationAPI. */
    protected PlanPresentationAPI planPresentationAPI;

    /** True if the swimlanes should be visible. */
    protected boolean showSwimlanes;

    private boolean deleteMode;

    /** A green light ImageIcon. */
    public ImageIcon greenLight;

    /** A grey light ImageIcon. */
    public ImageIcon greyLight;

    /** A red light ImageIcon. */
    public ImageIcon redLight; 


    /** Adds a ActionObjectConstraint to the view.<p>
     
       This function is called by the addConstraint-function,
       if it is no TransitionConstraint connected to this 
       ActionObjectConstraint
       @param c The constraint to add.
     */
    protected void addConstraint_internal(Constraint c)
    {
        if (c instanceof ActionObjectConstraint)
        {
            // make a new ActionSemanticGraphic Object
            ActionSemanticGraphic asg = new ActionSemanticGraphic(this,
                    (ActionObjectConstraint) c, getSwimlaneControl(),
                    multiLevelView);
            // Assign a fixed connectionType for all swimlanes which are
            // started by this ActionSemanticGraphic
            asg.setConnectionHeight(random.nextInt(ACTION_LEVEL_HEIGHT - 10));
            asg.setShouldVisible(true);

            // add ActionSemanticGraphic to the GUI Model
            multiLevelModel.addObject((ActionObjectConstraint) c, asg);

            // Check if a repaint is necessary.
            int beginCT = (int) asg.getBeginCT();
            int endCT = beginCT + (int) asg.getDuration();
            /*  View is already repainted by the Repaint-Thread
             * if (multiLevelView != null)
            {
                	
                
                 if (multiLevelView
                        .checkRevalidateMultiLevelView(beginCT, endCT))
                {
                    repaintActionCoordinatorView();
                }
            }*/
        }
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#absoluteY(int)
     */
    public int absoluteY(int relativeY)
    {
        if (multiLevelView == null)
        {
            return 0;
        } else
        {

            return planPresentationAPI.absoluteY(multiLevelView.getY())
                    + relativeY;
        }
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#addConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void addConstraint(Constraint c)
    {
        ActionObjectConstraint aoc = (ActionObjectConstraint) c;

        Variable actionTypeVariable = aoc.connections
                .getSpecificVariable(Registry.ACTIONTYPE_EDGE);
        // only add constraint, which are NOT a transition
        if (!TransitionConstraint
                .isTransition((int) actionTypeVariable.value()))
        {
            addConstraint_internal(c);
        }
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#addView(com.ai_center.planpresentation.plugins.RetractableView)
     */
    public void addView(RetractableView view)
    {
       // addView does nothing, because this coordinator has its own view,
       //  and does not coordinate other views.
        return; // do nothing
    }

/*
 *  (non-Javadoc)
 * @see com.ai_center.planpresentation.plugins.Coordinator#addViewChangeListener(javax.swing.event.ChangeListener)
 */    public void addViewChangeListener(ChangeListener l)
    {
        planPresentationAPI.addViewChangeListener(l);
    }

/** Delegate function. Adds an Action to the view.<p>
 
   @param asg The ActionSemanticGraphic to add.
   @see com.ai_center.planpresentation.utilities.MultiLevelModel#addVisibleObject
 */
 public synchronized void addVisibleAction(ActionSemanticGraphic asg)
    {
        multiLevelModel.addVisibleObject(asg);
    }

  

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#engineStop()
     */
    public void engineStop()
    {
        planPresentationAPI.engineStop();

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#getAccessTime()
     */
    public long getAccessTime()
    {

        return Long.MAX_VALUE / 2;
    }

    /** Delegate function. Gets all Actions between beginCT and
       beginCT + duration.<p>
     
        	
       @param beginCT The beginning time.
       @param durationCT The duration.
       @return A Map of all ActionSemanticGraphic which are in this interval. 
     */
    public synchronized Map getActions(long beginCT, long durationCT)
    {
        return multiLevelModel.getObjects(beginCT, durationCT);
    }


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#getGlobalSearchControl()
     */
    public GlobalSearchControl getGlobalSearchControl()
    {
        return planPresentationAPI.getGlobalSearchControl();

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.MultiLevelController#getLevel()
     */
    public synchronized int getLevel()
    {
        return multiLevelModel.getLevel();
    }

    /* (non-Javadoc)
     @see com.ai_center.planpresentation.utilities.MultiLevelController#getLevelcount()
     */
    public int getLevelcount()
    {
        return multiLevelModel.getLevelcount();

    }

    /**  @return mainViewModel.getExtent() */
    public int getMainViewModelExtent()
    {
        return multiLevelView.getMainViewModelExtent();
    }

    /**  @return mainViewModel.getValue() */
    public int getMainViewModelValue()
    {
        return multiLevelView.getMainViewModelValue();
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.MultiLevelController#getObjectsOnLevel(int, long, long)
     */
    public synchronized Map getObjectsOnLevel(int level, long beginCT,
            long durationCT)
    {
        return multiLevelModel.getObjectsOnLevel(level, beginCT, durationCT);
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#getPanelWidth()
     */
    public int getPanelWidth()
    {

        return planPresentationAPI.getViewPanelWidth();
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#getPlanPresentationAPI()
     */
    public PlanPresentationAPI getPlanPresentationAPI()
    {
        return planPresentationAPI;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#getPriority()
     */
    public Long getPriority()
    {

        return new Long(Long.MAX_VALUE); // stay always on top. maximum priority
    }

    /** @return True if the swimlanes should be visible. */
    public boolean getShowSwimlanes()
    {

        return showSwimlanes;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#getSwimlaneControl()
     */
    public SwimlaneControl getSwimlaneControl()
    {

        return planPresentationAPI.getSwimlaneControl();
    }

    /** @return The height of the view. */
    public int getViewHeight()
    {
        return multiLevelView.getHeight();
    }

    /** The width of the view. */
    public int getViewWidth()
    {
        return multiLevelView.getWidth();
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#initPlugin(com.ai_center.planpresentation.PlanPresentationAPI)
     */
    public void initPlugin(PlanPresentationAPI _p)
    {

        planPresentationAPI = _p;
        
        /** A green light ImageIcon. */
        greenLight = new ImageIcon(getPlanPresentationAPI().getClass().getResource("resources/greenLight.gif"));

        /** A grey light ImageIcon. */
        greyLight = new ImageIcon(getPlanPresentationAPI().getClass().getResource("resources/greyLight.gif"));

        /** A red light ImageIcon. */
        redLight = new ImageIcon(getPlanPresentationAPI().getClass().getResource("resources/redLight.gif"));


        // init model
        multiLevelModel = new MultiLevelModel(this);

        //init view
        multiLevelView = new ActionCoordinatorView(this, "Actions");
        this.addViewChangeListener(multiLevelView);
        planPresentationAPI.addView(multiLevelView);

        // register constraint types	
        planPresentationAPI.registerConstraintType(
                Registry.ACTION_OBJECTCONSTRAINT, (Plugin) this);

        // create action commands
        ActionCoordinatorAction showSwimlanes = new ActionCoordinatorAction(
                " Show action connections lines", null, "show swimlanes",
                new Integer(0),
                ActionCoordinatorControlPlugin.CHANGE_SWIMLANES, this);

        ActionCoordinatorAction addActionDialog = new ActionCoordinatorAction(
                "Action", null, "Adds an action", new Integer(0),
                ActionCoordinatorControlPlugin.SHOW_ADDACTIONDIALOG, this);
        
        ActionCoordinatorAction removeAction = new ActionCoordinatorAction(
                "Action", null, "Remove an action", new Integer(0),
                ActionCoordinatorControlPlugin.REMOVE_ACTION, this);
        
        // create menu entries
        planPresentationAPI.addMenu("View", KeyEvent.VK_V, 10);
        JMenuItem showSwimlanesItem = new JMenuItem(showSwimlanes);
        showSwimlanesItem.setAccelerator(KeyStroke.getKeyStroke('S',Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(),false));
        showSwimlanesItem.setMnemonic('S');
        planPresentationAPI.addMenuItem(showSwimlanesItem, "View");

        planPresentationAPI.addMenu("Add",KeyEvent.VK_A, 15);
        JMenuItem addActionMenuItem = new JMenuItem(addActionDialog);
        addActionMenuItem.setMnemonic('A');
        addActionMenuItem.setAccelerator(KeyStroke.getKeyStroke('A',Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(),false));
        planPresentationAPI.addMenuItem(addActionMenuItem, "Add");
        
        planPresentationAPI.addMenu("Remove", KeyEvent.VK_R, 16);
        JMenuItem removeActionItem = new JMenuItem(removeAction);
        removeActionItem.setAccelerator(KeyStroke.getKeyStroke('R',Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(),false));
        removeActionItem.setMnemonic('R');
        planPresentationAPI.addMenuItem(removeActionItem, "Remove");


    }

   
    /** Changes the mouse cursor to a crosshair.<p>
     
       Additional it enables catching of mouse clicks to the action panel.<p>
     */
    public void prepareActionDialog()
    {
        // show text: select the place, where the action should be placed..
        multiLevelView.setMouseCatcherOn(true);
        planPresentationAPI.getGuiFrame().setCursor(
                Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        //multiLevelView.setCursor();
        planPresentationAPI
                .setHelpText("Click on the place where the action shall be inserted");


    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#removeConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void removeConstraint(Constraint c)
    {
        if (c instanceof ActionObjectConstraint)
        {

            ActionSemanticGraphic asg = (ActionSemanticGraphic) multiLevelModel
                    .getSemanticGraphic((ActionObjectConstraint) c);
            if (asg != null)
            {

                // save coordinate data for view update
                int beginCT = (int) asg.getBeginCT();
                int duration = (int) asg.getDuration();

                multiLevelModel.removeObject((ActionObjectConstraint) c);
              /* View is already repainted by the Repaint-Thread
               *  if (multiLevelView != null)
                {

                    if (multiLevelView.checkRevalidateMultiLevelView(beginCT,
                            beginCT + duration))
                    {
                        repaintActionCoordinatorView();

                    }
                }*/
            }
        }
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#removeView(javax.swing.JPanel)
     */
    public void removeView(RetractableView view)
    {
      // do nothing. ActionCoordinatorView will never be removed.

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#removeViewChangeListener(javax.swing.event.ChangeListener)
     */
    public void removeViewChangeListener(ChangeListener l)
    {
        planPresentationAPI.removeViewChangeListener(l);
    }

    /** Delegate function. Removes a Action from the view.<p>
     
       @param asg The ActionSemanticGraphic to remove
       @see com.ai_center.planpresentation.utilities.MultiLevelModel#removeVisibleObject
     */
    public synchronized void removeVisibleAction(ActionSemanticGraphic asg)
    {
        multiLevelModel.removeVisibleObject(asg);
    }

    
   
 /*
  *  (non-Javadoc)
  * @see com.ai_center.planpresentation.plugins.Controller#scroll(int)
  */
    public void scroll(int scrollstep, boolean extendViewIfNecessary)
    {
        planPresentationAPI.scroll(scrollstep,extendViewIfNecessary);

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#setPriorityOffset(int)
     */
    public void setPriorityOffset(long _offset)
    {
        // do nothing. This view is always on top.

    }

    /** Starts the AddActionDialog.<p>
     
       @param beginCT The tick, where the action should be inserted.
     */
    public void showAddActionDialog(long beginCT)
    {
        AddActionDialog aad = new AddActionDialog(planPresentationAPI
                .getGuiFrame(), "Add a action", true, this, planPresentationAPI
                .getGlobalSearchControl(), beginCT);
        aad.show();
    }

    
    /**  Shows all swimlanes connected to the Actions.<p>
     
       @param _showSwimlanes True if all swimlanes should be visible.
       
     */
    public void showSwimlanes(boolean _showSwimlanes)
    {
        showSwimlanes = _showSwimlanes;
        // get all actions
        Map actions = multiLevelModel.getObjects(getMainViewModelValue(),
                getMainViewModelExtent());
        Iterator it = actions.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry entry = (Entry) it.next();
            ActionSemanticGraphic asg = (ActionSemanticGraphic) entry
                    .getValue();
            if (showSwimlanes)
            {
                // tell every ActionSemanticGraphic to show its swimlane
                asg.requestSwimlane();
            } else
            {
                // tell every ActionSemanticGraphic to remove its swimlane
                asg.removeSwimlane();
            }

        }

    }

    /** Updates this ActionSemanticGraphic in the view..<p>
     
      e.g. moves it to the left or right. 	
       @param asg The ActionSemanticGraphic to update.
     */
    public void updateAction(ActionSemanticGraphic asg)
    {
        multiLevelModel.updateVisibleObject(asg);

    }

    
    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.MultiLevelController#updateView()
     */
    public void updateView()
    {
        planPresentationAPI.updateView();
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#updateViewSize(int)
     */
    public void updateViewSize(int level)
    {
        if (multiLevelView != null)
        {

            multiLevelView.updateViewSize(level);
        }
        planPresentationAPI.updateView();

    }

    /**  Changes the mouse cursor to a crosshair and waits for a click
      on an action to delete it. 
       
     */
    public void prepareRemoveAction()
    {
        planPresentationAPI.getGuiFrame().setCursor(
                Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        //multiLevelView.setCursor();
        planPresentationAPI
                .setHelpText("Click on an action to remove it.");

       deleteMode = true;
        
    }

    /** Checks if the next click on an action should delete it.
       @return true if action shall be deleted by a click.
     */
    public boolean isDeleteMode()
    {
        return deleteMode;
        
    }

    /**  Sets the deleteMode to false and returns to normal planning mode.
     */
    public void resetDeleteMode()
    {
        planPresentationAPI.getGuiFrame().setCursor(
                Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        planPresentationAPI
                .setHelpText("");
       deleteMode = false;
        
    }


}



/** The ActionCoordinatorAction represents a the commands: show swimlanes
  and addAction. <p>
 
  @author Daniel Weiss
  @version $Revision: 1.6 $
 */
class ActionCoordinatorAction extends AbstractAction
        implements
            PlanPresentationConstants
{

    /** The link to the coordinator. */
    private ActionCoordinatorControlPlugin actionCoordinatorControl;

    /** The messageID of this action. */
    private int actionType;

    /** Creates a new action command.
     
       @param text The text to display.
       @param icon A optional ImageIcon
       @param desc A detailled description of this action. (for ToolTips)
       @param mnemonic The keyboard shortcut.
       @param _actionType The actionType/MessageID
       @param _acc A link to the ActionCoordinatorControlPlugin
     */
    public ActionCoordinatorAction(String text, ImageIcon icon, String desc,
            Integer mnemonic, int _actionType,
            ActionCoordinatorControlPlugin _acc)
    {
        super(text, icon);
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
        actionType = _actionType;
        actionCoordinatorControl = _acc;
    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */

    public void actionPerformed(ActionEvent e)
    {

        // check, which action is requested
        switch (actionType)
        {
            case CURRENT_TIME_CHANGED :
                break;
            case ActionCoordinatorControlPlugin.CHANGE_SWIMLANES :
                if (e.getSource() instanceof JMenuItem)
                {
                    JMenuItem menuItem = (JMenuItem) e.getSource();
                    if (menuItem.getText().equals(
                            " Show action connections lines"))
                    {
                        menuItem.setText(" Hide action connections lines");
                        actionCoordinatorControl.showSwimlanes(true);
                    } else if (menuItem.getText().equals(
                            " Hide action connections lines"))
                    {
                        menuItem.setText(" Show action connections lines");
                        actionCoordinatorControl.showSwimlanes(false);

                    }
                }

                break;
            case ActionCoordinatorControlPlugin.SHOW_ADDACTIONDIALOG :
                actionCoordinatorControl.prepareActionDialog();
                break;
            
           case ActionCoordinatorControlPlugin.REMOVE_ACTION:
               actionCoordinatorControl.prepareRemoveAction();
           		break;
        }

    }

}
