package com.ai_center.planpresentation.plugins.actionconstraint;


import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.ConnectedNode;
import com.ai_center.dragonbreath.constraintgraph.ConventionalConstraint;
import com.ai_center.dragonbreath.constraintgraph.ObjectConstraint;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.ActionObjectConstraint;
import com.ai_center.dragonbreath.constraints.ActionTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.PreconditionTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.StateTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.TaskConstraint;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.SwimlaneConnector;
import com.ai_center.planpresentation.SwimlaneControl;
import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.utilities.DragablePanel;
import com.ai_center.planpresentation.utilities.SemanticGraphic;




/** The ActionSemanticGraphic represents an ActionObjectConstraint. <p>

 For that purpose, it registers as ChangeListener to all connected
 variables and other constraints.

 @author Daniel Weiss
 @version $Revision: 1.10 $
 */
public class ActionSemanticGraphic extends DragablePanel
        implements
            ChangeListener,
            PlanPresentationConstants,
            SwimlaneConnector,
            SemanticGraphic
{


    /** The height, where the swimlane connects. */
    private int connectionHeight;

    /** The number of connected Precondition-, State- or ActionTasks. */
    public int connectedTasks;

    /** True if this ActionObjectConstraint should be visible from the
     datamodel point of view.*/
    private boolean shouldVisible;

    /** True if a swimlane should be painted. */
    private boolean paintSwimlane = false;

    /** True if a swimlane should connect to this Action. */
    private boolean swimlaneVisibility = true;

    /** The actionType of the ActionObjectConstraint. */
    private int actionType = -1;

    /** The name/description of the action. */
    private String actionDescription = null;

    /** True if this ActionSemanticGraphic object is in the GUI model. */
    private boolean isInModel = false;

    /** The variable with the smallest begin time. */
    private Variable minBeginTimeVariable = null;

    /** The time of the Task with the minimal begin time. */
    private double minBeginTime = -1.0;

    /** The time variable which is the begin-part of the begin+duration pair.*/
    private Variable maxEndTimeVariable = null;

    /** The time variable which is the duration-part of the begin+duration pair.*/
    private Variable maxEndTimeVariableDuration = null;

    /** The right boundary of the action. */
    private double maxEndTime = -1.0;

    /** The link to the corresponding object in the datamodell. */
    private ActionObjectConstraint actionObjectConstraint = null;

    /** The link to the actionCoordinatorControlPlugin. */
    private ActionCoordinatorControlPlugin actionCoordinatorControl = null;

    /** Mapping of knownPreconditions to there corresponding
     StateResourceConstraints */
    private Map knownPreconditionStateResourceMap = new HashMap();

    /** A map of all known temporal refernce variables. */
    private HashSet knownTemporalReferenceVariables = new HashSet();

    /** This hashset contains the ResourceConstraints which are indirectly
     (over a variable) connected to this ActionObjectConstraint.  */
    private HashSet knownResourceConstraints = new HashSet();

    /** Mapping of PreconditionTaskObjectConstraint to the value 0, if fullfilled
     or 1 if not fullfilled. */
    private Map preconditionFullfilledMap = new HashMap();

    /** The link to the TaskConstraint instance. */
    private TaskConstraint taskConstraint = null;

    /** The begin tick of this action */
    private int beginCT = -1;

    /** The ending tick of this action. */
    private int endCT = -1;

    // GUI Variables
    /** The JLabel, where the name of the action is diplayed. */
    private JLabel descriptionJLabel = new JLabel();

    /** The link to the view, where this action is contained. */
    private RetractableView retractableView = null;

    /** The upper light. */
    private JButton upperLight = new JButton();

    /** the lower light. */
    private JButton lowerLight = new JButton();

    /** Creats a ActionSemanticGraphic.<p>

     @param control The link to the controller (ActionCoordinatorControlPlugion)
     @param constraint A link to the corresponding constraint in the datamodel
     @param _swimlaneControl A link to the SwimlaneControl.
     @param _retractableView A link to the view.
     */
    public ActionSemanticGraphic(ActionCoordinatorControlPlugin control,
            ActionObjectConstraint constraint,
            SwimlaneControl _swimlaneControl, RetractableView _retractableView)
    {

        actionCoordinatorControl = control;
        actionObjectConstraint = constraint;
        swimlaneControl = _swimlaneControl;
        retractableView = _retractableView;

        // register as changelistener to the constraint
        actionObjectConstraint.addChangeListener(this);

        // register as changelistener to the connection class of the
        // constraint
        synchronized (actionObjectConstraint.connections)
        {
            actionObjectConstraint.connections.addChangeListener(this);
        }

        // compute the boundary times (beginCT and endCT)
        computeBoundaryTimes();

        // layout the action
        this.setLayout(null);
        setBorder(BorderFactory.createLineBorder(Color.black, 1));

        this.setBackground(ACTIONSEMANTICGRAPHIC_BACKGROUND);
        this.add(descriptionJLabel);
        descriptionJLabel.setLocation(20, 1);
        descriptionJLabel.setSize(200, ACTION_LEVEL_HEIGHT - 10);
        this.add(upperLight);
        this.add(lowerLight);
        int lightHeight = (ACTION_LEVEL_HEIGHT - 5) / 2;
        upperLight.setBorderPainted(false);
        upperLight.setOpaque(false);
        upperLight.setLocation(0, 0);
        upperLight.setSize(lightHeight, lightHeight);
        lowerLight.setBorderPainted(false);
        lowerLight.setOpaque(false);
        lowerLight.setLocation(0, 3 + lightHeight);
        lowerLight.setSize(lightHeight, lightHeight);

        // register as connector for the given constraint
        swimlaneControl.registerConnector(actionObjectConstraint, this);

    }


    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public synchronized void stateChanged(ChangeEvent e)
    {
        // check boundary times
        computeBoundaryTimes();

        // continious check for later connected variables
        if (actionType == -1)
        {
            checkActionType();
        }

        // Checking the event

        if (e.getSource() instanceof PreconditionTaskObjectConstraint)
        {
            // this signal comes from a precondition. check the preconditions
            updatePreconditionTaskObjectConstraint((PreconditionTaskObjectConstraint) e
                    .getSource());
        } else if (e.getSource() instanceof ConnectedNode)
        {
            // this signal comes from the ActionObjectConstraint.connections or
            // from a connected Variable.connections (second-level connection)

            ConnectedNode cn = (ConnectedNode) e.getSource();

            if (cn.node instanceof StateResourceConstraint)
            {
                // a StateResourceConstraint was dis/connected
                updateStateResouceConstraint((StateResourceConstraint) cn.node);

            } else if (cn.node instanceof ActionTaskObjectConstraint)
            {

                // add new ActionTaskObjectConstraint
                addActionTaskObjectConstraint((ActionTaskObjectConstraint) cn.node);

            } else if (cn.node instanceof PreconditionTaskObjectConstraint)
            {

                //add new PreconditionTaskObjectConstraint
                updatePreconditionTaskObjectConstraint((PreconditionTaskObjectConstraint) cn.node);

            } else if (cn.node instanceof TaskConstraint)
            {
                // the taskconstraint is now connected
                if (taskConstraint != null)
                    throw new Error("TaskConstraint is already connected");
                taskConstraint = (TaskConstraint) cn.node;

            }
        } else if (e.getSource() instanceof Variable)
        {

            // the changeevent was sent by an Variable
            // get the name of the variable
            Variable v = (Variable) e.getSource();
            String variablename = "";
            if (v != null)
            {
                variablename = v.getName();
            }

            /* if (variablename.equals("begin") || variablename.equals("duration")
                    || variablename.equals("temporal reference"))
            {
                // a temporal variable has changed.
                stateChangedTimeVariable(v);

            } else if (variablename.equals("state"))
            {
                //checkPreconditions();

            } else */ if (variablename.equals("Action Type"))
            {
                // the action type has changed
                actionType = (int) v.value();
                actionDescription = TaskConstraint
                        .getNameForActionType(actionType);

                if (actionDescription != null)
                {
                    descriptionJLabel.setText(actionDescription);
                    this.setToolTipText(actionDescription);

                }
            }
        }


    }

    /** This function is called, when a temporal variable has changed.<p>

     It computes, if the boundary of the action has changed or if the
     change was only inside of the action interval.<p>

     @param v The variable which has changed.
     */
    private void stateChangedTimeVariable(Variable v)
    {
        boolean isKnownVariable = false;

        // save values for later comparement
        int oldBeginCT = beginCT;
        int oldEndCT = endCT;
        //      computeBoundaryTimes();

        // check if the changed variable is on the left boundary
        if (v == minBeginTimeVariable)
        {
            // a boundary variable has changed.
            isKnownVariable = true;


            if (v.value() <= minBeginTime)
            {
                // boundary variable shifted to the left => no problem, the variable doesn't change
                minBeginTime = v.value();
            } else
            {
                // boundary variable shiftet to the right => problem, there could be
                // another variable with an lower value => recompute the boundaryTimes
                computeBoundaryTimes();
            }

        }


        if (v == maxEndTimeVariableDuration || v == maxEndTimeVariable)
        {
            // a variable on the right boundary has changed
            isKnownVariable = true;

            // set boundary variables new
            double maxEndTimeVariableDurationValue = 0;
            if (maxEndTimeVariableDuration != null)
            {
                maxEndTimeVariableDurationValue = maxEndTimeVariableDuration
                        .value();
            }

            double maxEndTimeVariableValue = 0;
            if (maxEndTimeVariable != null)
            {
                maxEndTimeVariableValue = maxEndTimeVariable.value();
            }


            if (maxEndTimeVariableDurationValue + maxEndTimeVariableValue >= maxEndTime)
            {

                //boundary variable shifted to the right => no problem, the variable doesn't change
                maxEndTime = maxEndTimeVariableValue
                        + maxEndTimeVariableDurationValue;
            } else
            {

                //      boundary variable shiftet to the left => problem, there could be
                // another variable with an higher value => recompute the boundaryTimes
                computeBoundaryTimes();
            }
        }

        if (!isKnownVariable)
        {
            // the event-Variable is unknown, probably this is a new boundary variable
            // => recompute boundary times new.
            computeBoundaryTimes();
        }
    }

    /** Checks if all Preconditions are fullfilled.<p>

     @return True if all Preconditions are fullfilled or false if not.
     */
    private boolean checkPreconditions()
    {
        Iterator it = preconditionFullfilledMap.entrySet().iterator();
        boolean preconditionsFulfilled = true;
        while (it.hasNext() && preconditionsFulfilled)
        {
            Map.Entry preconditionStatePair = (Entry) it.next();
            PreconditionTaskObjectConstraint ptoc = (PreconditionTaskObjectConstraint) preconditionStatePair
                    .getKey();
            int preconditionState = ((Integer) preconditionStatePair.getValue())
                    .intValue();

            if (preconditionState == -1)
            { // preconditionState is unknown
                // get the actual state (last chance that this state is now know,
                // otherwise
                StateResourceConstraint stoc = null;
                synchronized (ptoc.connections)
                {

                    stoc = (StateResourceConstraint) ptoc.connections
                            .getSpecificConstraint(
                                    Registry.STATERESOURCE_CONSTRAINT,
                                    Registry.PLAIN_EDGE);

                }

                if (stoc != null)
                {
                    preconditionState = stoc.checkPrecondition(ptoc);
                }
            }

            if (preconditionState != 0)
            {
                // AND conjunction, if one precondition is false, the whole action
                // cannot be executed.
                preconditionsFulfilled = false;
            }

        }

        return preconditionsFulfilled;
    }


    /** Computes the boundary times and sets the begintCT and endCT variables.<p>

     */
    private void computeBoundaryTimes()
    {
        connectedTasks = 0;

        int oldBeginCT = beginCT; // save values for later comparement
        int oldEndCT = endCT;
        knownResourceConstraints = new HashSet(knownResourceConstraints.size());

        // compute boundary times for ActionTasks
        synchronized (actionObjectConstraint.connections)
        {
            actionObjectConstraint.connections.initializeEnumeration(false,
                    true, false, false, null,
                    Registry.nodeType[Registry.ACTIONTASK_OBJECTCONSTRAINT], 0,
                    false);
            ConnectedNode cn = actionObjectConstraint.connections.getNext();
            minBeginTimeVariable = null;
            maxEndTimeVariable = null;
            minBeginTime = -1.0;
            maxEndTime = -1.0;
            while (cn != null)
            {

                ActionTaskObjectConstraint actionTaskObjectConstraint = (ActionTaskObjectConstraint) cn.node;

                if (actionTaskObjectConstraint != null)
                {

                    ConventionalConstraint arc = actionTaskObjectConstraint.connections
                            .getSpecificConstraint(
                                    Registry.DISCRETERESOURCE_CONSTRAINT,
                                    Registry.PLAIN_EDGE);

                    if (arc != null)
                    {
                        knownResourceConstraints.add(arc);
                        connectedTasks++;

                        Variable beginTime = actionTaskObjectConstraint.connections
                                .getSpecificVariable(Registry.BEGIN_EDGE);

                        // compute minBeginTime so that changes in beginCT Variables
                        // only results in an complete new computing of an ActionTaskObjectConstraints

                        if ((beginTime.value() < minBeginTime)
                                || minBeginTime == -1.0)
                        {
                            minBeginTime = beginTime.value();
                            minBeginTimeVariable = beginTime;
                            beginCT = (int) beginTime.value();

                        }

                        Variable duration = actionTaskObjectConstraint.connections
                                .getSpecificVariable(Registry.DURATION_EDGE);

                        if ((beginTime.value() + duration.value()) > maxEndTime)
                        {
                            maxEndTime = beginTime.value() + duration.value();
                            endCT = (int) maxEndTime;
                            maxEndTimeVariable = beginTime;
                            maxEndTimeVariableDuration = duration;

                        }

                    }
                    cn = actionObjectConstraint.connections.getNext();
                }
            }
        }

        // compute boundary times for PreconditonTasks
        computeTasksTemporalReference(Registry.PRECONDITIONTASK_OBJECTCONSTRAINT);

        // compute boundary times for StateTasks
        computeTasksTemporalReference(Registry.STATETASK_OBJECTCONSTRAINT);

        if (connectedTasks > 0)
        {
            // there is at least one task connected
            if ((beginCT != oldBeginCT) || (endCT != oldEndCT))
            {
                // Boundary times of this action changed
                if (isInModel)
                {
                    //check if the action was already in the model and has
                    // to be deleted now. (update = delete + insert)
                    actionCoordinatorControl.removeVisibleAction(this);
                    isInModel = false;

                }

                if (beginCT > 0 && endCT > 0)
                {
                    // add this action to the gui again
                    actionCoordinatorControl.addVisibleAction(this);
                    isInModel = true;
                    if (actionCoordinatorControl != null && actionCoordinatorControl.multiLevelView != null) {
                    	actionCoordinatorControl.multiLevelView.checkIfNewMaximumTime(endCT);
                    }
                }

            }
        } else
        {
            // no task is connected, this action is invisible
            if (isInModel)
            {
                //check if the action was already in the model an has to be deleted.
                actionCoordinatorControl.removeVisibleAction(this);
                isInModel = false;

            }
        }


    }


    /** Sets the beginCT and endCT new, if a temporal variable of a task
     is not in the existing beginCT-endCT interval.<p>

     <pre>
     *  computeTasksTemporalReference(Registry.PRECONDITIONTASK_OBJECTCONSTRAINT);
     </pre>

     @param nodeType The nodeType. e.g. Registry.PRECONDITIONTASK_OBJECTCONSTRAINT
     */
    private void computeTasksTemporalReference(int nodeType)
    {

        int oldBeginCT = beginCT; // save values for later comparement
        int oldEndCT = endCT;

        synchronized (actionObjectConstraint.connections)
        {
            actionObjectConstraint.connections.initializeEnumeration(false,
                    true, false, false, null, Registry.nodeType[nodeType], 0,
                    false);
            ConnectedNode cn = actionObjectConstraint.connections.getNext();

            while (cn != null)
            {

                boolean resourceConstraintFound = false;
                ObjectConstraint constraint = (ObjectConstraint) cn.node;

                if (constraint != null)
                {
                    synchronized (constraint.connections)
                    {
                        ConventionalConstraint src = constraint.connections
                                .getSpecificConstraint(
                                        Registry.STATERESOURCE_CONSTRAINT,
                                        Registry.PLAIN_EDGE);

                        if (src != null)
                        {
                            knownResourceConstraints.add(src);
                            resourceConstraintFound = true;
                        }

                    } // end synchronized

                    if (resourceConstraintFound)
                    {
                        connectedTasks++;

                        Variable temporalReference = constraint.connections
                                .getSpecificVariable(Registry.TEMPORALREFERENCE_EDGE);

                        if (temporalReference != null)
                        {
                            if (!knownTemporalReferenceVariables
                                    .contains(temporalReference))
                            {
                                temporalReference.addChangeListener(this);
                                knownTemporalReferenceVariables
                                        .add(temporalReference);
                            }

                            // compute minBeginTime so that changes in beginCT Variables
                            // only results in an complete new computing of an ActionTaskObjectConstraints

                            if ((temporalReference.value() < minBeginTime)
                                    || minBeginTime == -1.0)
                            {
                                minBeginTime = temporalReference.value();
                                minBeginTimeVariable = temporalReference;
                                beginCT = (int) temporalReference.value();

                            }

                            if ((temporalReference.value()) > maxEndTime)
                            {
                                maxEndTime = temporalReference.value();
                                endCT = (int) maxEndTime;
                                maxEndTimeVariable = temporalReference;
                                maxEndTimeVariableDuration = null;

                            }
                        }
                    }
                    cn = actionObjectConstraint.connections.getNext();

                }
            }
        }

    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#getBeginCT()
     */
    public synchronized long getBeginCT()
    {
        return beginCT;
    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#getDuration()
     */
    public synchronized long getDuration()
    {

        return endCT - beginCT;
    }


    /*
     *  (non-Javadoc)
     * @see java.awt.Component#paint(java.awt.Graphics)
     */
    public void paint(Graphics g)
    {
        int width = this.getWidth();
        // paint swimlanes if necessary
        if (actionCoordinatorControl.getShowSwimlanes() || paintSwimlane)
        {
            requestSwimlane();
        } else
        {
            removeSwimlane();
        }

        // adjust the actionDescription text (if the size has changed)
        if (actionDescription != null)
        {

            descriptionJLabel.setText(prepareText(actionDescription, width));
            
        }
        // System.out.println("ActionSemanticGraphic:paintComponent:beginCT "+beginCT+ "  Descr:"+actionDescription);
        // if width to small,make lights invisible to avoid graphics errors.
        if (width < 15)
        {
            upperLight.setVisible(false);
            lowerLight.setVisible(false);
        } else if (!upperLight.isVisible())
        {
            upperLight.setVisible(true);
            lowerLight.setVisible(true);
        }

        // check if this action should be green or red
        // FIXME Temproal Inconsistencies correct??
        if (checkPreconditions()
                && taskConstraint != null
                && (taskConstraint
                        .getCosts(TaskConstraint.TEMPORAL_INCONSISTENCY) == 0))
        {
            upperLight.setIcon(actionCoordinatorControl.greyLight);
            lowerLight.setIcon(actionCoordinatorControl.greenLight);
            upperLight.setToolTipText("All Preconditions are fulfilled. The action can be executed.");
            lowerLight.setToolTipText("All Preconditions are fulfilled. The action can be executed.");
            //this.setBackground(Color.green);
        } else
        {
            upperLight.setIcon(actionCoordinatorControl.redLight);
            lowerLight.setIcon(actionCoordinatorControl.greyLight);
            lowerLight.setToolTipText("Some Preconditions are not fulfilled or the temporal reference is inconsistent.");
            upperLight.setToolTipText("Some Preconditions are not fulfilled or the temporal reference is inconsistent.");
            //this.setBackground(Color.red);
        }
        super.paint(g);
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent evt)
    {
        actionCoordinatorControl.planPresentationAPI.getGuiFrame().setCursor(
                Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

        paintSwimlane = false;
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent evt)
    {
        actionCoordinatorControl.planPresentationAPI.getGuiFrame().setCursor(
                Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        paintSwimlane = true;
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent evt)
    {
        actionCoordinatorControl.engineStop();
        //overwrite DragablePanel function
        super.mouseDragged(evt);
        int width = actionCoordinatorControl.getViewWidth();
        Point location = this.getLocation();

        int scrollstep = 0;
        //check for normal speed
        if ((location.x + this.getWidth()) > (width - SCROLL_NORMAL_AREA))
        {
            scrollstep = SCROLL_NORMAL;
            // scroll to the right with normal speed
        } else if (location.x < SCROLL_NORMAL_AREA)
        {
            scrollstep = -SCROLL_NORMAL;
            // scroll to the lefft with normal speed
        }

        //check for fast speed
        if ((location.x + this.getWidth()) > (width - SCROLL_FAST_AREA))
        {
            scrollstep = SCROLL_FAST;
            // scroll to the right with normal speed
        } else if (location.x < SCROLL_FAST_AREA)
        {
            scrollstep = -SCROLL_FAST;
            // scroll to the lefft with normal speed
        }

        if (location.x < 0)
        {
            location.x = 0;

        }
        if (location.y < 0)
        {
            location.y = 0;

        }

        if (location.y + this.getHeight() > actionCoordinatorControl
                .getViewHeight())
        {
            location.y = actionCoordinatorControl.getViewHeight()
                    - this.getHeight();
        }

        if (location.x + this.getWidth() > actionCoordinatorControl
                .getViewWidth())
        {
            location.x = actionCoordinatorControl.getViewWidth()
                    - this.getWidth();
        }

        this.setLocation(location);

        if (scrollstep != 0)
        {
            actionCoordinatorControl.scroll(scrollstep, true);
        }

    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent evt)
    {
        super.mouseClicked(evt);
        sortToTop();
    }

    /**  This method sorts all connected ResourceConstraints to the top.*/
    private void sortToTop()
    {
        //
        swimlaneControl.setSortMode(PlanPresentationConstants.SORTMODE_UPDATE);
        Iterator it = knownResourceConstraints.iterator();
        while (it.hasNext())
        {
            Object resourceConstraint = it.next();
            //move all connected Resources to the top for 20 seconds
            swimlaneControl.setPriorityOffset(resourceConstraint, 20000);

        }

    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent evt)
    {
        if (this.isDragging())
        {
            long newBeginCT = convertCoordinatesToTime(this.getX());
            setBeginCTDatamodel(newBeginCT);

        }
        if (actionCoordinatorControl.isDeleteMode()) {
            // delete action

            removeAction();
        }
        super.mouseReleased(evt);

    }

    /** Removes the action from the data model. */
    private void removeAction()
    {

        if (taskConstraint != null) {
            taskConstraint.removeTaskConstraint();
        } else {
            throw new Error("No TaskConstraint connected");
        }
        actionCoordinatorControl.resetDeleteMode();

    }


    /**  Sets a new begin time in the datamodel.<p>

     For that purpose it moves all connected task by the difference time
     of the new and the old beginCT.<p>
     @param newBeginCT The new begin time of the action.
     */
    private void setBeginCTDatamodel(long newBeginCT)
    {

        double diffTime = newBeginCT - minBeginTime;
        moveAllActionTasks((long) diffTime);
        moveAllPreconditionTasks((long) diffTime);
        moveAllStateTasks((long) diffTime);

    }

    /** Moves all PreconditionTasks.<p>

     @param diffTime The time difference to move.
     */
    private void moveAllPreconditionTasks(long diffTime)
    {
        List pTs = new LinkedList();
        synchronized (actionObjectConstraint.connections)
        {
            actionObjectConstraint.connections
                    .initializeEnumeration(
                            false,
                            true,
                            false,
                            false,
                            null,
                            Registry.nodeType[Registry.PRECONDITIONTASK_OBJECTCONSTRAINT],
                            0, false);
            ConnectedNode cn = actionObjectConstraint.connections.getNext();

            while (cn != null)
            {
                pTs.add(cn);
                cn = actionObjectConstraint.connections.getNext();
            }
            Iterator it = pTs.iterator();

            while (it.hasNext()) {
                cn = (ConnectedNode) it.next();

                PreconditionTaskObjectConstraint preconditionTaskObjectConstraint = (PreconditionTaskObjectConstraint) cn.node;

                if (preconditionTaskObjectConstraint != null)
                {
                    Variable tempVar = preconditionTaskObjectConstraint.connections
                            .getSpecificVariable(Registry.TEMPORALREFERENCE_EDGE);
                    actionCoordinatorControl.getGlobalSearchControl()
                            .setVariable(tempVar, tempVar.value() + diffTime);

                }


            }

        }

    }

    /** Moves all ActionTasks.<p>

     @param diffTime The time difference to move.
     */
    private void moveAllActionTasks(long diffTime)
    {
        List aTs = new LinkedList();

        synchronized (actionObjectConstraint.connections)
        {
            actionObjectConstraint.connections.initializeEnumeration(false,
                    true, false, false, null,
                    Registry.nodeType[Registry.ACTIONTASK_OBJECTCONSTRAINT], 0,
                    false);
            ConnectedNode cn = actionObjectConstraint.connections.getNext();

            while (cn != null)
            {
                aTs.add(cn);
                cn = actionObjectConstraint.connections.getNext();
            }
            Iterator aIt = aTs.iterator();

            while (aIt.hasNext()) {
                cn = (ConnectedNode) aIt.next();
                ActionTaskObjectConstraint actionTaskObjectConstraint = (ActionTaskObjectConstraint) cn.node;

                if (actionTaskObjectConstraint != null)
                {
                    Variable beginTime = actionTaskObjectConstraint.connections
                            .getSpecificVariable(Registry.BEGIN_EDGE);
                    actionCoordinatorControl.getGlobalSearchControl()
                            .setVariable(beginTime,
                                    beginTime.value() + diffTime);


                }


            }

        }
    };

    /** Moves all StateTasks.<p>

     @param diffTime The time difference to move.
     */
    private void moveAllStateTasks(long diffTime)
    {
        synchronized (actionObjectConstraint.connections)
        {
            List sTs = new LinkedList();
            actionObjectConstraint.connections.initializeEnumeration(false,
                    true, false, false, null,
                    Registry.nodeType[Registry.STATETASK_OBJECTCONSTRAINT], 0,
                    false);
            ConnectedNode cn = actionObjectConstraint.connections.getNext();

            while (cn != null)
            {
                sTs.add(cn);
                cn = actionObjectConstraint.connections.getNext();
            }
            Iterator sIt = sTs.iterator();
            while (sIt.hasNext()) {
                cn = (ConnectedNode) sIt.next();

                StateTaskObjectConstraint stateTaskObjectConstraint = (StateTaskObjectConstraint) cn.node;

                if (stateTaskObjectConstraint != null)
                {
                    Variable tempVar = stateTaskObjectConstraint.connections
                            .getSpecificVariable(Registry.TEMPORALREFERENCE_EDGE);
                    actionCoordinatorControl.getGlobalSearchControl()
                            .setVariable(tempVar, tempVar.value() + diffTime);
                }


            }

        }
    }

    /** Converts the x coordinate of a pixel to a absolute tick-value.<p>

     @param x The x-coordinate.
     @return The tick.
     */
    private long convertCoordinatesToTime(int x)
    {

        int width = actionCoordinatorControl.getViewWidth();
        double pixelPerTimeUnit = (double) actionCoordinatorControl
                .getViewWidth()
                / (double) actionCoordinatorControl.getMainViewModelExtent();
        return (long) (actionCoordinatorControl.getMainViewModelValue() + ((x - RETRACT_BUTTON_WIDTH) / pixelPerTimeUnit));

    }

    /**  This function is called, if an ActionTask was added to this Action.<p>

     @param actionTaskObjectConstraint The new ActionTask.
     */
    private void addActionTaskObjectConstraint(
            ActionTaskObjectConstraint actionTaskObjectConstraint)
    {

        //adding ChangeListener to the new Task
        actionTaskObjectConstraint.addChangeListener(this);
        // for new Objects connected

        // observe the beginTime variable
        Variable beginTime = actionTaskObjectConstraint.connections
                .getSpecificVariable(Registry.BEGIN_EDGE);
        if (beginTime != null)
        {
            beginTime.addChangeListener(this);
        }

        // observe the duration variable
        Variable duration = actionTaskObjectConstraint.connections
                .getSpecificVariable(Registry.DURATION_EDGE);
        if (duration != null)
        {
            duration.addChangeListener(this);
        }

        // recompute the new boundary times.
        computeBoundaryTimes();
    }

    /** Updates a dis/connected PreconditionTask.<p>

     @param preconditionTaskObjectConstraint The PreconditionTask to check.
     */
    private void updatePreconditionTaskObjectConstraint(
            PreconditionTaskObjectConstraint preconditionTaskObjectConstraint)
    {

        // check if we know this Task already. If not, it must be new
        if (!knownPreconditionStateResourceMap
                .containsKey(preconditionTaskObjectConstraint))
        {
            // new PreconditionTaskObjectConstraint
            synchronized (preconditionTaskObjectConstraint.connections)
            {
                // observe it
                preconditionTaskObjectConstraint.connections
                        .addChangeListener(this);
            }
        }

        // get the connected StateResource
        StateResourceConstraint src = null;
        synchronized (preconditionTaskObjectConstraint.connections)
        {

            src = (StateResourceConstraint) preconditionTaskObjectConstraint.connections
                    .getSpecificConstraint(Registry.STATERESOURCE_CONSTRAINT,
                            Registry.PLAIN_EDGE);

        }
        StateResourceConstraint knownSrc = (StateResourceConstraint) knownPreconditionStateResourceMap
                .get(preconditionTaskObjectConstraint);


        if (knownSrc != src || knownSrc == null)
        {
            // the knownSrc is not the same, as the new Src. The Preconditon has relinked.

            if (knownSrc != null)
            {
                // there exists a knownSrc, so the preconditionTask has changed his connection
                // to another stateResourceConstraint

                // remove the changelistener from the old stateResourceConstraint
                knownSrc.removeChangeListener(this);
            }

            // update the preconditionTask constraint
            knownPreconditionStateResourceMap.put(
                    preconditionTaskObjectConstraint, src);

            //now the new src is the knownSrc
            knownSrc = src;

            if (knownSrc != null)
            {

                // there exists a connected Src. Add changelistener
                knownSrc.addChangeListener(this);
            }

        } // else connection did not change => do nothing

        if (knownSrc != null)
        {

            // get state of the precondition (if it is fullfilled or not)
            int preconditionState = knownSrc
                    .checkPrecondition(preconditionTaskObjectConstraint);

            if (preconditionState == -1)
            {
                // the stateResourceConstraint does not know the PreconditionConstraint
                throw new Error(
                        "StateResourceConstraint does not know the corresponding PreconditionTaskObjectConstraint!");

            }
            preconditionFullfilledMap.put(preconditionTaskObjectConstraint,
                    new Integer(preconditionState));
        }
    }

    /**  This function is called, if something (the state, the resourceType)
     of a stateResourceConstraint has changed. It then updates the
     preconditionFullfilledMap.
     @param stateResourceConstraint The StateResourceConstraint to check.
     */
    private void updateStateResouceConstraint(
            StateResourceConstraint stateResourceConstraint)
    {

        synchronized (stateResourceConstraint.connections)
        {
            //get corresponding preconditions

            stateResourceConstraint.connections
                    .initializeEnumeration(
                            false,
                            true,
                            false,
                            false,
                            null,
                            Registry.nodeType[Registry.PRECONDITIONTASK_OBJECTCONSTRAINT],
                            Registry.PLAIN_EDGE, false);
            ConnectedNode cn = stateResourceConstraint.connections.getNext();

            while (cn != null)
            {
                // check EVERY connected precondition, if it is fullfilled
                PreconditionTaskObjectConstraint preconditionTaskObjectConstraint = (PreconditionTaskObjectConstraint) cn.node;

                if (preconditionTaskObjectConstraint != null)
                {
                    // there exists a preconditionTask if
                    int preconditionState = stateResourceConstraint
                            .checkPrecondition(preconditionTaskObjectConstraint);

                    // check if this precondition is known by this ActionObjectConstraint
                    if (knownPreconditionStateResourceMap
                            .containsKey(preconditionTaskObjectConstraint))
                    {
                        //it is know, so update the state in the preconditionFullfilledMap
                        preconditionFullfilledMap.put(
                                preconditionTaskObjectConstraint, new Integer(
                                        preconditionState));
                    }

                } // end if
                cn = stateResourceConstraint.connections.getNext();

            } // end while
        } // end synchronized
    }

    /** This function checks if the action type has changed.<p>

     */
    private void checkActionType()
    {
        // Reading the Action Type

        Variable actionTypeVariable = null;
        synchronized (actionObjectConstraint.connections)
        {

            actionTypeVariable = (Variable) actionObjectConstraint.connections
                    .getSpecificVariable(Registry.ACTIONTYPE_EDGE);
        }

        if (actionTypeVariable != null)
        {

            if (actionTypeVariable.getName() == "Action Type")
            {

                // register changelister to get informend, when action type changes.
                actionTypeVariable.addChangeListener(this);

                // read action type variable
                actionType = (int) actionTypeVariable.value();
                actionDescription = TaskConstraint
                        .getNameForActionType(actionType);

                if (actionDescription != null)
                {

                    descriptionJLabel.setText(actionDescription);
                    this.setToolTipText(actionDescription);

                }

            }
        }

    }

    /** Connects all visible tasks with a swimlane to this action. */
    public void requestSwimlane()
    {
        List connectedList = new LinkedList();
        Set preconditions = knownPreconditionStateResourceMap.keySet();

        connectedList.add(actionObjectConstraint);
        connectedList.addAll(preconditions);
        connectedList.addAll(getTasks(Registry.STATETASK_OBJECTCONSTRAINT));
        connectedList.addAll(getTasks(Registry.ACTIONTASK_OBJECTCONSTRAINT));

        swimlaneControl.requestSwimlane(connectionHeight, this, connectedList);

    }

    /** Gets all connected tasks of this type.<p>

     @param nodeType The type of the tasks
     @return A List with all tasks of this type.
     */
    private List getTasks(int nodeType)
    {
        List taskList = new LinkedList();
        synchronized (actionObjectConstraint.connections)
        {
            actionObjectConstraint.connections.initializeEnumeration(false,
                    true, false, false, null, Registry.nodeType[nodeType], 0,
                    false);
            ConnectedNode cn = actionObjectConstraint.connections.getNext();

            while (cn != null)
            {

                taskList.add(cn.node);

                cn = actionObjectConstraint.connections.getNext();
            }

        }
        return taskList;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getConnectorX(int)
     */
    public int getConnectorX(int connectionType)
    {

        return this.getX();
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getConnectorY(int)
     */
    public int getConnectorY(int connectionType)
    {

        return absoluteY(connectionHeight);
    }


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.View#absoluteY(int)
     */
    private int absoluteY(int relativeY)
    {
        return actionCoordinatorControl.absoluteY(this.getY()) + relativeY;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#isVisible(int)
     */
    public boolean isVisible(int connectionType)
    {
        return swimlaneVisibility && !retractableView.isRetracted();
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getPosition(int)
     */
    public int getPosition(int connectionType)
    {

        return SwimlaneConnector.WEST;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#isOutgoing(int)
     */
    public boolean isOutgoing(int connectionType)
    {

        return true;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#terminate()
     */
    public void terminate()
    {
        swimlaneControl.removeConnector(actionObjectConstraint);
        swimlaneControl.removeSwimlane(this);
        swimlaneVisibility = false;

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#setSwimlaneVisibility(boolean)
     */
    public void setSwimlaneVisibility(boolean _swimlaneVisibilty)
    {
        swimlaneVisibility = _swimlaneVisibilty;

    }

    /** Removes the swimlane. */
    public void removeSwimlane()
    {
        swimlaneControl.removeSwimlane(this);

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#shouldVisible()
     */
    public boolean shouldVisible()
    {

        return shouldVisible;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#setShouldVisible(boolean)
     */
    public void setShouldVisible(boolean b)
    {
        shouldVisible = b;

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#setPriorityOffset(int)
     */
    public void setPriorityOffset(long offset)
    {
        // this is not a view. Nothing to do

    }

    /** Sets a new connectionHeight.
     @param _connectionHeight the new connectionHeight.
     */
    public void setConnectionHeight(int _connectionHeight)
    {
        connectionHeight = _connectionHeight;


    }

} // end ActionSemanticGraphic
