package com.ai_center.planpresentation.plugins.actionconstraint;


import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.ConventionalConstraint;
import com.ai_center.dragonbreath.constraintgraph.Embedding;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.DiscreteResourceConstraint;
import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.TaskConstraint;
import com.ai_center.dragonbreath.constraints.TransitionConstraint;
import com.ai_center.dragonbreath.constraints.taskconstraint.TCActionTask;
import com.ai_center.dragonbreath.constraints.taskconstraint.TCActionType;
import com.ai_center.dragonbreath.constraints.taskconstraint.TCPreconditionTask;
import com.ai_center.dragonbreath.constraints.taskconstraint.TCStateTask;
import com.ai_center.planpresentation.utilities.SpringUtilities;




/** The AddActionDialog is the dialog window, which appears, when clicking
 on Add->Action in the menu. <p>
 
 @author Daniel Weiss
 @version $Revision: 1.1 $
 */

public class AddActionDialog extends JDialog
        implements
            ItemListener,
            ActionListener
{


    /** The newly created constraint. */
    private ConventionalConstraint actionConstraint = null;

    /** The link to the ActionCoordinatorControlPlugin. */
    private ActionCoordinatorControlPlugin actionCoordinatorControlPlugin;

    /** The link to the global search control.  */
    private GlobalSearchControl gsc;

    /** The begin time, where the action should be inserted. */
    private long beginCT;

    /** The Mapping from index to actionType. */
    private HashMap indexActionTypeMap = new HashMap();

    /** The list with the possible actions. */
    JComboBox actionList = null;

    /** The main panel which contains  the action list. */
    JPanel mainPanel = new JPanel(new SpringLayout());

    /** The panel with the buttons. */
    JPanel buttonPanel = new JPanel();

    /** The cancel button. */
    JButton cancelButton = null;

    /** The okbutton */
    JButton okButton = null;


    /** Creates an AddActionDialog, but doens't open it.
     @param frame The parent frame.
     @param title The title of the dialog window.
     @param modal If it should be modal or not.
     @param _accp The link to the ActionCoordinatorControlPlugin.
     @param _gsc The link to the GlobalSearchControl.
     @param _beginCT The beginning time, where the action should be inserted.
     */
    public AddActionDialog(Frame frame, String title, boolean modal,
            ActionCoordinatorControlPlugin _accp, GlobalSearchControl _gsc,
            long _beginCT)
    {
        super(frame, title, modal);

        actionCoordinatorControlPlugin = _accp;
        gsc = _gsc;
        beginCT = _beginCT;

        this.setLocation(200, 200);
        try
        {

            JLabel selectActionLabel = new JLabel("Insert Action:",
                    JLabel.TRAILING);
            selectActionLabel.setLabelFor(actionList);
            mainPanel.add(selectActionLabel);
            actionList = new JComboBox(getActions());
            mainPanel.add(actionList);
            JLabel atTimeLabel = new JLabel("at Time:");

            JLabel currentTimeLabel = new JLabel("" + beginCT);
            atTimeLabel.setLabelFor(currentTimeLabel);
            mainPanel.add(atTimeLabel);
            mainPanel.add(currentTimeLabel);
            okButton = new JButton("OK");

            okButton.addActionListener(this);

            cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(this);
            mainPanel.add(okButton);
            mainPanel.add(cancelButton);

            SpringUtilities.makeCompactGrid(mainPanel, 3, 2, //rows, cols
                    6, 6, //initX, initY
                    6, 6); //xPad, yPad


            //	breakpointPanel.setLayout(new GridLayout(0, 1, 5, 20));

            //	   jScrollPane1.getViewport().add(jList1, null);
            this.getContentPane().add(mainPanel, BorderLayout.NORTH);
            pack();
        } catch (Exception e)
        {
            e.printStackTrace();
        }
    }


    /** Inserts a action at the given beginning time..<p>
     
     @param actionTypeID The ActionType to insert.
     @param beginCT The time, when the action should start.
     */
    private void addAction(int actionTypeID, long beginCT)
    {
        TCActionType tcActionType = TaskConstraint
                .getTCActionType(actionTypeID);
        ConventionalConstraint currentTimeConstraint = gsc
                .findConstraint(Registry.CURRENTTIME_OBJECTCONSTRAINT);

        // Creating the action object

        Variable v1 = gsc.createVariable_internal("Action Type");
        gsc.setVariable_internal(v1, actionTypeID);

        Embedding e = new Embedding(1);
        gsc.extendEmbedding_internal(e, v1, Registry.ACTIONTYPE_EDGE,
                Registry.INCOMING);
        actionConstraint = gsc.createConstraint_internal(
                Registry.ACTION_OBJECTCONSTRAINT, e);

        // Creating the precondition tasks 

        Iterator it = tcActionType.preconditionTasks.iterator();
        while (it.hasNext())
        {

            TCPreconditionTask tcPreconditionTask = (TCPreconditionTask) it
                    .next();

            addPreconditionTask(tcPreconditionTask);

        }

        // Creating the action tasks 

        it = tcActionType.actionTasks.iterator();
        while (it.hasNext())
        {

            TCActionTask tcActionTask = (TCActionTask) it.next();
            addActionTask(tcActionTask);
        }

        // Creating the state tasks 

        it = tcActionType.stateTasks.iterator();
        while (it.hasNext())
        {

            TCStateTask tcStateTask = (TCStateTask) it.next();
            addStateTask(tcStateTask);
        }


        // Creating the task constraint

        e = new Embedding(2);
        gsc.extendEmbedding_internal(e, currentTimeConstraint,
                Registry.PLAIN_EDGE, Registry.INCOMING);
        gsc.extendEmbedding_internal(e, actionConstraint, Registry.PLAIN_EDGE,
                Registry.INCOMING);

        // TO BE IMPROVED: Cost function selection should be decided
        // by the user!

        gsc.selectCostFunction_internal(gsc.getStructuralCostFunction());
        ConventionalConstraint taskConstraint = gsc.createConstraint_internal(
                Registry.TASK_CONSTRAINT, e);
        gsc.selectCostFunction_internal(gsc.getDefaultCostFunction());
        gsc.addCosts(taskConstraint, TaskConstraint.TEMPORAL_INCONSISTENCY);

    }

    /** Adds a action task to the constraint.
     @param actionTask  The task blueprint for the task.
     */
    private void addActionTask(TCActionTask actionTask)
    {

        // Adding and initializing new variables

        Variable operationVariable = gsc.createVariable_internal("operation");
        gsc.setVariable_internal(operationVariable, actionTask.operation);
        Variable beginVariable = gsc.createVariable_internal("begin");
        gsc.setVariable_internal(beginVariable, beginCT);
        Variable durationVariable = gsc.createVariable_internal("duration");
        gsc.setVariable_internal(durationVariable, actionTask.duration);
        Variable consumptionVariable = gsc
                .createVariable_internal("consumption");
        gsc.setVariable_internal(consumptionVariable, actionTask.consumption);
        Variable executionStateVariable = gsc
                .createVariable_internal("execution state");
        gsc.setVariable_internal(executionStateVariable,
                DiscreteResourceConstraint.PENDING);

        // Fetching/establishing resource type variable of corresponding
        // discrete resource constraint

        Variable resourceTypeVariable;
        Embedding e;

        ConventionalConstraint drc = gsc.findConstraint(
                Registry.DISCRETERESOURCE_CONSTRAINT,
                Registry.RESOURCETYPE_EDGE, actionTask.resourceType);

        if (drc == null)
        {

            // Creating a new DiscreteResourceConstraint and resource
            // type variable

            resourceTypeVariable = gsc.createVariable_internal("resource type");
            gsc.setVariable_internal(resourceTypeVariable,
                    actionTask.resourceType);
            Variable capacityVariable = gsc.createVariable_internal("capacity");
            gsc.setVariable_internal(capacityVariable, 1);
            ConventionalConstraint currentTimeObjectConstraint;
            synchronized (gsc.constraintSHT)
            {
                currentTimeObjectConstraint = (ConventionalConstraint) gsc.constraintSHT
                        .get(Registry.nodeType[Registry.CURRENTTIME_OBJECTCONSTRAINT]);
            }

            e = new Embedding(3);
            gsc.extendEmbedding_internal(e, resourceTypeVariable,
                    Registry.RESOURCETYPE_EDGE, Registry.INCOMING);
            gsc.extendEmbedding_internal(e, capacityVariable,
                    Registry.CAPACITY_EDGE, Registry.INCOMING);
            gsc.extendEmbedding_internal(e, currentTimeObjectConstraint,
                    Registry.PLAIN_EDGE, Registry.INCOMING);
            drc = gsc.createConstraint_internal(
                    Registry.DISCRETERESOURCE_CONSTRAINT, e);

        } else
        {

            // Fetching discrete resource constraint's resource type variable

            synchronized (drc.connections)
            {
                resourceTypeVariable = drc.connections
                        .getSpecificVariable(Registry.RESOURCETYPE_EDGE);
            }

        }

        // Creating the action task

        e = new Embedding(6);
        gsc.extendEmbedding_internal(e, operationVariable,
                Registry.OPERATION_EDGE, Registry.INCOMING);
        gsc.extendEmbedding_internal(e, beginVariable, Registry.BEGIN_EDGE,
                Registry.INCOMING);
        gsc.extendEmbedding_internal(e, durationVariable,
                Registry.DURATION_EDGE, Registry.INCOMING);
        gsc.extendEmbedding_internal(e, consumptionVariable,
                Registry.CONSUMPTION_EDGE, Registry.INCOMING);
        gsc.extendEmbedding_internal(e, resourceTypeVariable,
                Registry.RESOURCETYPE_EDGE, Registry.INCOMING);
        gsc.extendEmbedding_internal(e, executionStateVariable,
                Registry.EXECUTIONSTATE_EDGE, Registry.INCOMING);

        ConventionalConstraint actionTaskConstraint = gsc
                .createConstraint_internal(
                        Registry.ACTIONTASK_OBJECTCONSTRAINT, e);

        // Connecting the action task

        e = new Embedding(1);
        gsc.extendEmbedding_internal(e, actionTaskConstraint,
                Registry.PLAIN_EDGE, Registry.INCOMING);
        gsc.extendConstraint_internal(actionConstraint, e);
        gsc.extendConstraint_internal(drc, e);

    }

    /** Adds a precondition task to the constraint.
     @param tcPreconditionTask  The task blueprint for the task.
     */
    private void addPreconditionTask(TCPreconditionTask tcPreconditionTask)
    {
        Variable v1 = gsc.createVariable_internal("Temporal Reference");
        gsc.setVariable_internal(v1, beginCT);
        Variable v2 = gsc.createVariable_internal("State");
        gsc.setVariable_internal(v2, tcPreconditionTask.state);
        ConventionalConstraint src = gsc.findConstraint(
                Registry.STATERESOURCE_CONSTRAINT, Registry.RESOURCETYPE_EDGE,
                tcPreconditionTask.resourceType);
        Variable v3;
        synchronized (src.connections)
        {
            v3 = src.connections
                    .getSpecificVariable(Registry.RESOURCETYPE_EDGE);
        }

        Embedding e = new Embedding(3);
        gsc.extendEmbedding_internal(e, v1, Registry.TEMPORALREFERENCE_EDGE,
                Registry.INCOMING);
        gsc.extendEmbedding_internal(e, v2, Registry.STATE_EDGE,
                Registry.INCOMING);
        gsc.extendEmbedding_internal(e, v3, Registry.RESOURCETYPE_EDGE,
                Registry.INCOMING);
        ConventionalConstraint task = gsc.createConstraint_internal(
                Registry.PRECONDITIONTASK_OBJECTCONSTRAINT, e);

        //			Connecting the task to the action object

        Embedding eTask = new Embedding(1);
        gsc.extendEmbedding_internal(eTask, task, Registry.PLAIN_EDGE,
                Registry.INCOMING);
        gsc.extendConstraint_internal(actionConstraint, eTask);

        // Connecting the task to the state resource constraint

        gsc.extendConstraint_internal(src, eTask);

    }

    /** Adds a state task to the constraint.
     @param stateTask  The task blueprint for the task.
     */
    private void addStateTask(TCStateTask stateTask)
    {

        // Adding and initializing new variables

        Variable temporalReferenceVariable = gsc
                .createVariable_internal("temporal reference");
        gsc.setVariable_internal(temporalReferenceVariable, beginCT);
        Variable contributionVariable = gsc
                .createVariable_internal("contribution");
        gsc.setVariable_internal(contributionVariable, stateTask.contribution);

        // Fetching/establishing resource type variable of corresponding
        // state resource constraint

        Variable resourceTypeVariable;
        Embedding e;

        ConventionalConstraint src = gsc.findConstraint(
                Registry.STATERESOURCE_CONSTRAINT, Registry.RESOURCETYPE_EDGE,
                stateTask.resourceType);

        if (src == null)
        {

            // Creating a new StateResourceConstraint and resource
            // type variable

            resourceTypeVariable = gsc.createVariable_internal("resource type");
            gsc.setVariable_internal(resourceTypeVariable,
                    stateTask.resourceType);
            Variable stateVariable = gsc.createVariable_internal("state");
            ConventionalConstraint currentTimeObjectConstraint;
            synchronized (gsc.constraintSHT)
            {
                currentTimeObjectConstraint = (ConventionalConstraint) gsc.constraintSHT
                        .get(Registry.nodeType[Registry.CURRENTTIME_OBJECTCONSTRAINT]);
            }

            e = new Embedding(3);
            gsc.extendEmbedding_internal(e, resourceTypeVariable,
                    Registry.RESOURCETYPE_EDGE, Registry.INCOMING);
            gsc.extendEmbedding_internal(e, stateVariable, Registry.STATE_EDGE,
                    Registry.INCOMING);
            gsc.extendEmbedding_internal(e, currentTimeObjectConstraint,
                    Registry.PLAIN_EDGE, Registry.INCOMING);

            gsc.selectCostFunction(gsc.getStructuralCostFunction());
            src = gsc.createConstraint_internal(
                    Registry.STATERESOURCE_CONSTRAINT, e);

            gsc.selectCostFunction(gsc.getDefaultCostFunction());
            gsc.addCosts(src, StateResourceConstraint.PRECONDITION_FAILURE);

        } else
        {

            // Fetching state resource constraint's resource type variable

            synchronized (src.connections)
            {
                resourceTypeVariable = src.connections
                        .getSpecificVariable(Registry.RESOURCETYPE_EDGE);
            }

        }

        // Creating the state task

        e = new Embedding(3);
        gsc.extendEmbedding_internal(e, temporalReferenceVariable,
                Registry.TEMPORALREFERENCE_EDGE, Registry.INCOMING);
        gsc.extendEmbedding_internal(e, contributionVariable,
                Registry.CONTRIBUTION_EDGE, Registry.INCOMING);
        gsc.extendEmbedding_internal(e, resourceTypeVariable,
                Registry.RESOURCETYPE_EDGE, Registry.INCOMING);
        ConventionalConstraint stateTaskConstraint = gsc
                .createConstraint_internal(Registry.STATETASK_OBJECTCONSTRAINT,
                        e);

        // Connecting the state task

        e = new Embedding(1);
        gsc.extendEmbedding_internal(e, stateTaskConstraint,
                Registry.PLAIN_EDGE, Registry.INCOMING);
        gsc.extendConstraint_internal(actionConstraint, e);
        gsc.extendConstraint_internal(src, e);

    }

    /** Returns a list of all Action Names. one line description.<p>
     
     @return A list with all action names.
     */
    private String[] getActions()
    {
        List actionList = new LinkedList();

        // get all actions without transitions
        for (int i = 0; i < TaskConstraint.getCurrentActionTypeNumber(); i++)
        {
            if (!TransitionConstraint.isTransition(i))
            {
                String actionName = TaskConstraint.getNameForActionType(i);
                if (actionName == null)
                {
                    actionName = "Action Type " + i;
                }

                actionList.add(actionName);
                indexActionTypeMap.put(new Integer(actionList.size()),
                        new Integer(i));
            }
        }
        Iterator it = actionList.iterator();
        String[] actionArray = new String[actionList.size()];
        for (int j = 0; j < actionList.size(); j++)
        {
            actionArray[j] = (String) it.next();
        }

        return actionArray;

    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();
        if (source instanceof JButton)
        {

            if (source == okButton)
            {
                javax.swing.SwingUtilities.invokeLater(new Runnable()
                {


                    public void run()
                    {
                        Integer actionTypeInteger = (Integer) indexActionTypeMap
                                .get(new Integer(
                                        actionList.getSelectedIndex() + 1));
                        if (actionTypeInteger != null)
                        {

                            addAction(actionTypeInteger.intValue(), beginCT);
                        }
                    }
                });

            }
            //else if (source == cancelButton) {}
            this.setVisible(false);
        }
    }

    /* (non-Javadoc)
     * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
     */
    public void itemStateChanged(ItemEvent e)
    {
        Object source = e.getSource();

        //Object source = e.getItemSelectable();
        if (source instanceof JCheckBox)
        {
        }

    }


}