package com.ai_center.planpresentation.plugins.actionresourceconstraint;


import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BoundedRangeModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.constraints.DiscreteResourceConstraint;
import com.ai_center.dragonbreath.constraints.discreteresourceconstraint.DRCTask;
import com.ai_center.dragonbreath.constraints.discreteresourceconstraint.OrderedCapacityCostsTaskList;
import com.ai_center.dragonbreath.utilities.OrderedShiftDLLElement;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.plugins.Controller;
import com.ai_center.planpresentation.plugins.Coordinator;
import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.utilities.DefaultResourceController;
import com.ai_center.planpresentation.utilities.MultiLevelController;
import com.ai_center.planpresentation.utilities.MultiLevelModel;
import com.ai_center.planpresentation.utilities.MultiLevelView;
import com.ai_center.planpresentation.utilities.PanelCache;
import com.ai_center.planpresentation.utilities.ResourceController;




/** The ActionResourceController represents one DiscreteResourceConstraint. <p>
 
 It is one of three parts of a bottom-level agent of the presentation-
 abstraction-controll pattern. <p>
 
 The controller is connected to the coordinator (ActionResourceCoordinatorPlugin). 
 
 @author Daniel Weiss
 @version $Revision: 1.3 $
 */
public class ActionResourceController extends DefaultResourceController
        implements
            Comparable,
            Controller,
            ResourceController,
            MultiLevelController,
            ChangeListener,
            PlanPresentationConstants
{


    /**The corresponding DiscreteResourceConstraint/ActionResourceConstriant.*/
    private DiscreteResourceConstraint discreteResourceConstraint = null;

    /** The time of the last update in milliseconds. */
    private long lastUpdate;

    /** The gui model which arranges the tasks. */
    private MultiLevelModel multiLevelModel = null;

    /** The view which shows the tasks. */
    private MultiLevelView multiLevelView;

    /** A cache for gui elements. */
    private PanelCache taskCache = new PanelCache();

    public ActionResourceController(Coordinator _c,
            DiscreteResourceConstraint _drc)
    {
        super(_c);
        discreteResourceConstraint = _drc;

        coordinator = _c;


    }

    /** The updateDatamodel is called by the stateChange-function, whenever
     something has changed in the model.<p>
     
     */
    private void updateDatamodel()
    {
        // check if a update is really necessary or if
        // another update was short before
        //  allow max. 10 updates per second
        accessTime = System.currentTimeMillis();

        if ((System.currentTimeMillis() - lastUpdate > 100))
        {
            lastUpdate = System.currentTimeMillis();
            // get the task list from the datamodel
            OrderedCapacityCostsTaskList taskList = (OrderedCapacityCostsTaskList) getDatamodel();


            // add new tasks and update the other
            OrderedShiftDLLElement dllElement = taskList.first;
            while (dllElement != null)
            {
                DRCTask drcTask = (DRCTask) dllElement.element;

                ActionTaskSemanticGraphic tsg = (ActionTaskSemanticGraphic) taskCache
                        .getObject(drcTask.taskConstraint);


                // check, if this task is already known.
                if (tsg == null)
                {
                    // if not, create a new taskSematicGraphic
                    tsg = new ActionTaskSemanticGraphic(coordinator
                            .getSwimlaneControl(), drcTask, multiLevelView,
                            this);
                    if (drcTask.beginVar != null) {
                        drcTask.beginVar.addChangeListener(this);
                    }
                    taskCache.addObject(drcTask.taskConstraint, tsg);
                    multiLevelModel.addVisibleObject(tsg);
                }

            /* more cpu power is used without this if-condition, but with it, drag&drop isn't working
              correctly. (update comes to late)
                
                if ((tsg.getBeginCT() != drcTask.begin)
                        || (tsg.getDuration() != drcTask.duration))
                {*/
                    // object has moved, update it and the model
                    tsg.setBeginCT(drcTask.begin);
                    tsg.setDuration(drcTask.duration);
                    multiLevelModel.updateVisibleObject(tsg);
                    lastUpdate = -1; // something has changed repeat the update
            /*    }*/
                if (drcTask.operationVar != null) {
                    tsg.setOperation(discreteResourceConstraint
                            .getDescription(drcTask.operationVar.integerValue));
                }
                
             
                dllElement = dllElement.next;
            }


            // remove deleted tasks
            Collection deletedTasks = taskCache.getRemainingPanels();
            Iterator it = deletedTasks.iterator();
            while (it.hasNext())
            {
                ActionTaskSemanticGraphic tsg = (ActionTaskSemanticGraphic) it
                        .next();
                tsg.terminate();
                multiLevelModel.removeVisibleObject(tsg);
            }


         
        }

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceController#expandView()
     */
    protected void expandView()
    {
        multiLevelModel = new MultiLevelModel((Controller) this);
        multiLevelView = new MultiLevelView(this, getName());
        getSwimlaneControl().registerConnector(discreteResourceConstraint,
                multiLevelView);
        coordinator.addView((RetractableView) multiLevelView);
        coordinator.addViewChangeListener((ChangeListener) multiLevelView);
        discreteResourceConstraint.addChangeListener(this);
        discreteResourceConstraint.getIntervalList().addChangeListener(this);
        discreteResourceConstraint.fireIntervalListUpdate();


    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceController#removeView()
     */
    protected void removeView()
    {
        discreteResourceConstraint.getIntervalList().removeChangeListener(this);
        coordinator.removeViewChangeListener((ChangeListener) multiLevelView);
        coordinator.removeView((RetractableView) multiLevelView);
        getSwimlaneControl().removeConnector(discreteResourceConstraint);

        multiLevelModel = null;
        multiLevelView = null;

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#getAccessTime()
     */
    public long getAccessTime()
    {
        if (multiLevelModel != null)
        {
            return accessTime;
        } else
        {
            return 0;
        }

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.ResourceController#getDatamodel()
     */
    public Object getDatamodel()
    {
        return discreteResourceConstraint.getTaskList();

    }

    /** Returns a link to the GlobalSearchControl.<p> 	
     @return The link to the GlobalSearchControl.
     */
    public GlobalSearchControl getGlobalSearchControl()
    {
        return coordinator.getGlobalSearchControl();

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.MultiLevelController#getLevel()
     */
    public int getLevel()
    {
        if (multiLevelModel != null)
        {
            return multiLevelModel.getLevel();
        } else
        {
            return 0;
        }
    }

    /* (non-Javadoc)
     @see com.ai_center.planpresentation.utilities.MultiLevelController#getLevelcount()
     */
    public int getLevelcount()
    {

        return multiLevelModel.getLevelcount();
    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.MultiLevelController#getObjectsOnLevel(int, long, long)
     */
    public Map getObjectsOnLevel(int level, long beginCT, long durationCT)
    {
        return multiLevelModel.getObjectsOnLevel(level, beginCT, durationCT);
    }

   
    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public synchronized void stateChanged(ChangeEvent e)
    {
        if (e.getSource() instanceof BoundedRangeModel)
        {
            multiLevelView.stateChanged(e);
        } else if (multiLevelModel != null)
        {
            // view is expanded, redirect stateChanged call
            updateDatamodel();
        }

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#updateViewSize(int)
     */
    public void updateViewSize(int level)
    {
        multiLevelView.updateViewSize(level);
        coordinator.updateView();

    }

}