package com.ai_center.planpresentation.plugins.actionresourceconstraint;


import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.Constraint;
import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraints.DiscreteResourceConstraint;
import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.plugins.Coordinator;
import com.ai_center.planpresentation.plugins.Plugin;
import com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl;
import com.ai_center.planpresentation.utilities.ResourceController;




/** The ActionResourceCoordinatorPlugin represents all ActionResourceConstraints. <p>
  
 For that purpose it installs a JList on the left side with all available action
 resources. The user can click on one and this opens a full view of this
 resource.<p>
 
  @author Daniel Weiss
  @version $Revision: 1.2 $
 */
public class ActionResourceCoordinatorPlugin
        extends
            DefaultResourceCoordinatorControl
        implements
            PlanPresentationConstants,
            Coordinator,
            Plugin,
            ChangeListener
{


    /** Adds a ActionResourceConstraint/DiscreteResourceConstraint to the JList.<p>
     
       @param actionResourceConstraint The ActionResourceConstraint to add.
     */
    private void addActionResourceConstraint(
            DiscreteResourceConstraint actionResourceConstraint)
    {

        ResourceController resourceController = new ActionResourceController(
                (Coordinator) this, actionResourceConstraint);
        defaultResourceModel.ensureCapacity(defaultResourceModel.getSize() + 1);
        resourceController.setName(actionResourceConstraint.getName());
        defaultResourceModel.addElement(resourceController);
        int indexOf = defaultResourceModel.indexOf(resourceController);
        resourceController.addChangeListener(defaultResourceModel);
        constraintIndexMap.put(actionResourceConstraint, new Integer(indexOf));
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl#addConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void addConstraint(Constraint constraint)
    {
        if (constraint instanceof DiscreteResourceConstraint)
        {

            addActionResourceConstraint((DiscreteResourceConstraint) constraint);

        }

    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#getGlobalSearchControl()
     */
    public GlobalSearchControl getGlobalSearchControl()
    {
        return planPresentationAPI.getGlobalSearchControl();
    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl#initPlugin(com.ai_center.planpresentation.PlanPresentationAPI)
     */
    public void initPlugin(PlanPresentationAPI _pc)
    {
        planPresentationAPI = _pc;

        // register constraint types	
        planPresentationAPI.registerConstraintType(
                Registry.DISCRETERESOURCE_CONSTRAINT, (Plugin) this);

        planPresentationAPI.addControl(
                initDefaultResourceList("Action Resources"), 15);

        planPresentationAPI.addChangeListener((ChangeListener) this);

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl#removeConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void removeConstraint(Constraint constraint)
    {
        // FIXME Constraint removal is not supported.

    }


}