package com.ai_center.planpresentation.plugins.actionresourceconstraint;


import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.JLabel;

import com.ai_center.dragonbreath.constraints.discreteresourceconstraint.DRCTask;
import com.ai_center.planpresentation.SwimlaneConnector;
import com.ai_center.planpresentation.SwimlaneControl;
import com.ai_center.planpresentation.plugins.View;
import com.ai_center.planpresentation.utilities.SemanticGraphic;
import com.ai_center.planpresentation.utilities.TaskSemanticGraphic;




/** The ActionTaskSemanticGraphic represents one ActionTask. <p>
 
 
 @author Daniel Weiss
 @version $Revision: 1.4 $
 */
public class ActionTaskSemanticGraphic extends TaskSemanticGraphic
        implements
            SwimlaneConnector,
            SemanticGraphic
{


    /** The link to the ActionResourceController. */
    private ActionResourceController actionResourceController;

    /** The begin time of this task. */
    private long beginCT = 0;

    /** The duration of this task. */
    private long duration = 0;

    /** A link to the task. */
    private DRCTask drcTask;

    /** A description label to show the name of the task. */
    private JLabel descriptionJLabel = new JLabel();

    /** The name of the task. */
    private String operationName;


    /** Creates a ActionTaskSemanticGraphic.<p>
     
     @param _swimlaneControl A link to the swimlanecontrol
     @param _drcTask The link to the task.
     @param _view The link to the view, where this task is contained.
     @param _arc A link to the ActionResourceController.
     */
    public ActionTaskSemanticGraphic(SwimlaneControl _swimlaneControl,
            DRCTask _drcTask, View _view, ActionResourceController _arc)
    {
        super(_swimlaneControl, _drcTask.taskConstraint, _view);

        this.add(descriptionJLabel);
        drcTask = _drcTask;
        actionResourceController = _arc;
        swimlaneControl.registerConnector(drcTask.taskConstraint, this);
        setBackground(ACTIONTASK_COLOR);
        setBorder(BorderFactory.createLineBorder(Color.black, 1));
        


    }


    /** Sets a new beginning time for this task.<p>
     
     @param newBeginCT The new beginning time.
     */
    private void setBeginCTDatamodel(long newBeginCT)
    {

        actionResourceController.getGlobalSearchControl().setVariable(
                drcTask.beginVar, newBeginCT);
        //        drcTask.begin = newBeginCT;
      //  beginCT = newBeginCT;

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#getBeginCT()
     */
    public long getBeginCT()
    {
        return beginCT;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#getDuration()
     */
    public long getDuration()
    {

        return duration;
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent evt)
    {
        super.mouseReleased(evt);

        long newBeginCT = convertCoordinatesToTime(this.getX());
        setBeginCTDatamodel(newBeginCT);

    }

    /*
     *  (non-Javadoc)
     * @see java.awt.Component#paint(java.awt.Graphics)
     */
    public void paint(Graphics g)
    {
        if (operationName != null)
        {
            int width = this.getWidth();
            descriptionJLabel.setText(prepareText(operationName, width));


        }
        super.paint(g);
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.TaskSemanticGraphic#requestSwimlane(int)
     */
    public void requestSwimlane(int connectionType)
    {
        // do nothing. ActionTask never  start a swimlane.

    }


    /** Sets a new operation name.<p>
     
     @param _operationName The new name of the task.
     */
    public void setOperation(String _operationName)
    {
        operationName = _operationName;
        this.setToolTipText(operationName);

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#setPriorityOffset(int)
     */
    public void setPriorityOffset(long offset)
    {
        // this is not a view. Nothing to do

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#setShouldVisible(boolean)
     */
    public void setShouldVisible(boolean _shouldVisible)
    {
        if (_shouldVisible == false)
        {
            this.terminate(); // remove ActionTask (only one-time usage)
        }

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#setSwimlaneVisibility(boolean)
     */
    public void setSwimlaneVisibility(boolean b)
    {
        // do nothing, swimlane visibilty never changes and is always true.

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#isVisibility()
     */
    public boolean shouldVisible()
    {

        return true;
    }

    /**
     @param beginCT The beginCT to set.
     */
    public void setBeginCT(long beginCT)
    {
        this.beginCT = beginCT;
    }
    /**
     @param duration The duration to set.
     */
    public void setDuration(long duration)
    {
        this.duration = duration;
    }
} // end ActionTaskSematicGraphic
