package com.ai_center.planpresentation.plugins.planplugin;


import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.*;

import javax.swing.*;

import com.ai_center.dragonbreath.RealTimeControl;




/** The Breakpoint dialog lets the user de/activate breakpoints. <p>
 
 It is accessible over View->Breakpoints.
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class BreakpointDialog extends JDialog
        implements
            ItemListener,
            ActionListener
{


    /** The panel with the breakpoint list inside. */
    JPanel breakpointPanel = new JPanel();

    /** The scrollpane which containts the breakpoint panel. */
    JScrollPane jScrollPane1 = new JScrollPane(breakpointPanel);

    /** This Map contains the new state of the breakpointIDs. */
    HashMap breakpointStateMap = new HashMap();

    /** The panel which contains the buttons. */
    JPanel buttonPanel = new JPanel();

    /** The panel which containts the breakpointpanel. */
    JPanel mainPanel = new JPanel();

    /** The cancel button. */
    JButton cancelButton = null;

    /** The ok button. */
    JButton okButton = null;

    /** This maps from the JCheckBox button to the breakpointIDs. */
    HashMap checkBoxBreakpointMap = new HashMap();

    /** The label for the help text. */
    JLabel jLabel1 = new JLabel();


    /** A link the RealTimeControl. */
    private RealTimeControl realTimeControl = null;

    public BreakpointDialog(Frame frame, String title, boolean modal,
            RealTimeControl _rtc)
    {
        super(frame, title, modal);
        realTimeControl = _rtc;

        this.setLocation(200, 200);
        try
        {
            mainPanel.setLayout(new BorderLayout());

            mainPanel.setMaximumSize(new Dimension(32767, 32767));
            mainPanel.setMinimumSize(new Dimension(400, 400));
            mainPanel.setPreferredSize(new Dimension(400, 400));

            jScrollPane1.setMinimumSize(new Dimension(400, 400));
            jScrollPane1.setPreferredSize(new Dimension(400, 400));

            jLabel1.setText("");
            mainPanel.add(jLabel1, BorderLayout.NORTH);

            mainPanel.add(jScrollPane1, BorderLayout.CENTER);
            mainPanel.add(buttonPanel, BorderLayout.SOUTH);

            okButton = new JButton("OK");
            okButton.addActionListener(this);
            buttonPanel.add(okButton);

            cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(this);
            buttonPanel.add(cancelButton);

            breakpointPanel.setLayout(new GridLayout(0, 1, 5, 20));
            showBreakpoints();

            this.getContentPane().add(mainPanel, BorderLayout.NORTH);
            pack();
        } catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**  Adds all Breakpoints to the breakpointPanel.  */
    private void showBreakpoints()
    {
        String[] breakpoints = realTimeControl.getBreakpoints();
        int i = 0;
        while (breakpoints[i] != null)
        {
            JCheckBox b = new JCheckBox(breakpoints[i]);

            breakpointPanel.add(b);
            if (realTimeControl.getBreakpointState(i) == true)
            {
                b.setSelected(true);
            }
            b.addItemListener(this);
            checkBoxBreakpointMap.put(b, new Integer(i));
            i++;
        }
    }

    /** Updates the breakpoints in the datamodel with the breakpoint-decisions
     of the user.
     */
    private void updateBreakpointStates()
    {
        Iterator it = breakpointStateMap.entrySet().iterator();
        while (it.hasNext())
        {
            Entry entry = (Entry) it.next();
            Integer i = (Integer) entry.getKey();
            Boolean state = (Boolean) entry.getValue();
            realTimeControl.setBeakpointActive(i.intValue(), state
                    .booleanValue());
        }

    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();
        if (source instanceof JButton)
        {
            if (source == okButton)
            {
                updateBreakpointStates();
            }
            //else if (source == cancelButton) {}
            this.setVisible(false);
        }
    }

    /* (non-Javadoc)
     * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
     */
    public void itemStateChanged(ItemEvent e)
    {
        Object source = e.getSource();
        if (source instanceof JCheckBox)
        {

            Integer i = (Integer) checkBoxBreakpointMap.get(source);
            if (i == null)
            {
                throw new Error(
                        "Bug in BreakpointDialog.java: unknown CheckBox sends signal.");
            }
            if (e.getStateChange() == ItemEvent.DESELECTED)
            {
                breakpointStateMap.put(i, new Boolean(false));
            } else if (e.getStateChange() == ItemEvent.SELECTED)
            {
                breakpointStateMap.put(i, new Boolean(true));
            }
        }
    }

}