package com.ai_center.planpresentation.plugins.planplugin;


import javax.swing.BoundedRangeModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.CurrentTimeObjectConstraint;




/** The CurrentTimeSemanticGraphic represents the CurrentTimeObjectConstraint.
   <p>
  
 For that purpose, it paints a dashed line and controls the time stepping.
 To paint the dashed line, it uses the SwimlaneControl.<p>
 
  @author Daniel Weiss
  @version $Revision: 1.2 $
 */
public class CurrentTimeSemanticGraphic implements ChangeListener
{


    /** The present time. */
    private long currentTime;

    /** A link to the CurrentTimeObjectConstraint. */
    CurrentTimeObjectConstraint currentTimeObjectConstraint = null;

    /** A link to the mainViewModel. */
    private BoundedRangeModel mainViewModel;

    /** A link to the plan plugin. */
    PlanPlugin pcc = null;

    /** Creates the representation of the CurrentTimeObjectConstraint.<p>
     
     @param _pcc A link to the PlanPlugin
     @param _currentTimeObjectConstraint The link to the 
     CurrentTimeObjectConstraint.
     */
    public CurrentTimeSemanticGraphic(PlanPlugin _pcc,
            CurrentTimeObjectConstraint _currentTimeObjectConstraint)
    {
        pcc = _pcc;
        currentTimeObjectConstraint = _currentTimeObjectConstraint;

        // docking currentTimeSemanticGraphic on ObjectConstraint
        // and observe it
        currentTimeObjectConstraint.addChangeListener(this);

    }

    /** Select the x-value of the vertical dashed line, which represents
     the current time.<p>
     
     */
    private void checkCTLine()
    {
        if (mainViewModel != null)
        {

            int value = mainViewModel.getValue();
            int extent = mainViewModel.getExtent();
            if (value < currentTime && ((value + extent) > currentTime))
            {
                double ctPercent = (double) (currentTime - value) / extent;
                // swimlanceControl only has to
                // multiply with "width"
                pcc.getSwimlaneControl().setCTPercent(ctPercent);
            } else
            {
                pcc.getSwimlaneControl().setCTPercent(0);
            }
        }
    }

    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e)
    {
        if (e.getSource() == currentTimeObjectConstraint)
        {

            Variable v = null;
            synchronized (currentTimeObjectConstraint.connections)
            {
                v = (Variable) currentTimeObjectConstraint.connections
                        .getSpecificVariable(Registry.PLAIN_EDGE);
            }

            if (v != null)
            {
                currentTime = (long) v.value();
                if (currentTime < 0)
                {
                    throw new Error("Invalid value " + currentTime
                            + " for current Time");
                }
                pcc.setCurrentTime(currentTime);
                checkCTLine();
            } else
            {
                throw new Error("The connected CurrentTimeObjectConstraint"
                        + currentTimeObjectConstraint + " has no variable");
            }

        } else if (e.getSource() instanceof BoundedRangeModel)
        {
            mainViewModel = (BoundedRangeModel) e.getSource();
            checkCTLine();

        }

    }

}