package com.ai_center.planpresentation.plugins.planplugin;


import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.Constraint;
import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.RealTimeControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraints.CurrentTimeObjectConstraint;
import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.SwimlaneControl;
import com.ai_center.planpresentation.plugins.Plugin;




/** The PlanPlugin creates most the the menus and controls. <p>
 
 It traces the CurrentTime, creates the File and the View menu and
 controls the speed mode (play, fastforward, stepping).
 
 @author Daniel Weiss
 @version $$
 */
public class PlanPlugin
        implements
            PlanPresentationConstants,
            Plugin,
            ChangeListener
{


    /** commandID for fastforward. */
    public static final int REALTIMECONTROL_FASTFORWARD = 176001;

    /** commandID for jumping to the next breakpoint. */
    public static final int REALTIMECONTROL_NEXT = 176004;

    /** commandID for pause. */
    public static final int REALTIMECONTROL_PAUSE = 176005;

    /** commandID for play mode. */
    public static final int REALTIMECONTROL_PLAY = 176003;

    /** ID for the command to show the breakpoint dialog. */
    public static final int SHOW_BREAKPOINTDIALOG = 176010;

    /** The link to the CurrentTimeSemanticGraphic. */
    private CurrentTimeSemanticGraphic currentTimeSemanticGraphic = null;

    /** The link to the GlobalSearchControl. */
    private GlobalSearchControl gsc = null;

    /** The fastforward Button. */
    private JToggleButton fastforwardButton;

    /** The pause Button. */
    private JToggleButton pauseButton;

    /** The play Button. */
    private JToggleButton playButton;

    /** The single step Button. */
    private JButton steppingButton;

    /** The link to the planPresentationAPI. */
    public PlanPresentationAPI planPresentationAPI;

    /** Layouts a JToogleButton.<p>
     
     @param button The Button to layout.
     */
    private void initButton(JToggleButton button)
    {
        button.setText("");
        button.setMaximumSize(new Dimension(35, 30));
        button.setMinimumSize(new Dimension(35, 30));
        button.setPreferredSize(new Dimension(35, 30));

    }

    /** Makes the pause slider and layouts it.<p>
     
     @return A JSlider with TitledBorder.
     */
    private JSlider makePauseSlider()
    {

        JSlider pauseJSlider = new JSlider(0, 5000, 1000);
        pauseJSlider.setBounds(new Rectangle(7, 17, 200, 74));

        pauseJSlider.setMajorTickSpacing(2500);
        pauseJSlider.setMinorTickSpacing(1000);
        pauseJSlider.setInverted(true);

        pauseJSlider.addChangeListener(new ChangeListener()
        {


            public void stateChanged(ChangeEvent event)
            {
                JSlider source = (JSlider) event.getSource();

                int pauseMs = source.getValue();
                if (pauseMs < 1)
                {
                    source.setValue(1);
                    pauseMs = 1;
                }

                if (!source.getValueIsAdjusting())
                {
                    gsc.getRealTimeControl().setPlayPauseTime(pauseMs);
                }

            }

        });
        return pauseJSlider;
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#addConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void addConstraint(Constraint c)
    {

        if (c instanceof CurrentTimeObjectConstraint)
        {
            currentTimeSemanticGraphic = new CurrentTimeSemanticGraphic(this,
                    (CurrentTimeObjectConstraint) c);
            planPresentationAPI
                    .addViewChangeListener(currentTimeSemanticGraphic);
        }

    }

    /** Returns a link to the GlobalSearchControl. 
     @return The GlobalSearchControl. */
    public GlobalSearchControl getGsc()
    {
        return gsc;
    }

    /** Returns a link to the SwimlaneControl. 
     @return The SwimlaneControl. */

    public SwimlaneControl getSwimlaneControl()
    {
        return planPresentationAPI.getSwimlaneControl();

    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#initPlugin(com.ai_center.planpresentation.PlanPresentationAPI)
     */
    public void initPlugin(PlanPresentationAPI _pluginController)
    {

        planPresentationAPI = _pluginController;

        planPresentationAPI.registerConstraintType(
                Registry.CURRENTTIME_OBJECTCONSTRAINT, (Plugin) this);
        // GUI Action definition    

        PlanPluginAction sortmodeUpdateAction = new PlanPluginAction(
                "Newest on top",
                new ImageIcon(planPresentationAPI.getClass().getResource("resources/updatemode.gif")),
                "sorts accessed resources to the top", new Integer(0),
                SORTMODE_UPDATE, this);

        PlanPluginAction sortmodeGrouped = new PlanPluginAction(
                "Group Views",
                new ImageIcon(planPresentationAPI.getClass().getResource("resources/groupmode.gif")),
                "groups state and action resources", new Integer(0),
                SORTMODE_GROUPED, this);


        PlanPluginAction showBreakpointDialogAction = new PlanPluginAction(
                "Show Breakpoints", new ImageIcon(""),
                "Shows the breakpoint dialog", new Integer(0),
                SHOW_BREAKPOINTDIALOG, this);

        PlanPluginAction exitAction = new PlanPluginAction("Exit",
                new ImageIcon(""), "exit the proramm", new Integer(0),
                EXIT_ACTION, this);

        // Create Toolbar
        ButtonGroup group1 = new ButtonGroup();

        JToggleButton sortmodeUpdateButton = new JToggleButton(
                sortmodeUpdateAction);
        sortmodeUpdateButton.setText("");
        group1.add(sortmodeUpdateButton);
        planPresentationAPI.addToolButton(sortmodeUpdateButton, 10);

        JToggleButton groupedButton = new JToggleButton(sortmodeGrouped);
        groupedButton.setText("");
        group1.add(groupedButton);
        planPresentationAPI.addToolButton(groupedButton, 11);

        // Create RealTimeControlButtons
        JPanel rtcPanel = new JPanel(new FlowLayout());
        rtcPanel.setBorder(BorderFactory.createTitledBorder("Speed"));

        ButtonGroup group2 = new ButtonGroup();

        PlanPluginAction rtcPlay = new PlanPluginAction("Play", new ImageIcon(planPresentationAPI.getClass().getResource("resources/play.gif")), "Play",
                new Integer(0), PlanPlugin.REALTIMECONTROL_PLAY, this);

        playButton = new JToggleButton(rtcPlay);
        initButton(playButton);
        group2.add(playButton);

        // pluginController.addToolButton(playButton, 12);
        rtcPanel.add(playButton);

        PlanPluginAction rtcPause = new PlanPluginAction("Pause",
                new ImageIcon(planPresentationAPI.getClass().getResource("resources/pause.gif")),
                "Pause", new Integer(0), PlanPlugin.REALTIMECONTROL_PAUSE, this);
        pauseButton = new JToggleButton(rtcPause);

        initButton(pauseButton);

        group2.add(pauseButton);
        //pluginController.addToolButton(steppingButton, 13);
        rtcPanel.add(pauseButton);

        PlanPluginAction rtcFastforward = new PlanPluginAction(
                "Fastforward",
                new ImageIcon(planPresentationAPI.getClass().getResource("resources/fastforward.gif")),
                "Fastforward", new Integer(0),
                PlanPlugin.REALTIMECONTROL_FASTFORWARD, this);

        fastforwardButton = new JToggleButton(rtcFastforward);
        initButton(fastforwardButton);
        group2.add(fastforwardButton);
        rtcPanel.add(fastforwardButton);

        PlanPluginAction rtcStepping = new PlanPluginAction(
                "Next Breakpoint",
                new ImageIcon(planPresentationAPI.getClass().getResource("resources/StepForward.gif")),
                "Next Breakpoint", new Integer(0),
                PlanPlugin.REALTIMECONTROL_NEXT, this);
        steppingButton = new JButton(rtcStepping);
        steppingButton.setText("");

        steppingButton.setMaximumSize(new Dimension(35, 30));
        steppingButton.setMinimumSize(new Dimension(35, 30));
        steppingButton.setPreferredSize(new Dimension(35, 30));
        //pluginController.addToolButton(steppingButton, 13);
        rtcPanel.add(steppingButton);

        //pluginController.addToolButton(fastforwardButton, 14);
        rtcPanel.add(makePauseSlider());
        //pluginController.addControl(makePauseSlider());
        rtcPanel.setMaximumSize(new Dimension(210, 100));
        rtcPanel.setMinimumSize(new Dimension(210, 100));
        rtcPanel.setPreferredSize(new Dimension(210, 100));
        rtcPanel.setSize(new Dimension(210, 100));
        planPresentationAPI.addControl(rtcPanel, 10);

        // Create Menu

        planPresentationAPI.addMenu("File", KeyEvent.VK_F, 0); // menu on the left side
        JMenuItem exitActionItem = new JMenuItem(exitAction);
        exitActionItem.setAccelerator(KeyStroke.getKeyStroke('X', Toolkit
                .getDefaultToolkit().getMenuShortcutKeyMask(), false));
        exitActionItem.setMnemonic('X');
        planPresentationAPI.addMenuItem(exitActionItem, "File");

        planPresentationAPI.addMenu("View", KeyEvent.VK_V, 10);
        JMenuItem sortmodeUpdateItem = new JMenuItem(sortmodeUpdateAction);
        sortmodeUpdateItem.setAccelerator(KeyStroke.getKeyStroke('U', Toolkit
                .getDefaultToolkit().getMenuShortcutKeyMask(), false));
        sortmodeUpdateItem.setMnemonic('U');
        planPresentationAPI.addMenuItem(sortmodeUpdateItem, "View");


        JMenuItem sortmodeGroupedItem = new JMenuItem(sortmodeGrouped);
        sortmodeGroupedItem.setAccelerator(KeyStroke.getKeyStroke('G', Toolkit
                .getDefaultToolkit().getMenuShortcutKeyMask(), false));
        sortmodeGroupedItem.setMnemonic('G');
        planPresentationAPI.addMenuItem(sortmodeGroupedItem, "View");

        
        JMenuItem showBreakpointsItem = new JMenuItem(
                showBreakpointDialogAction);
        showBreakpointsItem.setAccelerator(KeyStroke.getKeyStroke('B', Toolkit
                .getDefaultToolkit().getMenuShortcutKeyMask(), false));
        showBreakpointsItem.setMnemonic('B');
        planPresentationAPI.addMenuItem(showBreakpointsItem, "View");

        gsc = planPresentationAPI.getGlobalSearchControl();

        gsc.getRealTimeControl().addChangeListener((ChangeListener) this);
        gsc.getRealTimeControl().setMode(RealTimeControl.PAUSE);

        planPresentationAPI
                .setHelpText("Press Play to start the planning process");


    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.CoordinatorControl#removeConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void removeConstraint(Constraint constraint)
    {
        // CurrentTimeObjectConstraint will never be removed

    }

    /** Delegates setCurrentTime call to the PlanPresentationAPI.
     @param ct The new current time.
     */
    public void setCurrentTime(long ct)
    {
        planPresentationAPI.setCurrentTime(ct);

    } // end dispatchAction   

    /** Sets a new sort mode.<p>
     
     @param sortmode The new sortmode.
     @see com.ai_center.planpresentation.PlanPresentationConstaints#SORTMODE_UPDATE
     @see com.ai_center.planpresentation.PlanPresentationConstaints#SORTMODE_GROUP
     */
    public void setSortMode(int sortmode)
    {
        planPresentationAPI.setSortMode(sortmode);

    }

    /** Creates the Breakpoint Dialog and shows it.<p>
     
     */
    public void showBreakpointDialog()
    {
        BreakpointDialog bd = new BreakpointDialog(planPresentationAPI
                .getGuiFrame(), "Select active breakpoints", true, getGsc()
                .getRealTimeControl());
        bd.show();
    }

    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e)
    {
        if (e.getSource() instanceof RealTimeControl)
        {
            RealTimeControl rtc = (RealTimeControl) e.getSource();
            int mode = rtc.getMode();
            switch (mode)
            {
                case RealTimeControl.PLAY :
                    playButton.setSelected(true);
                    break;
                case RealTimeControl.FASTFORWARD :
                    fastforwardButton.setSelected(true);
                    break;
                case RealTimeControl.STEPPING :
                    pauseButton.setSelected(true);

            }
        }

    }

} // end PlanPlugin


/** This PlanPluginAction represents the commands, which are sent, when 
 someone clicks on the File->Exit or Play-Button, etc. <p>
 
 
 @author Daniel Weiss
 @version $Revision: 1.4 $
 */


class PlanPluginAction extends AbstractAction
        implements
            PlanPresentationConstants
{


    /** The type of this action. */
    private int actionType;

    /** The link to the PlanPlugin. */
    private PlanPlugin planPlugin;

    /** Creats a PlanPluginAction.
     
     @param text The name of the action.
     @param icon A optional ImageIcon.
     @param desc A description for this action. 
     @param mnemonic The keyboard shortcut.
     @param _actionType The action type of this action.
     @param _pcc A link to the PlanPlugin.
     */
    public PlanPluginAction(String text, ImageIcon icon, String desc,
            Integer mnemonic, int _actionType, PlanPlugin _pcc)
    {
        super(text, icon);
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
        actionType = _actionType;
        planPlugin = _pcc;
    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */

    public void actionPerformed(ActionEvent e)
    {

        switch (actionType)
        {
            case CURRENT_TIME_CHANGED :
                break;

            case SORTMODE_UPDATE :
                planPlugin.setSortMode(SORTMODE_UPDATE);
                break;

            case SORTMODE_GROUPED :
                planPlugin.setSortMode(SORTMODE_GROUPED);
                break;

            case RESTART_ACTION :
                break;

            case PlanPlugin.REALTIMECONTROL_PLAY :
                planPlugin.getGsc().getRealTimeControl().setMode(
                        RealTimeControl.PLAY);
                planPlugin.planPresentationAPI.setHelpText("");
                break;
            case PlanPlugin.REALTIMECONTROL_NEXT :
            case PlanPlugin.REALTIMECONTROL_PAUSE :
                planPlugin.getGsc().getRealTimeControl().setMode(
                        RealTimeControl.PAUSE);
                break;
            case PlanPlugin.REALTIMECONTROL_FASTFORWARD :
                planPlugin.getGsc().getRealTimeControl().setMode(
                        RealTimeControl.FASTFORWARD);
                break;

            case PlanPlugin.SHOW_BREAKPOINTDIALOG :
                planPlugin.showBreakpointDialog();
                break;
            case EXIT_ACTION :
                System.exit(0);
                break;

        }

    }

}