package com.ai_center.planpresentation.plugins.stateresourceconstraint;


import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.Constraint;
import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.NumericalHeapSRC;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.SRCResourceType;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.SymbolicSRC;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCResourceType;
import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.plugins.Coordinator;
import com.ai_center.planpresentation.plugins.Plugin;
import com.ai_center.planpresentation.plugins.stateresourceconstraint.numericalSRC.NumericalSRCControl;
import com.ai_center.planpresentation.plugins.stateresourceconstraint.symbolicSRC.SymbolicStateResourceController;
import com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl;
import com.ai_center.planpresentation.utilities.ResourceController;




/** The StateResourceCoordinatorPlugin represents all StateResources. <p>
  
 For that purpose it lists all resources in an JList on the left side.
 Furthermore it controlls the communication between the PlanPresentationAPI
 and the single StateResources.
 
  @author Daniel Weiss
  @version $Revision: 1.2 $
 */
public class StateResourceCoordinatorPlugin
        extends
            DefaultResourceCoordinatorControl
        implements
            PlanPresentationConstants,
            Coordinator,
            Plugin,
            ChangeListener
{


    /** Adds a StateResourceConstraint to the Coordinator-Model.<p>
     
       Additional it adds the StateResourceConstraint to the JList on the 
       left side.
       
       @param stateResourceConstraint The StateResourceConstraint to add.
       
     */
    private void addStateResourceConstraint(
            StateResourceConstraint stateResourceConstraint)
    {

        Variable resourceTypeVariable = stateResourceConstraint.resourceTypeVariable;
        String srcName = null;

        if (resourceTypeVariable != null)
        {

            if (stateResourceConstraint.subClass instanceof SymbolicSRC)
            {

                ResourceController stateResourceController = new SymbolicStateResourceController(
                        (Coordinator) this, stateResourceConstraint);

                SymSRCResourceType type = (SymSRCResourceType) StateResourceConstraint
                        .getResourceTypeDefinition((int) resourceTypeVariable
                                .value(), StateResourceConstraint.SYMBOLIC_SRC);

                defaultResourceModel.ensureCapacity(defaultResourceModel
                        .getSize() + 1);
                stateResourceController.setName(type.name);
                defaultResourceModel.addElement(stateResourceController);
                int indexOf = defaultResourceModel
                        .indexOf(stateResourceController);
                stateResourceController.addChangeListener(defaultResourceModel);
                constraintIndexMap.put(stateResourceConstraint, new Integer(
                        indexOf));

            } else if (stateResourceConstraint.subClass instanceof NumericalHeapSRC)
            {

                ResourceController stateResourceController = new NumericalSRCControl(
                        (Coordinator) this, stateResourceConstraint);

                SRCResourceType type = (SRCResourceType) StateResourceConstraint
                        .getResourceTypeDefinition((int) resourceTypeVariable
                                .value(),
                                StateResourceConstraint.NUMERICALHEAP_SRC);

                defaultResourceModel.ensureCapacity(defaultResourceModel
                        .getSize() + 1);
                if (type != null)
                {
                    stateResourceController.setName(type.getName());
                } else
                {
                    stateResourceController.setName("Unknown");
                }

                defaultResourceModel.addElement(stateResourceController);
                int indexOf = defaultResourceModel
                        .indexOf(stateResourceController);
                stateResourceController.addChangeListener(defaultResourceModel);
                constraintIndexMap.put(stateResourceConstraint, new Integer(
                        indexOf));

            }
        }
    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl#addConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void addConstraint(Constraint constraint)
    {
        if (constraint instanceof StateResourceConstraint)
        {

            addStateResourceConstraint((StateResourceConstraint) constraint);

        }

    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#getGlobalSearchControl()
     */
    public GlobalSearchControl getGlobalSearchControl()
    {

        return planPresentationAPI.getGlobalSearchControl();
    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl#initPlugin(com.ai_center.planpresentation.PlanPresentationAPI)
     */
    public void initPlugin(PlanPresentationAPI _pc)
    {
        planPresentationAPI = _pc;

        // register constraint types	
        planPresentationAPI.registerConstraintType(
                Registry.STATERESOURCE_CONSTRAINT, (Plugin) this);

        planPresentationAPI.addControl(
                initDefaultResourceList("State Resources"), 16);

        planPresentationAPI.addChangeListener((ChangeListener) this);


    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceCoordinatorControl#removeConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void removeConstraint(Constraint constraint)
    {
        //  FIXME Constraint removal is not supported.

    }

    
}