package com.ai_center.planpresentation.plugins.stateresourceconstraint.numericalSRC;


import java.util.HashMap;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.NumericalHeapSRC;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.numericalheapsrc.NumHSRCStateTask;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.numericalheapsrc.OrderedHeapCostsTaskList;
import com.ai_center.dragonbreath.utilities.OrderedShiftDLLElement;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.plugins.Controller;
import com.ai_center.planpresentation.plugins.Coordinator;
import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.utilities.DefaultResourceController;
import com.ai_center.planpresentation.utilities.MultiLevelController;
import com.ai_center.planpresentation.utilities.MultiLevelModel;
import com.ai_center.planpresentation.utilities.ResourceController;
import com.ai_center.planpresentation.utilities.SemanticGraphic;




/** The NumercialSRCControl represents one NumericalHeapStateResource. <p>
 
 It is represented by a entry in the StateResource list on the left side.
 When clicking on it, this NumericalSRCControl opens a view to show more
 details.
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class NumericalSRCControl extends DefaultResourceController
        implements
            Comparable,
            ResourceController,
            Controller,
            ChangeListener,
            PlanPresentationConstants,
            MultiLevelController
{


    /** A link to the MultiLevelModel, which contains the tasks.*/
    private MultiLevelModel multiLevelModel = null;

    /** The link to the NumercialHeapSRC. */
    private NumericalHeapSRC numHSRC = null;

    /** The corresponding StateResourceConstraint.  */
    private StateResourceConstraint stateResourceConstraint = null;

    /** The link to the NumericalSRCView if expanded.*/
    private NumericalSRCView stateResourceView = null;

    /** A mapping from state tasks to their corresponding
     TaskSemanticGraphic objects. */
    private HashMap taskSemanticGraphicMap = null;


    /** Creates a NumericalSRCControl
     
     @param _c The coordinator who creates this object.
     @param _src The link to the corresponding StateResourceConstraint.
     */
    public NumericalSRCControl(Coordinator _c, StateResourceConstraint _src)
    {
        super(_c);
        stateResourceConstraint = _src;
        numHSRC = (NumericalHeapSRC) _src.subClass;

        coordinator = _c;
    }

    /** Adds a task to the gui.<p>
     
     @param stateTask The stateTask to add.
     */
    private void addStateTask(NumHSRCStateTask stateTask)
    {
        // this task is not known, add it to the taskList
        NumericalSRCTaskSemanticGraphic taskSemanticGraphic = new NumericalSRCTaskSemanticGraphic(
                getSwimlaneControl(), stateResourceView,
                stateResourceConstraint, numHSRC);
        if (taskSemanticGraphicMap != null)
        {

            taskSemanticGraphicMap.put(stateTask, taskSemanticGraphic);
        } else
        {
            System.err
                    .println("NumercialSRCControl.addStateTask(): taskSemanticGraphicMap does not exist.");
        }
        taskSemanticGraphic.setDuration(stateResourceView
                .convertTickToPixel(20));

        if (multiLevelModel != null)
        {

            multiLevelModel.addVisibleObject(taskSemanticGraphic);
        }
    }

    /** Removes a stateTask from the gui.<p>
     
     @param stateTask The stateTask to remove.
     */
    private void removeStateTask(NumHSRCStateTask stateTask)
    {
        if (stateTask != null)
        {
            NumericalSRCTaskSemanticGraphic taskSemanticGraphic = (NumericalSRCTaskSemanticGraphic) taskSemanticGraphicMap
                    .get(stateTask);
            if (taskSemanticGraphic != null && multiLevelModel != null)
            {
                multiLevelModel.removeVisibleObject(taskSemanticGraphic);
            }

        }

    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceController#expandView()
     */
    protected void expandView()
    {
        stateResourceView = new NumericalSRCView(this, getName());
        multiLevelModel = new MultiLevelModel(this);
        resetStateTasks();

        coordinator.addView((RetractableView) stateResourceView);
        // show view on the plan panel
        coordinator.addViewChangeListener((ChangeListener) stateResourceView);
        // register to get informed about zoomchanges etc.
        numHSRC.addChangeListener((ChangeListener) this);
        // add changelistener to symbolicSRC

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.DefaultResourceController#removeView()
     */
    protected void removeView()
    {
        coordinator.removeView((RetractableView) stateResourceView);
        coordinator
                .removeViewChangeListener((ChangeListener) stateResourceView);
        // register to get informed about zoomchanges etc.
        numHSRC.removeChangeListener(this);
        // remove changelistener from symbolicSRC

        stateResourceView.terminate();
        stateResourceView = null;
        multiLevelModel = null;
        taskSemanticGraphicMap = null;
    }

    /** Sets a new value for the heap.<p>
     
     @param heapValue The new value for the heap.*/
    protected void setHeap(double heapValue)
    {
        coordinator.getGlobalSearchControl().setVariable(
                stateResourceConstraint.stateVariable, heapValue);


    }

    /** Adds a new SemanticGraphic to the MultiLevelModel.<p>
     
     First it searches for a level, where enough place on the time scala is,
     to insert the SemanticGraphic. Then it puts the SemanticGraphic in this
     level.
     @param sg The SemanticGraphic to insert in the model.
     */
    public synchronized void addVisibleObject(SemanticGraphic sg)
    {
        multiLevelModel.addVisibleObject(sg);
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#getAccessTime()
     */
    public long getAccessTime()
    {
        return accessTime;

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.ResourceController#getDatamodel()
     */
    public Object getDatamodel()
    {
        return null;
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.MultiLevelController#getLevel()
     */
    public int getLevel()
    {
        if (multiLevelModel != null)
        {
            return multiLevelModel.getLevel();
        } else
        {
            return 0;
        }

    }

    /* (non-Javadoc)
     @see com.ai_center.planpresentation.utilities.MultiLevelController#getLevelcount()
     */
    public int getLevelcount()
    {

        return multiLevelModel.getLevelcount();
    }

    /** Returns a link to the NumericalHeapSRC.<p>
     
     @return The numericalHeapSRC.
     
     */
    public NumericalHeapSRC getNumericalHeapSRC()
    {
        return numHSRC;
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.MultiLevelController#getObjectsOnLevel(int, long, long)
     */
    public Map getObjectsOnLevel(int level, long beginCT, long durationCT)
    {
        return multiLevelModel.getObjectsOnLevel(level, beginCT, durationCT);
    }

    /** Returns a link to the StateResourceConstraint.<p>
     
     @return The corresponding StateResourceConstraint.
     */
    public StateResourceConstraint getStateResourceConstraint()
    {
        return stateResourceConstraint;
    }

    /** Remove a SemanticGraphic element only from the visible 
     area.<p>
     
     @param sg The SemanticGraphic to remove.
     */
    public synchronized void removeVisibleObject(SemanticGraphic sg)
    {
        multiLevelModel.removeVisibleObject(sg);
    }


    /** Removes all StateTasks and reads them new from the datamodel.<p>
     
     */
    public void resetStateTasks()
    {
        taskSemanticGraphicMap = new HashMap();
        OrderedHeapCostsTaskList taskList = numHSRC.getTaskList();
        OrderedShiftDLLElement listElement = taskList.first;
        while (listElement != null)
        {
            addStateTask((NumHSRCStateTask) listElement.element);

            listElement = listElement.next;
        }

    }

    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public synchronized void stateChanged(ChangeEvent e)
    {
        if (e.getSource() instanceof NumHSRCStateTask)
        {
            // a stateTask was added, deleted or updated

            NumHSRCStateTask numHSRCStateTask = (NumHSRCStateTask) e
                    .getSource();

            if (taskSemanticGraphicMap.containsKey(numHSRCStateTask))
            {
                // this task is already known. delete it
                removeStateTask(numHSRCStateTask);

            }

            if (numHSRCStateTask.taskLE != null)
            {
                if (numHSRCStateTask.taskLE.element != null)
                {
                    // this task exists anymore (either update or add-stateTask)
                    addStateTask(numHSRCStateTask);

                }
            }

        } else if (e.getSource() instanceof NumericalHeapSRC)
        {
            // the headDescription has changed.
            if (stateResourceView != null)
            {
                stateResourceView.setHeap(numHSRC.getHeap());
            }

        }

    }
}