package com.ai_center.planpresentation.plugins.stateresourceconstraint.numericalSRC;


import java.util.LinkedList;

import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.SwimlaneConnector;
import com.ai_center.planpresentation.SwimlaneControl;
import com.ai_center.planpresentation.plugins.View;
import com.ai_center.planpresentation.utilities.TaskSemanticGraphic;




/** This NumericalSRCTaskSemanticGraphic represents one state task
 which is connected to a NumericalHeapSRC. <p>
 
 
 @author Daniel Weiss
 @version $Revision: 1.3 $
 */
public class NumericalSRCTaskSemanticGraphic extends TaskSemanticGraphic
        implements
            SwimlaneConnector,
            PlanPresentationConstants
{


    /** The beginning time. */
    private long beginCT;

    /** The duration. */
    private long duration;

    /** A link  to the NumericalHeapSRC. */
    private Object specializedTask = null;

    /** A link to the view. */
    private NumericalSRCView stateResourceView;

    /** The temporalReferenceVariable. */
    private Variable temporalReferenceVariable = null;

    /** Creates a NumericalSRCTaskSemanticGraphic.<p>
     
     <pre>
     * NumericalSRCTaskSemanticGraphic taskSemanticGraphic = new NumericalSRCTaskSemanticGraphic(
     *          getSwimlaneControl(), stateResourceView,
     *          stateResourceConstraint, numHSRC);
     </pre>
     
     @param _swimlaneControl A link to the SwimlaneControl.
     @param _stateResourceView The link to the NumericalSRCView.
     @param _task The task which is represented as ObjectConstraint.
     @param _specializedTask The specialized, interal version of the task.
     */
    public NumericalSRCTaskSemanticGraphic(SwimlaneControl _swimlaneControl,
            NumericalSRCView _stateResourceView, Object _task,
            Object _specializedTask)
    {
        super(_swimlaneControl, _task, (View) _stateResourceView);
        stateResourceView = _stateResourceView;
        specializedTask = _specializedTask;


    }


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#getBeginCT()
     */
    public long getBeginCT()
    {

        return beginCT;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getConnectorX(int)
     */
    public int getConnectorX(int connectionType)
    {

        if (connectionType < 0)
        {
            // state <-> task connection on the left side.
            return this.getX();
        } else
        {
            // probably action <-> task connection.
            return this.getX();
        }

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getConnectorY(int)
     */
    public int getConnectorY(int connectionType)
    {
        if (connectionType < 0)
        {
            // this is a task <-> state connection (connectionType == -x coordinate)
            return absoluteY(0); // connection from the left bottom end. 

        } else
        {
            return absoluteY(this.getHeight() / 2);
        }

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#getDuration()
     */
    public long getDuration()
    {

        return duration;
    }


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getPosition(int)
     */
    public int getPosition(int connectionType)
    {
        if (connectionType < 0)
        {
            // this is a connection between task <-> state. No x difference,
            // connection on the SOUTH end.
            return SwimlaneConnector.NORTH;
        }
        return 0;
    }

    /** Requests a swimlane for this task to the StateResourceConstraint.<p>
     
     */
    public void requestSwimlane()
    {
        // connectionType = -1 * x, because SymbolicView doesn't need a model for the connectionTypes.
        int x = this.getX();
        this.requestSwimlane(-x);

    }

    /** Requests a swimlane for this task to the StateResourceConstraint.<p>
     
     @param connectionType The connection Type. 
     */
    public void requestSwimlane(int connectionType)
    {
        if (task != null)
        {

            connectorList = new LinkedList();
            connectorList.add(task);
            connectorList.add(stateResourceView.getStateResourceConstraint());
            swimlaneControl.requestSwimlane(connectionType,
                    (SwimlaneConnector) this, connectorList);
        }
    }

    /** Sets the beginCT.<p>
     
     @param _beginCT The new beginCT
     */
    public void setBeginCT(long _beginCT)
    {
        beginCT = _beginCT;

    }

    /** Sets a new duration..<p>
     
     @param _duration The new duration.
     */
    public void setDuration(long _duration)
    {
        duration = _duration;

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#setPriorityOffset(int)
     */
    public void setPriorityOffset(long offset)
    {
        // this is not a view. Nothing to do

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#setShouldVisible(boolean)
     */
    public void setShouldVisible(boolean b)
    {


    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#setSwimlaneVisibility(boolean)
     */
    public void setSwimlaneVisibility(boolean b)
    {
        // 	always visible

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.SemanticGraphic#shouldVisible()
     */
    public boolean shouldVisible()
    {

        return true;
    }

}

