package com.ai_center.planpresentation.plugins.stateresourceconstraint.numericalSRC;


import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JButton;

import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.planpresentation.utilities.MultiLevelController;
import com.ai_center.planpresentation.utilities.MultiLevelView;




/** The NumericalSRCView shows the content of one NumericalHeapSRC. <p>
 
 For that purpose, it extends the MultiLevelView (which shows the tasks)
 to add a view for the heap, represented as a horizontal line.
 
 @author Daniel Weiss
 @version $Revision: 1.5 $
 */
public class NumericalSRCView extends MultiLevelView
{


    /** The link to the Panel, which shows the line and the heap. */
    private NumHSRCValuePanel loadDisplayPanel = new NumHSRCValuePanel();


    /** The link to the NumericalSRCControl. */
    private NumericalSRCControl numericalSRCControl = null;

    /** Creates the NumericalSRCView.
     
     @param _nsc The link to the NumericalSRCControl
     @param _viewName The name of the view.
     */
    public NumericalSRCView(NumericalSRCControl _nsc, String _viewName)
    {
        super((MultiLevelController) _nsc, _viewName);
        numericalSRCControl = _nsc;

        JButton b = new JButton(new AbstractAction()
        {


            /*
             *  (non-Javadoc)
             * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
             */
            public void actionPerformed(ActionEvent arg0)
            {
                EditNumericalSensorDialog esd = new EditNumericalSensorDialog(
                        numericalSRCControl.getPlanPresentationAPI()
                                .getGuiFrame(), "Sensor data", true,
                        (NumericalSRCControl) controller, numericalSRCControl
                                .getPlanPresentationAPI()
                                .getGlobalSearchControl());
                esd.show();

            }
        });
        b.setIcon(new ImageIcon(controller.getPlanPresentationAPI().getClass().getResource("resources/EditSensorData.gif")));
        b.setSize(20, 20);
        b.setOpaque(false);
        b.setBorderPainted(false);
        b.setToolTipText("Enter sensor data");
        this.addControl(b);
        this.setBaseOffset(40); // height of the load-display
        this.setSize(controller.getPanelWidth(), 40 + ACTION_LEVEL_HEIGHT + 20
                + HEADER_HEIGHT);


    }

    /** @return A link to the StateResourceConstraint.*/
    public StateResourceConstraint getStateResourceConstraint()
    {

        return numericalSRCControl.getStateResourceConstraint();
    }

    /*
     *  (non-Javadoc)
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    public void paintComponent(Graphics g)
    {
        loadDisplayPanel.setLocation(0, HEADER_HEIGHT);
        loadDisplayPanel.setSize(this.getWidth(), 20);
        loadDisplayPanel.setBackground(Color.white);
        this.add(loadDisplayPanel);
        super.paintComponent(g);
    }

    /** Sets a new value for the heap.
     @param _heap The new heap value.
     */
    public void setHeap(double _heap)
    {
        loadDisplayPanel.setHeap(_heap);

    }
}