package com.ai_center.planpresentation.plugins.stateresourceconstraint.symbolicSRC;


import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.CurrentTimeObjectConstraint;
import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.SymbolicSRC;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCResourceType;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCState;
import com.ai_center.planpresentation.utilities.SpringUtilities;




/** This dialog represents the goal adding function. <p>
  
 It opens, when clicking on the checked flag at any StateResource.
 
  @author Daniel Weiss
  @version $Revision: 1.1 $
 */
public class AddGoalDialog extends JDialog
        implements 
            ActionListener
{


    /** The link to the GlobalSearchControl. */
    private GlobalSearchControl gsc;

    /** The link to the corresponding SymbolicStateResourceController. */
    private SymbolicStateResourceController symbolicStateResourceController;

    /** The main panel inside of the dialog window. */
    JPanel mainPanel = new JPanel();

    /** The panel which contains the ok and cancel buttos. */
    JPanel buttonPanel = new JPanel();

    /** The cancel button. */
    JButton cancelButton = null;

    /** The ok button. */
    JButton okButton = null;

    /** The comboBox with all possible states. */
    JComboBox stateList = null;

    /** The textfield which contains the time when this goal should be reached.*/
    JTextField textField = null;
    
    /** Creates an AddGoalDialog but does not open it.
     
       @param frame The parent frame.
       @param title The title.
       @param modal If it should be modal.
       @param _symbolicStateResourceController The link to the controller.
       @param _gsc A link to the GlobalSearchControl.
     */
    public AddGoalDialog(Frame frame, String title, boolean modal,
            SymbolicStateResourceController _symbolicStateResourceController,
            GlobalSearchControl _gsc)
    {
        super(frame, title, modal);

        symbolicStateResourceController = _symbolicStateResourceController;
        gsc = _gsc;


        this.setLocation(200, 200);


        try
        {

            stateList = new JComboBox(getStates());

            textField = new JTextField("" + getCurrentTime());

            //only allow the input of digits
            textField.addKeyListener(new KeyAdapter()
            {


                public void keyTyped(KeyEvent e)
                {
                    char c = e.getKeyChar();
                    if (!((Character.isDigit(c)
                            || (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE))))
                    {
                        getToolkit().beep();
                        e.consume();
                    }
                }
            });


            mainPanel.setLayout(new BorderLayout());
            JPanel inputPanel = new JPanel(new SpringLayout());


            JLabel selectStateLabel = new JLabel("Goal state:", JLabel.TRAILING);
            selectStateLabel.setLabelFor(stateList);

            inputPanel.add(selectStateLabel);
            inputPanel.add(stateList);

            JLabel enterTimePoint = new JLabel("at Time:", JLabel.TRAILING);
            enterTimePoint.setLabelFor(textField);
            inputPanel.add(enterTimePoint);
            inputPanel.add(textField);

            SpringUtilities.makeCompactGrid(inputPanel, 2, 2, //rows, cols
                    6, 6, //initX, initY
                    6, 6); //xPad, yPad

            mainPanel.add(inputPanel, BorderLayout.CENTER);
            mainPanel.add(buttonPanel, BorderLayout.SOUTH);

            okButton = new JButton("OK");
            okButton.addActionListener(this);
            buttonPanel.add(okButton);

            cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(this);
            buttonPanel.add(cancelButton);

            this.getContentPane().add(mainPanel, BorderLayout.NORTH);
            pack();
        } catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /** Reads the current time from the datamodel.<p>

       @return The current time.
     */
    private long getCurrentTime()
    {
        CurrentTimeObjectConstraint ctoc = (CurrentTimeObjectConstraint) symbolicStateResourceController
                .getGlobalSearchControl().findConstraint(
                        Registry.CURRENTTIME_OBJECTCONSTRAINT);
        long currentTime = 0;
        Variable v = null;
        synchronized (ctoc.connections)
        {
            v = (Variable) ctoc.connections
                    .getSpecificVariable(Registry.PLAIN_EDGE);
        }
        if (v != null)
        {
            currentTime = (long) (v.value() * 1.10);
        }
        return currentTime;
    }

    /** Returns all possible states for this StateResourceConstraint.<p>
      	
       @return An array with all possible states as string.
       
     */
    private Object[] getStates()
    {
        StateResourceConstraint src = symbolicStateResourceController
                .getStateResourceConstraint();
        int resourceTypeID = (int) src.resourceTypeVariable.value();
        SymSRCResourceType type = (SymSRCResourceType) StateResourceConstraint
                .getResourceTypeDefinition(resourceTypeID,
                        StateResourceConstraint.SYMBOLIC_SRC);
        return type.states.toArray();

    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();
        if (source instanceof JButton)
        {

            if (source == okButton)
            {
                javax.swing.SwingUtilities.invokeLater(new Runnable()
                {


                    public void run()
                    {
                        SymSRCState symSRCState = (SymSRCState) stateList
                                .getSelectedItem();
                        long currentTime = Long.parseLong(textField.getText());


                        StateResourceConstraint src = symbolicStateResourceController
                                .getStateResourceConstraint();
                        SymbolicSRC symbolicSRC = (SymbolicSRC) src.subClass;
                        symbolicSRC.addGoal(symSRCState.ID, currentTime);

                    }

                });

            }
            //else if (source == cancelButton) {}
            this.setVisible(false);
        }
    }

  
}