package com.ai_center.planpresentation.plugins.stateresourceconstraint.symbolicSRC;


import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.CurrentTimeObjectConstraint;
import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCResourceType;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCState;
import com.ai_center.planpresentation.utilities.SpringUtilities;




/** The EditSensorDialog allows the input of sensor data for the 
 StateResourceConstraint.<p>
 
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class EditSensorDialog extends JDialog implements ActionListener
{


    /** A link to the SymbolicStateResourceController. */
    private SymbolicStateResourceController symbolicStateResourceController;

    /** A link to the GlobalSearchControl. */
    private GlobalSearchControl gsc;

    /** The main panel inside of the JDialog. */
    JPanel mainPanel = new JPanel(new SpringLayout());

    /** The ok button. */
    JButton okButton = null;

    /** The cancle button. */
    JButton cancelButton = null;

    /** The list of all possible states inside of a JComboBox. */
    JComboBox stateList;


    /** Creates an EditSensorDialog but does not open it.
     
     @param frame The parent frame.
     @param title The title.
     @param modal If it should be modal.
     @param _symbolicStateResourceController The link to the controller.
     @param _gsc A link to the GlobalSearchControl.
     */
    public EditSensorDialog(Frame frame, String title, boolean modal,
            SymbolicStateResourceController _symbolicStateResourceController,
            GlobalSearchControl _gsc)
    {
        super(frame, title, modal);

        symbolicStateResourceController = _symbolicStateResourceController;
        gsc = _gsc;


        this.setLocation(200, 200);
        try
        {

            stateList = new JComboBox(getStates());
            JLabel stateLabel = new JLabel("State:");
            mainPanel.add(stateLabel);
            mainPanel.add(stateList);


            JLabel atTimeLabel = new JLabel("at Time:");

            JLabel currentTimeLabel = new JLabel("" + getCurrentTime());
            atTimeLabel.setLabelFor(currentTimeLabel);
          //  mainPanel.add(atTimeLabel);
          //  mainPanel.add(currentTimeLabel);
            okButton = new JButton("OK");

            okButton.addActionListener(this);

            cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(this);
            mainPanel.add(okButton);
            mainPanel.add(cancelButton);

            SpringUtilities.makeCompactGrid(mainPanel, 2, 2, //rows, cols
                    6, 6, //initX, initY
                    6, 6); //xPad, yPad

            this.getContentPane().add(mainPanel, BorderLayout.NORTH);
            pack();
        } catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /** Reads the current time from the datamodel.<p>

     @return The current time.
     */
    private long getCurrentTime()
    {
        CurrentTimeObjectConstraint ctoc = (CurrentTimeObjectConstraint) symbolicStateResourceController
                .getGlobalSearchControl().findConstraint(
                        Registry.CURRENTTIME_OBJECTCONSTRAINT);
        long currentTime = 0;
        Variable v = null;
        synchronized (ctoc.connections)
        {
            v = (Variable) ctoc.connections
                    .getSpecificVariable(Registry.PLAIN_EDGE);
        }
        if (v != null)
        {
            currentTime = (long) (v.value());
        }
        return currentTime;
    }

    /** Returns all possible states for this StateResourceConstraint.<p>
     
     @return An array with all possible states as string.
     
     */
    private Object[] getStates()
    {
        StateResourceConstraint src = symbolicStateResourceController
                .getStateResourceConstraint();
        int resourceTypeID = (int) src.resourceTypeVariable.value();
        SymSRCResourceType type = (SymSRCResourceType) StateResourceConstraint
                .getResourceTypeDefinition(resourceTypeID,
                        StateResourceConstraint.SYMBOLIC_SRC);
        return type.states.toArray();

    }


    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();
        if (source instanceof JButton)
        {

            if (source == okButton)
            {
                javax.swing.SwingUtilities.invokeLater(new Runnable()
                {


                    public void run()
                    {
                        SymSRCState symSRCState = (SymSRCState) stateList
                                .getSelectedItem();

                        StateResourceConstraint src = symbolicStateResourceController
                                .getStateResourceConstraint();
                        gsc.setVariable(src.stateVariable, symSRCState.ID);


                    }
                });

            }
            this.setVisible(false);
        }
    }


}