package com.ai_center.planpresentation.plugins.stateresourceconstraint.symbolicSRC;


import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.SymbolicSRC;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.plugins.Controller;
import com.ai_center.planpresentation.plugins.Coordinator;
import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.utilities.DefaultResourceController;
import com.ai_center.planpresentation.utilities.MultiLevelModel;
import com.ai_center.planpresentation.utilities.ResourceController;
import com.ai_center.planpresentation.utilities.SemanticGraphic;




/** The SymbolicStateResourceController represents one 
 SymbolicStateResource.<p>
 
 The SymbolicStateResourceController is represented as an entry in the
 JList on the left side. If the user clicks on it, this controller
 instantiates a SymbolicStateResourceView to show the detailled information
 about this constraint.
 
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class SymbolicStateResourceController extends DefaultResourceController
        implements
            Comparable,
            ResourceController,
            Controller,
            ChangeListener,
            PlanPresentationConstants
{


    /** The link to the gui model, where all tasks are contained. */
    private MultiLevelModel multiLevelModel = null;

    /** A specialized model, which contains a copy of the 
     SymbolicStateResourceConstraint.intervalList.*/
    private SymbolicStateResourceModel stateResourceModel = null;

    /** The corresponding StateResourceConstraint.   */
    private StateResourceConstraint stateResourceConstraint = null;

    /** The link to the SymbolicStateResourceView. */
    private SymbolicStateResourceView stateResourceView = null;

    /** The link to the specialized StateResourceConstraint.subclass. */
    private SymbolicSRC symbolicSRC = null;

    /** Creates the SymbolicStateResourceController. 
     
     The constructor is called by the StateResourceCoordinatorPlugin.
     
     @param _c The link to the StateResourceCoordinatorPlugin.
     @param _src A link to the corresponding StateResourceConstraint.
     */
    public SymbolicStateResourceController(Coordinator _c,
            StateResourceConstraint _src)
    {
        super(_c);
        stateResourceConstraint = _src;
        symbolicSRC = (SymbolicSRC) _src.subClass;

        coordinator = _c;

    }

    /** Adds the view to the plan panel  */
    protected void expandView()
    {
        stateResourceView = new SymbolicStateResourceView(this, getName());
        stateResourceModel = new SymbolicStateResourceModel(this);
        multiLevelModel = new MultiLevelModel(this);

        coordinator.addView((RetractableView) stateResourceView);
        // show view on the plan panel
        coordinator.addViewChangeListener((ChangeListener) stateResourceView);
        // register to get informed about zoomchanges etc.
        symbolicSRC.addChangeListener((ChangeListener) this);
        // add changelistener to symbolicSRC

        symbolicSRC.getIntervalList().addChangeListener((ChangeListener) this);
        symbolicSRC.getIntervalList().fireIntervalListUpdate();
        // add this line, if direct reading from SymbolicSRC symbolicSRC.addIntervalListChangeListener((ChangeListener)this);
    }

    /**  Removes the view from the plan panel. */
    protected void removeView()
    {
        coordinator.removeView((RetractableView) stateResourceView);
        coordinator
                .removeViewChangeListener((ChangeListener) stateResourceView);
        // register to get informed about zoomchanges etc.
        symbolicSRC.removeChangeListener(this);
        // remove changelistener from symbolicSRC
        symbolicSRC.getIntervalList().removeChangeListener(this);

        stateResourceView.terminate();
        stateResourceView = null;
        stateResourceModel = null;
        multiLevelModel = null;
    }

    /** Delegate function. Adds a SemanticGraphic to the gui model.
     @param sg The SemanticGraphic to add.
     @see com.ai_center.planpresentation.utilities.MultiLevelModel#addVisibleObject
     */
    public synchronized void addVisibleObject(SemanticGraphic sg)
    {
        multiLevelModel.addVisibleObject(sg);
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#getAccessTime()
     */
    public long getAccessTime()
    {
        if (stateResourceModel != null)
        {
            return stateResourceModel.getAccesTime();
        } else
        {
            return 0;
        }
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.utilities.ResourceController#getDatamodel()
     */
    public Object getDatamodel()
    {
        return symbolicSRC.getIntervalList();

    }

    /** @return A link to the GlobalSearchControl. */
    public GlobalSearchControl getGlobalSearchControl()
    {
        return coordinator.getGlobalSearchControl();

    }

    /** Returns the actual level.<p>
     
     The actual level is the value of how many levels exist in parallel.<p>
     @return The actual level.
     @see com.ai_center.planpresentation.utilities.MultiLevelModel#getLevel
     */
    public int getLevel()
    {
        if (multiLevelModel != null)
        {
            return multiLevelModel.getLevel();
        } else
        {
            return 0;
        }

    }

    /** Returns all object between beginCT and beginCT+durationCT on the given
     level.<p>
     
     @param beginCT The begin Tick.
     @param durationCT The duration.
     @param The level to check.
     @return A Map of all SemanticGraphics in this interval.
     */
    public synchronized Map getObjectsOnLevel(int level, long beginCT,
            long durationCT)
    {
        return multiLevelModel.getObjectsOnLevel(level, beginCT, durationCT);
    }

    /** Returns a reference to the original element for a given copy..<p>
     
     This function is used, to recognize elements again. 
     @param copy The copied element.
     @return The reference to the original element, if existent.
     */
    public Object getOriginal(Object copy)
    {
        return stateResourceModel.getOriginal(copy);
    }

    /** Returns a link to the StateResourceConstraint.<p>
     
     
     @return The StateResourceConstraint.
     */
    public StateResourceConstraint getStateResourceConstraint()
    {
        return stateResourceConstraint;
    }

    /**Returns all states, tasks and preconditions in the interval between
     beginTick and endTick.<p>
     
     @param beginTick The beginTick on the left side.
     @param endTick The endTick on the right side of the view.
     @return List with IntervalListElements between the beginTick and the endTick.
     */
    public List getStatesAndTasksBetween(int beginTick, int endTick)
    {
        //		add this line, if direct reading from SymbolicSRC         return symbolicSRC.getStatesAndTasksBetween(beginTick, endTick);
        if (stateResourceModel != null)
        {

            return stateResourceModel.getStatesAndTasksBetween(beginTick,
                    endTick);
        } else
        {
            return (List) new LinkedList();
        }
    }

    /** Returns a link to the SymbolicSRC
     @return The SymbolicSRC.
     */
    public SymbolicSRC getSymbolicSRC()
    {
        return symbolicSRC;
    }

    /** Remove a SemanticGraphic element only from the visible 
     area.<p>
     
     @param sg The SemanticGraphic to remove.
     */
    public synchronized void removeVisibleObject(SemanticGraphic sg)
    {
        multiLevelModel.removeVisibleObject(sg);
    }

    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public synchronized void stateChanged(ChangeEvent e)
    {
        if (stateResourceModel != null)
        {
            // view is expanded, redirect stateChanged call
            stateResourceModel.stateChanged(e);
        }
    }


}