package com.ai_center.planpresentation.plugins.stateresourceconstraint.symbolicSRC;


import java.util.HashSet;
import java.util.Iterator;

import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCEvent;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCInterval;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCPreconditionTask;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCState;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCStateTask;
import com.ai_center.planpresentation.utilities.DefaultResourceModel;
import com.ai_center.planpresentation.utilities.ResourceController;




/** The SymbolicStateResourceModel is mainly used to clone the IntervalList. <p>
  
  The entry point for the cloning operation is the copyElement function, 
  which needs to be overwritten by the model.
  
  TODO If something changes below the SymSRCInterval, this class must be
  adjusted.
  
  @author Daniel Weiss
  @version $Revision: 1.3 $
 */
public class SymbolicStateResourceModel extends DefaultResourceModel
{


    /** Creates a new SymbolicStateResourceModel
     
       @param _controller The link to the SymbolicStateResourceController.
     */
    public SymbolicStateResourceModel(
            SymbolicStateResourceController _controller)
    {
        super((ResourceController) _controller);

    }

    /** Copies one SymSRCEvent.<p>
     
       @param event The SymSRCEvent to copy.
       @return The copied SymSRCEvent.
     */
    private SymSRCEvent copySymSRCEvent(SymSRCEvent event)
    {
        SymSRCEvent copy = new SymSRCEvent(event.name);
        if (event != null)
        {
            copy.ID = event.ID;

        }
        return copy;
    }

    /** Copies one SymSRCPreconditionTask.<p>
     
       @param preconditionTask The SymSRCPreconditionTask to copy.
       @return The copied SymSRCPreconditionTask.
     */private SymSRCPreconditionTask copySymSRCPreconditionTask(
            SymSRCPreconditionTask preconditionTask)
    {

        SymSRCPreconditionTask copy = new SymSRCPreconditionTask();
        if (preconditionTask != null)
        {

            
            // do not clone
            copy.tempVariable = preconditionTask.tempVariable; // do not clone
            copy.resourceTypeVariable = preconditionTask.resourceTypeVariable;
            copy.intervalReference = preconditionTask.intervalReference;
            copy.preconditionFailureLE = preconditionTask.preconditionFailureLE;
            // do not clone

            copy.tempValue = preconditionTask.tempValue;
            copy.resourceType = preconditionTask.resourceType;
            copy.preconditionCosts = preconditionTask.preconditionCosts;
            copy.state = preconditionTask.state;
            copy.partOfWorldTransition = preconditionTask.partOfWorldTransition;
            copy.taskConstraint = preconditionTask.taskConstraint;

            copiedObjectToOriginalMap.put(preconditionTask.tempVariable,
                    preconditionTask);

        }
        return copy;
    }

    /** Copies one SymSRCState.<p>
     
       @param state The SymSRCState to copy. 
       @return The copied SymSRCState.
     */
     private SymSRCState copySymSRCState(SymSRCState state)
    {
        SymSRCState copy = null;
        if (state != null)
        {

            copy = new SymSRCState(new String(state.name));
            copy.ID = state.ID;

        }
        return copy;
    }

     
    /** Copies one SymSRCStateTask. <p>
     @param stateTask The SymSRCStateTask to copy.
     @return The copied SymSRCStateTask.
     */
    private SymSRCStateTask copySymSRCStateTask(SymSRCStateTask stateTask)
    {
        SymSRCStateTask copy = new SymSRCStateTask();
        if (stateTask != null)
        {


            copy.contributionVariable = stateTask.contributionVariable;
            // do not clone
            copy.tempVariable = stateTask.tempVariable; // do not clone
            copy.resourceTypeVariable = stateTask.resourceTypeVariable;
            copy.intervalReference = stateTask.intervalReference;
            copy.taskConstraint = stateTask.taskConstraint;
            // do not clone

            copy.tempValue = stateTask.tempValue;
            copy.resourceType = stateTask.resourceType;

            copy.contribution = copySymSRCEvent(stateTask.contribution);

            copiedObjectToOriginalMap.put(stateTask.tempVariable, stateTask);
        }

        return copy;

    }

    /** This function needs to be overwritten.
      
      @param element The element of the IntervalList to copy.
      @return  The copied element.
     */
    protected Object copyElement(Object element)
    {
        SymSRCInterval symSRCInterval = (SymSRCInterval) element;
        SymSRCInterval copy = null;
        if (symSRCInterval != null)
        {

            copy = new SymSRCInterval();
            copy.state = copySymSRCState(symSRCInterval.state);
            copy.stateTasks = new HashSet();

            if (symSRCInterval.stateTasks != null)
            {
                Iterator it = symSRCInterval.stateTasks.iterator();
                while (it.hasNext())
                {
                    SymSRCStateTask stateTask = (SymSRCStateTask) it.next();
                    copy.stateTasks.add(copySymSRCStateTask(stateTask));

                }
            }

            copy.preconditionTasks = new HashSet();
            if (symSRCInterval.preconditionTasks != null)
            {
                Iterator it = symSRCInterval.preconditionTasks.iterator();
                while (it.hasNext())
                {
                    SymSRCPreconditionTask preconditionTask = (SymSRCPreconditionTask) it
                            .next();
                    copy.preconditionTasks
                            .add(copySymSRCPreconditionTask(preconditionTask));

                }
            }

        }
        return copy;

    }


}