package com.ai_center.planpresentation.plugins.stateresourceconstraint.symbolicSRC;


import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.BoundedRangeModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.GlobalSearchControl;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.ConventionalConstraint;
import com.ai_center.dragonbreath.constraintgraph.Embedding;
import com.ai_center.dragonbreath.constraintgraph.Node;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.PreconditionTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.StateResourceConstraint;
import com.ai_center.dragonbreath.constraints.StateTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.SymbolicSRC;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCInterval;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCPreconditionTask;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCState;
import com.ai_center.dragonbreath.constraints.stateresourceconstraint.symbolicsrc.SymSRCStateTask;
import com.ai_center.dragonbreath.utilities.IntervalListElement;
import com.ai_center.planpresentation.SwimlaneConnector;
import com.ai_center.planpresentation.SwimlaneControl;
import com.ai_center.planpresentation.plugins.Controller;
import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.plugins.View;
import com.ai_center.planpresentation.utilities.PanelCache;
import com.ai_center.planpresentation.utilities.SemanticGraphic;




/** The SymbolicStateResourceView shows all states and tasks
 of a StateResourceConstraint. <p>
 
 Additional this View supports Drag'n'Drop for StateTasks and
 PreconditionTasks.
 
 @author Daniel Weiss
 @version $Revision: 1.10 $
 */
public class SymbolicStateResourceView extends RetractableView
        implements
            ChangeListener,
            SwimlaneConnector,
            View,
            MouseListener
{


    /** DTListener<p> 
     a listener that tracks the state of the operation.
     @see java.awt.dnd.DropTargetListener
     @see java.awt.dnd.DropTarget
     */
    class DTListener implements DropTargetListener
    {


        /** Called by dragEnter and dragOver.<p>
         Checks the flavors and operations
         @param e the event object
         @return whether the flavor and operation is ok
         */
        private boolean isDragOk(DropTargetDragEvent e)
        {

            int resourceTypeID = (int) stateResourceConstraint.resourceTypeVariable
                    .value();
            DataFlavor[] dataFlavor = e.getCurrentDataFlavors();
            if (dataFlavor.length >= 1)
            {

                String humanPresentableName = dataFlavor[0]
                        .getHumanPresentableName();
                if (humanPresentableName
                        .equals("PreconditionTaskObjectConstraint-ResourceTypeID="
                                + resourceTypeID)
                        || humanPresentableName
                                .equals("StateTaskObjectConstraint-ResourceTypeID="
                                        + resourceTypeID))
                {
                    return true;
                }
            }
            return false;
        }

        /*
         *  (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragEnter(DropTargetDragEvent e)
        {
            if (isDragOk(e) == false)
            {
                e.rejectDrag();
                return;
            }
            e.acceptDrag(e.getDropAction());
        }

        /*
         *  (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
         */
        public void dragExit(DropTargetEvent e)
        {

        }

        /*
         *  (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragOver(DropTargetDragEvent e)
        {
            swimlaneControl.paintVerticalLine(e.getLocation().x);
            if (isDragOk(e) == false)
            {
                e.rejectDrag();
                return;
            }
            e.acceptDrag(e.getDropAction());
        }

        /*
         *  (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
         */
        public void drop(DropTargetDropEvent e)
        {
            // the actual action
            int da = e.getDropAction();
            // the actions that the source has specified with DragGestureRecognizer
            int sa = e.getSourceActions();

            if ((da) == 0)
            {
                System.err.println("No action match found");
                e.rejectDrop();
                return;
            }

            Object data = null;
            try
            {
                e.acceptDrop(DnDConstants.ACTION_MOVE);
                DataFlavor[] dataFlavors = e.getCurrentDataFlavors();

                data = e.getTransferable().getTransferData(dataFlavors[0]);
                if (data == null)
                    throw new NullPointerException();
            } catch (Throwable t)
            {
                System.err.println("Couldn't get transfer data: "
                        + t.getMessage());
                t.printStackTrace();
                e.dropComplete(false);
                return;
            }

            if ((data instanceof PreconditionTaskObjectConstraint)
                    || (data instanceof StateTaskObjectConstraint))
            {

                GlobalSearchControl gsc = stateResourceControl
                        .getGlobalSearchControl();

                ConventionalConstraint oldStateResourceConstraint;
                SymbolicSRC symSRC = (SymbolicSRC) stateResourceConstraint.subClass;
                Node objectConstraint = (Node) data;

                synchronized (objectConstraint.connections)
                {

                    oldStateResourceConstraint = objectConstraint.connections
                            .getSpecificConstraint(
                                    Registry.STATERESOURCE_CONSTRAINT,
                                    Registry.PLAIN_EDGE);

                }


                // disconnect node from old resource


                Embedding em = new Embedding(1);
                gsc.extendEmbedding(em, objectConstraint, Registry.PLAIN_EDGE,
                        Registry.INCOMING);

                gsc.reduceConstraint(oldStateResourceConstraint, em);

                // connect node to new resource

                Embedding emb = new Embedding(1);
                gsc.extendEmbedding(emb, objectConstraint, Registry.PLAIN_EDGE,
                        Registry.INCOMING);
                gsc.extendConstraint(stateResourceConstraint, emb);


                Variable temporalReference = null;

                synchronized (objectConstraint.connections)
                {

                    temporalReference = objectConstraint.connections
                            .getSpecificVariable(Registry.TEMPORALREFERENCE_EDGE);

                    long time = convertCoordinatesToTime(e.getLocation().x);
                    gsc.setVariable(temporalReference, time);
                    oldStateResourceConstraint = objectConstraint.connections
                            .getSpecificConstraint(
                                    Registry.STATERESOURCE_CONSTRAINT,
                                    Registry.PLAIN_EDGE);

                }


                symSRC.getIntervalList().fireIntervalListUpdate();
                stateResourceControl.updateView();
            } else
            {
                e.dropComplete(false);
                return;
            }

            e.dropComplete(true);
        }

        /*
         *  (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
         */
        public void dropActionChanged(DropTargetDragEvent e)
        {
            if (isDragOk(e) == false)
            {
                e.rejectDrag();
                return;
            }
            e.acceptDrag(e.getDropAction());
        }

    }

    /** Only a move is accepted. */
    private int acceptableActions = DnDConstants.ACTION_MOVE;

    /** The link to the dropTarget. */
    private DropTarget dropTarget;

    /** The DropTargetListener. */
    private DropTargetListener dtListener;

    /** True if the process is painting at the moment. */
    private boolean isPainting = false;

    /** The last repaint request. */
    private long lastPaintRequest;

    /** mainViewModel.getExtent() */
    private int mainViewModelExtent;

    /** mainViewModel.getValue() */
    private int mainViewModelValue;

    /** The level how height this view is. */
    private int maxlevel;


    /** This is a special mapping from SymSRCPreconditionTask to 
     SymSRCTaskSemanticGraphic. To identify a PreconditionTask, 
     the tempVariable-reference in the PreconditionTask is used.
     */
    private PanelCache preconditionTaskCache = new PanelCache();

    /** This is a special mapping from a SymSRCState to
     SymSRCStateSemanticGraphic.To identify a state, the first SymSRCState
     is used which starts this state.
     */
    private PanelCache stateCache = new PanelCache();

    /**This is a special mapping from SymSRCStateTask to 
     SymSRCTaskSemanticGraphic. To identify a StateTask, the 
     tempVariable-reference in the StateTask is used. */
    private PanelCache stateTaskCache = new PanelCache();


    /** True if a repaintRequest is started. */
    private boolean repaintRequest = false;

    /** The SemanticGraphic which is dragged at the moment. */
    private SemanticGraphic semanticGraphicInDragMode;

    /** A mapping from state.ID to state.name */
    private Hashtable stateNameHash = new Hashtable();

    /** The link to the corresponding StateResourceConstraint. */
    private StateResourceConstraint stateResourceConstraint = null;

    /** The link to the SymbolicStateResourceController. */
    private SymbolicStateResourceController stateResourceControl;

    /** A link to the SwimlaneControl. */
    private SwimlaneControl swimlaneControl = null;

    /** True if all Swimlanes should be visible. */
    private boolean swimlaneVisibility = true;

    /** Creats a SymbolicStateResourceView
     * 
     @param _controller The link to the SymbolicStateResourceController.
     @param _viewName The name of the view.
     */
    public SymbolicStateResourceView(
            SymbolicStateResourceController _controller, String _viewName)
    {

        super((Controller) _controller, _viewName);
        JButton b = new JButton(new AbstractAction()
        {


            /*
             *  (non-Javadoc)
             * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
             */
            public void actionPerformed(ActionEvent arg0)
            {
                EditSensorDialog esd = new EditSensorDialog(
                        stateResourceControl.getPlanPresentationAPI()
                                .getGuiFrame(), "Enter Sensor Data", true,
                        (SymbolicStateResourceController) controller,
                        stateResourceControl.getPlanPresentationAPI()
                                .getGlobalSearchControl());
                esd.show();

            }
        });

        b.setIcon(new ImageIcon(controller.getPlanPresentationAPI().getClass()
                .getResource("resources/EditSensorData.gif")));

        b.setSize(20, 20);
        b.setOpaque(false);
        b.setBorderPainted(false);
        b.setToolTipText("Enter sensor data");
        this.addControl(b);

        b = new JButton(new AbstractAction()
        {


            public void actionPerformed(ActionEvent arg0)
            {
                AddGoalDialog agd = new AddGoalDialog(stateResourceControl
                        .getPlanPresentationAPI().getGuiFrame(), "Add a Goal",
                        true, (SymbolicStateResourceController) controller,
                        stateResourceControl.getPlanPresentationAPI()
                                .getGlobalSearchControl());
                agd.show();

            }
        });
        b.setIcon(new ImageIcon(controller.getPlanPresentationAPI().getClass()
                .getResource("resources/checkeredFlag.gif")));
        b.setSize(20, 20);
        b.setOpaque(false);
        b.setBorderPainted(false);
        b.setToolTipText("Enter a goal for this resource");
        this.addControl(b);

        stateResourceControl = _controller;
        stateResourceConstraint = stateResourceControl
                .getStateResourceConstraint();

        //  this.setBackground(Color.green);
        setLevel(1); // start with one level

        this.setLayout(null);

        swimlaneControl = stateResourceControl.getSwimlaneControl();
        swimlaneControl.registerConnector(stateResourceConstraint, this);
        this.addMouseListener(this);
        this.dtListener = new DTListener();

        // component, ops, listener, accepting
        this.dropTarget = new DropTarget(this, this.acceptableActions,
                this.dtListener, true);

        //stateResourceControl.getSymbolicSRC().

    }


    /** Checks if the level must be adjusted.<p>
     
     @param level The actual level.
     */
    private void adjustLevel(int level)
    {
        boolean changeSize = false;
        if (level > maxlevel)
        {
            // level is bigger than maxlevel, make the panel bigger
            maxlevel = level + 2;
            changeSize = true;
        } else if (level < maxlevel - 3)
        {
            // level is much smaller than, size down the panel
            maxlevel = level;
            changeSize = true;

        }

        if (changeSize)
        {
            setLevel(maxlevel);
        }

    }

    /** Converts a tick to a x-coordinate.<p>
     
     @param tick The tick to convert.
     @return The x-coordinate.
     */
    private int convertTickToPixel(long tick)
    {
        int extent = mainViewModelExtent;
        int width = this.getWidth();

        double pixelPerTimeUnit = (double) width / (double) mainViewModelExtent;

        // avoid Java-Bug: Long.MAX_VALUE * 2.0 == - Long.MAX_VALUE;
    /*    if (tick > 100000000)
        {
            tick = 10000000;
        }*/
        double xdouble = (double) RETRACT_BUTTON_WIDTH
        + ((double) (tick - mainViewModelValue) * pixelPerTimeUnit);
        int x = (int) xdouble;
//        int x = RETRACT_BUTTON_WIDTH
  //      + (int) ((double) (tick - mainViewModelValue) * pixelPerTimeUnit);

        if (x < 0 && tick > mainViewModelValue && mainViewModelValue > 100)
        {
            System.err
                    .println("SymbolicStateResourceView.convertTickToPixel(): Illegal value. Resetting to zero");
            x = 0;
        }
        // protection against extreme coordinate values and the following jpanel memory bug
        if (x > 2 * width)
        {
            x = 2 * width;
        }
        if (x < (-2 * width))
        {
            x = -2 * width;
        }
        return x;

    }


    /** Paint all States which are in the given intervalList..<p>
     
     @param intervalList The States to paint.
     */
    private void paintAllStates(List intervalList)
    {

        Iterator it = intervalList.iterator();
        int lastStateID = -1;
        long lastEnd = -1;
        long beginning = -1;
        SymSRCState beginningSequenceState = null;

        boolean firstElement = true;
        IntervalListElement currentILE = null;

        while (it.hasNext())
        {
            currentILE = (IntervalListElement) it.next();

            if (firstElement)
            { // check for update mechanism
                firstElement = false;
                //  minBeginTime = currentILE.getBeginning();
            }

            SymSRCInterval symSRCInterval = (SymSRCInterval) currentILE
                    .getElement();

            if (symSRCInterval != null)
            {
                // check to avoid NullPointerExceptions

                if (!stateNameHash
                        .contains(new Integer(symSRCInterval.state.ID)))
                {
                    stateNameHash.put(new Integer(symSRCInterval.state.ID),
                            symSRCInterval.state.name);
                }

                SymSRCStateSemanticGraphic symSRCSemanticGraphic = null;

                if (symSRCInterval.state.ID != lastStateID || lastStateID == -1)
                {
                    // here begins a new state

                    if (lastEnd != -1)
                    {
                        // get the state before
                        paintState(beginningSequenceState, beginning, lastEnd);
                    } // end lastID != -1 => here ends a new state
                    beginning = currentILE.getBeginning();
                    beginningSequenceState = symSRCInterval.state;

                }
                lastEnd = currentILE.getEnd();
                lastStateID = symSRCInterval.state.ID;

            } else
            {
                //the state hasn't changed
                lastEnd = currentILE.getEnd(); // progress the end

            } // end if (lastEnd != -1)

        } // end while

        if (lastEnd != -1)
        {
            // get the state before
            paintState(beginningSequenceState, beginning, lastEnd);
        } // end lastID != -1 => here ends a new state


        removeRemainingPanels(stateCache.getRemainingPanels());

    }

    /** Paints all tasks which are in the intervalList.<p>
     
     @param intervalList The tasks to paint.
     */
    private void paintAllTasks(List intervalList)
    {
        Iterator it = intervalList.iterator();
        IntervalListElement currentILE = null;
        int newMaxLevel = 0;
        long lastBeginning = 0;
        while (it.hasNext())
        {
            currentILE = (IntervalListElement) it.next();

            SymSRCInterval symSRCInterval = (SymSRCInterval) currentILE
                    .getElement();

            if (symSRCInterval != null)
            {
                // check to avoid NullPointerExceptions

                lastBeginning = currentILE.getBeginning();
                // paint all StateTasks
                Iterator itStateTasks = symSRCInterval.stateTasks.iterator();
                while (itStateTasks.hasNext())
                {
                    SymSRCStateTask stateTask = null;

                    stateTask = (SymSRCStateTask) itStateTasks.next();

                    paintStateTask(stateTask);
                }

                // paint all PreconditionTasks
                Iterator itPreconditionTasks = symSRCInterval.preconditionTasks
                        .iterator();
                while (itPreconditionTasks.hasNext())
                {
                    SymSRCPreconditionTask preconditionTask = (SymSRCPreconditionTask) itPreconditionTasks
                            .next();
                    paintPreconditionTask(preconditionTask);
                }

            }

        } // end while

        adjustLevel(newMaxLevel);

        removeRemainingPanels(stateTaskCache.getRemainingPanels());
        removeRemainingPanels(preconditionTaskCache.getRemainingPanels());

    }

    /** Paints one PreconditionTask.<p>
     
     @param preconditionTask The SymSRCPreconditionTask to paint.
     */
private void paintPreconditionTask(SymSRCPreconditionTask preconditionTask)
    {
        if (preconditionTask.taskConstraint != null)
        {

            //check if it is a new state
            SymSRCTaskSemanticGraphic taskSemanticGraphic = (SymSRCTaskSemanticGraphic) preconditionTaskCache
                    .getObject(preconditionTask.tempVariable);

            if (taskSemanticGraphic == null)
            {
                // graphic for this state does not exist. create a new one.

                taskSemanticGraphic = new SymSRCTaskSemanticGraphic(
                        swimlaneControl, this, preconditionTask.taskConstraint,
                        preconditionTask);
                taskSemanticGraphic.setBackground(PRECONDITIONTASK_COLOR);

                if (!stateNameHash
                        .contains(new Integer(preconditionTask.state)))
                {
                    
                    SymSRCState state = SymbolicSRC.retrieveState((long)stateResourceConstraint.resourceTypeVariable.value(),preconditionTask.state);
                    if (state != null) {
                    stateNameHash.put(new Integer(preconditionTask.state),
                          state.name );
                    }
                }
                String stateName = (String) stateNameHash.get(new Integer(
                        preconditionTask.state));
              
                taskSemanticGraphic.setText(stateName);
                
                taskSemanticGraphic
                        .setToolTipText("Precondition Task: Is State "
                                + stateName + "?");
                preconditionTaskCache.addObject(preconditionTask.tempVariable,
                        taskSemanticGraphic);

            } // end if semantic graphic == null

            taskSemanticGraphic.setBeginCT(preconditionTask.tempValue);
            String text = taskSemanticGraphic.getText();
            
            taskSemanticGraphic.setDuration(convertPixelToCT((text.length() * 8 ) + 20)); // 10 pixel per letter
            
            stateResourceControl
                    .removeVisibleObject((SemanticGraphic) taskSemanticGraphic);
            stateResourceControl
                    .addVisibleObject((SemanticGraphic) taskSemanticGraphic);

            taskSemanticGraphic.registerConnector();
            taskSemanticGraphic.requestSwimlane();
        }
    }
    /** Paints one state.<p>
     
     @param beginningSequenceState The SymSRCState which represents all other
     following SymSRCState with the same stateID.
     @param beginning The beginning of this State Sequence.
     @param lastEnd The end of thhis State Sequence.
     */
    private void paintState(SymSRCState beginningSequenceState, long beginning,
            long lastEnd)
    {
        //check if it is a new state
        SymSRCStateSemanticGraphic symSRCSemanticGraphic = (SymSRCStateSemanticGraphic) stateCache
                .getObject(beginningSequenceState);

        if (symSRCSemanticGraphic == null)
        {
            // graphic for this state does not exist. create a new one.

            symSRCSemanticGraphic = new SymSRCStateSemanticGraphic(
                    beginningSequenceState, beginning, lastEnd);

            stateCache.addObject(beginningSequenceState, symSRCSemanticGraphic);

            this.add(symSRCSemanticGraphic);
        } // end if semantic graphic == null

        int x1 = convertTickToPixel(beginning);
        int x2 = convertTickToPixel(lastEnd);
        if (x1 < RETRACT_BUTTON_WIDTH)
        {
            // move label to the right side in the visible area
            symSRCSemanticGraphic.stateNameLabelLocation((-1
                    * (x1 - RETRACT_BUTTON_WIDTH) + 5));
        }
        symSRCSemanticGraphic.setLocation(x1, MARGIN_TOP + HEADER_HEIGHT);
        symSRCSemanticGraphic.setSize(x2 - x1, STATE_HEIGHT);

    }

    /** Paints one StateTask.<p>
     
     @param stateTask The SymSRCStateTask to paint.
     
     */
    private void paintStateTask(SymSRCStateTask stateTask)
    {

        //check if it is a new state
        if (stateTask != null)
        {
            if (stateTask.taskConstraint != null)
            {

                SymSRCTaskSemanticGraphic taskSemanticGraphic = (SymSRCTaskSemanticGraphic) stateTaskCache
                        .getObject(stateTask.tempVariable);

                if (taskSemanticGraphic == null)
                {
                    // graphic for this state does not exist. create a new one.

                    taskSemanticGraphic = new SymSRCTaskSemanticGraphic(
                            swimlaneControl, this, stateTask.taskConstraint,
                            stateTask);
                    taskSemanticGraphic.setBackground(STATETASK_COLOR);
                    stateTaskCache.addObject(stateTask.tempVariable,
                            taskSemanticGraphic);
                    taskSemanticGraphic.setText(""
                            + stateTask.contribution.name);
                    taskSemanticGraphic
                            .setToolTipText("State Task: Contribution "
                                    + stateTask.contribution.name);

                } // end if semantic graphic == null
                taskSemanticGraphic.setBeginCT(stateTask.tempValue);
                //taskSemanticGraphic.setDuration(convertPixelToCT(20));
                String text = taskSemanticGraphic.getText();
                taskSemanticGraphic.setDuration(convertPixelToCT((text.length() * 8) + 20  ) ); // 10 pixel per letter
                
                
                stateResourceControl
                        .removeVisibleObject((SemanticGraphic) taskSemanticGraphic);
                stateResourceControl
                        .addVisibleObject((SemanticGraphic) taskSemanticGraphic);

                taskSemanticGraphic.registerConnector();
                taskSemanticGraphic.requestSwimlane();
            } 
        }
    }

    /** Remove all gui elements which are listed in panels

     @param panels The panels to remove.
     */
    private void removeRemainingPanels(Collection panels)
    {

        Iterator remainingIt = panels.iterator();
        while (remainingIt.hasNext())
        {
            Object nextObject = remainingIt.next();
            JPanel sg = (JPanel) nextObject;
            if (nextObject instanceof SwimlaneConnector)
            {
                SwimlaneConnector sc = (SwimlaneConnector) nextObject;
                sc.terminate();
            }
            this.remove(sg);

            if (nextObject instanceof SemanticGraphic)
            {

                stateResourceControl
                        .removeVisibleObject((SemanticGraphic) nextObject);
            }

        }

    }

    /** Sets a new level and updates the view.
     
     @param _maxlevel The new maxlevel.
     */
    private void setLevel(int _maxlevel)
    {
        maxlevel = _maxlevel;

        updateViewSize(maxlevel);

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.View#absoluteY(int)
     */
    public int absoluteY(int relativeY)
    {
        return stateResourceControl.absoluteY(this.getY()) + relativeY;
    }

    /** Converts the x coordinate of a pixel to a absolute tick-value.<p>
     
     @param x The x-coordinate.
     @return The tick.
     */
    public long convertCoordinatesToTime(int x)
    {
        // mainViewModelValue + (x-pixel / pixelPerTimeUnit)
        int width = this.getWidth();
        double pixelPerTimeUnit = (double) this.getWidth()
                / (double) mainViewModelExtent;

        return (long) (mainViewModelValue + ((x -RETRACT_BUTTON_WIDTH) / pixelPerTimeUnit));

    }

    /** Converts a distance of pixel to a tick value.<p>
     
     @param pixel The pixel number to convert.
     @return The distance as tick.
     */
    public long convertPixelToCT(int pixel)
    {
        int width = this.getWidth();
        double pixelPerTimeUnit = (double) this.getWidth()
                / (double) mainViewModelExtent;
        return (long) Math.round(pixel / pixelPerTimeUnit);
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.View#engineStop()
     */
    public void engineStop()
    {
        stateResourceControl.engineStop();

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getConnectorX(int)
     */
    public int getConnectorX(int connectionType)
    {
        if (connectionType < 0)
        {
            // simple translation because connectionType is -x;
            return -connectionType;
        }
        return -1;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getConnectorY(int)
     */
    public int getConnectorY(int connectionType)
    {
        if (connectionType < 0)
        {
            return absoluteY(5 + HEADER_HEIGHT + STATE_HEIGHT);
        }
        return 0;
    }

    /** @return A link to the GlobalSearchControl
     */
    public GlobalSearchControl getGlobalSearchControl()
    {

        return stateResourceControl.getGlobalSearchControl();
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.View#getMainViewModelExtent()
     */
    public int getMainViewModelExtent()
    {
        return mainViewModelExtent;
    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.View#getMainViewModelValue()
     */
    public int getMainViewModelValue()
    {
        return mainViewModelValue;
    }


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#getPosition(int)
     */
    public int getPosition(int connectionType)
    {
        if (connectionType < 0)
        {
            return SwimlaneConnector.SOUTH;
        }

        return 0;
    }

    /** Returns a link to the StateResourceConstraint.<p>
     
     @return The corresponding StateResourceConstraint
     */
    public StateResourceConstraint getStateResourceConstraint()
    {

        return stateResourceConstraint;
    }


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#isOutgoing(int)
     */
    public boolean isOutgoing(int connectionType)
    {
        return false;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#isVisible(int)
     */
    public boolean isVisible(int connectionType)
    {
        return swimlaneVisibility;
    }

    /* (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent arg0)
    {

    }

    /* (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent e)
    {
    }

    /* (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent arg0)
    {

    }

    /* (non-Javadoc)
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent arg0)
    {

    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e)
    {

    }

    /*
     *  (non-Javadoc)
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    public synchronized void paintComponent(Graphics g)
    {

        if (!isPainting)
        {
            repaintRequest = false;
            isPainting = true;

            List intervalList = stateResourceControl.getStatesAndTasksBetween(
                    mainViewModelValue, mainViewModelValue
                            + mainViewModelExtent);

            paintAllStates(intervalList);
            paintAllTasks(intervalList);

            int reallevel = stateResourceControl.getLevel();
            adjustLevel(reallevel);
            // copied from multiLevelView

            //paint all tasks
            for (int level = 0; level < reallevel; level++)
            {

                //get all actions on this level
                Set objects = stateResourceControl.getObjectsOnLevel(level,
                        mainViewModelValue, mainViewModelExtent).entrySet();
                Iterator it = objects.iterator();

                while (it.hasNext())
                {
                    // paint alle tasks on this level
                    Map.Entry entry = (Map.Entry) it.next();

                    SemanticGraphic sg = (SemanticGraphic) entry.getValue();

                    if (sg.shouldVisible())
                    // "!sg.isDragging() &&" removed, because this is only for 
                    // local drag & drop. But here we use global drag & drop.
                    {

                        double x1 = convertTickToPixel(sg.getBeginCT());

                        double y1 = (level * ACTION_LEVEL_HEIGHT) + 5
                                + HEADER_HEIGHT + STATE_HEIGHT + 10;
                        //+5 margin in top

                        double x2 = convertTickToPixel(sg.getBeginCT()
                                + sg.getDuration());


                        sg.setSize((int) (x2 - x1), ACTION_LEVEL_HEIGHT - 5);
                        sg.setLocation((int) x1, (int) y1);

                        this.add((JPanel) sg);

                    } else
                    {
                        semanticGraphicInDragMode = sg;
                    }
                }

                if (semanticGraphicInDragMode != null)
                {

                    if (semanticGraphicInDragMode.isDragging())
                    {

                        semanticGraphicInDragMode.setSize(
                                convertTickToPixel(semanticGraphicInDragMode
                                        .getDuration()),
                                ACTION_LEVEL_HEIGHT - 5);

                    } else
                    {
                        semanticGraphicInDragMode = null;
                    }

                }

            }

            isPainting = false;

        }
        super.paintComponent(g);

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.View#scroll(int)
     */
    public void scroll(int scrollstep, boolean extendViewIfNecessary)
    {
        stateResourceControl.scroll(scrollstep, extendViewIfNecessary);
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#setPriorityOffset(int)
     */
    public void setPriorityOffset(long offset)
    {
        controller.setPriorityOffset(offset);
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#setShouldVisible(boolean)
     */
    public void setShouldVisible(boolean b)
    {
        //if extracted, always visible

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#shouldVisible()
     */
    public boolean shouldVisible()
    {
        // if extracted, always visible
        return true;
    }

    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public synchronized void stateChanged(ChangeEvent e)
    {
        if (e.getSource() instanceof BoundedRangeModel)
        {
            BoundedRangeModel mainViewModelEvent = (BoundedRangeModel) e
                    .getSource();
            if (!mainViewModelEvent.getValueIsAdjusting())
            {
                if ((mainViewModelEvent.getValue() != mainViewModelValue)
                        || (mainViewModelEvent.getExtent() != mainViewModelExtent))
                {
                    mainViewModelValue = mainViewModelEvent.getValue();
                    mainViewModelExtent = mainViewModelEvent.getExtent();
                }
            }
        } else if ((e.getSource() instanceof IntervalListElement)
                || (e.getSource() instanceof SymbolicSRC))
        {
            if (!repaintRequest)
            {
                if (System.currentTimeMillis() - lastPaintRequest > 100)
                {
                    lastPaintRequest = System.currentTimeMillis();
                    repaintRequest = true;

                }
            }

        }

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.SwimlaneConnector#terminate()
     */
    public void terminate()
    {
        swimlaneControl.removeConnector(stateResourceConstraint);
        removeRemainingPanels(stateTaskCache.getRemainingPanels());
        removeRemainingPanels(preconditionTaskCache.getRemainingPanels());
        swimlaneVisibility = false;

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.RetractableView#updateViewSize()
     */
    public void updateViewSize()
    {
        updateViewSize(maxlevel);
    }

    /** Requests a view update.<p>
     
     Requests a update for an adjustment of a new height. The levelcount
     is used to check, if this update is really necessary.
     @param levelcount The actual number of levels.
     */
    public void updateViewSize(int level)
    {
        int width = controller.getPanelWidth();
        int panelHeight = (maxlevel * ACTION_LEVEL_HEIGHT) + 5 + HEADER_HEIGHT
                + STATE_HEIGHT + 10;
        this.setMinimumSize(new Dimension(width, panelHeight));
        this.setPreferredSize(new Dimension(width, panelHeight));
        this.setMaximumSize(new Dimension(width, panelHeight));
        stateResourceControl.updateViewSize(level);
    }

}