package com.ai_center.planpresentation.plugins.transitionconstraint;


import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import com.ai_center.dragonbreath.Constraint;
import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.ActionObjectConstraint;
import com.ai_center.dragonbreath.constraints.TransitionConstraint;
import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.plugins.Plugin;
import com.ai_center.planpresentation.plugins.actionconstraint.ActionCoordinatorControlPlugin;
import com.ai_center.planpresentation.plugins.actionconstraint.ActionCoordinatorView;
import com.ai_center.planpresentation.utilities.MultiLevelModel;




/** The TransistionCoordinatorPlugin shows external effects on the 
 planning. e.g. that a basin overflows if too much water is inside.<p>
 
 This is for the most part equal to the ActionCoordinatorControlPlugin and
 extends it therefore. Only the addConstraint functions differs from the 
 ActionCoordinatorControlPlugin.
 
 @author Daniel Weiss
 @version $$
 */
public class TransitionCoordinatorPlugin extends ActionCoordinatorControlPlugin
{


    /** The commandID to shwo transitions. */
    final static int SHOW_TRANSITIONS = 3423465;

    /** The link to the TransitionConstraint. */
    private TransitionConstraint transitionConstraint;

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#addConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void addConstraint(Constraint c)
    {

        if (c instanceof TransitionConstraint)
        {
            transitionConstraint = (TransitionConstraint) c;

        } else if (c instanceof ActionObjectConstraint)
        {
            ActionObjectConstraint aoc = (ActionObjectConstraint) c;
            Variable actionTypeVariable = null;
            synchronized (aoc.connections)
            {

                actionTypeVariable = aoc.connections
                        .getSpecificVariable(Registry.ACTIONTYPE_EDGE);
            }
            if (actionTypeVariable != null)
            {

                if (TransitionConstraint.isTransition((int) actionTypeVariable
                        .value()))
                {
                    addConstraint_internal(c);
                }
            }

        }
    }

    /** Expands the transition view. */
    public void expandView()
    {

        //init view
        multiLevelView = new ActionCoordinatorView(this, "Transitions");
        planPresentationAPI.addViewChangeListener(multiLevelView);
        planPresentationAPI.addView(multiLevelView);

    }

    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#initPlugin(com.ai_center.planpresentation.PlanPresentationAPI)
     */
    public void initPlugin(PlanPresentationAPI _p)
    {

        planPresentationAPI = _p;

        // init model
        multiLevelModel = new MultiLevelModel(this);

        // register constraint types	
        planPresentationAPI.registerConstraintType(
                Registry.TRANSITION_CONSTRAINT, (Plugin) this);

        planPresentationAPI.registerConstraintType(
                Registry.ACTION_OBJECTCONSTRAINT, (Plugin) this);

        TransitionCoordinatorAction showTransitions = new TransitionCoordinatorAction(
                " Show Transitions", null, "show transitions", new Integer(0),
                TransitionCoordinatorPlugin.SHOW_TRANSITIONS, this);
        
        planPresentationAPI.addMenu("View", KeyEvent.VK_V,  10);
        JMenuItem showTransitionsItem = new JMenuItem(showTransitions);
        showTransitionsItem.setAccelerator(KeyStroke.getKeyStroke('T', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
        showTransitionsItem.setMnemonic('T');
        planPresentationAPI.addMenuItem(showTransitionsItem , "View");

    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Plugin#removeConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public void removeConstraint(Constraint c)
    {
        if (c instanceof ActionObjectConstraint)
        {
            ActionObjectConstraint aoc = (ActionObjectConstraint) c;
            super.removeConstraint(aoc);

        } else if (c instanceof TransitionConstraint)
        {
            transitionConstraint = null;
        }
    }

    /** Removes the Transition View.*/
    public void retractView()
    {
        planPresentationAPI.removeView(multiLevelView);
        multiLevelView = null;

    }


}

/** The TransitionCoordinatorAction is only for the menu entry
 View->show/hide transitions. <p>
 
 @author Daniel Weiss
 @version $$
 */


class TransitionCoordinatorAction extends AbstractAction
        implements
            PlanPresentationConstants
{


    /** The action Type. */
    private int actionType;

    /** The link to the TransitionCoordinatorPlugin. */
    private TransitionCoordinatorPlugin transitionCoordinatorPlugin;

    /** Creates a new action command.
     
     @param text The text to display.
     @param icon A optional ImageIcon
     @param desc A detailled description of this action. (for ToolTips)
     @param mnemonic The keyboard shortcut.
     @param _actionType The actionType/MessageID
     @param _acc A link to the TransitionCoordinatorPlugin
     */
    public TransitionCoordinatorAction(String text, ImageIcon icon,
            String desc, Integer mnemonic, int _actionType,
            TransitionCoordinatorPlugin _tcp)
    {
        super(text, icon);
        putValue(SHORT_DESCRIPTION, desc);
        putValue(MNEMONIC_KEY, mnemonic);
        actionType = _actionType;
        transitionCoordinatorPlugin = _tcp;
    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */

    public void actionPerformed(ActionEvent e)
    {

        switch (actionType)
        {

            case TransitionCoordinatorPlugin.SHOW_TRANSITIONS :
                if (e.getSource() instanceof JMenuItem)
                {
                    JMenuItem menuItem = (JMenuItem) e.getSource();
                    if (menuItem.getText().equals(" Show Transitions"))
                    {
                        menuItem.setText(" Hide Transitions");
                        transitionCoordinatorPlugin.expandView();
                    } else if (menuItem.getText().equals(" Hide Transitions"))
                    {
                        menuItem.setText(" Show Transitions");
                        transitionCoordinatorPlugin.retractView();

                    }
                }

                break;

        }

    }

}