package com.ai_center.planpresentation.utilities;


import javax.swing.DefaultListModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;




/** This ChangeListenerListModel receives changes in elements and updates the
 corresponding gui list.<p>
 
 In a normal DefaultListModel the programmer has to take care, which
 element has changed its content. The ChangeListenerListModel simply
 subscribes to the correspondig element as changelistener and gets
 informed about all changed.<p>
 
 <pre>
 * changeListenerListModel.addElement(element);
 * element.addChangeListener(changeListenerListModel);
 </pre>
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class ChangeListenerListModel extends DefaultListModel
        implements
            ChangeListener
{


    /*
     *  (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e)
    {
        // find the index of the changed element
        int indexOf = this.indexOf(e.getSource());
        // make a view update, e.g. that a JList can change its representation
        super.fireContentsChanged(this, indexOf, indexOf + 1);

    }

}