package com.ai_center.planpresentation.utilities;


/** The ChangeMessage adds additional information to a ChangeEvent.<p>
 
 A normal ChangeEvent informs only about that something has changed,
 but the ChangeMessage can give more information what exactly has changed.<p>
 The problem is, that this class is not standardised and can only be used by
 special ChangeListeners.
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class ChangeMessage
{


    /** The message in int-format for more performance.
     e.g. Sortmode is now update would be: message = SORTMODE_UPDATE 
     */
    private int message = 0;

    /** The message type format to filter messages.
     e.g. messageType = SORTMODE_CHANGE */
    private int messageType = 0;

    /** The source who sends this message. This is can also be a ChangeEvent,
     to add more information to it. Than the ChangeEvent.getSource()
     contains the real source.  */
    private Object source = null;

    /** Create a new change message.
     @param _messageType The message type. 
     @see #messageType
     */
    public ChangeMessage(int _messageType)
    {
        messageType = _messageType;
    }

    /** Returns the message.<p> 	
     @return The message in int-format.
     @see #message
     */
    public int getMessage()
    {
        return message;
    }

    /** Returns the message type.<p>
     @return the message type.
     */
    public int getMessageType()
    {
        return messageType;
    }

    /** Returns the message source.<p>
     @return The message source.
     @see #source
     */
    public Object getSource()
    {
        return source;
    }

    /** Sets the message.<p>
     @param _message The new message.
     */
    public void setMessage(int _message)
    {
        message = _message;
    }

    /** Sets the source.<p>
     @param object The new source.
     */
    public void setSource(Object object)
    {
        source = object;
    }

}