package com.ai_center.planpresentation.utilities;


import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import com.ai_center.dragonbreath.Registry;
import com.ai_center.dragonbreath.constraintgraph.Node;
import com.ai_center.dragonbreath.constraintgraph.Variable;
import com.ai_center.dragonbreath.constraints.PreconditionTaskObjectConstraint;
import com.ai_center.dragonbreath.constraints.StateTaskObjectConstraint;




/** This ConstraintTransferable can contain a PreconditionTask- or
 StateTaskObjectConstraint for Drag&Drop purpose. <p>
 
 This Transerable supports only local transfer within one virtual machine.

 @author Daniel Weiss
 @version $Revision: 1.1 $
 */
public class ConstraintTransferable implements Transferable
{


    /** The list of supported flavors. (Some type of dataformat).*/
    private List flavorList;

    /** The node to transfer. */
    private Node node;

    /** The list of supported flavors as array. */
    public DataFlavor[] flavors = new DataFlavor[1];


    /** Creates the Transferable object.
     @param _node The node to transfer.
     */
    public ConstraintTransferable(Node _node)
    {

        this.node = _node;

        // get the resource type of the connected constraint.
        int resourceTypeID = 0;
        Variable resourceTypeVariable;
        synchronized (node.connections)
        {
            resourceTypeVariable = node.connections
                    .getSpecificVariable(Registry.RESOURCETYPE_EDGE);
        }
        if (resourceTypeVariable != null)
        {
            resourceTypeID = (int) resourceTypeVariable.value();
        }

        // check if this a PreconditionTask or a StateTask
        String nodeType = "";
        if (node instanceof PreconditionTaskObjectConstraint)
        {
            nodeType = "PreconditionTaskObjectConstraint";
        }
        if (node instanceof StateTaskObjectConstraint)
        {
            nodeType = "StateTaskObjectConstraint";
        }

        //create the flavor. (The resource type and the TaskType is coded in
        //the flavor-name-string.
        DataFlavor df = new DataFlavor(
                com.ai_center.dragonbreath.constraintgraph.ConventionalConstraint.class,
                nodeType + "-ResourceTypeID=" + resourceTypeID);

        // make the flavor lists.
        flavors[0] = df;
        flavorList = Arrays.asList(flavors);

    }

    /*
     *  (non-Javadoc)
     * @see java.awt.datatransfer.Transferable#getTransferData(java.awt.datatransfer.DataFlavor)
     */public synchronized Object getTransferData(DataFlavor flavor)
            throws UnsupportedFlavorException, IOException
    {
        return node;
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
     */
    public synchronized DataFlavor[] getTransferDataFlavors()
    {

        return flavors;
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(java.awt.datatransfer.DataFlavor)
     */
    public boolean isDataFlavorSupported(DataFlavor flavor)
    {
        return (flavorList.contains(flavor));
    }


}