package com.ai_center.planpresentation.utilities;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.ai_center.dragonbreath.Constraint;
import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PlanPresentationConstants;
import com.ai_center.planpresentation.SwimlaneControl;
import com.ai_center.planpresentation.plugins.RetractableView;
import com.ai_center.planpresentation.plugins.actionresourceconstraint.ActionResourceCoordinatorPlugin;
import com.ai_center.planpresentation.plugins.stateresourceconstraint.StateResourceCoordinatorPlugin;




/**  The DefaultResourceCoordinatorControl coordinates one ore more 
 controllers. <p>
 
 It is specialized to DefaultResourceControllers. For more details see the
 description at the Controller interface.
 
 @author Daniel Weiss
 @version $Revision: 1.4 $
 @see com.ai_center.planpresentation.plugins.Controller
 */

public abstract class DefaultResourceCoordinatorControl
        implements
            PlanPresentationConstants,
            ChangeListener
{


    /** The keys are the Constraints and values the indexes in the 
     stateResourceList. */
    protected Map constraintIndexMap = new HashMap();

    /** The JList where the controllers are listed for a overview.*/
    protected JList defaultResourceList = null;

    /** The listmodel which contains all controllers. */
    protected ChangeListenerListModel defaultResourceModel = new ChangeListenerListModel();


    /** The mouselistener to track clicks on the JList. */
    MouseListener mouseListener = new MouseAdapter()
    {


        /*
         *  (non-Javadoc)
         * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
         */
        public void mouseClicked(MouseEvent e)
        {
            // if it is clicked one or more times. 
            // (if doubleclick is required, change value to 2)
            if (e.getClickCount() >= 2)
            {
                // get the index, of the element which was clicked
                int index = defaultResourceList.locationToIndex(e.getPoint());
                ResourceController resourceController = (ResourceController) defaultResourceModel
                        .get(index);
                // inform the element, that it was clicked.
                resourceController.onClick();

            }
        }
    };

    /** The link to the planPresentationAPI. */
    protected PlanPresentationAPI planPresentationAPI;

    /** The actual sortmode. */
    protected int sortMode = SORTMODE_GROUPED;

    /** Sets a new sort mode.<p>
     
     @param sortMode The new sortmode
     @see com.ai_center.planpresentation.PlanPresentationConstants#SORTMODE_GROUPED
     @see com.ai_center.planpresentation.PlanPresentationConstants#SORTMODE_UPDATE
     */
    private void setSortMode(int sortMode)
    {
        int priorityOffset = -1;
        switch (sortMode)
        {
            case SORTMODE_GROUPED :
                // we have to check if we have a StateResource or
                // an ActionResource, to give the correct
                // priorityOffset
                if (this instanceof StateResourceCoordinatorPlugin)
                {
                    priorityOffset = STATERESOURCE_GROUP;
                } else if (this instanceof ActionResourceCoordinatorPlugin)
                {
                    priorityOffset = ACTIONRESOURCE_GROUP;
                } else
                {
                    priorityOffset = 0;
                }

                break;
            case SORTMODE_UPDATE :
                // update mode: only the access time is crucial, set the offset
                // to zero
                priorityOffset = 0;

        }
        // inform all controllers about their new priority.
        for (int i = 0; i < defaultResourceModel.getSize(); i++)
        {
            ResourceController resourceController = (ResourceController) defaultResourceModel
                    .get(i);
            resourceController.setPriorityOffset(priorityOffset);
        }

    }

    /** Makes a new JList inside a JScrollPane.<p>

     Additional it adds a TitledBoader and adds MouseListener etc.
     @param title The title of this list.
     @return A ready JScrollPane to insert in a JPanel.
     */
    protected JScrollPane initDefaultResourceList(String title)
    {
        // new JList
        defaultResourceList = new JList();
        // link it with the model
        defaultResourceList.setModel(defaultResourceModel);

        // initialize JList
        defaultResourceList
                .setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        defaultResourceList.setAutoscrolls(true);

        // add a JScrollPanel
        JScrollPane defaultResourceScrollPane = new JScrollPane(
                defaultResourceList);

        // layout the JScrollPane
        defaultResourceScrollPane.setPreferredSize(new Dimension(146, 170));

        // add a title to the border of the JScrollPane
        TitledBorder titledBorder1 = new TitledBorder(BorderFactory
                .createEtchedBorder(Color.white, new Color(165, 163, 151)),
                title);
        defaultResourceScrollPane.setBorder(titledBorder1);
        defaultResourceScrollPane.setMinimumSize(new Dimension(146, 170));

        defaultResourceList.addMouseListener(mouseListener);
        return defaultResourceScrollPane;

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#absoluteY(int)
     */
    public int absoluteY(int relativeY)
    {
        return planPresentationAPI.absoluteY(relativeY);
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.CoordinatorControl#addConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public abstract void addConstraint(Constraint constraint);

    /** Delegate function. Adds a view.<p>
     
     @param view The view to add.
     @see com.ai_center.planpresentation.PlanPresentationAPI#addView(RetractableView)
     */
    public void addView(RetractableView view)
    {
        planPresentationAPI.addView(view);

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#addViewChangeListener(javax.swing.event.ChangeListener)
     */
    public void addViewChangeListener(ChangeListener l)
    {
        planPresentationAPI.addViewChangeListener(l);

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#engineStop()
     */
    public void engineStop()
    {
        planPresentationAPI.engineStop();

    }


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#getPanelWidth()
     */
    public int getPanelWidth()
    {
        return planPresentationAPI.getViewPanelWidth();
    }


    /** Returns a link to the PlanPresentationAPI.<p> 
     
     @return The link to the PlanPresentationAPI */
    public PlanPresentationAPI getPlanPresentationAPI()
    {
        return planPresentationAPI;
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#getSwimlaneControl()
     */
    public SwimlaneControl getSwimlaneControl()
    {
        return planPresentationAPI.getSwimlaneControl();
    }

    public abstract void initPlugin(PlanPresentationAPI _pc);


    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#removeConstraint(com.ai_center.dragonbreath.Constraint)
     */
    public abstract void removeConstraint(Constraint constraint);


    /** Delegate function. Removes a view.<p>
     
     @param view The view to be removed.
     @see com.ai_center.planpresentation.PlanPresentationAPI#removeView(RetractableView)
     */
    public void removeView(RetractableView view)
    {
        planPresentationAPI.removeView(view);

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#removeViewChangeListener(javax.swing.event.ChangeListener)
     */
    public void removeViewChangeListener(ChangeListener l)
    {
        planPresentationAPI.removeViewChangeListener(l);

    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Controller#scroll(int)
     */
    public void scroll(int scrollstep)
    {
        planPresentationAPI.scroll(scrollstep, false);

    }

    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent event)
    {

        if (event.getSource() instanceof ChangeMessage)
        {
            ChangeMessage message = (ChangeMessage) event.getSource();
            int messageType = message.getMessageType();

            switch (messageType)
            {
                case SORTMODE_CHANGED :
                    sortMode = planPresentationAPI.getSortMode();
                    // do sortmode update
                    setSortMode(sortMode);
                    break;

                default :
            // unknown message. ignore it

            }

        }
    }

    /* (non-Javadoc)
     * @see com.ai_center.planpresentation.plugins.Coordinator#updateView()
     */
    public void updateView()
    {
        planPresentationAPI.updateView();
    }
}