package com.ai_center.planpresentation.utilities;


import java.awt.Cursor;
import java.awt.LayoutManager;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

import com.ai_center.planpresentation.SwimlaneControl;




/** This is a dragable panel. <p>
 
 The panel can be moved inside the parent panel. It follows the mouse cursor
 after it was clicked.
 
 @author Daniel Weiss
 @version $$
 */

public class DragablePanel extends JPanel
        implements
            MouseListener,
            MouseMotionListener
{


    /** The x-value, when the draggin was started. */
    private int dragStartx;

    /** Theyx-value, when the draggin was started. */
    private int dragStarty;

    /** A tracing variable, if we are in drag mode or not. */
    private boolean isDragging = false;

    /** The link to the SwimlaneControl. Needed to paint vertical lines at the
     cursor.  */
    protected SwimlaneControl swimlaneControl;


    /** Creates the DragablePanel. */
    public DragablePanel()
    {

        this.addMouseListener(this);
        this.addMouseMotionListener(this);

        this.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
    }


    /** Creates a Dragable Panel with a special Layoutmanager.<p>
     * 
     @param l The LayoutManager.
     */
    public DragablePanel(LayoutManager s)
    {
        super(s);

        this.addMouseListener(this);
        this.addMouseMotionListener(this);

        this.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
    }

    /** Prunes a string, if it is too long.<p>
     
     Additional it adds dots, if the string was pruned.<p>
     @param text The text to check.
     @param width The maximal with of the text.
     @return The ready text.
     */
    protected String prepareText(String text, int width)
    {
        if (text != null)
        {
            // 8 pixel per letter
            int maxchars = (width - 10) / 8; 
            try
            {
                
                if (maxchars > text.length())
                {
                    // the text is below max width, so do nothing
                    return text;
                } else if (maxchars > 2)
                {
                    // the text must be pruned.
                    return text.substring(0, maxchars - 2) + "..";
                } else
                {
                    // the width is two small, return nothing.
                    return "";
                }
            } catch (Exception e)
            {
                System.out.println("Textconversion error: " + e.getMessage());
            }
        }
        return null;

    }

    /** Returns true, when in dragging mode..<p>
     
       @return True if in dragging mode. */
    public boolean isDragging()
    {
        return isDragging;
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent evt)
    {
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent evt)
    {
        int x = this.getLocation().x + evt.getX() - dragStartx; // + locationStartx - dragStartx;  
        int y = this.getLocation().y + evt.getY() - dragStarty; // + locationStarty - dragStarty; //

        this.setLocation(x, y);
        // request a vertical line at the cursor
        if (swimlaneControl != null)
        {
            swimlaneControl.paintVerticalLine(x - 1);
        }

    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent evt)
    {
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent evt)
    {
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    public void mouseMoved(MouseEvent evt)
    {
    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent evt)
    {
        dragStartx = dragStartx + evt.getX();
        dragStarty = dragStarty + evt.getY();
        this.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
        isDragging = true;

    }

    /*
     *  (non-Javadoc)
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent evt)
    {
        isDragging = false;
        dragStartx = 0;
        dragStarty = 0;

        this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
};
