package com.ai_center.planpresentation.utilities;


import java.util.Map;




/** This MultiLevelController works in context with the MultiLevelView.<p>
 
 It is one of three parts of a bottom-level-agent at the pac-architectur.
 A controller, which instantiates a multiLevelView must implement this
 interface, so that the view can communicate with this controller. 
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 @see com.ai_center.planpresentation.plugins.Controller
 */

public interface MultiLevelController
{


    /** Returns the absolut y value of the coordinate.<p>
     
     absoluteY(relativeY) computes the absolute value of y values in the
     planning area.<p>
     
     @param relativeY The local y value in the planning area 
     @return the absolute y value in the JFrame.
     */
    int absoluteY(int relativeY);

    /** Returns the actual maximum level from the multiLevelModel.<p> 
     
     @return The actual maximum level.
     @see com.ai_center.planpresentation.utilities.MultiLevelModel#getLevel
     */
    int getLevel();


    /** Returns the actual levelcount from the multiLevelModel.<p>
     
     @return The actual levelcount.
     @see com.ai_center.planpresentation.utilities.MultiLevelModel#getLevelcount
     */
    int getLevelcount();

    /** Returns all objects which are on this level and in this interval.<p>
     
     @param level The level from where the objects are requested.
     @param mainViewModelValue The left boundary tick.
     @param mainViewModelExtent The right boundary tick.
     */
    Map getObjectsOnLevel(int level, long mainViewModelValue,
            long mainViewModelExtent);

    /** Requests a full view update.<p>
     
     e.g.when the vertical size of the view has changed is this call necessary.<p>
     */
    void updateView();
}