package com.ai_center.planpresentation.utilities;


import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;




/** This class is used to remember objects and recognize which are new
 and which were not used..<p>
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class PanelCache
{


    /**  Mapping from objects to panels  */
    private Map objects = Collections.synchronizedMap(new HashMap());

    /** A map with the used objects and panels. */
    private Map usedObjects = Collections.synchronizedMap(new HashMap());

    /** Adds an object-panel pair to the cache and marks it as used.
     
     @param obj The object to represent.
     @param panel The panel which represents the object  */
    public synchronized void addObject(Object obj, Object panel)
    {
        objects.put(obj, panel);
        usedObjects.put(obj, panel);

    };

    /** Returns the value to the give object and marks this object as used.
     @param obj The object from the datamodel.
     @return value The corresponding panel
     */
    public synchronized Object getObject(Object obj)
    {
        Object panel = objects.get(obj);
        if (panel != null)
        {
            usedObjects.put(obj, panel);
        }

        return panel;
    }

    /** Returns a list of objects, which were not used since the last
     getRemainingPanel()-call.<p>
     @return A Collections of not used panels.
     */
    public synchronized Collection getRemainingPanels()
    {

        List deleteObjects = new LinkedList();
        // get all keys from object list
        Set keys = objects.keySet();

        Iterator keyIterator = keys.iterator();
        while (keyIterator.hasNext())
        {
            Object obj = null;

            obj = keyIterator.next();

            //check if this object was used in this round
            if (usedObjects.get(obj) == null)
            {
                // no, it was not used
                Object panel = objects.get(obj);
                // add to delete list
                deleteObjects.add(panel);
                // remove object from cache
                keyIterator.remove();
            }

        }

        // reset used objects 
        usedObjects = new HashMap();

        return Collections.unmodifiableList(deleteObjects);
    }

}