package com.ai_center.planpresentation.utilities;


import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PluginLoader;
import com.ai_center.planpresentation.plugins.Plugin;




/** This PluginLoader loads all plugins defined in the array.<p>
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class PluginInitConstant implements PluginLoader
{


    private static final String[] classnames = {
            "com.ai_center.planpresentation.plugins.planplugin.PlanPlugin",
            "com.ai_center.planpresentation.plugins.actionconstraint.ActionCoordinatorControlPlugin",
            "com.ai_center.planpresentation.plugins.actionresourceconstraint.ActionResourceCoordinatorPlugin",
            "com.ai_center.planpresentation.plugins.stateresourceconstraint.StateResourceCoordinatorPlugin",
            "com.ai_center.planpresentation.plugins.transitionconstraint.TransitionCoordinatorPlugin"};

    private PlanPresentationAPI planPresentationAPI = null;


    /** Loads all plugins */
    public void loadPlugins()
    {
        if (planPresentationAPI == null)
            throw new Error(
                    "PluginInitDirectory:PluginController not defined. Call setPluginController first!");
        if (classnames != null)
        {
            for (int i = 0; i < classnames.length; i++)
            {
                Plugin plugin = null;
                try
                {
                    // load class
                    Class pluginClass = Class.forName(classnames[i]);

                    // instantiate it
                    plugin = (Plugin) pluginClass.newInstance();

                } catch (Exception e)
                {
                    System.out.println("Exception: Can't instantiate "
                            + classnames[i] + " : " + e.getMessage());
                }


                if (plugin != null)
                {
                    // call the initPlugin function
                    plugin.initPlugin(planPresentationAPI);
                }
            }
        }

    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.PluginLoader#setPlanPresentationAPI(com.ai_center.planpresentation.PlanPresentationAPI)
     */public void setPlanPresentationAPI(PlanPresentationAPI pc)
    {
        planPresentationAPI = pc;
    }

}