package com.ai_center.planpresentation.utilities;


import java.io.File;

import com.ai_center.planpresentation.PlanPresentationAPI;
import com.ai_center.planpresentation.PluginLoader;
import com.ai_center.planpresentation.plugins.Plugin;




/** This PluginLoader parses directories to find plugins.<p>
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public class PluginInitDirectory implements PluginLoader
{


    private PlanPresentationAPI planPresentationAPI = null;

    /** Parses the directories.<p>
     
     loadClassDirectory parses the  directory and all subdirectories to find
     all *Plugin.class files and instantiates them.
     TODO Known Bugs: Fileseparator hard coded with "/", because problems with replaceAll.
     
     @param directoryName The directory to parse
     */
    private void loadClassDirectory(String directoryName)
    {

        File directory = new File(directoryName);
        File tempFile = null;
        String[] classnames = directory.list();
        if (classnames != null)
        {
            for (int i = 0; i < classnames.length; i++)
            {

                tempFile = new File(directoryName + File.separator
                        + classnames[i]);
                // check if it is a directory or a file
                if (tempFile.isDirectory())
                {

                    // go recursive into directory structure
                    loadClassDirectory(directoryName + "/" + classnames[i]);

                } else
                {
                    // it is a file
                    // check if filename ends with *Plugin.class
                    if (classnames[i].endsWith("Plugin.class")
                            && !classnames[i]
                                    .equals("CoordinatorControl.class"))
                    {

                        // autoload file found. Load class and instantiate it.
                        String classname = directoryName + "/"
                                + classnames[i].replaceAll(".class", "");
                        Plugin plugin = null;
                        try
                        {

                            // compute classname from filenames.
                            String packageclass = classname
                                    .replaceAll("/", ".");

                            // load class
                            Class pluginClass = Class.forName(packageclass);

                            // instantiate it
                            plugin = (Plugin) pluginClass.newInstance();

                        } catch (Exception e)
                        {
                            System.out.println("Exception: Can't instantiate "
                                    + classname + " : " + e.getMessage());
                        }


                        if (plugin != null)
                        {
                            // call the initPlugin function
                            plugin.initPlugin(planPresentationAPI);
                        }
                    }
                }
            }
        }

    }

    /** calls the loadClassDirectory function with the correct directoryname  */
    public void loadPlugins()
    {
        if (planPresentationAPI == null)
            throw new Error(
                    "PluginInitDirectory:PluginController not defined. Call setPluginController first!");
        loadClassDirectory("com/ai_center/planpresentation/plugins");
    }


    /*
     *  (non-Javadoc)
     * @see com.ai_center.planpresentation.PluginLoader#setPlanPresentationAPI(com.ai_center.planpresentation.PlanPresentationAPI)
     */public void setPlanPresentationAPI(PlanPresentationAPI pc)
    {
        planPresentationAPI = pc;
    }

}