package com.ai_center.planpresentation.utilities;


import java.awt.Color;




/** The SemanticGraphic interface is for gui objects. <p>
 
 It is mainly used in context with the MultiLevelView and 
 MultiLevelModel.
 
 @author Daniel Weiss
 @version $Revision: 1.2 $
 */
public interface SemanticGraphic
{


    /** @return The beginCT of this object.  */
    long getBeginCT();

    /** @return The duration of this object.  */
    long getDuration();

    /** @return true if this object is dragging in this moment. */
    boolean isDragging();

    /** Sets the backgroundcolor.<p> 
     
     @param color The new background color.
     */
    void setBackground(Color color);

    /** Sets a new location.<p>
        	
       @param x The x-value.
       @param y The y-value.
     */
    void setLocation(int x, int y);

    /** Sets shouldVisible from the datamodel point of view.<p>
       
       @param shouldVisible The shouldVisible-value
     */
    void setShouldVisible(boolean shouldVisible);

    /** Sets a new size for the SemanticGraphic.<p>
      	
       @param x The width.
       @param y The height.
     */
    void setSize(int x, int y);

    /** Sets the SwimlaneVisibility.<p>
     
      @param b True if the swimlane should be visible
     */
    void setSwimlaneVisibility(boolean b);

    
    /** Gets the shouldVisible value.<p>
       
       @return True if this SemanticGraphic should be visible from the
       datamodel point of view.
     */
    boolean shouldVisible();

    /** Terminates the SemanticGraphic object. */
    void terminate();
}