
# The DragonBreath Development Tool
# Author: Alexander Nareyek


### Setting default configuration values

set engine_directory "C:/Work/DragonBreath/engine/"
set classpath ".;org/apache/xerces/xerces.jar;org/jdom/jdom.jar"
set wc_command "C:/AppLinks/wc.exe"
set startup_filename "StartExamples 3"
set display_width 80
set display_height 30


### Setting internal configuration values

set required_configuration_file_version 1
set configuration_file_version -1


### Window Creation

wm title . "DragonBreath Development Tool"

frame .menu
frame .buttons
frame .display

# Menu

frame .menu.f -relief groove -borderwidth 2

tk_menuBar .menu.f .menu.f.file
menubutton .menu.f.file -text File -menu .menu.f.file.menu

menu .menu.f.file.menu
.menu.f.file.menu add command -label Exit -command exit

tk_menuBar .menu.f .menu.f.info
menubutton .menu.f.info -text Info -menu .menu.f.info.menu

menu .menu.f.info.menu
.menu.f.info.menu add command -label "Line Count" -command line_count

tk_menuBar .menu.f .menu.f.display
menubutton .menu.f.display -text Display -menu .menu.f.display.menu

menu .menu.f.display.menu
.menu.f.display.menu add command -label "Increase Display Width" -command {
    incr display_width 10
    eval $displaypath configure -width $display_width
}
.menu.f.display.menu add command -label "Decrease Display Width" -command {
    incr display_width -10
    if {$display_width < 10} {
	set display_width 10
    }
    eval $displaypath configure -width $display_width
}
.menu.f.display.menu add command -label "Increase Display Height" -command {
    incr display_height 5
    eval $displaypath configure -height $display_height
}
.menu.f.display.menu add command -label "Decrease Display Height" -command {
    incr display_height -5
    if {$display_height < 5} {
	set display_height 5
    }
    eval $displaypath configure -height $display_height
}

pack .menu.f.file .menu.f.info .menu.f.display -side left -ipadx 1m
pack .menu.f -padx 1m -pady 1m -fill x 

# Buttons

frame .buttons.f

button .buttons.f.compile -text " Compile " -command compile
pack .buttons.f.compile -side left -padx 2m

button .buttons.f.compile_run -text " Compile & Run " -command compile_and_run
pack .buttons.f.compile_run -side left -padx 2m

button .buttons.f.run -text " Run " -command { 
    run $startup_filename 
}
pack .buttons.f.run -side right

entry .buttons.f.filename -width 20 -font courier -relief sunken -bd 2 -textvariable startup_filename
bind .buttons.f.filename <Return> { 
    .buttons.f.run invoke 
}
pack .buttons.f.filename -side right

pack .buttons.f -padx 5m -pady 2m -fill x

# Display

set displaypath ".display.f.text"

frame .display.f
text .display.f.text -yscroll ".display.f.scrollbar set" -font courier -foreground "dark blue" -width $display_width -height $display_height
scrollbar .display.f.scrollbar -command ".display.f.text yview"
pack .display.f.text .display.f.scrollbar -side left -fill y
pack .display.f -padx 1m -pady 2m

pack .menu .buttons .display -fill x


### Functions


proc display {text} {
    global displaypath
    eval $displaypath insert end {$text\n}
    update idletasks
}


set display_tag_counter 0

proc display_with_tag {text} {

    global display_tag_counter
    global displaypath

    set begin "[eval $displaypath index end] - 1 lines"
    eval $displaypath insert end {$text\n}
    set end "[eval $displaypath index end] - 1 lines"
    eval $displaypath tag add tag$display_tag_counter {$begin} {$end}
    incr display_tag_counter

    update idletasks

}


proc change_color {color} {

    global display_tag_counter
    global displaypath

    set tag tag[expr $display_tag_counter - 1]
    eval $displaypath tag configure $tag -foreground $color

    update idletasks

}


proc change_boldcolor {color} {

    global display_tag_counter
    global displaypath

    set tag tag[expr $display_tag_counter - 1]
    eval $displaypath tag configure $tag -foreground $color
    eval $displaypath tag configure $tag -font bold

    update idletasks

}


proc display_errorinfo {} {

    global display_tag_counter
    global displaypath
    global errorInfo

    set info $errorInfo
    set end [expr [string first "   while executing" $info] - 3]

    display_with_tag [string range $info 0 $end]
    change_color black

}


proc clear_display {} {

    global display_tag_counter
    global displaypath

    for {set i 0} {$i < $display_tag_counter} {incr i} {
	eval $displaypath tag delete tag$i
    }
    set display_tag_counter 0

    eval $displaypath delete 1.0 end
    display ""

    update idletasks

}


proc compile {} {

    global directories
    global classpath

    clear_display
    display "Compiling source files..."
    display ""

    set all ""
    foreach dir $directories {
	append all "$dir*java "
    }

    if {[catch { 
	eval exec javac -classpath {$classpath} $all
    }]} {
	display_with_tag "Source code contains errors!"
	change_boldcolor red
	display ""
	display_errorinfo
    } else {
	display_with_tag "Compilation successful."
	change_boldcolor green
    }

    display ""
    display "Done."

}


proc run {filename} {

    global classpath

    clear_display
    display "Running \"$filename\"..."
    display ""

    if {[catch { 
	set result [eval exec java -classpath {$classpath} $filename]
    }]} {
	display_with_tag "Execution error!"
	change_boldcolor red
	display ""
	display_errorinfo
    } else {
	display_with_tag "Execution successful."
	change_boldcolor green
	display ""
	display_with_tag $result
	change_color black
    }

    display ""
    display "Done."

}


proc compile_and_run {} {

    global directories
    global classpath
    global startup_filename

    clear_display
    display "Compiling source files..."
    display ""

    set all ""
    foreach dir $directories {
	append all "$dir*java "
    }

    if {[catch { 
	eval exec javac -classpath {$classpath} $all
    }]} {
	display_with_tag "Source code contains errors!"
	change_boldcolor red
	display ""
	display_errorinfo
    } else {

	display_with_tag "Compilation successful."
	change_boldcolor green
	display ""
	display "Running \"$startup_filename\"..."
	display ""

	if {[catch { 
	    set result [eval exec java -classpath {$classpath} $startup_filename]
	}]} {
	    display_with_tag "Execution error!"
	    change_boldcolor red
	    display ""
	    display_errorinfo
	} else {
	    display_with_tag "Execution successful."
	    change_boldcolor green
	    display ""
	    display_with_tag $result
	    change_color black
	}

    }

    display ""
    display "Done."

}


proc line_count {} {

    global directories
    global wc_command

    clear_display
    display "Processing line-count information..."
    display ""

    set output ""
    set source_lines 0

    if {[catch {
	foreach dir $directories {
	    append output "\n\n~/$dir*java:\n\n"
	    append output "[exec $wc_command -l $dir*java]"
	    incr source_lines [lindex $output [expr [llength $output] - 2]]
	}
    }]} {
	display_with_tag "Line-count command execution error!"
	change_boldcolor red
	display ""
	display_errorinfo
    } else {

	display_with_tag "Line-count command execution successful."
	change_boldcolor green
	display ""
	set output "All source files (*.java): $source_lines lines$output"
	display_with_tag $output
	change_color black

    }

    display ""
    display "Done."

}


### Initialization

set directories {}

proc add_directory {dir_name} {
    global directories
    lappend directories $dir_name
}

add_directory ""
add_directory "com/ai_center/dragonbreath/"
add_directory "com/ai_center/dragonbreath/constraintgraph/"
add_directory "com/ai_center/dragonbreath/constraints/"
add_directory "com/ai_center/dragonbreath/constraints/discreteresourceconstraint/"
add_directory "com/ai_center/dragonbreath/constraints/stateresourceconstraint/"
add_directory "com/ai_center/dragonbreath/constraints/stateresourceconstraint/numericalheapsrc/"
add_directory "com/ai_center/dragonbreath/constraints/stateresourceconstraint/symbolicsrc/"
add_directory "com/ai_center/dragonbreath/constraints/taskconstraint/"
add_directory "com/ai_center/dragonbreath/constraints/transitionconstraint/"
add_directory "com/ai_center/dragonbreath/costs/"
add_directory "com/ai_center/dragonbreath/examples/interaction/"
add_directory "com/ai_center/dragonbreath/examples/scheduling/"
add_directory "com/ai_center/dragonbreath/examples/specificationbasics/"
add_directory "com/ai_center/dragonbreath/examples/structuralConstraintExample/"
add_directory "com/ai_center/dragonbreath/examples/structuralConstraintExample/fluffExampleConstraints/"
add_directory "com/ai_center/dragonbreath/examples/taskconstraintdemo/"
add_directory "com/ai_center/dragonbreath/examples/utilities/"
add_directory "com/ai_center/dragonbreath/graphRepair/"
add_directory "com/ai_center/dragonbreath/loader/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/applicationCondition/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/dockingGraph/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/dockingGraph/applicationCondition/instance/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/dockingGraph/applicationCondition/type/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/movableSubgraph/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/testGraph/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/testGraph/applicationCondition/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/testGraph/applicationCondition/instance/"
add_directory "com/ai_center/dragonbreath/structuralConstraintHandling/graph/testGraph/applicationCondition/type/"
add_directory "com/ai_center/dragonbreath/utilities/"
add_directory "com/ai_center/planpresentation/"
add_directory "com/ai_center/planpresentation/plugins/"
add_directory "com/ai_center/planpresentation/plugins/actionconstraint/"
add_directory "com/ai_center/planpresentation/plugins/actionresourceconstraint/"
add_directory "com/ai_center/planpresentation/plugins/planplugin/"
add_directory "com/ai_center/planpresentation/plugins/stateresourceconstraint/"
add_directory "com/ai_center/planpresentation/plugins/stateresourceconstraint/numericalSRC/"
add_directory "com/ai_center/planpresentation/plugins/stateresourceconstraint/symbolicSRC/"
add_directory "com/ai_center/planpresentation/plugins/transitionconstraint/"
add_directory "com/ai_center/planpresentation/utilities/"


clear_display


### Potentially creating sample configuration file

set config_ok 1

proc create_sample_config {} {
    if {[catch { 
	set configfile [open config.ddt w]
	puts $configfile "\#\#\#"
	puts $configfile "\#\#\# Configuration file for the DragonBreath Development Tool"
	puts $configfile "\#\#\#"
	puts $configfile ""
	puts $configfile ""
	puts $configfile "\# Change engine_directory variable below to reflect the"
	puts $configfile "\# path to your engine directory."
	puts $configfile ""
	puts $configfile "set engine_directory \"C:/Work/DragonBreath/engine/\""
	puts $configfile ""
	puts $configfile ""
	puts $configfile "\# Change classpath variable below depending on your OS"
	puts $configfile "\# (Windows: \";\"  Unix: \":\") and further inclusion needs."
	puts $configfile ""
	puts $configfile "set classpath \".;org/apache/xerces/xerces.jar;org/jdom/jdom.jar\""
	puts $configfile ""
	puts $configfile ""
	puts $configfile "\# If you need additional directories to be compiled, please"
	puts $configfile "\# add for each one a line like:"
	puts $configfile "\# add_directory \"com/ai_center/dragonbreath/constraints/\""
	puts $configfile "\#"
	puts $configfile "\# Note that regular engine directories need to be included"
	puts $configfile "\# with this command in DragonBreath_Development_Tool.tcl"
	puts $configfile "\# instead."
	puts $configfile "\# (Don't forget to update the CVS repository in the latter"
	puts $configfile "\# case.)"
	puts $configfile ""
	puts $configfile ""
	puts $configfile "\# Point the wc_command variable below to the \"wc\" word"
	puts $configfile "\# count utility."
	puts $configfile ""
	puts $configfile "set wc_command \"C:/AppLinks/wc.exe\""
	puts $configfile ""
	puts $configfile ""
	puts $configfile "\# Change the default_filename variable below to set the"
	puts $configfile "\# default execution class file."
	puts $configfile ""
	puts $configfile "set default_filename \"StartExamples 3\""
	puts $configfile ""
	puts $configfile ""
	puts $configfile "\# Change width and height variables below to reflect your"
	puts $configfile "\# display preferences."
	puts $configfile ""
	puts $configfile "set display_width 80"
	puts $configfile "set display_height 30"
	puts $configfile ""
	puts $configfile ""
	puts $configfile "\#\#\#"
	puts $configfile "\#\#\# Do NOT change the version variable below!"
	puts $configfile "\#\#\#"
	puts $configfile "set configuration_file_version 1"
	close $configfile
    }]} {
	return 0
    } else {
	return 1
    }
}

if {![file exists config.ddt]} {

    display "Welcome to the DragonBreath Development Tool!"
    display ""

    if {[file exists DragonBreath_Development_Tool.tcl]} {
	display "Sample configuration file is created..."
	display ""
	if {[create_sample_config]} {
	    display_with_tag "Configuration file successfully created."
	    change_boldcolor green
	    display ""
	    display "Please review the file \"config.ddt\", adapt it to your environment, and"
	    display "restart the tool. Do NOT add the configuration file to the general CVS!"
	} else {
	    display_with_tag "Error during creation!"
	    change_boldcolor red
	    display ""
	    display_errorinfo
	}
    } else {
	display "Please restart the tool from the directory in which"
	display_with_tag "DragonBreath_Development_Tool.tcl"
	change_color green
	display "is located."
    }

    set config_ok 0

}


### Processing configuration file

if {$config_ok} {

    display "Loading configuration file..."
    display ""

    if {[catch { 
	source config.ddt	
    }]} {
	display_with_tag "Error during load process!"
	change_boldcolor red
	display ""
	display_errorinfo
	set config_ok 0
    } else {

	display_with_tag "Configuration file successfully loaded."
	change_boldcolor green
	display ""
	display "Processing configuration file..."
	display ""

	if {$required_configuration_file_version 
	    != $configuration_file_version} {

	    display_with_tag "Incompatible configuration file version detected!"
	    change_boldcolor red
	    display ""
	    display "Renaming old configuration file..."
	    display ""

	    if {[catch { 
		file rename config.ddt old_config.ddt
	    }]} {
		display_with_tag "File renaming error!"
		change_boldcolor red
		display ""
		display_errorinfo
	    } else {

		display_with_tag "Old configuration file successfully renamed."
		change_boldcolor green
		display ""
		display "Creating new configuration file..."
		display ""

		if {[create_sample_config]} {
		    display_with_tag "New configuration file successfully created."
		    change_boldcolor green
		    display ""
		    display "Please review the new file \"config.ddt\", adapt it to your environment,"
		    display "and restart the tool. The old configuration file has been renamed to"
		    display "\"old_config.ddt\"."
		} else {
		    display_with_tag "Error during creation!"
		    change_boldcolor red
		    display ""
		    display_errorinfo
		}

	    }

	    set config_ok 0

	} else {
	    eval $displaypath configure -width $display_width -height $display_height
	    set startup_filename $default_filename
	    display_with_tag "Configuration file successfully processed."
	    change_boldcolor green
	    display ""
	}

    }
    
}



### Changing to Engine Directory

if {$config_ok} {

    display "Entering engine directory..."
    display ""
    
    if {[catch { 
	cd $engine_directory
    }]} {
	display_with_tag "Changing directory not possible!"
	change_boldcolor red
	display ""
	display_errorinfo
    } else {
	display_with_tag "Directory successfully entered."
	change_boldcolor green
    }
    
}

display ""
display "Done."
